SystemOrganization addCategory: #'Tools-Base'!
SystemOrganization addCategory: #'Tools-Browser'!
SystemOrganization addCategory: #'Tools-Changes'!
SystemOrganization addCategory: #'Tools-ConfigurationBrowser'!
SystemOrganization addCategory: #'Tools-Debugger'!
SystemOrganization addCategory: #'Tools-Explorer'!
SystemOrganization addCategory: #'Tools-File Contents Browser'!
SystemOrganization addCategory: #'Tools-FileList'!
SystemOrganization addCategory: #'Tools-Inspector'!
SystemOrganization addCategory: #'Tools-Process Browser'!

!FileStream class methodsFor: '*Tools-FileList' stamp: 'nk 7/16/2003 15:49'!
fileReaderServicesForFile: fullName suffix: suffix
	"Answer services for the given file"

	^ ((self isSourceFileSuffix: suffix) or: [ suffix = '*' ])
		ifTrue:
			[{self serviceRemoveLineFeeds.
			self serviceFileIn}]
		ifFalse:
			[#()]! !

!FileStream class methodsFor: '*Tools-FileList' stamp: 'tbn 8/11/2010 10:18'!
serviceFileIn
	"Answer a service for filing in an entire file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'FileIn entire file'
		selector: #fileIn:
		description: 'File in the entire contents of the file, which is expected to contain Smalltalk code in fileout ("chunk") format'
		buttonLabel: 'Filein'! !

!FileStream class methodsFor: '*Tools-FileList' stamp: 'tbn 8/11/2010 10:33'!
serviceRemoveLineFeeds
	"Answer a service for removing linefeeds from a file"

	^ FileModifyingSimpleServiceEntry
		provider: self 
		label: 'Remove line feeds'
		selector: #removeLineFeeds:	
		description: 'Remove line feeds in file'
		buttonLabel: 'Remove lfs'! !

!FileStream class methodsFor: '*Tools-FileList' stamp: 'sd 2/1/2002 22:28'!
services

	^ Array 
			with: self serviceRemoveLineFeeds
			with: self serviceFileIn
	! !

!FileStream methodsFor: '*Tools' stamp: 'IgorStasenko 3/6/2011 18:52'!
edit
	"Create and schedule an editor on this file."

	Smalltalk tools fileList openEditorOn: self editString: nil.
! !

!ChangeSet class methodsFor: '*Tools-Changes' stamp: 'IgorStasenko 3/6/2011 18:18'!
browseChangedMessages
	"Create and schedule a message browser on each method that has been 
	changed."

	current isEmpty ifTrue: [^ self inform: 'There are no changed messages
in the current change set.'].
	Smalltalk tools openChangedMessageSet: current.! !

!ChangeSet class methodsFor: '*Tools-FileList' stamp: 'ar 7/15/2005 21:36'!
fileReaderServicesForFile: fullName suffix: suffix

	^ (FileStream isSourceFileSuffix: suffix)
		ifTrue: [ self services]
		ifFalse: [#()]! !

!ChangeSet class methodsFor: '*Tools-FileList' stamp: 'tbn 8/11/2010 10:11'!
serviceFileIntoNewChangeSet
	"Answer a service for installing a file into a new change set"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Install into new change set'
		selector: #fileIntoNewChangeSet:
		description: 'Install the file as a body of code in the image: create a new change set and file-in the selected file into it'
		buttonLabel: 'Install'! !

!ChangeSet class methodsFor: '*Tools-FileList' stamp: 'ar 7/15/2005 21:36'!
services
	^ Array with: self serviceFileIntoNewChangeSet! !

ListItemWrapper subclass: #ObjectExplorerWrapper
	instanceVariableNames: 'itemName parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!ObjectExplorerWrapper commentStamp: '<historical>' prior: 0!
Contributed by Bob Arning as part of the ObjectExplorer package.
!

!ObjectExplorerWrapper class methodsFor: 'as yet unclassified' stamp: 'RAA 6/21/1999 10:50'!
with: anObject name: aString model: aModel

	^self new 
		setItem: anObject name: aString model: aModel! !

!ObjectExplorerWrapper class methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 09:16'!
with: anObject name: aString model: aModel parent: aParent

	^self new 
		setItem: anObject name: aString model: aModel parent: aParent
! !

!ObjectExplorerWrapper methodsFor: 'converting' stamp: 'sge 4/12/2001 08:24'!
asString
	| explorerString string |
	explorerString := 
		[item asExplorerString]
			on: Error 
			do: ['<error in asExplorerString: evaluate "' , itemName , ' asExplorerString" to debug>'].
	string := itemName , ': ' , explorerString.
	(string includes: Character cr)
		ifTrue: [^ string withSeparatorsCompacted].
	^ string! !

!ObjectExplorerWrapper methodsFor: 'accessing' stamp: 'yo 8/27/2008 23:39'!
contents

	(item customizeExplorerContents) ifTrue: [^item explorerContents].
	"For all others, show named vars first, then indexed vars"
	^(item class allInstVarNames asOrderedCollection withIndexCollect: [:each :index |
		self class
			with: (item instVarAt: index)
			name: each
			model: item
			parent: self]) ,
	((1 to: item basicSize) collect: [:index |
		self class
			with: (item basicAt: index)
			name: index printString
			model: item
			parent: self])! !

!ObjectExplorerWrapper methodsFor: 'accessing' stamp: 'RAA 6/21/1999 11:27'!
hasContents

	^item hasContentsInExplorer
	
! !

!ObjectExplorerWrapper methodsFor: 'accessing' stamp: 'AlainPlantec 12/6/2009 22:13'!
icon
	"Answer a form to be used as icon"
	^ ObjectExplorer showIcons
		ifTrue: [item iconOrThumbnailOfSize: 16]
		ifFalse: [nil]! !

!ObjectExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 09:14'!
parent
	^parent! !

!ObjectExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 09:14'!
parent: anObject
	parent := anObject! !

!ObjectExplorerWrapper methodsFor: 'monitoring' stamp: 'nk 7/12/2003 18:28'!
refresh
	"hack to refresh item given an object and a string that is either an index or an instance variable name."
	[ | index |
		(model class allInstVarNames includes: itemName)
			ifTrue: [ item := model instVarNamed: itemName ]
			ifFalse: [ index := itemName asNumber.
				(index between: 1 and: model basicSize) ifTrue: [ item := model basicAt: index]]
	] on: Error do: [ :ex | item := nil ]! !

!ObjectExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 09:49'!
selector
	parent ifNil: [ ^nil ].
	^(parent withoutListWrapper class allInstVarNames includes: itemName) ifTrue: [ itemName asSymbol ]! !

!ObjectExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'RAA 6/21/1999 10:49'!
setItem: anObject name: aString model: aModel

	item := anObject.
	model := aModel.
	itemName := aString.! !

!ObjectExplorerWrapper methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 09:14'!
setItem: anObject name: aString model: aModel parent: itemParent
	parent := itemParent.
	self setItem: anObject name: aString model: aModel! !

ObjectExplorerWrapper subclass: #PointerExplorerWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!PointerExplorerWrapper commentStamp: 'avi 8/21/2004 19:58' prior: 0!
A subclass of ObjectExplorerWrapper for use with PointerExplorer.  #contents is overridden to work backwards: it returns wrappers for the objects pointing to item rather than for the objects that item points to.!

!PointerExplorerWrapper methodsFor: 'accessing' stamp: 'AndyKellens 6/11/2010 14:15'!
contents
	| objects |
	objects := item pointersToExcept: (Array with: self with: model).	
	^(objects reject: [:ea | ea class = self class])
		collect: [:ea| self class with: ea name: ea identityHash asString model: item]! !

!PointerExplorerWrapper methodsFor: 'testing' stamp: 'ab 8/22/2003 18:39'!
hasContents
	^true! !

!Morph methodsFor: '*Tools-ViewHierarchyExplorer' stamp: 'SW 5/22/2009 12:07'!
exploreViewHierarchy
	^ToolSet exploreViewHierarchy: self! !

!Set methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:33'!
inspectorClass 
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ SetInspector! !

!CompiledMethod methodsFor: '*Tools-Inspector' stamp: 'eem 5/15/2008 13:14'!
explorerContents
	"(CompiledMethod compiledMethodAt: #explorerContents) explore"
	
	^Array streamContents:
		[:s| | tokens |
		tokens := Scanner new scanTokens: (self headerDescription readStream skipTo: $"; upTo: $").
		s nextPut: (ObjectExplorerWrapper
						with: ((0 to: tokens size by: 2) collect:
								[:i| i = 0 ifTrue: [self header] ifFalse: [{tokens at: i - 1. tokens at: i}]])
						name: 'header'
						model: self).
		(1 to: self numLiterals) do:
			[:key|
			s nextPut: (ObjectExplorerWrapper
							with: (self literalAt: key)
							name: ('literal', key printString contractTo: 32)
							model: self)].
		self isQuick
			ifTrue: [s nextPut: (ObjectExplorerWrapper
									with: self symbolic
									name: #symbolic
									model: self)]
			ifFalse:
				[self symbolicLinesDo:
					[:pc :line|
					pc <= 1
						ifTrue:
							[s nextPut: (ObjectExplorerWrapper
											with: line
											name: 'pragma'
											model: self)]
						ifFalse:
							[s nextPut: (ObjectExplorerWrapper
											with: line
											name: pc printString
											model: self)]]].
				"should be self numLiterals + 1 * Smalltalk wordSize + 1"
		self endPC + 1
			to: self basicSize
			do: [:key|
				s nextPut: (ObjectExplorerWrapper
								with: (self basicAt: key)
								name: key printString
								model: self)]]! !

!CompiledMethod methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:32'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ CompiledMethodInspector! !

!Number methodsFor: '*Tools-Inspector' stamp: ''!
defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the receiver."

	^ super defaultLabelForInspector, ': ', self printString! !

!Behavior methodsFor: '*Tools-Inspector' stamp: 'sw 5/21/2001 22:51'!
inspectAllInstances 
	"Inpsect all instances of the receiver.  1/26/96 sw"

	| all allSize prefix |
	all := self allInstances.
	(allSize := all size) == 0 ifTrue: [^ self inform: 'There are no 
instances of ', self name].
	prefix := allSize == 1
		ifTrue: 	['The lone instance']
		ifFalse:	['The ', allSize printString, ' instances'].
	
	all asArray inspectWithLabel: (prefix, ' of ', self name)! !

!Behavior methodsFor: '*Tools-Inspector' stamp: 'sw 5/21/2001 22:51'!
inspectSubInstances 
	"Inspect all instances of the receiver and all its subclasses.  CAUTION - don't do this for something as generic as Object!!  1/26/96 sw"

	| all allSize prefix |
	all := self allSubInstances.
	(allSize := all size) == 0 ifTrue: [^ self inform: 'There are no 
instances of ', self name, '
or any of its subclasses'].
	prefix := allSize == 1
		ifTrue: 	['The lone instance']
		ifFalse:	['The ', allSize printString, ' instances'].
	
	all asArray inspectWithLabel: (prefix, ' of ', self name, ' & its subclasses')! !

!WeakSet methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:33'!
inspectorClass 
	^ WeakSetInspector! !

SystemWindow subclass: #PreDebugWindow
	instanceVariableNames: 'proceedButton debugButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!PreDebugWindow commentStamp: 'LaurentLaffont 3/15/2011 20:51' prior: 0!
I am the window that shows up when the debugger is launched.  I show you the first lines of the stack trace that caused my apparition, and buttons to allow you to:
 - resume the execution
 - abandon the execution
 - debug this error
 - create the method if it is MessageNotUnderstood exception

To see me, evaluate:
2/0!

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'nice 3/5/2010 22:40'!
adjustBookControls
	| inner |
	proceedButton ifNil: [^ self].
	proceedButton align: proceedButton topLeft with: (inner := self innerBounds) topLeft + (35 @ -4).
	debugButton align: debugButton topRight with: inner topRight - (16 @ 4).! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'nk 2/12/2003 23:00'!
createMethod
	model createMethod! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 10/15/1998 13:00'!
debug
	model debug! !

!PreDebugWindow methodsFor: 'geometry' stamp: 'sw 11/4/1998 09:50'!
extent: newExtent
	super extent: (newExtent max: 100 @ 50).
	self adjustBookControls! !

!PreDebugWindow methodsFor: 'initialization' stamp: 'tbn 7/29/2010 21:30'!
initialize
	| aFont proceedLabel debugLabel aWidth |
	super initialize.
	true 
		ifFalse: 
			[
			(aWidth := self widthOfFullLabelText) > 280 ifTrue: [^self].	"No proceed/debug buttons if title too long"
			debugLabel := aWidth > 210 
				ifTrue: 
					["Abbreviated buttons if title pretty long"

					proceedLabel := 'p'.
					'd']
				ifFalse: 
					["Full buttons if title short enough"

					proceedLabel := 'proceed'.
					'debug'].
			aFont := StandardFonts buttonFont.
			self addMorph: (proceedButton := (SimpleButtonMorph new)
								borderWidth: 0;
								label: proceedLabel font: aFont;
								color: Color transparent;
								actionSelector: #proceed;
								target: self).
			proceedButton setBalloonText: 'Continue execution'.
			self addMorph: (debugButton := (SimpleButtonMorph new)
								borderWidth: 0;
								label: debugLabel font: aFont;
								color: Color transparent;
								actionSelector: #debug;
								target: self).
			debugButton setBalloonText: 'Bring up a debugger'.
			proceedButton submorphs first color: Color blue.
			debugButton submorphs first color: Color red].
	self adjustBookControls! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/11/2011 17:26'!
paneColor
	^ Color veryLightGray! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'sw 10/15/1998 13:00'!
proceed
	model proceed! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'tbn 7/29/2010 21:30'!
setBalloonTextForCloseBox
	closeBox ifNotNil:
		[closeBox setBalloonText: 'Abandon this execution by closing this window' translated].
! !

!PreDebugWindow methodsFor: 'label' stamp: 'AlainPlantec 12/1/2009 22:50'!
setLabelWidgetAllowance
	^ labelWidgetAllowance := CodeHolder optionalButtons
				ifTrue: [super setLabelWidgetAllowance]
				ifFalse: [180]! !

!PreDebugWindow methodsFor: 'as yet unclassified' stamp: 'mir 11/10/2003 15:15'!
storeLog
	model storeLog! !

!Integer methodsFor: '*Tools-Explorer' stamp: 'laza 3/17/2005 13:37'!
explorerContents
	^{
		'hexadecimal' -> 16.
		'octal' -> 8.
		'binary' -> 2
	} collect: [:each |
		ObjectExplorerWrapper with: each key translated name: (self printStringBase: each value) model: self]! !

!Integer methodsFor: '*Tools-Explorer' stamp: 'laza 3/17/2005 13:38'!
hasContentsInExplorer
	^true! !

!SystemNavigation methodsFor: '*Tools' stamp: 'BenjaminVanRyseghem 11/6/2010 22:31'!
allUnsentMessagesWithProgressBar
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBarForAllUnsentMessages: [:class :selector| 
			unsentMessages add: (SourcedMethodReference class: class selector: selector)].
	^unsentMessages! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 11/6/2010 22:31'!
browseAllAccessesTo: instVarName from: aClass
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."

	"self new browseAllAccessesTo: 'contents' from: Collection."
	
	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsAccess: instVarName) do: [:sel |
				sel isDoIt ifFalse: [
					coll add: (
						SourcedMethodReference new
							setClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self 
		browseMessageList: coll 
		name: 'Accesses to ' , instVarName 
		autoSelect: instVarName! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/6/2011 15:20'!
browseAllCallsOn: aLiteral 
	"Create and schedule a message browser on each method that refers to 
	aLiteral. For example, SystemNavigation new browseAllCallsOn: #open:label:."

	self headingAndAutoselectForLiteral: aLiteral do:
		[:label :autoSelect|
		self
			browseMessageList: (self allCallsOn: aLiteral) asSortedCollection
			name: label
			autoSelect: autoSelect]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'nk 6/26/2003 22:32'!
browseAllCallsOn: literal1 and: literal2 
	"Create and schedule a message browser on each method that calls on the 
	two Symbols, literal1 and literal2. For example, SystemNavigation new 
	browseAllCallsOn: #at: and: #at:put:."

	^self 
		browseMessageList: (self allCallsOn: literal1 and: literal2)
		name: literal1 printString , ' -and- ' , literal2 printString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'eem 8/30/2010 11:27'!
browseAllCallsOn: aLiteral from: aBehavior
	"Create and schedule a Message Set browser for
	 all the methods that call on aLiteral within aBehavior."

	"self new browseAllCallsOn: #/ from: Number"

	^self headingAndAutoselectForLiteral: aLiteral do:
		[:label :autoSelect|
		self 
			browseMessageList: (self  allCallsOn: aLiteral from: aBehavior)
			name: label, ' from ', aBehavior name
			autoSelect: autoSelect]

	! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'eem 8/30/2010 11:15'!
browseAllCallsOn: aLiteral localTo: aClass
	"Create and schedule a message browser on each method in or below the given class that refers to
	aLiteral. For example, SystemNavigation new browseAllCallsOn: #open:label: localTo: CodeHolder."

	aClass ifNil: [ ^self inform: 'no selected class' ].
	self headingAndAutoselectForLiteral: aLiteral do:
		[:label :autoSelect|
		self browseMessageList: (aClass allLocalCallsOn: aLiteral) asSortedCollection
			name: label, ' local to ', aClass name
			autoSelect: autoSelect]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'eem 8/30/2010 11:16'!
browseAllCallsOn: aLiteral localToPackage: packageNameOrInfo
	"Create and schedule a message browser on each method in the given package
	 that refers to aLiteral. For example,
		SystemNavigation new browseAllCallsOn: #open:label: localToPackage: 'Tools'."

	self headingAndAutoselectForLiteral: aLiteral do:
		[:label :autoSelect|
		self browseMessageList: (self allCallsOn: aLiteral localToPackage: packageNameOrInfo) asSortedCollection
			name: label, ' local to package ', (self packageInfoFor: packageNameOrInfo) name
			autoSelect: autoSelect]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 4/4/2011 13:38'!
browseAllCallsOnClass: aClass
	"Create and schedule a message browser on each method that refers to 
	aClass. For example, SystemNavigation new browseAllCallsOnClass: Object."
	self
		browseMessageList: ((aClass allCallsOnIn: self) asSortedCollection)
		name: 'Users of class ' , aClass theNonMetaClass name
		autoSelect: aClass theNonMetaClass name! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/16/2011 14:03'!
browseAllImplementorsOf: selector 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is the argument, selector. For example,  
	Smalltalk browseAllImplementorsOf: #at:put:."
	^self browseImplementorsOf: selector name: 'Implementors of ' , selector autoSelect: selector! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 7/21/2010 12:37'!
browseAllImplementorsOf: selector localTo: aClass 
	"Create and schedule a message browser on each method in or below the  given class that implements the message whose selector is the argument, selector.  
	For example,  
		SystemNavigation new browseAllImplementorsOf: #at:put: localTo: Dictionary."
		
	aClass ifNil: [^ self inform: 'no class selected'].
	^ self
		browseMessageList: ((self allImplementorsOf: selector localTo: aClass)
				collect: [:methRef | methRef actualClass name , ' ' , methRef methodSymbol])
		name: 'Implementors of ' , selector , ' local to ' , aClass name! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'eem 8/30/2010 11:13'!
browseAllImplementorsOf: selector localToPackage: packageNameOrInfo
	"Create and schedule a message browser on each method in the given package
	that implements the message whose selector is the argument, selector. For example, 
	SystemNavigation new browseAllImplementorsOf: #at:put: localToPackage: 'Collections'."

	self browseMessageList: (self
								allImplementorsOf: selector
								localToPackage: packageNameOrInfo) asSortedCollection
		name: 'Implementors of ' , selector,
				' local to package ', (self packageInfoFor: packageNameOrInfo) name! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 7/21/2010 12:39'!
browseAllImplementorsOfList: selectorList
	"Create and schedule a message browser on each method that implements 
	the message whose selector is in the argument selectorList. For example, 
		SystemNavigation default browseAllImplementorsOfList: #(at:put: size)."

	self browseAllImplementorsOfList: selectorList title: 'Implementors of all'! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 7/21/2010 12:41'!
browseAllImplementorsOfList: selectorList title: aTitle 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is in the argument selectorList. For 
	example,  
	self new browseAllImplementorsOfList: #(at:put: size) title: 'Implementors of',  #(at:put: size) asString."
	| implementorLists flattenedList |
	implementorLists := selectorList collect: [:each | self allImplementorsOf: each].
	flattenedList := SortedCollection new.
	implementorLists
		do: [:each | flattenedList addAll: each].
	Cursor normal show.
	^ self browseMessageList: flattenedList name: aTitle! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 7/21/2010 12:42'!
browseAllMethodsInCategory: category 

	^self
		browseMessageList: (self allMethodsInCategory: category)
		name: category! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'AndyKellens 6/11/2010 14:15'!
browseAllObjectReferencesTo: anObject except: objectsToExclude ifNone: aBlock 
	"Bring up a list inspector on the objects that point to anObject.
	If there are none, then evaluate aBlock on anObject.  "

	| aList shortName |
	aList := anObject pointersToExcept: objectsToExclude.
	aList size > 0 ifFalse: [^aBlock value: anObject].
	shortName := (anObject name ifNil: [anObject printString]) contractTo: 20.
	aList inspectWithLabel: 'Objects pointing to ' , shortName! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'lr 3/14/2010 21:13'!
browseAllReferencesToPool: poolOrName from: aClass
	"Open a message list on all messages referencing the given pool"

	| pool list |
	poolOrName isString
		ifTrue: [ pool := Smalltalk globals at: poolOrName asSymbol ]
		ifFalse: [ pool := poolOrName ].
	list := self allReferencesToPool: pool from: aClass.
	self browseMessageList: list name: 'users of ' , poolOrName name.
	^ list! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'sd 4/29/2003 20:43'!
browseAllSelect: aBlock 
	"Create and schedule a message browser on each method that, when used 
	as the block argument to aBlock gives a true result. For example,  
	SystemNavigation new browseAllSelect: [:method | method numLiterals >  
	10]."
	^ self
		browseMessageList: (self allMethodsSelect: aBlock)
		name: 'selected messages'! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 2/19/2010 11:06'!
browseAllSelect: aBlock name: aName autoSelect: autoSelectString 
	"Create and schedule a message browser on each method that, when used 
	as the block argument to aBlock gives a true result."
	"self new browseAllSelect: [:method | method numLiterals > 10] name:  
	'Methods with more than 10 literals' autoSelect: 'isDigit'"
	
	^ self
		browseMessageList: (self allMethodsSelect: aBlock)
		name: aName
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 11/6/2010 22:31'!
browseAllStoresInto: instVarName from: aClass
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."
	
	"self new browseAllStoresInto: 'contents' from: Collection."

	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsStoreInto: instVarName) do: [:sel |
				sel isDoIt ifFalse: [
					coll add: (
						SourcedMethodReference new
							setClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self
		browseMessageList: coll 
		name: 'Stores into ' , instVarName 
		autoSelect: instVarName! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'marcus.denker 9/14/2008 18:53'!
browseAllUnimplementedCalls
	"Create and schedule a message browser on each method that includes a 
	message that is not implemented in any object in the system."

	"self new browseAllUnimplementedCalls"

	^self browseMessageList: self allUnimplementedCalls name: 'Unimplemented calls'! !

!SystemNavigation methodsFor: '*Tools' stamp: 'MarcusDenker 10/28/2010 13:33'!
browseAllUnsentMessages
	"SystemNavigation default browseAllUnsentMessages"
	| unsentMessages |
	unsentMessages := self allUnsentMessagesWithProgressBar.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'All unsent messages' 
 ! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 2/16/2010 16:19'!
browseAllUsersOfTrait: aTrait
	"Create and schedule a message browser on each class or trait that imports aTrait"

	ToolSet 
		openClassListBrowser: (aTrait users collect: #name) asArray sort
		title: 'Users of trait ' , aTrait theNonMetaClass name
	! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'ar 9/27/2005 20:34'!
browseClass: aBehavior
	| targetClass |
	targetClass := aBehavior isMeta
				ifTrue: [aBehavior theNonMetaClass]
				ifFalse: [aBehavior ].
	ToolSet browse: targetClass selector: nil! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 4/22/2011 09:22'!
browseClassCommentsWithString: aString
	"Smalltalk browseClassCommentsWithString: 'my instances' "
	"Launch a message list browser on all class comments containing aString as a substring."

	| caseSensitive suffix list |

	suffix := (caseSensitive := Sensor shiftPressed)
		ifTrue: [' (case-sensitive)']
		ifFalse: [' (use shift for case-sensitive)'].
	list := Set new.
	Cursor wait showWhile: [
		Smalltalk allClassesDo: [:class | 
			(class organization classComment asString 
							includesSubstring: aString caseSensitive: caseSensitive) ifTrue: [
								list add: (
									CommentReference class: class
								)
							]
		]
	].
	^ self 
		browseMessageList: list asSortedCollection
		name: 'Class comments containing ' , aString printString , suffix
		autoSelect: aString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'nice 1/5/2010 15:59'!
browseClassVarRefs: aClass
	"Put up a menu offering all class variable names; if the user chooses one, open up a message-list browser on all methods 
	that refer to the selected class variable"

	| lines labelStream allVars index owningClasses |
	lines := OrderedCollection new.
	allVars := OrderedCollection new.
	owningClasses := OrderedCollection new.
	labelStream := (String new: 200) writeStream.
	aClass withAllSuperclasses reverseDo:
		[:class | | vars |
		vars := class classVarNames.
		vars do:
			[:var |
			labelStream nextPutAll: var; cr.
			allVars add: var.
			owningClasses add: class].
		vars isEmpty ifFalse: [lines add: allVars size]].
	labelStream contents isEmpty ifTrue: [^Beeper beep]. "handle nil superclass better"
	labelStream skip: -1 "cut last CR".
	index := (UIManager default chooseFrom: (labelStream contents substrings) lines: lines).
	index = 0 ifTrue: [^ self].
	self browseAllCallsOn:
		((owningClasses at: index) classPool associationAt: (allVars at: index))! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'sd 3/28/2003 18:49'!
browseClassVariables: aClass
	aClass classPool inspectWithLabel: 'Class Variables in ' , aClass name! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 3/5/2010 14:36'!
browseClassesWithNamesContaining: aString caseSensitive: caseSensitive 
	"Smalltalk browseClassesWithNamesContaining: 'eMorph' caseSensitive: true "
	"Launch a class-list list browser on all classes whose names containg aString as a substring."

	| suffix aList |
	suffix := caseSensitive
				ifTrue: [' (case-sensitive)']
				ifFalse: [' (use shift for case-sensitive)'].
	aList := OrderedCollection new.
	Cursor wait
		showWhile: [Smalltalk
				allClassesDo: [:class | (class name includesSubstring: aString caseSensitive: caseSensitive)
						ifTrue: [aList add: class name]]].
	aList size > 0
		ifTrue: [ToolSet openClassListBrowser: aList asSet asArray sort title: 'Classes whose names contain ' , aString , suffix]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 2/16/2010 16:32'!
browseHierarchy: aBehavior
	self browseHierarchy: aBehavior selector: nil! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'pavel.krivanek 2/24/2010 12:27'!
browseHierarchy: aClass selector: aSelector
	
	ToolSet browseHierarchy: aClass selector: aSelector.! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/18/2011 17:19'!
browseImplementorsOf: aSelector name: labelString autoSelect: autoSelectString
	"Create and schedule a senders browser for aSelector."	
	| implementors title size ml|
	
	implementors := self allImplementorsOf: aSelector.
	implementors size = 0 ifTrue: 
		[^ self inform: 'There are no ' , labelString].
	implementors size = 1 ifTrue: [^  implementors first browse].
	
	title := (size := implementors size) > 1
		ifFalse:	[labelString]
		ifTrue:	[ labelString, ' [', size printString, ']'].
		
	ml := MessageList new methodReferenceList: implementors.

	(FlatMessageListBrowser
		on: ml
		named: title
		autoSelect: aSelector) openInWorld.
	
"	ToolSet 
		browseImplementorsOf: aSelector
		name: title
		autoSelect: autoSelectString"! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 9/1/2010 17:12'!
browseInstVarDefs: aClass

	self chooseInstVarFrom: aClass thenDo: 
		[:aVar | self browseAllStoresInto: aVar from: aClass]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 9/1/2010 17:12'!
browseInstVarRefs: aClass
	
	self chooseInstVarFrom: aClass thenDo: 
		[:aVar | self browseAllAccessesTo: aVar from: aClass]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'sd 4/16/2003 09:18'!
browseMessageList: messageList name: label 
	"Create and schedule a MessageSet browser on messageList."
	^ self   
		browseMessageList: messageList 
		name: label 
		autoSelect: nil! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/16/2011 14:15'!
browseMessageList: messageList name: labelString autoSelect: autoSelectString
	| title aSize ml |
	"Create and schedule a MessageSet browser on the message list."

	messageList size = 0 ifTrue: 
		[^ self inform: 'There are no
' , labelString].

	title := (aSize := messageList size) > 1
		ifFalse:	[labelString]
		ifTrue:	[ labelString, ' [', aSize printString, ']'].

	ml := MessageList new methodReferenceList: messageList.

	(FlatMessageListBrowser
		on: ml
		named: title
		autoSelect: autoSelectString) forceAlphabeticalOrder: false; openInWorld.
		
	"ToolSet 
		browseMessageSet: messageList 
		name: title 
		autoSelect: autoSelectString"! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'stephane.ducasse 5/23/2009 14:01'!
browseMethodsWhoseNamesContain: aString
	"Launch a tool which shows all methods whose names contain the given 	string; case-insensitive."
	ToolSet browseMessageNames: aString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'yo 7/31/2004 18:40'!
browseMethodsWithLiteral: aString
	"Launch a browser on all methods that contain string literals with aString as a substring. Make the search case-sensitive or insensitive as dictated by the caseSensitive boolean parameter"

	self browseAllSelect:
			[:method |
				method hasLiteralSuchThat: [:lit |
					(lit isString and: [lit isSymbol not]) and:
					[lit = aString]]]
		name:  'Methods with string ', aString printString
		autoSelect: aString.
! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'sd 4/20/2003 14:11'!
browseMethodsWithSourceString: aString 
	"SystemNavigation new browseMethodsWithSourceString: 'SourceString'"
	"Launch a browser on all methods whose source code contains aString as 
	a substring."
	| caseSensitive suffix |
	suffix := (caseSensitive := Sensor shiftPressed)
				ifTrue: [' (case-sensitive)']
				ifFalse: [' (use shift for case-sensitive)'].
	^ self
		browseMessageList: (self allMethodsWithSourceString: aString matchCase: caseSensitive)
		name: 'Methods containing ' , aString printString , suffix
		autoSelect: aString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'sd 4/15/2003 22:28'!
browseMethodsWithString: aString
	"Launch a browser on all methods that contain string literals with aString as a substring. The search is case-insensitive, unless the shift key is pressed, in which case the search is case-sensitive."

	'string for testing'.
	^ self browseMethodsWithString: aString matchCase: Sensor shiftPressed

	"SystemNavigation new browseMethodsWithString: 'Testing' matchCase: false"
	"SystemNavigation new browseMethodsWithString: 'Testing' matchCase: true"! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'bf 4/26/2005 11:44'!
browseMethodsWithString: aString matchCase: caseSensitive
	"Launch a browser on all methods that contain string literals with aString as a substring. Make the search case-sensitive or insensitive as dictated by the caseSensitive boolean parameter"

	self browseAllSelect:
			[:method |
				method  hasLiteralSuchThat: [:lit |
					(lit isString and: [lit isSymbol not]) and:
					[lit includesSubstring: aString caseSensitive: caseSensitive]]]
		name:  'Methods with string ', aString printString, (caseSensitive ifTrue: [' (case-sensitive)'] ifFalse: [' (case-insensitive)'])
		autoSelect: aString.
! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 8/8/2010 18:24'!
browseMyChanges
	"Browse only the changes (in the changes file) by the current author."
	self browseAllSelect: [ :method |
		method fileIndex > 1 "only look at changes file"
			and: [ method timeStamp beginsWith: Author fullName ]]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'ar 9/27/2005 20:34'!
browseObsoleteMethodReferences
	"Open a browser on all referenced behaviors that are obsolete"

	"SystemNavigation default browseObsoleteMethodReferences"

	| list |
	list := self obsoleteMethodReferences.
	self 
		browseMessageList: list
		name: 'Method referencing obsoletes'
		autoSelect: nil! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'Alexandre Bergel 11/21/2009 12:04'!
browseObsoleteReferences  
	"self new browseObsoleteReferences"

	| references |
	references := OrderedCollection new.
	(LookupKey allSubInstances select:
		[:x | ((x value isKindOf: Behavior) and: ['AnOb*' match: x value name]) or:
		['AnOb*' match: x value class name]]) 
		do: [:x | references addAll: (self allReferencesTo: x)].
	self  
		browseMessageList: references 
		name: 'References to Obsolete Classes'! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/6/2011 15:20'!
browseSendersOf: aSelector name: labelString autoSelect: autoSelectString
	| title size senders |
	"Create and schedule a senders browser for aSelector."

	senders := self allCallsOn: aSelector.
	senders size = 0 ifTrue: 
		[^ self inform: 'There are no ' , labelString].

	title := (size := senders size) > 1
		ifFalse:	[labelString]
		ifTrue:	[labelString, ' [', size printString, ']'].
		
	ToolSet 
		browseSendersOf: aSelector
		name: title 
		autoSelect: autoSelectString! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 11/6/2010 22:32'!
browseUncommentedMethodsWithInitials: targetInitials
	"Browse uncommented methods whose initials (in the time-stamp, as logged to disk) match the given initials.  Present them in chronological order.  CAUTION: It will take several minutes for this to complete."
	"Time millisecondsToRun: [SystemNavigation default browseUncommentedMethodsWithInitials: 'jm']"

	| methodReferences |
	methodReferences := OrderedCollection new.
	self  allBehaviorsDo:
		[:aClass | aClass selectorsAndMethodsDo: [:sel :cm |
			| timeStamp initials |
			timeStamp := cm timeStamp.
			timeStamp isEmpty ifFalse:
				[initials := timeStamp substrings first.
				initials first isDigit ifFalse:
					[((initials = targetInitials) and: [(aClass firstPrecodeCommentFor: sel) isNil])
						ifTrue:
							[methodReferences add: (SourcedMethodReference new
								setClass: aClass 
								methodSymbol: sel)]]]]].

	ToolSet
		browseMessageSet: methodReferences 
		name: 'Uncommented methods with initials ', targetInitials
		autoSelect: nil! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'MarcusDenker 12/13/2010 20:56'!
browseUndeclaredReferences
	"SystemNavigation default browseUndeclaredReferences"
	
	Undeclared removeUnreferencedKeys.
	Undeclared keys do: [ :k |
		self
			browseMessageList: (self allReferencesTo: (Undeclared associationAt: k))
			name: 'References to Undeclared: ', k printString ]! !

!SystemNavigation methodsFor: '*Tools' stamp: 'MarcusDenker 10/28/2010 13:35'!
browseUnsentMessagesInClass: aClass
	"SystemNavigation default browseUnsentMessagesInClass: Browser"
	| unsentMessages |
	unsentMessages := self unsentMessagesWithProgressBarInClass: aClass.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'Unsent messages in class ', aClass name 
 ! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:59'!
browseUnsentMessagesInPackageNamed: packageName
	"SystemNavigation default browseUnsentMessagesWithProgressBarInPackageNamed: 'Kernel-Contexts'"
	| unsentMessages |
	unsentMessages := self unsentMessagesWithProgressBarInPackageNamed: packageName.
	^self 
		browseMessageList: unsentMessages asSortedCollection
		name: 'Unsent messages in package ', packageName 
 ! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 15:46'!
browserClass
	browserClass ifNil: [browserClass := self defaultBrowserClass].
	^browserClass! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 15:46'!
browserClass: aBrowserClass
	browserClass := aBrowserClass! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 3/25/2011 15:57'!
chooseInstVarAlphabeticallyFrom: aClass thenDo: aBlock
	| allVars index |
	"Put up a menu of all the instance variables in the receiver, presented in alphabetical order, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter."

	allVars := aClass allInstVarNames sort.
	allVars isEmpty ifTrue: [^ self inform: 'There are no
instance variables'].

	index := (UIManager default chooseFrom: allVars lines: #() title: 'Instance variables in ', aClass name).
	index = 0 ifTrue: [^ aClass].
	aBlock value: (allVars at: index)! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'StephaneDucasse 9/1/2010 17:14'!
chooseInstVarFrom: aClass thenDo: aBlock 
	"Put up a menu of all the instance variables in the receiver, and when
the user chooses one, evaluate aBlock with the chosen variable as its
parameter.  If the list is 6 or larger, then offer an alphabetical
formulation as an alternative. triggered by a 'show alphabetically' item
at the top of the list."

	| lines labelStream allVars index count offerAlpha |
	(count := aClass allInstVarNames size) = 0 ifTrue: 
		[^ self inform: 'There are no 
instance variables.'].

	allVars := OrderedCollection new.
	lines := OrderedCollection new.
	labelStream := (String new: 200) writeStream.
	(offerAlpha := count > 5)
		ifTrue:
			[lines add: 1.
			allVars add: 'show alphabetically'.
			labelStream nextPutAll: allVars first; cr].
	aClass withAllSuperclasses reverseDo:
		[:class | | vars |
		vars := class instVarNames.
		vars do:
			[:var |
			labelStream nextPutAll: var; cr.
			allVars add: var].
		vars isEmpty ifFalse: [lines add: allVars size]].
	labelStream skip: -1 "cut last CR".
	(lines size > 0 and: [lines last = allVars size]) ifTrue:
		[lines removeLast].  "dispense with inelegant line beneath last item"
	index := (UIManager default chooseFrom: (labelStream contents subStrings: {Character cr}) lines: lines
title: 'Instance variables in ', aClass name).
	index = 0 ifTrue: [^ self].
	(index = 1 and: [offerAlpha]) ifTrue: [^ self
chooseInstVarAlphabeticallyFrom: aClass thenDo: aBlock].
	aBlock value: (allVars at: index)! !

!SystemNavigation methodsFor: '*Tools-Inspector' stamp: 'BenjaminVanRyseghem 4/4/2011 14:28'!
classFromPattern: pattern withCaption: aCaption
	"If there is a class whose name exactly given by pattern, return it.
	If there is only one class in the system whose name matches pattern, return it.
	Otherwise, put up a menu offering the names of all classes that match pattern, and return the class chosen, else nil if nothing chosen.
	This method ignores tab, space, & cr characters in the pattern"

	| toMatch potentialClassNames classNames exactMatch index |
	(toMatch := pattern
		copyWithoutAll:
			{(Character space).
			(Character cr).
			(Character tab)}) isEmpty
		ifTrue: [ ^ nil ].
	Symbol
		hasInterned: toMatch
		ifTrue: [ :patternSymbol | 
			self environment
				at: patternSymbol
				ifPresent: [ :maybeClass | 
					((maybeClass isKindOf: Class) or: [ maybeClass isKindOf: Trait ])
						ifTrue: [ ^ maybeClass ] ] ].
	toMatch := (toMatch copyWithout: $.) asLowercase.
	potentialClassNames := (self environment classNames , self environment traitNames) asOrderedCollection.
	classNames := pattern last = $.
		ifTrue: [ potentialClassNames select: [ :nm | nm asLowercase = toMatch ] ]
		ifFalse: [ potentialClassNames select: [ :n | n includesSubstring: toMatch caseSensitive: false ] ].
	classNames isEmpty
		ifTrue: [ ^ nil ].
	exactMatch := classNames detect: [ :each | each asLowercase = toMatch ] ifNone: [ nil ].
	index := classNames size = 1
		ifTrue: [ 1 ]
		ifFalse: [ 
			exactMatch
				ifNil: [ UIManager default chooseFrom: classNames lines: #() title: aCaption ]
				ifNotNil: [ 
					classNames addFirst: exactMatch.
					UIManager default chooseFrom: classNames lines: #(1) title: aCaption ] ].
	index = 0
		ifTrue: [ ^ nil ].
	^ self environment at: (classNames at: index) asSymbol	"
	self default classFromPattern: 'znak' withCaption: ''
	self default classFromPattern: 'orph' withCaption: ''
	self default classFromPattern: 'TCompil' withCaption: ''
"! !

!SystemNavigation methodsFor: '*Tools-Inspector' stamp: 'BenjaminVanRyseghem 3/29/2011 16:32'!
confirmRemovalOf: aSelector on: aClass 
	"Determine if it is okay to remove the given selector. Answer 1 if it  
	should be removed, 2 if it should be removed followed by a senders  
	browse, and 3 if it should not be removed."
	| count answer caption allCalls |
	allCalls := self allCallsOn: aSelector.
	(count := allCalls size) == 0
		ifTrue: [^ 1].
	"no senders -- let the removal happen without warning"
	count == 1
		ifTrue: [(allCalls first actualClass == aClass
					and: [allCalls first methodSymbol == aSelector])
				ifTrue: [^ 1]].
	"only sender is itself"
	caption := 'The message ', aSelector printString ,' has ' , count printString , ' sender'.
	count > 1
		ifTrue: [caption := caption copyWith: $s].
	answer := UIManager default 
		chooseFrom: #('Remove it'
				'Remove, then browse senders'
				'Don''t remove, but show me those senders'
				'Forget it -- do nothing -- sorry I asked') title: caption.
	answer == 3
		ifTrue: [self
				browseMessageList: allCalls
				name: 'Senders of ' , aSelector
				autoSelect: aSelector keywords first].
	answer == 0
		ifTrue: [answer := 3].
	"If user didn't answer, treat it as cancel"
	^ answer min: 3! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'nk 2/25/2005 09:42'!
defaultBrowserClass
	^SystemBrowser default! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 15:49'!
defaultHierarchyBrowserClass
	^self class environment at: #HierarchyBrowser ifAbsent:[]! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:37'!
doWithProgressBar: aBlock forUnsentMessagesInClass: class
	| progressMessages selectors |
	progressMessages := 'Unsent messages in class ', class name.
	selectors := class selectors asArray.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: selectors size
		during: [:bar | 
			selectors with: (0 to: selectors size - 1) do: [:selector :index|
				bar value: index.
				(self isUnsentMessage: selector) ifTrue: [aBlock value: class value: selector] ]]
! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:37'!
doWithProgressBar: aBlock forUnsentMessagesInPackageNamed: packageName
	|progressMessages classes |
	Cursor wait showWhile: [classes := self allClassesInPackageNamed: packageName].
	progressMessages := 'Unsent messages in package ', packageName.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: classes size
		during: [:bar | 
			classes with: (0 to: classes size - 1) do: [:class :index|
				bar value: index.
				self doWithProgressBar: aBlock forUnsentMessagesInClass: class]]! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:55'!
doWithProgressBarForAllUnsentMessages: aBlock
	|progressMessages classes |
	Cursor wait showWhile: [classes := self allClasses].
	progressMessages := 'Unsent messages in all classes'.
	progressMessages
		displayProgressAt: Display center
		from: 0 to: classes size
		during: [:bar | 
			classes with: (0 to: classes size - 1) do: [:class :index|
				bar value: index.
				self doWithProgressBar: aBlock forUnsentMessagesInClass: class]]! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 15:50'!
hierarchyBrowserClass
	hierarchyBrowserClass ifNil: [hierarchyBrowserClass := self defaultHierarchyBrowserClass].
	^hierarchyBrowserClass! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 15:47'!
hierarchyBrowserClass: aBrowserClass
	hierarchyBrowserClass := aBrowserClass! !

!SystemNavigation methodsFor: '*Tools-Browser' stamp: 'BenjaminVanRyseghem 5/6/2011 16:04'!
methodHierarchyBrowserForClass: aClass selector: sel
	"Create and schedule a message set browser on all implementors of the 
	currently selected message selector. Do nothing if no message is selected."
	"SystemNavigation default 
		methodHierarchyBrowserForClass: ParagraphEditor 
		selector: #isControlActive"
	
	| list aClassNonMeta isMeta tab |
	aClass ifNil: [^ self].
	aClass isTrait ifTrue: [^ self].
	sel ifNil: [^ self].
	aClassNonMeta := aClass theNonMetaClass.
	isMeta := aClassNonMeta ~~ aClass.
	list := OrderedCollection new.
	tab := ''.
	
	aClass allSuperclasses reverseDo:
		[:cl |
		(cl includesSelector: sel) ifTrue:
			[list addLast: ( (cl>>sel) methodReference )]].
"		tab := tab , '  '].
	aClassNonMeta allSubclassesWithLevelDo:
		[:cl :level | | theClassOrMeta stab |
		theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
		(theClassOrMeta includesSelector: sel) ifTrue:
			[stab := ''.  1 to: level do: [:i | stab := stab , '  '].
			list addLast: tab , stab , theClassOrMeta name, ' ', sel]]
	 	startingLevel: 0."
	list addLast: ((aClass>>sel) methodReference).

	self browseMessageList: list name: 'Inheritance of ' , sel

! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:49'!
removeUnsentMessagesWithProgressBarInClass: aClass
	self
		doWithProgressBar: [:class :selector| 
			class remove: selector] 
		forUnsentMessagesInClass: aClass! !

!SystemNavigation methodsFor: '*Tools' stamp: 'Noury 10/26/2008 22:48'!
removeUnsentMessagesWithProgressBarInPackageNamed: packageName
	self
		doWithProgressBar: [:class :selector| 
			class removeSelector: selector] 
		forUnsentMessagesInPackageNamed: packageName.
! !

!SystemNavigation methodsFor: '*Tools-Inspector' stamp: 'sd 4/15/2003 15:34'!
showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock
	"Show a sorted menu of the given selectors, preceded by firstItem, and all
	abbreviated to 40 characters.  Evaluate choiceBlock if a message is chosen."

	^ self showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock withCaption: nil! !

!SystemNavigation methodsFor: '*Tools-Inspector' stamp: 'StephaneDucasse 10/4/2010 19:35'!
showMenuOf: selectorCollection withFirstItem: firstItem ifChosenDo: choiceBlock withCaption: aCaption
	"Show a sorted menu of the given selectors, preceded by firstItem, and all abbreviated to 40 characters.  Use aCaption as the menu title, if it is not nil.  Evaluate choiceBlock if a message is chosen."
 
	| index menuLabels sortedList |
	sortedList := selectorCollection asSortedCollection.
	menuLabels := Array streamContents: 
		[:strm | strm nextPut: (firstItem contractTo: 40).
			sortedList do: [:sel | strm nextPut: (sel contractTo: 40)]].
	index := UIManager default chooseFrom: menuLabels lines: #(1).	
	index = 1 ifTrue: [choiceBlock value: firstItem].
	index > 1 ifTrue: [choiceBlock value: (sortedList at: index - 1)]! !

!SystemNavigation methodsFor: '*Tools' stamp: 'BenjaminVanRyseghem 11/6/2010 22:32'!
unsentMessagesWithProgressBarInClass: aClass
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBar: [:class :selector| 
			unsentMessages add: (SourcedMethodReference class: class selector: selector)] 
		forUnsentMessagesInClass: aClass.
	^unsentMessages! !

!SystemNavigation methodsFor: '*Tools' stamp: 'BenjaminVanRyseghem 11/6/2010 22:32'!
unsentMessagesWithProgressBarInPackageNamed: packageName
	| unsentMessages |
	unsentMessages := Set new.
	self
		doWithProgressBar: [:class :selector| 
			unsentMessages add: (SourcedMethodReference class: class selector: selector)] 
		forUnsentMessagesInPackageNamed: packageName.
	^unsentMessages! !

!BlockClosure methodsFor: '*Tools' stamp: 'StephaneDucasse 5/22/2010 18:23'!
timeProfile
	^TimeProfileBrowser onBlock: self! !

!BlockClosure methodsFor: '*Tools' stamp: 'StephaneDucasse 5/22/2010 18:24'!
timeToRunWithoutGC
	"Answer the number of milliseconds taken to execute this block without GC time."
 
	^(Smalltalk vmParameterAt: 8) + (Smalltalk vmParameterAt: 10) +
           	self timeToRun -  (Smalltalk vmParameterAt: 8) - (Smalltalk vmParameterAt: 10)
! !

!OrderedCollection methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:33'!
inspectorClass 
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^OrderedCollectionInspector! !

AppRegistry subclass: #SystemBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!SystemBrowser commentStamp: '<historical>' prior: 0!
This is the AppRegistry class for class browsing!

!SystemBrowser class methodsFor: 'registration' stamp: 'hpt 9/30/2004 20:53'!
addRegistryMenuItemsTo: aMenu inAccountOf: aBrowser 
	"Add some useful options related Browser registry to the
	browsers windows menu"
	aMenu addLine;
		add: 'Register this Browser as default'
		target: [self default: aBrowser class]
		action: #value;
		add: 'Choose new default Browser'
		target: self
		action: #askForDefault! !

!SystemBrowser class methodsFor: 'instance creation' stamp: 'hpt 8/5/2004 20:27'!
defaultOpenBrowser
	^self default openBrowser! !

!SystemBrowser class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:50'!
registerToolsOn: registry
	"Add the default browser to registry. See [Smalltalk tools]" 
	registry register: [ self default ] as: #browser
! !

AppRegistry subclass: #TestRunnerBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!TestRunnerBrowser class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 04:00'!
registerToolsOn: registry
	registry register: [ self default ] as: #testRunner! !

!Collection methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:45'!
explorerContents

	^self explorerContentsWithIndexCollect: [:value :index |
		ObjectExplorerWrapper
			with: value
			name: index printString
			model: self]! !

!Collection methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:29'!
explorerContentsWithIndexCollect: twoArgBlock

	^ self asOrderedCollection withIndexCollect: twoArgBlock
! !

Model subclass: #CPUWatcher
	instanceVariableNames: 'tally watcher threshold'
	classVariableNames: 'CpuWatcherEnabled CurrentCPUWatcher'
	poolDictionaries: ''
	category: 'Tools-Process Browser'!

!CPUWatcher commentStamp: 'VeronicaUquillas 6/11/2010 12:47' prior: 0!
CPUWatcher implements a simple runaway process monitoring tool
that will suspend a process that is taking up too much of Pharo's
time and allow user interaction. By default it watches for a Process that
is taking more than 80% of the time; this threshold can be changed.

CPUWatcher can also be used to show cpu percentages for each process 
from within the ProcessBrowser.

	CPUWatcher startMonitoring.	"process period 20 seconds, sample rate 100 msec"
	CPUWatcher current monitorProcessPeriod: 10 sampleRate: 20.
	CPUWatcher current threshold: 0.5.	"change from 80% to 50%"
	CPUWatcher stopMonitoring.
!

!CPUWatcher class methodsFor: 'setting' stamp: 'AlainPlantec 12/5/2009 19:42'!
cpuWatcherEnabled
	^ CpuWatcherEnabled ifNil: [CpuWatcherEnabled := false]! !

!CPUWatcher class methodsFor: 'setting' stamp: 'AlainPlantec 12/5/2009 19:44'!
cpuWatcherEnabled: aBoolean
	CpuWatcherEnabled = aBoolean ifTrue: [^ self].
	CpuWatcherEnabled := aBoolean.
	self monitorPreferenceChanged
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/8/2001 18:45'!
current
	^CurrentCPUWatcher
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:28'!
currentWatcherProcess
	^CurrentCPUWatcher ifNotNil: [ CurrentCPUWatcher watcherProcess ]
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/6/2011 18:11'!
dumpTallyOnTranscript
	self current ifNotNil: [
		self processBrowser dumpTallyOnTranscript: self current tally
	]! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:15'!
initialize
	"CPUWatcher initialize"
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:06'!
isMonitoring

	^CurrentCPUWatcher notNil and: [ CurrentCPUWatcher isMonitoring ]
! !

!CPUWatcher class methodsFor: 'setting' stamp: 'AlainPlantec 12/5/2009 19:43'!
monitorPreferenceChanged
	self cpuWatcherEnabled
		ifTrue: [ self startMonitoring ]
		ifFalse: [ self stopMonitoring ]! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:14'!
shutDown
	self stopMonitoring.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:17'!
startMonitoring
	"CPUWatcher startMonitoring"

	^self startMonitoringPeriod: 20 rate: 100 threshold: 0.8! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'sd 11/20/2005 21:28'!
startMonitoringPeriod: pd rate: rt threshold: th
	"CPUWatcher startMonitoring"

	CurrentCPUWatcher ifNotNil: [ ^CurrentCPUWatcher startMonitoring. ].
	CurrentCPUWatcher := (self new)
		monitorProcessPeriod: pd sampleRate: rt;
		threshold: th;
		yourself.
	^CurrentCPUWatcher
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:14'!
startUp
	self monitorPreferenceChanged.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'sd 11/20/2005 21:28'!
stopMonitoring
	"CPUWatcher stopMonitoring"

	CurrentCPUWatcher ifNotNil: [ CurrentCPUWatcher stopMonitoring. ].
	CurrentCPUWatcher := nil.
! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'IgorStasenko 3/6/2011 18:10'!
catchThePig: aProcess
	| rules  |
	"nickname, allow-stop, allow-debug"
	rules := self processBrowser nameAndRulesFor: aProcess.

	(self processBrowser isUIProcess: aProcess)
		ifTrue: [ "aProcess debugWithTitle: 'Interrupted from the CPUWatcher'." ]
		ifFalse: [ rules second ifFalse: [ ^self ].
				self processBrowser suspendProcess: aProcess.
				self openWindowForSuspendedProcess: aProcess ]
! !

!CPUWatcher methodsFor: 'process operations' stamp: 'IgorStasenko 3/6/2011 18:10'!
debugProcess: aProcess
	| uiPriority oldPriority  |
	uiPriority := Processor activeProcess priority.
	aProcess priority >= uiPriority ifTrue: [
		oldPriority := self processBrowser setProcess: aProcess toPriority: uiPriority - 1
	].
	self processBrowser debugProcess: aProcess.! !

!CPUWatcher methodsFor: 'process operations' stamp: 'nk 3/8/2001 17:27'!
debugProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	self debugProcess: aProcess.! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'sd 11/20/2005 21:27'!
findThePig
	"tally has been updated. Look at it to see if there is a bad process.
	This runs at a very high priority, so make it fast"
	| countAndProcess | 
	countAndProcess := tally sortedCounts first.
	(countAndProcess key / tally size > self threshold) ifTrue: [ | proc |
		proc := countAndProcess value.
		proc == Processor backgroundProcess ifTrue: [ ^self ].	"idle process? OK"
		self catchThePig: proc
	].
! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/14/2001 07:56'!
isMonitoring
	^watcher notNil! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'sd 11/20/2005 21:27'!
monitorProcessPeriod: secs sampleRate: msecs
	self stopMonitoring.

	watcher := [ [ | promise |
		promise := Processor tallyCPUUsageFor: secs every: msecs.
		tally := promise value.
		promise := nil.
		self findThePig.
	] repeat ] forkAt: Processor highestPriority.
	Processor yield ! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'IgorStasenko 3/6/2011 18:10'!
openMorphicWindowForSuspendedProcess: aProcess
	| menu rules |
	menu := MenuMorph new.
	"nickname  allow-stop  allow-debug"
	rules := self  processBrowser nameAndRulesFor: aProcess.
	menu add: 'Dismiss this menu' target: menu selector: #delete; addLine.
	menu add: 'Open Process Browser' target: self processBrowser selector: #open.
	menu add: 'Resume'
		target: self
		selector: #resumeProcess:fromMenu:
		argumentList: { aProcess . menu }.
	menu add: 'Terminate'
		target: self
		selector: #terminateProcess:fromMenu:
		argumentList: { aProcess . menu }.
	rules third ifTrue: [
		menu add: 'Debug at a lower priority'
			target: self
			selector: #debugProcess:fromMenu:
			argumentList: { aProcess . menu }.
	].
	menu addTitle: aProcess identityHash asString,
		' ', rules first,
		' is taking too much time and has been suspended.
What do you want to do with it?'.
	menu stayUp: true.
	menu popUpInWorld
! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'alain.plantec 5/30/2008 10:35'!
openWindowForSuspendedProcess: aProcess

	 WorldState addDeferredUIMessage: [ self openMorphicWindowForSuspendedProcess: aProcess ] 
	! !

!CPUWatcher methodsFor: 'accessing' stamp: 'IgorStasenko 3/6/2011 18:10'!
processBrowser
	^ Smalltalk tools processBrowser! !

!CPUWatcher methodsFor: 'process operations' stamp: 'IgorStasenko 3/6/2011 18:11'!
resumeProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	self processBrowser resumeProcess: aProcess.! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'nk 3/14/2001 08:07'!
startMonitoring
	self
		monitorProcessPeriod: 20 sampleRate: 100! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'IgorStasenko 3/6/2011 18:11'!
stopMonitoring
	watcher ifNotNil: [
		self processBrowser terminateProcess: watcher.
		watcher := nil.
	]! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/8/2001 18:36'!
tally
	^tally copy! !

!CPUWatcher methodsFor: 'process operations' stamp: 'IgorStasenko 3/6/2011 18:11'!
terminateProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	self processBrowser terminateProcess: aProcess.! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/8/2001 18:49'!
threshold
	"What fraction of the time can a process be the active process before we stop it?"
	^threshold! !

!CPUWatcher methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
threshold: thresh
	"What fraction of the time can a process be the active process before we stop it?"
	threshold := (thresh max: 0.02) min: 1.0! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/14/2001 08:26'!
watcherProcess
	^watcher! !

Model subclass: #DualChangeSorter
	instanceVariableNames: 'leftCngSorter rightCngSorter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!DualChangeSorter commentStamp: '<historical>' prior: 0!
This class presents a view of a two change sets at once, and supports copying changes between change sets.
!

!DualChangeSorter class methodsFor: 'world menu' stamp: 'IgorStasenko 3/6/2011 18:48'!
menuCommandOn: aBuilder 
	<worldMenu> 
	(aBuilder group: #SystemChanges)
		parent: #Tools;  
		with: [
			(aBuilder item: #'Change Sorter')
				action:[self new morphicWindow openInWorld]; 
				icon: self taskbarIcon.
			(aBuilder item: #'Recover lost changes...') 
				action: [Smalltalk tools changeList browseRecentLog]]! !

!DualChangeSorter class methodsFor: 'opening' stamp: 'sw 3/24/1999 17:50'!
open
	"Open a new instance of the receiver's class"

	self new open! !

!DualChangeSorter class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color paleBlue
! !

!DualChangeSorter class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #dualChangeSorter
! !

!DualChangeSorter class methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 09:50'!
theme	
	^ UITheme current! !

!DualChangeSorter methodsFor: 'toolbuilder' stamp: 'IgorStasenko 3/6/2011 18:50'!
buildWith: builder
	| windowSpec |
	self setDefaultChangeSorters.
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Change Sorter'.
	windowSpec children: OrderedCollection new.
	leftCngSorter buildWith: builder in: windowSpec rect: (0@0 extent: 0.5@1).
	rightCngSorter buildWith: builder in: windowSpec rect: (0.5@0 extent: 0.5@1).
	^builder build: windowSpec
! !

!DualChangeSorter methodsFor: 'other' stamp: ''!
isLeftSide: theOne
	"Which side am I?"
	^ theOne == leftCngSorter! !

!DualChangeSorter methodsFor: 'other' stamp: 'StephaneDucasse 3/30/2010 22:50'!
labelString
	"The window label"

	^ 'Changes go to "', ChangeSet current name,  '"'.
	! !

!DualChangeSorter methodsFor: 'accessing' stamp: 'JuanVuletich 10/21/2010 17:35'!
leftCngSorter
	^leftCngSorter! !

!DualChangeSorter methodsFor: 'other' stamp: 'gvc 1/23/2009 16:31'!
modelWakeUp
	"A window with me as model is being entered.  Make sure I am up-to-date with the changeSets.
	Treat each side individually rather than going through the . Changed here to avoid endless confirm dialogs."

	leftCngSorter canDiscardEdits ifTrue: [leftCngSorter updateIfNecessary].
	rightCngSorter canDiscardEdits ifTrue: [rightCngSorter updateIfNecessary]! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'IgorStasenko 3/6/2011 18:50'!
morphicWindow
	
	| window |
	self setDefaultChangeSorters.

	window := (SystemWindow labelled: leftCngSorter label) model: self.
	"topView minimumSize: 300 @ 200."
	leftCngSorter openAsMorphIn: window rect: (0@0 extent: 0.5@1).
	rightCngSorter openAsMorphIn: window rect: (0.5@0 extent: 0.5@1).
	^ window
! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'di 5/20/1998 21:44'!
okToChange
	^ leftCngSorter okToChange & rightCngSorter okToChange! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'alain.plantec 5/30/2008 13:00'!
open
	^ self openAsMorph.
! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 17:08'!
openAsMorph
	^ UIManager default openDualChangeSorterAsMorph: self
! !

!DualChangeSorter methodsFor: 'other' stamp: ''!
other: theOne
	"Return the other side's ChangeSorter"
	^ theOne == leftCngSorter
		ifTrue: [rightCngSorter]
		ifFalse: [leftCngSorter]! !

!DualChangeSorter methodsFor: 'initialization' stamp: ''!
release
	leftCngSorter release.
	rightCngSorter release.! !

!DualChangeSorter methodsFor: 'accessing' stamp: 'JuanVuletich 10/21/2010 17:36'!
rightCngSorter
	^rightCngSorter! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'IgorStasenko 3/6/2011 18:49'!
setDefaultChangeSorters

	leftCngSorter := Smalltalk tools changeSorter new myChangeSet: ChangeSet current.
	leftCngSorter parent: self.
	rightCngSorter := Smalltalk tools changeSorter new myChangeSet: ChangeSet secondaryChangeSet.
	rightCngSorter parent: self.! !

Model subclass: #FileList
	instanceVariableNames: 'fileName directory volList volListIndex list listIndex pattern sortMode brevityState currentDirectorySelected dirSelectionBlock optionalButtonSpecs modalView ok contents'
	classVariableNames: 'FileReaderRegistry RecentDirs'
	poolDictionaries: ''
	category: 'Tools-FileList'!

!FileList commentStamp: 'StephaneDucasse 3/28/2010 20:44' prior: 0!
A FileList is a tool to deal with files.

FileList open				"open FileList"!

!FileList class methodsFor: 'utility' stamp: 'nice 1/5/2010 15:59'!
addFullPanesTo: window from: aCollection

	

	aCollection do: [ :each | | frame |
		frame := LayoutFrame 
			fractions: each second 
			offsets: each third.
		window addMorph: each first fullFrame: frame.
	]! !

!FileList class methodsFor: 'file reader registration' stamp: 'hfm 11/29/2008 19:35'!
allRegisteredServices
	"self allRegisteredServices"

	| col |
	col := OrderedCollection new.
	self registeredFileReaderClasses do: [:each | col addAll: (each services)].
	^ col! !

!FileList class methodsFor: 'initialization' stamp: 'MarcusDenker 12/12/2009 07:36'!
initialize
	"FileList initialize"

	RecentDirs := OrderedCollection new.
	(self systemNavigation allClassesImplementing: #fileReaderServicesForFile:suffix:) do: 		[:providerMetaclass |
			self registerFileReader: providerMetaclass soleInstance]! !

!FileList class methodsFor: 'file reader registration' stamp: 'hfm 11/29/2008 19:36'!
isReaderNamedRegistered: aSymbol
	"return if a given reader class has been registered. Note that this is on purpose that the argument is
	a symbol and not a class"

	 ^ (self registeredFileReaderClasses collect: [:each | each name]) includes: aSymbol
! !

!FileList class methodsFor: 'utility' stamp: 'hfm 11/29/2008 18:58'!
itemsForDirectory: aFileDirectory
	"Answer a list of services appropriate when no file is selected."

	| services |
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForDirectory: aFileDirectory) ]].
	^ services! !

!FileList class methodsFor: 'utility' stamp: 'hfm 11/29/2008 18:35'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"

	| services suffix |
	suffix := self suffixOf: fullName.
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForFile: fullName suffix: suffix)]].
	^ services! !

!FileList class methodsFor: 'world menu' stamp: 'AlainPlantec 2/18/2010 11:20'!
menuCommandOn: aBuilder 
	<worldMenu> 
	(aBuilder item: #'File Browser')
		parent: #Tools;
		action:[self open]; 
		icon: self taskbarIcon.! !

!FileList class methodsFor: 'modal dialogs' stamp: 'miki 8/15/2005 18:34'!
modalFileSelectorForSuffixes: aList

	| window aFileList |

	window := self morphicViewFileSelectorForSuffixes: aList.
	aFileList := window valueOfProperty: #fileListModel.
	window openCenteredInWorld.
	self modalLoopOn: window.
	^aFileList getSelectedFile! !

!FileList class methodsFor: 'modal dialogs' stamp: 'Mariano 12/27/2009 16:21'!
modalFolderSelector

	^self modalFolderSelector: FileDirectory default! !

!FileList class methodsFor: 'modal dialogs' stamp: 'miki 8/15/2005 18:33'!
modalFolderSelector: aDir

	| window fileModel |
	window := self morphicViewFolderSelector: aDir.
	fileModel := window model.
	window openInWorld: self currentWorld extent: 300@400.
	self modalLoopOn: window.
	^fileModel getSelectedDirectory withoutListWrapper! !

!FileList class methodsFor: 'utility' stamp: 'miki 8/15/2005 18:34'!
modalLoopOn: aMorph
	[aMorph world notNil] whileTrue: [
		aMorph outermostWorldMorph doOneCycle.
	].! !

!FileList class methodsFor: 'morphic ui' stamp: 'btr 1/30/2004 00:56'!
morphicView
	^ self morphicViewOnDirectory: FileDirectory default! !

!FileList class methodsFor: 'morphic ui' stamp: 'miki 8/14/2005 21:21'!
morphicViewFileSelectorForSuffixes: aList
	"Answer a morphic file-selector tool for the given suffix list."

	^self 
		morphicViewFileSelectorForSuffixes: aList 
		directory: FileDirectory default.! !

!FileList class methodsFor: 'morphic ui' stamp: 'MarcusDenker 8/15/2010 15:04'!
morphicViewFileSelectorForSuffixes: aList directory: dir
	"Answer a morphic file-selector tool for the given suffix list and the given directory."

	| aFileList window fixedSize midLine gap |
	aFileList := self new directory: dir.
	aFileList optionalButtonSpecs: aFileList okayAndCancelServices.
	window := BorderedMorph new
		layoutPolicy: ProportionalLayout new;
		color: Color lightBlue;
		borderColor: Color blue;
		borderWidth: 4;
		layoutInset: 4;
		extent: 600@400;
		useRoundedCorners.
	window setProperty: #fileListModel toValue: aFileList.
	aFileList modalView: window.
	midLine := 0.4.
	fixedSize := 25.
	gap := 5.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a file'. 0 @ 0 corner: 1 @ 0. 0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 
					gap @(fixedSize * 2) corner: gap negated@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 
					gap@(fixedSize * 2) corner: gap negated@0}.
	}.

	aFileList postOpen.

	^ window ! !

!FileList class methodsFor: 'morphic ui' stamp: 'bkv 11/12/2002 16:55'!
morphicViewFolderSelector: aDir
	"Answer a tool that allows the user to select a folder"

	| aFileList window fixedSize |
	aFileList := self new directory: aDir.
	aFileList optionalButtonSpecs: aFileList servicesForFolderSelector.
	window := (SystemWindow labelled: aDir pathName) model: aFileList.
	aFileList modalView: window.

	fixedSize := 25.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a folder'. 0 @ 0 corner: 1 @ 0. 
				0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 
				0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: 1@1.
				0@(fixedSize * 2) corner: 0@0}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList class methodsFor: 'morphic ui' stamp: 'gk 5/5/2006 02:05'!
morphicViewOnDirectory: aFileDirectory
	| aFileList window fileListBottom midLine fileListTopOffset buttonPane |

	aFileList := self new directory: aFileDirectory.
	window := (SystemWindow labelled: aFileDirectory pathName) model: aFileList.

	fileListTopOffset := (TextStyle defaultFont pointSize * 2) + 14.
	fileListBottom := 0.4.
	midLine := 0.4.
	buttonPane := aFileList optionalButtonRow addMorph:
		(aFileList morphicPatternPane vResizing: #spaceFill; yourself).
	self addFullPanesTo: window from: {
		{buttonPane. 0@0 corner: 1@0. 0@0 corner: 0@fileListTopOffset}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@fileListBottom. 
					0@fileListTopOffset corner: 0@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@fileListBottom. 
					0@fileListTopOffset corner: 0@0}.
		{aFileList morphicFileContentsPane. 0@fileListBottom corner: 1@1. nil}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList class methodsFor: 'instance creation' stamp: 'MarcusDenker 12/12/2009 07:29'!
open
	"Open a view of an instance of me on the default directory."
	"FileList open openInWorld"
	^ self morphicView openInWorld! !

!FileList class methodsFor: 'instance creation' stamp: 'hfm 11/29/2008 19:37'!
openEditorOn: aFileStream editString: editString 
	"Open an editor on the given FileStream."
	^ (self openMorphOn: aFileStream editString: editString) openInWorld! !

!FileList class methodsFor: 'instance creation' stamp: 'MarcusDenker 9/30/2010 14:31'!
openMorphOn: aFileStream editString: editString 
	"Open a morphic view of a FileList on the given file."
	| fileModel window fileContentsView |

	fileModel := self new setFileStream: aFileStream.	"closes the stream"
	window := (SystemWindow labelled: aFileStream fullName) model: fileModel.

	window addMorph: (fileContentsView := PluggableTextMorph on: fileModel 
			text: #contents accept: #put:
			readSelection: #contentsSelection 
			menu: #fileContentsMenu:shifted:)
		frame: (0@0 corner: 1@1).
	editString ifNotNil: [fileContentsView editString: editString.
			fileContentsView hasUnacceptedEdits: true].

	^ window! !

!FileList class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:09'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightMagenta
! !

!FileList class methodsFor: 'instance creation' stamp: 'hfm 11/29/2008 19:04'!
registerFileReader: aProviderClass
	"register the given class as providing services for reading files"

	| registeredReaders |
	registeredReaders := self registeredFileReaderClasses.
	(registeredReaders includes: aProviderClass) 
			ifFalse: [ registeredReaders addLast: aProviderClass ]! !

!FileList class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:00'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #fileList
! !

!FileList class methodsFor: 'utility' stamp: 'hfm 11/29/2008 18:37'!
registeredFileReaderClasses
	FileReaderRegistry := nil. "wipe it out"
	^FileServices registeredFileReaderClasses
	! !

!FileList class methodsFor: 'initialization' stamp: 'hfm 11/29/2008 19:38'!
removeObsolete
	"FileList removeObsolete"
	self registeredFileReaderClasses copy 
		do:[:cls| cls isObsolete ifTrue:[self unregisterFileReader: cls]]! !

!FileList class methodsFor: 'utility' stamp: 'hfm 11/29/2008 18:35'!
suffixOf: aName
	"Answer the file extension of the given file"
	^ aName
		ifNil:
			['']
		ifNotNil:
			[(FileDirectory extensionFor: aName) asLowercase]! !

!FileList class methodsFor: 'utility' stamp: 'RAA 3/6/2001 12:39'!
textRow: aString 

	^AlignmentMorph newRow 
		wrapCentering: #center; cellPositioning: #leftCenter;
		color: Color transparent;
		layoutInset: 0;
		addMorph: (
			AlignmentMorph newColumn
			wrapCentering: #center; cellPositioning: #topCenter;
			color: Color transparent;
			vResizing: #shrinkWrap;
			layoutInset: 0;
			addMorph: (
				AlignmentMorph newRow
				wrapCentering: #center; cellPositioning: #leftCenter;
				color: Color transparent;
				hResizing: #shrinkWrap;
				vResizing: #shrinkWrap;
				layoutInset: 0;
				addMorph: ((StringMorph contents: aString) color: Color blue; lock)
			)
		)! !

!FileList class methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 09:51'!
theme
	^ UITheme current ! !

!FileList class methodsFor: 'file reader registration' stamp: 'hfm 11/29/2008 19:39'!
unregisterFileReader: aProviderClass
	"unregister the given class as providing services for reading files"

	self registeredFileReaderClasses remove: aProviderClass ifAbsent: [nil]! !

!FileList methodsFor: 'drag''n''drop' stamp: 'hfm 11/29/2008 19:22'!
acceptDroppingMorph: aTransferMorph event: evt inMorph: dest
	| oldName oldEntry destDirectory newName newEntry baseName response |
	destDirectory := self dropDestinationDirectory: dest event: evt.
	oldName := aTransferMorph passenger.
	baseName := FileDirectory localNameFor: oldName.
	newName := destDirectory fullNameFor: baseName.
	newName = oldName ifTrue: [ "Transcript nextPutAll: 'same as old name'; cr." ^ true ].
	oldEntry := FileDirectory directoryEntryFor: oldName.
	newEntry := FileDirectory directoryEntryFor: newName.
	newEntry ifNotNil: [ | msg |
		msg := String streamContents: [ :s |
			s nextPutAll: 'destination file ';
				nextPutAll: newName;
				nextPutAll: ' exists already,';
				cr;
				nextPutAll: 'and is ';
				nextPutAll: (oldEntry modificationTime < newEntry modificationTime
					ifTrue: [ 'newer' ] ifFalse: [ 'not newer' ]);
				nextPutAll: ' than source file ';
				nextPutAll: oldName;
				nextPut: $.;
				cr;
				nextPutAll: 'Overwrite file ';
				nextPutAll: newName;
				nextPut: $?
		].
		response := self confirm: msg.
		response ifFalse: [ ^false ].
	].

	aTransferMorph shouldCopy
		ifTrue: [ self primitiveCopyFileNamed: oldName to: newName ]
		ifFalse: [ directory rename: oldName toBe: newName ].

	self updateFileList; fileListIndex: 0.

	aTransferMorph source model ~= self
		ifTrue: [ aTransferMorph source model updateFileList; fileListIndex: 0 ].
	"Transcript nextPutAll: 'copied'; cr."
	^true! !

!FileList methodsFor: 'file menu action' stamp: 'CarloTeixeira 7/2/2010 23:27'!
addNew: aString byEvaluating: aBlock
	"A parameterization of earlier versions of #addNewDirectory and
	#addNewFile.  Fixes the bug in each that pushing the cancel button
	in the FillInTheBlank dialog gave a walkback."

	| response newName index ending |
	self okToChange ifFalse: [^ self].
	(response := UIManager default
						request: ('New {1} Name?' translated format: {aString translated})
						initialAnswer: ('{1}Name' translated format: {aString translated}))
		isEmptyOrNil ifTrue: [^ self].
	newName := response asFileName.
	Cursor wait showWhile: [
		aBlock value: newName].
	self updateFileList.
	self updateDirectory.
	index := list indexOf: newName.
	index = 0 ifTrue: [ending := ') ',newName.
		index := list findFirst: [:line | line endsWith: ending]].
	self fileListIndex: index.
! !

!FileList methodsFor: 'own services' stamp: 'hfm 11/29/2008 18:59'!
addNewDirectory
	self 
		addNew: 'Directory'
		byEvaluating: [:newName | directory createDirectory: newName]
! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 19:07'!
addNewFile
	self 
		addNew: 'File'
		byEvaluating: [:newName | (directory newFileNamed: newName) close]
! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 19:01'!
addPath: aString
	"Add the given string to the list of recently visited directories."

	| full |
	aString ifNil: [^self].
	full := String streamContents: 
		[ :strm | 2 to: volList size do: 
			[ :i | strm nextPutAll: (volList at: i) withBlanksTrimmed.
			strm nextPut: FileDirectory pathNameDelimiter]].
	full := full, aString.
"Remove and super-directories of aString from the collection."
	RecentDirs removeAllSuchThat: [ :aDir | ((aDir, '*') match: full)].

"If a sub-directory is in the list, do nothing."
	(RecentDirs detect: [ :aDir | ((full, '*') match: aDir)] ifNone: [nil])
		ifNotNil: [^self].

	[RecentDirs size >= 10]
		whileTrue: [RecentDirs removeFirst].
	RecentDirs addLast: full! !

!FileList methodsFor: 'server list' stamp: 'stephane.ducasse 7/3/2009 21:38'!
askServerInfo
	"Get the user to create a ServerDirectory for a new server.  Fill in and say Accept."
	| template |
	template := '"Please fill in the following info, then select all text and choose DoIt."

	| aa | 
	self flag: #ViolateNonReferenceToOtherClasses.
	aa := ServerDirectory new.
	aa server: ''st.cs.uiuc.edu''.    "host"
	aa user: ''anonymous''.
	aa password: ''yourEmail@school.edu''.
	aa directory: ''/Smalltalk/Squeak/Goodies''.
	aa url: ''''.    "<- this is optional.  Only used when *writing* update files."
	ServerDirectory addServer: aa named: ''UIUCArchive''.  "<- known by this name in Squeak"'.

	(StringHolder new contents: template) openLabel: 'FTP Server Form'
	! !

!FileList methodsFor: 'own services' stamp: 'MarcusDenker 1/23/2011 09:26'!
basicDeleteDirectory
	"Remove the currently selected directory"
	| localDirName |
	directory entries isEmpty ifFalse:[^self inform:'Directory must be empty'].
	localDirName := directory localName.
	(self confirm: 'Really delete ' , localDirName , '?') ifFalse: [^ self].
	self volumeListIndex: self volumeListIndex-1.
	directory deleteDirectory: localDirName.
! !

!FileList methodsFor: 'initialization' stamp: 'MarcusDenker 5/25/2011 12:21'!
buttonSelectorsToSuppress
	"Answer a list of action selectors whose corresponding services we would prefer *not* to have appear in the filelist's button pane; this can be hand-jimmied to suit personal taste."

	^ #(removeLineFeeds: addFileToNewZip: compressFile:)! !

!FileList methodsFor: 'private' stamp: 'RAA 4/6/2001 12:45'!
cancelHit

	modalView delete.
	directory := fileName := currentDirectorySelected := nil.! !

!FileList methodsFor: 'volume list and pattern' stamp: 'nk 6/14/2004 09:45'!
changeDirectoryTo: aFileDirectory
	"Change directory as requested."

	self directory: aFileDirectory.
	self updateDirectory! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 19:16'!
compressFile
	"Compress the currently selected file"

	| f |
	f := StandardFileStream
				readOnlyFileNamed: (directory fullNameFor: self fullName).
	f compressFile.
	self updateFileList! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 3/27/2011 17:20'!
contents
	"Answer the contents of the file, reading it first if needed."
	"Possible brevityState values:
		FileList,
		fullFile, briefFile, needToGetFull, needToGetBrief,
		fullHex, briefHex, needToGetFullHex, needToGetBriefHex"

	(listIndex = 0) | (brevityState == #FileList) ifTrue: [^ self defaultContents].  "no file selected"
	brevityState == #fullFile ifTrue: [^ contents]. 
	brevityState == #fullHex ifTrue: [^ contents].
	brevityState == #briefFile ifTrue: [^ contents].
	brevityState == #briefHex ifTrue: [^ contents].

	brevityState == #needToGetFullHex ifTrue: [^ self readContentsHex: false].
	brevityState == #needToGetBriefHex ifTrue: [^ self readContentsHex: true].

	brevityState == #needToGetFull ifTrue: [^ self readContentsBrief: false].
	brevityState == #needToGetBrief ifTrue: [^ self readContentsBrief: true].  "default"

	(TextConverter allEncodingNames includes: brevityState) 
		ifTrue: [ ^self readContentsAsEncoding: brevityState].

	self halt: 'unknown state ' , brevityState printString! !

!FileList methodsFor: 'menu messages' stamp: 'hfm 11/29/2008 19:22'!
copyName

	listIndex = 0 ifTrue: [^ self].
	Clipboard clipboardText: self fullName asText.
! !

!FileList methodsFor: 'private' stamp: 'LC 1/6/2002 06:50'!
currentDirectorySelected
	^ currentDirectorySelected
! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 8/15/2010 15:02'!
defaultContents
	contents := list isNil
		ifTrue: [String new]
		ifFalse: [String streamContents:
					[:s | s nextPutAll: 'NO FILE SELECTED' translated; cr.
					s nextPutAll: '  -- Folder Summary --' translated; cr.
					list do: [:item | s nextPutAll: item; cr]]].
	brevityState := #FileList.
	^ contents! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 2/14/2010 09:36'!
defaultEncoderFor: aFileName

	"This method just illustrates the stupidest possible implementation of encoder selection."
	| l |
	l := aFileName asLowercase.
	((l endsWith: 'cs') or: [l endsWith: 'st']) ifTrue: [
		^ MacRomanTextConverter new].
	^ Latin1TextConverter new.

	! !

!FileList methodsFor: 'own services' stamp: 'CarloTeixeira 7/3/2010 22:25'!
deleteDirectory
	self basicDeleteDirectory.
	self updateFileList.
	self updateDirectory.
! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 19:10'!
deleteFile
	"Delete the currently selected file"
	listIndex = 0 ifTrue: [^ self].
	(self confirm: ('Really delete {1}?' translated format:{fileName})) ifFalse: [^ self].
	directory deleteFileNamed: fileName.
	self updateFileList.
	brevityState := #FileList.
	self get! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:47'!
directory

	^ directory! !

!FileList methodsFor: 'accessing' stamp: 'MarcusDenker 8/15/2010 15:02'!
directory: dir
	"Set the path of the volume to be displayed."
	
	self okToChange ifFalse: [^ self].
	self modelSleep.
	directory := dir ifNil: [FileDirectory on: ''].
	self modelWakeUp.
	sortMode isNil ifTrue: [sortMode := #date].
		volList := ((Array with: '[]'), directory pathParts)  
				withIndexCollect: [:each :i | ( String new: i-1 withAll: $ ), each].
	volListIndex := volList size.	
	self changed: #relabel.
	self changed: #volumeList.
	self pattern: pattern! !

!FileList methodsFor: 'private' stamp: 'ar 2/12/2001 16:20'!
directoryNamesFor: item
	"item may be file directory or server directory"
	| entries |
	entries := item directoryNames.
	dirSelectionBlock ifNotNil:[entries := entries select: dirSelectionBlock].
	^entries! !

!FileList methodsFor: 'drag''n''drop' stamp: 'hfm 11/29/2008 19:20'!
dragPassengerFor: item inMorph: dragSource
	^self directory fullNameFor: ((self fileNameFromFormattedItem: item contents copy)
		copyReplaceAll: self folderString with: '').
! !

!FileList methodsFor: 'drag''n''drop' stamp: 'hfm 11/29/2008 19:21'!
dragTransferTypeForMorph: aMorph
	^#file! !

!FileList methodsFor: 'drag''n''drop' stamp: 'nk 6/15/2003 13:07'!
dropDestinationDirectory: dest event: evt
	"Answer a FileDirectory representing the drop destination in the directory hierarchy morph dest"
	^ (dest itemFromPoint: evt position) withoutListWrapper! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 18:31'!
dynamicButtonServices
	"Answer services for buttons that may come and go in the button pane, depending on selection"

	^ fileName isEmptyOrNil
		ifTrue:
			[#()]
		ifFalse:
			[ | toReject |
				toReject := self buttonSelectorsToSuppress.
				(self itemsForFile: self fullName) reject:
					[:svc | toReject includes: svc selector]]! !

!FileList methodsFor: 'private' stamp: 'pmm 3/13/2010 11:06'!
entriesMatching: patternString
	"Answer a list of directory entries which match the patternString.
	The patternString may consist of multiple patterns separated by ';'.
	Each pattern can include a '*' or '#' as wildcards - see String>>match:"

	| entries patterns |
	entries := directory entries.
	patterns := patternString findTokens: ';'.
	(patterns anySatisfy: [:each | each = '*'])
		ifTrue: [^ entries].
	^ entries select: [:entry | 
		entry isDirectory or: [patterns anySatisfy: [:each | each match: entry name]]]! !

!FileList methodsFor: 'file list menu' stamp: 'AlainPlantec 10/19/2010 23:14'!
fileContentsMenu: aMenu shifted: shifted
	"Construct aMenu to have items appropriate for the file browser's code pane, given the shift state provided"

	| shiftMenu services maybeLine extraLines |
	shifted ifTrue:
		[shiftMenu := SmalltalkEditor shiftedYellowButtonMenu.
		^ aMenu addAllFrom: shiftMenu].
	fileName ifNotNil:
		[services := OrderedCollection new.
		(#(briefHex briefFile needToGetBriefHex needToGetBrief) includes: brevityState) ifTrue:
			[services add: self serviceGet].
		(#(fullHex briefHex needToGetFullHex needToGetBriefHex) includes: brevityState) ifFalse:
			[services add: self serviceGetHex].
		(#(needToGetShiftJIS needToGetEUCJP needToGetCNGB needToGetEUCKR needToGetUTF8) includes: brevityState) ifFalse:
			[services add: self serviceGetEncodedText].
		maybeLine := services size.
		(FileStream sourceFileSuffixes includes: self suffixOfSelectedFile) ifTrue:
			[services addAll:
				(self servicesFromSelectorSpecs:
					#(fileIntoNewChangeSet: fileIn: browseChangesFile: browseFile:))].

		extraLines := OrderedCollection new.
		maybeLine > 0 ifTrue: [extraLines add: maybeLine].
		services size > maybeLine ifTrue: [extraLines add: services size].
		aMenu 
			addServices: services
			for: self fullName
			extraLines: extraLines].

	aMenu addList: {
			{'Find...(f)' translated.		#find. nil. #smallFindIcon}.
			{'Find again (g)' translated.		#findAgain. nil. #smallFindIcon}.
			{'Set search string (h)' translated.	#setSearchString}.
			#-.
			{'Do again (j)' translated.		#again. nil. #smallRedoIcon}.
			{'Undo (z)' translated.			#undo. nil. #smallUndoIcon}.
			#-.
			{'Copy (c)' translated.			#copySelection. nil. #smallCopyIcon}.
			{'Cut (x)' translated.			#cut. nil. #smallCutIcon}.
			{'Paste (v)' translated.		#paste. nil. #smallPasteIcon}.
			{'Paste...' translated.			#pasteRecent. nil. #smallPasteIcon}.
			#-.
			{'Do it (d)' translated.		#doIt. nil. #smallDoItIcon}.
			{'Print it (p)' translated.		#printIt. nil. #smallPrintIcon}.
			{'Inspect it (i)' translated.		#inspectIt. nil. #smallInspectItIcon}.
			{'FileIn selection (G)' translated.	#fileItIn}.
			#-.
			{'Accept (s)' translated.		#accept. nil. #smallOkIcon}.
			{'Cancel (l)' translated.		#cancel. nil. #smallCancelIcon}.
			#-.
			{'More...' translated.			#shiftedYellowButtonActivity}}.


	^ aMenu
! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:30'!
fileList
	"Answer the list of files in the current volume."

	^ list! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:31'!
fileListIndex
	"Answer the index of the currently selected file."

	^ listIndex! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:32'!
fileListIndex: anInteger
	"Select the file name having the given index, and display its contents."

	| item name |
	self okToChange ifFalse: [^ self].
	listIndex := anInteger.
	listIndex = 0 
		ifTrue: [fileName := nil]
		ifFalse:
			[item := self fileNameFromFormattedItem: (list at: anInteger).
			(item endsWith: self folderString)
				ifTrue:
					["remove [...] folder string and open the folder"
					name := item copyFrom: 1 to: item size - self folderString size.
					listIndex := 0.
					brevityState := #FileList.
					self addPath: name.
					name first = $^
						ifTrue: [self directory: (ServerDirectory serverNamed: name allButFirst)]
						ifFalse: [volListIndex = 1 ifTrue: [name := name, directory slash].
							self directory: (directory directoryNamed: name)]]
				ifFalse: [fileName := item]].  "open the file selected"

	brevityState := #needToGetBrief.
	self changed: #fileListIndex.
	self changed: #contents.
	self updateButtonRow! !

!FileList methodsFor: 'file list menu' stamp: 'hfm 11/29/2008 18:41'!
fileListMenu: aMenu

	fileName
		ifNil: [^ self noFileSelectedMenu: aMenu]
		ifNotNil: [^ self fileSelectedMenu: aMenu].
! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:48'!
fileName

	^ fileName! !

!FileList methodsFor: 'volume list and pattern' stamp: 'nice 3/14/2010 20:18'!
fileNameFormattedFrom: entry sizePad: sizePad
	"entry is a 5-element array of the form:
		(name creationTime modificationTime dirFlag fileSize)"
	| sizeStr nameStr dateStr |
	nameStr := (entry isDirectory)
		ifTrue: [(entry name) , self folderString]
		ifFalse: [entry name].
	dateStr := ((Date fromSeconds: (entry modificationTime) )
					printFormat: #(3 2 1 $. 1 1 2)) , ' ' ,
				(String streamContents: [:s |
					(Time fromSeconds: (entry modificationTime) \\ 86400)
						print24: true on: s]).
	sizeStr := (entry fileSize) asStringWithCommas.
	sortMode = #name ifTrue:
		[^ nameStr , '    (' , dateStr , ' ' , sizeStr , ')'].
	sortMode = #date ifTrue:
		[^ '(' , dateStr , ' ' , sizeStr , ') ' , nameStr].
	sortMode = #size ifTrue:
		[^ '(' , ((sizeStr size to: sizePad) collect: [:i | $ ]) , sizeStr , ' ' , dateStr , ') ' , nameStr].
! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:32'!
fileNameFromFormattedItem: item
	"Extract fileName and folderString from a formatted fileList item string"

	| from to |
	self sortingByName
		ifTrue: [
			from := item lastIndexOf: $( ifAbsent: [0].
			to := item lastIndexOf: $) ifAbsent: [0]]
		ifFalse: [
			from := item indexOf: $( ifAbsent: [0].
			to := item indexOf: $) ifAbsent: [0]].
	^ (from * to = 0
		ifTrue: [item]
		ifFalse: [item copyReplaceFrom: from to: to with: '']) withBlanksTrimmed! !

!FileList methodsFor: 'file list menu' stamp: 'tbn 8/11/2010 10:17'!
fileSelectedMenu: aMenu
	| n1 n2 n3 services |
	services := OrderedCollection withAll: (self itemsForFile: self fullName).
	n1 := services size.
	services addAll: self itemsForAnyFile.
	n2 := services size.
	services addAll: self itemsForNoFile.
	n3 := services size.
	services := services collect: [ :svc | svc copy.].
	services do: [ :svc | svc addDependent: self; doNotUseLineAfter ].
	^ aMenu 
		addServices2: services 
		for: self
		extraLines: (Array with: n1 with: n2 with: n3)
! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:27'!
folderString
	^ ' [...]'! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:33'!
fullName
	"Answer the full name for the currently selected file; answer nil if no file is selected."

	^ fileName ifNotNil: [directory
		ifNil:
			[FileDirectory default fullNameFor: fileName]
		ifNotNil:
			[directory fullNameFor: fileName]]
! !

!FileList methodsFor: 'file menu action' stamp: 'MarcusDenker 8/15/2010 15:06'!
get
	"Get contents of file again, it may have changed. Do this by making the cancel string be the contents, and doing a cancel."

	Cursor read
		showWhile: [ 
			self okToChange
				ifFalse: [ ^ nil ].
			brevityState := brevityState == #briefHex
				ifTrue: [ #needToGetFullHex ]
				ifFalse: [ #needToGetFull ].
			self changed: #contents ]! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 19:24'!
getEncodedText

	Cursor read showWhile: [
		self selectEncoding.
		self changed: #contents].
! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 19:24'!
getHex
	"Get contents of file again, and display in Hex. Do this by making the cancel string be the contents, and doing a cancel."

	Cursor read showWhile: [
		brevityState := #needToGetBriefHex.
		self changed: #contents].
! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 8/15/2010 14:59'!
getSelectedDirectory
	ok ifFalse: [^ nil].
	^ currentDirectorySelected
! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 8/15/2010 14:59'!
getSelectedFile
	"Answer a filestream on the selected file.  If it cannot be opened for read/write, try read-only before giving up; answer nil if unsuccessful"

	ok ifFalse: [^ nil].
	directory ifNil: [^ nil].
	fileName ifNil: [^ nil].
	^ (directory oldFileNamed: fileName) ifNil:
		[directory readOnlyFileNamed: fileName]! !

!FileList methodsFor: 'initialization' stamp: 'nice 1/5/2010 15:59'!
initialDirectoryList

	| dirList |
	dirList := (FileDirectory on: '') directoryNames collect: [ :each |
		FileDirectoryWrapper with: (FileDirectory on: each) name: each model: self].
	dirList isEmpty ifTrue:[
		dirList := Array with: (FileDirectoryWrapper 
			with: FileDirectory default 
			name: FileDirectory default localName 
			model: self)].
	dirList := dirList,(
		ServerDirectory serverNames collect: [ :n | | dir nameToShow | 
			dir := ServerDirectory serverNamed: n.
			nameToShow := n.
			(dir directoryWrapperClass with: dir name: nameToShow model: self)
				balloonText: dir realUrl
		]
	).
	^dirList! !

!FileList methodsFor: 'initialize-release' stamp: 'MarcusDenker 8/15/2010 15:05'!
initialize
	super initialize.
	ok := false.
	dirSelectionBlock := [ :dirName | true].! !

!FileList methodsFor: 'drag''n''drop' stamp: 'nk 6/15/2003 22:00'!
isDirectoryList: aMorph
	^aMorph isKindOf: SimpleHierarchicalListMorph! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:44'!
isFileSelected
	"return if a file is currently selected"

	^ fileName notNil! !

!FileList methodsFor: 'file list menu' stamp: 'hfm 11/29/2008 18:42'!
itemsForAnyFile
	"Answer a list of universal services that could apply to any file"
	
	| services |
	services := OrderedCollection new: 4.
	services add: self serviceCopyName. 
	services add: self serviceRenameFile. 
	services add: self serviceDeleteFile.
	services add: self serviceViewContentsInWorkspace.
	^ services! !

!FileList methodsFor: 'file list menu' stamp: 'StephaneDucasse 3/5/2010 14:42'!
itemsForDirectory: dir 
	| services |
	services := OrderedCollection new.
	dir ifNotNil: [
		services
			addAll: (self class itemsForDirectory: dir).
		services last useLineAfter. ].
	services add: self serviceAddNewFile.
	services add: self serviceAddNewDirectory.
	^ services! !

!FileList methodsFor: 'file list menu' stamp: 'MarcusDenker 5/25/2011 12:24'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"
	| suffix |
	suffix := self class suffixOf: fullName.
	^ (self class itemsForFile: fullName).! !

!FileList methodsFor: 'file list menu' stamp: 'StephaneDucasse 3/5/2010 14:42'!
itemsForNoFile

	| services |
	services := OrderedCollection new.
	services add: self serviceSortByName.
	services add: self serviceSortBySize.
	services add: (self serviceSortByDate useLineAfter).
	services addAll: (self itemsForDirectory: (self isFileSelected ifFalse: [ self directory ] ifTrue: [])).
	^ services

		! !

!FileList methodsFor: 'initialization' stamp: 'RAA 6/16/2000 10:40'!
labelString
	^ (directory ifNil: [^'[]']) pathName contractTo: 50! !

!FileList methodsFor: 'volume list and pattern' stamp: 'StephaneDucasse 3/28/2010 20:47'!
listForPatterns: anArray
	"Make the list be those file names which match the pattern."

	| sizePad newList |
	newList := Set new.
	anArray do: [ :pat | newList addAll: (self entriesMatching: pat) ].
	newList := newList asArray sort: self sortBlock.
	sizePad := (newList inject: 0 into: [:mx :entry | mx max: (entry fileSize)])
					asStringWithCommas size - 1.
	newList := newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^ newList , (ServerDirectory serverNames collect: [:n | '^' , n , self folderString])].
	^ newList ! !

!FileList methodsFor: 'private' stamp: 'RAA 6/21/2000 12:06'!
modalView: aSystemWindowOrSuch

	modalView := aSystemWindowOrSuch! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 19:25'!
modelSleep
	"User has exited or collapsed the window -- close any remote connection."

	directory ifNotNil: [directory sleep]! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 19:26'!
modelWakeUp
	"User has entered or expanded the window -- reopen any remote connection."

	(directory notNil and:[directory isRemoteDirectory])
		ifTrue: [[directory wakeUp] on: TelnetProtocolError do: [ :ex | self inform: ex printString ]] "It would be good to implement a null method wakeUp on the root of directory"! !

!FileList methodsFor: 'user interface' stamp: 'RAA 2/17/2001 12:18'!
morphicDirectoryTreePane

	^self morphicDirectoryTreePaneFiltered: #initialDirectoryList
! !

!FileList methodsFor: 'user interface' stamp: 'rww 12/13/2003 13:07'!
morphicDirectoryTreePaneFiltered: aSymbol
	^(SimpleHierarchicalListMorph 
		on: self
		list: aSymbol
		selected: #currentDirectorySelected
		changeSelected: #setSelectedDirectoryTo:
		menu: #volumeMenu:
		keystroke: nil)
			autoDeselect: false;
			enableDrag: false;
			enableDrop: true;
			yourself
		
! !

!FileList methodsFor: 'user interface' stamp: 'BenjaminVanRyseghem 3/2/2011 10:37'!
morphicFileContentsPane

	^PluggableTextMorph 
		on: self 
		text: #contents 
		accept: #put:
		readSelection: nil 
		menu: #fileContentsMenu:shifted:
! !

!FileList methodsFor: 'user interface' stamp: 'nk 6/15/2003 13:05'!
morphicFileListPane

	^(PluggableListMorph 
		on: self 
		list: #fileList 
		selected: #fileListIndex
		changeSelected: #fileListIndex: 
		menu: #fileListMenu:)
			enableDrag: true;
			enableDrop: false;
			yourself

! !

!FileList methodsFor: 'user interface' stamp: 'gvc 2/6/2009 12:34'!
morphicPatternPane
	"Remove the vertical scrollbar since the minHeight would otherwise
	be too large to fit the layout frame. Added here for Pharo since
	FileList2 has been merged into FileList."
	
	|pane|
 	pane := PluggableTextMorph 
		on: self 
		text: #pattern 
		accept: #pattern:.
 	pane
		acceptOnCR: true;
		hideVScrollBarIndefinitely: true.
 	^pane! !

!FileList methodsFor: 'file list menu' stamp: 'hfm 11/29/2008 19:26'!
noFileSelectedMenu: aMenu

	^ aMenu
		addServices: self itemsForNoFile 
		for: self
		extraLines: #()
		
! !

!FileList methodsFor: 'private' stamp: 'md 10/22/2003 15:27'!
okHit
	ok := true.
	currentDirectorySelected
		ifNil: [Beeper beep]
		ifNotNil: [modalView delete]! !

!FileList methodsFor: 'own services' stamp: 'sw 2/22/2002 02:35'!
okayAndCancelServices
	"Answer ok and cancel services"

	^ {self serviceOkay. self serviceCancel}! !

!FileList methodsFor: 'own services' stamp: 'alain.plantec 5/30/2008 13:12'!
openImageInWindow
	"Handle five file formats: GIF, JPG, PNG, Form stoteOn: (run coded), and
	BMP. Fail if file format is not recognized."
	| image myStream |
	myStream := (directory readOnlyFileNamed: fileName) binary.
	image := Form fromBinaryStream: myStream.
	myStream close.
	(World drawingClass withForm: image) openInWorld! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 18:29'!
optionalButtonRow
	"Answer the button row associated with a file list"

	| aRow |
	aRow := AlignmentMorph newRow beSticky.
	aRow color: Color transparent.
	aRow clipSubmorphs: true.
	aRow layoutInset: 5@1; cellInset: 6.
	self universalButtonServices do:  "just the three sort-by items"
			[:service |
				aRow addMorphBack: (service buttonToTriggerIn: self).
				(service selector  == #sortBySize)
					ifTrue:
						[aRow addTransparentSpacerOfSize: (4@0)]].
	aRow setNameTo: 'buttons'.
	aRow setProperty: #buttonRow toValue: true.  "Used for dynamic retrieval later on"
	^ aRow! !

!FileList methodsFor: 'initialization' stamp: 'hfm 12/12/2008 13:30'!
optionalButtonSpecs
	"Answer a list of services underlying the optional buttons in their initial inception."

	^ optionalButtonSpecs 
		ifNil: [ { self serviceSortByName . self serviceSortByDate . self serviceSortBySize } ]
	
	! !

!FileList methodsFor: 'initialization' stamp: 'RAA 6/16/2000 13:01'!
optionalButtonSpecs: anArray

	optionalButtonSpecs := anArray! !

!FileList methodsFor: 'accessing' stamp: 'hfm 11/29/2008 18:22'!
pattern

	^ pattern ifNil: ['*']
! !

!FileList methodsFor: 'accessing' stamp: 'StephaneDucasse 3/24/2010 10:14'!
pattern: textOrStringOrNil

	textOrStringOrNil
		ifNil: [pattern := '*']
		ifNotNil: [pattern := textOrStringOrNil asString].
	pattern isEmpty ifTrue: [pattern := '*'].
	self updateFileList.
	^ true
! !

!FileList methodsFor: 'menu messages' stamp: 'MarcusDenker 3/23/2011 18:28'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user.
	If it's one of the three sort-by items, handle it specially.
	If I can respond myself, then perform it on myself. 
	If not, send it to otherTarget, presumably the editPane from which the menu was invoked."

	^ (#(sortByDate sortBySize sortByName) includes: selector)
		ifTrue:
			[self resort: selector]
		ifFalse:
			[(#(get getHex copyName openImageInWindow renameFile deleteFile addNewFile) includes: selector)
				ifTrue: [self perform: selector]
				ifFalse: [super perform: selector orSendTo: otherTarget]]! !

!FileList methodsFor: 'private' stamp: 'RAA 6/16/2000 10:48'!
postOpen

	directory ifNotNil: [
		self changed: #(openPath) , directory pathParts. 
	].
! !

!FileList methodsFor: 'drag''n''drop' stamp: 'nice 1/5/2010 15:59'!
primitiveCopyFileNamed: srcName to: dstName 
	"Copied from VMMaker code.
	This really ought to be a facility in file system. The major annoyance 
	here is that file types and permissions are not handled by current 
	Squeak code.
	NOTE that this will clobber the destination file!!"
	| src dst |
	<primitive: 'primitiveFileCopyNamedTo' module:'FileCopyPlugin'> "primitiveExternalCall" 
	"If the plugin doesn't do it, go the slow way and lose the filetype info"
	"This method may signal FileDoesNotExistException if either the source or 
	dest files cannnot be opened; possibly permissions or bad name problems"
	[ | buffer |[src := FileStream readOnlyFileNamed: srcName]
		on: FileDoesNotExistException
		do: [^ self error: ('could not open file ', srcName)].
	[dst := FileStream forceNewFileNamed: dstName]
		on: FileDoesNotExistException
		do: [^ self error: ('could not open file ', dstName)].
	buffer := String new: 50000.
	[src atEnd]
		whileFalse: [dst
				nextPutAll: (src nextInto: buffer)]]
		ensure: [src
				ifNotNil: [src close].
			dst
				ifNotNil: [dst close]]! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 19:27'!
put: aText
	"Private - put the supplied text onto the file"

	| ff type |
	brevityState == #fullFile ifTrue:
		[ff := directory newFileNamed: self fullName.
		Cursor write showWhile: [ff nextPutAll: aText asString; close].
		fileName = ff localName 
			ifTrue: [contents := aText asString]
			ifFalse: [self updateFileList].		"user renamed the file"
		^ true  "accepted"].

	listIndex = 0 ifTrue:
		[self inform: 'No fileName is selected' translated.
		^ false  "failed"].
	type := 'These'.
	brevityState = #briefFile ifTrue: [type := 'Abbreviated'].
	brevityState = #briefHex ifTrue: [type := 'Abbreviated'].
	brevityState = #fullHex ifTrue: [type := 'Hexadecimal'].
	brevityState = #FileList ifTrue: [type := 'Directory'].
	self inform: ('{1} contents cannot
meaningfully be saved at present.' translated format:{type translated}).
	^ false  "failed"
! !

!FileList methodsFor: 'private' stamp: 'PeterHugossonMiller 9/3/2009 01:26'!
readContentsAsEncoding: encodingName
	| f writeStream converter c |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := String new writeStream.
	converter := TextConverter defaultConverterClassForEncoding: encodingName.
	converter ifNil: [^ 'This encoding is not supported'].
	f converter: converter new.
	f wantsLineEndConversion: true.
	[f atEnd or: [(c := f next) isNil]]
		whileFalse: [writeStream nextPut: c].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 19:18'!
readContentsBrief: brevityFlag
	"Read the contents of the receiver's selected file, unless it is too long, in which case show just the first 5000 characters. Don't create a file if it doesn't already exist."
	| f fileSize first5000 |

	brevityFlag ifTrue: [
		directory isRemoteDirectory ifTrue: [^ self readServerBrief]].
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read' translated].
	f converter: (self defaultEncoderFor: self fullName).
	(brevityFlag not or: [(fileSize := f size) <= 100000]) ifTrue:
		[contents := f contentsOfEntireFile.
		brevityState := #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 := f next: 5000.
	f close.
	contents := 'File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 5000 characters.' translated format: {fileName. fileSize. first5000}.
	brevityState := #briefFile.   "don't change till actually read"
	^ contents.
! !

!FileList methodsFor: 'private' stamp: 'MarcusDenker 8/15/2010 15:02'!
readContentsHex: brevity
	"retrieve the contents from the external file unless it is too long.
	  Don't create a file here.  Check if exists."
	| f size data hexData s |

	f := directory oldFileOrNoneNamed: self fullName. 
	f isNil ifTrue: [^ 'For some reason, this file cannot be read' translated].
	f binary.
	((size := f size)) > 5000 & brevity
		ifTrue: [data := f next: 10000. f close. brevityState := #briefHex]
		ifFalse: [data := f contentsOfEntireFile. brevityState := #fullHex].

	s := (String new: data size*4) writeStream.
	0 to: data size-1 by: 16 do:
		[:loc | s nextPutAll: loc printStringHex; space;
			nextPut: $(; print: loc; nextPut: $); space; tab.
		loc+1 to: (loc+16 min: data size) do: [:i | s nextPutAll: (data at: i) printStringHex; space].
		s cr].
	hexData := s contents.

	^ contents := ((size > 5000) & brevity
		ifTrue: ['File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 5000 characters.' translated format: {fileName. size. hexData}]
		ifFalse: [hexData]).
! !

!FileList methodsFor: 'file list' stamp: 'hfm 11/29/2008 18:47'!
readOnlyStream
	"Answer a read-only stream on the selected file. For the various stream-reading services."

	^self directory ifNotNil: [ :dir | dir readOnlyFileNamed: self fileName ]! !

!FileList methodsFor: 'private' stamp: 'dgd 12/27/2003 12:09'!
readServerBrief
	| lString sizeStr fsize ff first5000 parts |
	"If file on server is known to be long, just read the beginning.  Cheat badly by reading the fileList string."

	listIndex = 0 ifTrue: [^ self].
	"Get size from file list entry"
	lString := list at: listIndex.
	parts := lString findTokens: '()'.
	sortMode = #name ifTrue: [sizeStr := (parts second findTokens: ' ') third].
	sortMode = #date ifTrue: [sizeStr := (parts first findTokens: ' ') third].
	sortMode = #size ifTrue: [sizeStr := (parts first findTokens: ' ') first].
	fsize := (sizeStr copyWithout: $,) asNumber.

	fsize <= 50000 ifTrue:
		[ff := directory oldFileOrNoneNamed: self fullName.
		ff ifNil: [^ 'For some reason, this file cannot be read' translated].
		contents := ff contentsOfEntireFile.
		brevityState := #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 := directory getOnly: 3500 from: fileName.
	contents := 'File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 3500 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 3500 characters.' translated format: {fileName. sizeStr. first5000}.
	brevityState := #briefFile.   "don't change till actually read"
	^ contents.

! !

!FileList methodsFor: 'private' stamp: 'alain.plantec 2/6/2009 16:55'!
recentDirs
	"Put up a menu and let the user select from the list of recently visited directories."

	| dirName |
	RecentDirs isEmpty ifTrue: [^self].
	dirName := UIManager default chooseFrom: RecentDirs values: RecentDirs.
	dirName ifNil: [^self].
	self directory: (FileDirectory on: dirName)! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:38'!
registeredFileReaderClasses
	"return the list of classes that provide file reader services"

	^ self class registeredFileReaderClasses! !

!FileList methodsFor: 'initialization' stamp: 'MarcusDenker 8/15/2010 15:07'!
release
	super release.
	self modelSleep! !

!FileList methodsFor: 'server list' stamp: 'MarcusDenker 3/5/2010 14:34'!
removeServer

	| choice names |
	names := ServerDirectory serverNames.
	choice := UIManager default chooseFrom: names values: names.
	choice ifNil: [^ self].
	ServerDirectory removeServerNamed: choice! !

!FileList methodsFor: 'file menu action' stamp: 'DamienCassou 9/29/2009 12:57'!
renameFile
	"Rename the currently selected file"
	| newName response |
	listIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(response := UIManager default request: 'NewFileName?' translated
 					initialAnswer: fileName)
		isEmptyOrNil ifTrue: [^ self].
	newName := response asFileName.
	newName = fileName ifTrue: [^ self].
	directory rename: fileName toBe: newName.
	self updateFileList.
	listIndex := list findFirst: [:item | (self fileNameFromFormattedItem: item) = newName].
	listIndex > 0 ifTrue: [fileName := newName].
	self changed: #fileListIndex.
! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:50'!
resort: newMode
	"Re-sort the list of files."

	| name |
	listIndex > 0
		ifTrue: [name := self fileNameFromFormattedItem: (list at: listIndex)].
	sortMode := newMode.
	self pattern: pattern.
	name ifNotNil: [
		fileName := name.
		listIndex := list findFirst: [:item | (self fileNameFromFormattedItem: item) = name. ].
		self changed: #fileListIndex].
	listIndex = 0 ifTrue: [self changed: #contents].
	self updateButtonRow
! !

!FileList methodsFor: 'private' stamp: 'StephaneDucasse 2/25/2011 18:42'!
selectEncoding
	"self new selectEncoding"
	
	| aMenu  |
	aMenu := UIManager default 
				newMenuIn: self for: self.
	TextConverter allSubclasses do: [:each | | names |
		names := each encodingNames.
		names notEmpty ifTrue: [ | label |
			label := '' writeStream.
			names 
				do: [:eachName | label nextPutAll: eachName ] 
				separatedBy: [ label nextPutAll: ', '].
			aMenu  add: label contents action: names first asSymbol.
		].
	].
	aMenu popUpInWorld
	! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:12'!
serviceAddNewDirectory
	"Answer a service entry characterizing the 'add new directory' command"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Add new directory' 
		selector: #addNewDirectory
		description: 'Adds a new, empty directory (folder)' ! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:59'!
serviceAddNewFile
	"Answer a service entry characterizing the 'add new file' command"

	^(SimpleServiceEntry 
		provider: self 
		label: 'Add new file' 
		selector: #addNewFile 
		description: 'Create a new,. empty file, and add it to the current directory.')
		usingLineAfter: false! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:13'!
serviceCancel
	"Answer a service for hitting the cancel button"

	^ (SimpleServiceEntry new
		provider: self label: 'Cancel' 
		selector: #cancelHit 
		description: 'Hit here to cancel ')
		buttonLabel: 'Cancel'! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:13'!
serviceCompressFile
	"Answer a service for compressing a file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Compress' 
		selector: #compressFile 
		description: 'Compress file' 
		buttonLabel: 'Compress'! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:13'!
serviceCopyName

	^ (SimpleServiceEntry 
		provider: self 
		label: 'Copy name to clipboard' 
		selector: #copyName 
		description: 'Copy name to clipboard' )! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:14'!
serviceDeleteFile

	^ (SimpleServiceEntry 
		provider: self 
		label: 'Delete' 
		selector: #deleteFile)
		description: 'Delete the seleted item'! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:14'!
serviceGet
	"Answer a service for getting the entire file"

	^  (SimpleServiceEntry 
			provider: self 
			label: 'Get entire file' 
			selector: #get
			description: 'If the file has only been partially read in, because it is very large, read the entire file in at this time.')! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:14'!
serviceGetEncodedText

	^  (SimpleServiceEntry 
			provider: self 
			label: 'View as encoded text'
			selector: #getEncodedText
			description: 'View as encoded text')

! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:14'!
serviceGetHex

	^  (SimpleServiceEntry 
			provider: self 
			label: 'View as hex' 
			selector: #getHex
			description: 'View as hex')
			
! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:14'!
serviceOkay
	"Answer a service for hitting the okay button"

	^ (SimpleServiceEntry new
		provider: self 
		label: 'Okay' 
		selector: #okHit 
		description: 'Hit here to accept the current selection')
		buttonLabel: 'Ok'! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:15'!
serviceRenameFile

	^ (SimpleServiceEntry 
		provider: self 
		label: 'Rename' 
		selector: #renameFile 
		description: 'Rename file')! !

!FileList methodsFor: 'initialization' stamp: 'tbn 8/11/2010 10:15'!
serviceSortByDate
	"Answer a service for sorting by date"

	^  (SimpleServiceEntry new
			provider: self 
			label: 'Sort by date' 
			selector: #sortByDate 
			description: 'Sort entries by date')
			extraSelector: #sortingByDate;
			buttonLabel: 'Date'! !

!FileList methodsFor: 'initialization' stamp: 'tbn 8/11/2010 10:53'!
serviceSortByName
	"Answer a service for soring by name"

	^ (SimpleServiceEntry new
		provider: self 
		label: 'Sort by name' 
		selector: #sortByName 
		description: 'Sort entries by name')
			extraSelector: #sortingByName;
			buttonLabel: 'Name';
			usingLineAfter: false! !

!FileList methodsFor: 'initialization' stamp: 'tbn 8/11/2010 10:53'!
serviceSortBySize
	"Answer a service for sorting by size"

	^  (SimpleServiceEntry 
			provider: self 
			label: 'Sort by size' 
			selector: #sortBySize
			description: 'Sort entries by size')
				extraSelector: #sortingBySize;
				buttonLabel: 'Size';
				usingLineAfter: false! !

!FileList methodsFor: 'own services' stamp: 'tbn 8/11/2010 10:17'!
serviceViewContentsInWorkspace
	"Answer a service for viewing the contents of a file in a workspace"
	
	^ (SimpleServiceEntry 
		provider: self 
		label: 'Workspace with contents' 
		selector: #viewContentsInWorkspace)
		description: 'Open a new Workspace whose contents are set to the contents of this file'! !

!FileList methodsFor: 'own services' stamp: 'sw 2/22/2002 02:36'!
servicesForFolderSelector
	"Answer the ok and cancel servies for the folder selector"

	^ self okayAndCancelServices! !

!FileList methodsFor: 'own services' stamp: 'MarcusDenker 5/25/2011 12:24'!
servicesFromSelectorSpecs: symbolArray
	"Answer an array of services represented by the incoming symbols, eliminating any that do not have a currently-registered service.  Pass the symbol #- along unchanged to serve as a separator between services"

	"FileList new servicesFromSelectorSpecs: #(fileIn: fileIntoNewChangeSet: browseChangesFile:)"

	| services col | 
	col := OrderedCollection new.
	services := self class allRegisteredServices.
	symbolArray do: 
		[:sel | | res | 
			sel == #-
				ifTrue:
					[col add: sel]
				ifFalse:
					[res := services
							detect: [:each | each selector = sel] ifNone: [nil].
					res notNil
							ifTrue: [col add: res]]].
	^ col! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 19:33'!
setFileStream: aStream
	"Used to initialize a spawned file editor.  Sets directory too."

	self directory: aStream directory.
	fileName := aStream localName.
	pattern := '*'.
	listIndex := 1.  "pretend a file is selected"
	aStream close.
	brevityState := #needToGetBrief.
	self changed: #contents.
! !

!FileList methodsFor: 'private' stamp: 'LC 1/6/2002 09:03'!
setSelectedDirectoryTo: aFileDirectoryWrapper
	currentDirectorySelected := aFileDirectoryWrapper.
	self directory: aFileDirectoryWrapper withoutListWrapper.
	brevityState := #FileList.
	"self addPath: path."
	self changed: #fileList.
	self changed: #contents.
	self changed: #currentDirectorySelected.! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:26'!
sortBlock
	"Answer block to decide what order to display the directory entries."

	^ [ :x :y |
			(x isDirectory = y isDirectory) 
				ifTrue: [  
					"sort by user-specified criterion"
					sortMode = #name 
						ifTrue: [(x name compare: y name) <= 2]
						ifFalse: [ sortMode = #date
							ifTrue: [ x modificationTime = y modificationTime
									ifTrue: [ (x name compare: y name) <= 2 ]
									ifFalse: [ x modificationTime > y modificationTime ] ]
							ifFalse: [ "size"
								x fileSize = y fileSize 
									ifTrue: [ (x name compare: y name) <= 2 ]
									ifFalse: [ x fileSize > y fileSize ] ] ] ]
				ifFalse: [
					"directories always precede files"
					x isDirectory ] ]! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 18:51'!
sortByDate
	self resort: #date! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 18:49'!
sortByName
	self resort: #name! !

!FileList methodsFor: 'file menu action' stamp: 'hfm 11/29/2008 18:50'!
sortBySize
	self resort: #size! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:50'!
sortingByDate
	^ sortMode == #date! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:33'!
sortingByName
	^ sortMode == #name! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 19:33'!
sortingBySize
	^ sortMode == #size! !

!FileList methodsFor: 'file list menu' stamp: 'hfm 11/29/2008 19:33'!
suffixOfSelectedFile
	"Answer the file extension of the receiver's selected file"
	^ self class suffixOf: self fullName.! !

!FileList methodsFor: 'initialization' stamp: 'sw 2/22/2002 02:34'!
universalButtonServices
	"Answer the services to be reflected in the receiver's buttons"

	^ self optionalButtonSpecs! !

!FileList methodsFor: 'updating' stamp: 'hfm 11/29/2008 19:33'!
update: aParameter
	"Receive a change notice from an object of whom the receiver is a dependent"

	(aParameter == #fileListChanged) ifTrue: [self updateFileList].
	super update: aParameter! !

!FileList methodsFor: 'initialization' stamp: 'hfm 11/29/2008 18:28'!
updateButtonRow
	"Dynamically update the contents of the button row, if any."

	| aWindow aRow |
	aWindow := self dependents 
				detect: [:m | (m isSystemWindow) and: [m model == self]]
				ifNone: [^self].
	aRow := aWindow findDeepSubmorphThat: [:m | m hasProperty: #buttonRow]
				ifAbsent: [^self].
	aRow submorphs size - 4 timesRepeat: [aRow submorphs last delete].
	self dynamicButtonServices do: 
			[:service | 
			aRow addMorphBack: (service buttonToTriggerIn: self).
			service addDependent: self]! !

!FileList methodsFor: 'initialization' stamp: 'nk 6/14/2004 09:39'!
updateDirectory
	"directory has been changed externally, by calling directory:.
	Now change the view to reflect the change."
	self changed: #currentDirectorySelected.
	self postOpen.! !

!FileList methodsFor: 'private' stamp: 'hfm 11/29/2008 18:24'!
updateFileList
	"Update my files list with file names in the current directory  
	that match the pattern.
	The pattern string may have embedded newlines or semicolons; these separate different patterns."
	| patterns |
	patterns := OrderedCollection new.
	Cursor wait showWhile: [
	(pattern findTokens: (String with: Character cr with: Character lf with: $;))
		do: [ :each |
			(each includes: $*) | (each includes: $#)
					ifTrue: [ patterns add: each]
					ifFalse: [each isEmpty
										ifTrue: [ patterns add: '*']
										ifFalse: [ patterns add: '*' , each , '*']]].

	list := self listForPatterns: patterns.
	listIndex := 0.
	volListIndex := volList size.
	fileName := nil.
	contents := ''.
	self changed: #volumeListIndex.
	self changed: #fileList.
	self updateButtonRow]! !

!FileList methodsFor: 'volume list and pattern' stamp: 'hfm 11/29/2008 19:34'!
veryDeepFixupWith: deepCopier
	super veryDeepFixupWith: deepCopier.
	volListIndex := 1.
	self directory: FileDirectory default.
	self updateFileList! !

!FileList methodsFor: 'own services' stamp: 'hfm 11/29/2008 18:51'!
viewContentsInWorkspace
	"View the contents of my selected file in a new workspace"
	
	| aString aFileStream aName |
	aString := (aFileStream := directory readOnlyFileNamed: self fullName) setConverterForCode contentsOfEntireFile.
	aName := aFileStream localName.
	aFileStream close.
	UIManager default edit: aString label: 'Workspace from ', aName! !

!FileList methodsFor: 'volume list and pattern' stamp: 'hfm 11/29/2008 19:34'!
volumeList
	"Answer the current list of volumes."

	^ volList
! !

!FileList methodsFor: 'volume list and pattern' stamp: 'hfm 11/29/2008 19:34'!
volumeListIndex
	"Answer the index of the currently selected volume."

	^ volListIndex
! !

!FileList methodsFor: 'volume list and pattern' stamp: 'hfm 11/29/2008 19:34'!
volumeListIndex: index
	"Select the volume name having the given index."

	| delim path |
	volListIndex := index.
	index = 1 
		ifTrue: [self directory: (FileDirectory on: '')]
		ifFalse: [delim := directory pathNameDelimiter.
				path := String streamContents: [:strm |
					2 to: index do: [:i |
						strm nextPutAll: (volList at: i) withBlanksTrimmed.
						i < index ifTrue: [strm nextPut: delim]]].
				self directory: (directory on: path)].
	brevityState := #FileList.
	self addPath: path.
	self changed: #fileList.
	self changed: #contents.
	self updateButtonRow! !

!FileList methodsFor: 'volume menu' stamp: 'tbn 7/29/2010 22:06'!
volumeMenu: aMenu
	aMenu addList: {
			{'Recent...' translated.		#recentDirs}.
			#-.
			{'Add server...' translated.		#askServerInfo}.
			{'Remove server...' translated.		#removeServer}.
			#-.
			{'Delete directory...' translated.	#deleteDirectory}.
			#-}.
	aMenu
		addServices: (self itemsForDirectory: self directory)
		for: self
		extraLines: #().
	^aMenu.! !

!FileList methodsFor: 'drag''n''drop' stamp: 'MarcusDenker 8/15/2010 15:08'!
wantsDroppedMorph: aTransferMorph event: evt inMorph: dest
	| retval |
	retval := (aTransferMorph isKindOf: TransferMorph)
		and: [ (aTransferMorph dragTransferType == #file)
				and: [ self isDirectoryList: dest ]].
	^retval! !

!Model methodsFor: '*Tools-Base' stamp: 'alain.plantec 6/10/2008 20:22'!
containingWindow
	"Answer the window that holds the receiver. The dependents technique is
	odious and may not be airtight, if multiple windows have the same
	model. "
	^ self dependents
		detect: [:d | (d isSystemWindow)
				and: [d model == self]]
		ifNone: []! !

!Model methodsFor: '*Tools-Base' stamp: 'MarcusDenker 3/18/2011 16:03'!
topView
	"Find the first top view on me. Is there any danger of their being two
	with the same model? Any danger from ungarbage collected old views?
	Ask if schedulled?"
	dependents ifNil: [^ nil].
	dependents do: [:v | (v isSystemWindow and: [v isInWorld]) ifTrue: [^ v]].
	^ nil! !

Model subclass: #ObjectExplorer
	instanceVariableNames: 'rootObject inspector monitorList currentSelection'
	classVariableNames: 'ShowIcons'
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!ObjectExplorer commentStamp: '<historical>' prior: 0!
ObjectExplorer provides a hierarchical alternative to #inspect. Simply evaluate an expression like:

World explore

and enjoy.!

!ObjectExplorer class methodsFor: 'as yet unclassified' stamp: 'RAA 6/21/1999 15:55'!
about

	StringHolder new textContents: self comment; openLabel: 'about ',self asString! !

!ObjectExplorer class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/20/2011 15:07'!
openOn: anObject
" -- common protocol for inspector/explorer "

	^ self new openExplorerFor: anObject! !

!ObjectExplorer class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 02:59'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #objectExplorer
! !

!ObjectExplorer class methodsFor: 'setting' stamp: 'AlainPlantec 12/6/2009 22:12'!
showIcons
	^ ShowIcons ifNil: [ShowIcons := true]! !

!ObjectExplorer class methodsFor: 'setting' stamp: 'AlainPlantec 12/6/2009 22:13'!
showIcons: aBoolean
	ShowIcons := aBoolean! !

!ObjectExplorer methodsFor: 'menus' stamp: 'RAA 9/23/1999 13:22'!
codePaneMenu: aMenu shifted: shifted
	"Note that unless we override perform:orSendTo:, PluggableTextController will respond to all menu items"
	^ StringHolder basicNew codePaneMenu: aMenu shifted: shifted
! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'RAA 9/23/1999 13:11'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0  "null selection"! !

!ObjectExplorer methodsFor: 'menus' stamp: 'sd 11/20/2005 21:27'!
defsOfSelection
	"Open a browser on all defining references to the selected instance variable, if that's what's currently selected."
	| aClass sel |

	(aClass := self parentObject class) isVariable ifTrue: [^ self changed: #flash].
	sel := self selector.
	self systemNavigation  browseAllStoresInto: sel from: aClass! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'RAA 9/23/1999 13:15'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'RAA 9/23/1999 13:19'!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	currentSelection ifNil: [^rootObject].
	^currentSelection withoutListWrapper
! !

!ObjectExplorer methodsFor: 'error handling' stamp: 'nk 7/24/2003 09:29'!
doesNotUnderstand: aMessage
	inspector ifNotNil: [ (inspector respondsTo: aMessage selector) ifTrue: [ ^inspector perform: aMessage selector withArguments: aMessage arguments ]].
	^super doesNotUnderstand: aMessage! !

!ObjectExplorer methodsFor: 'menus' stamp: 'mtf 4/25/2008 17:41'!
explorePointers
	"Open a PointerExplorer on the current selection"
	PointerExplorer new openExplorerFor: self object! !

!ObjectExplorer methodsFor: 'menus' stamp: 'nk 7/24/2003 10:26'!
exploreSelection
	"Open an ObjectExplorer on the current selection"
	self object explore! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'JuanVuletich 10/26/2010 14:52'!
explorerFor: anObject
	^UIManager default explorer: self for: anObject withLabel: (anObject printStringLimitedTo: 32)! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'JuanVuletich 10/26/2010 14:53'!
explorerFor: anObject withLabel: label 
	^UIManager default explorer: self for: anObject withLabel: label! !

!ObjectExplorer methodsFor: 'menus' stamp: 'stephane.ducasse 9/20/2008 22:23'!
explorerKey: aChar from: view

	"Similar to #genericMenu:..."
	| insideObject parentObject |
	currentSelection ifNotNil: [
		insideObject := self object.
		parentObject := self parentObject.
		inspector ifNil: [inspector := Inspector new].
		inspector
			inspect: parentObject;
			object: insideObject.

		aChar == $i ifTrue: [^ self inspectSelection].
		aChar == $I ifTrue: [^ self exploreSelection].

		aChar == $b ifTrue:	[^ inspector browseMethodFull].
		aChar == $h ifTrue:	[^ inspector classHierarchy].
		aChar == $c ifTrue: [^ inspector copyName].
		aChar == $p ifTrue: [^ inspector browseFullProtocol].
		aChar == $N ifTrue: [^ inspector browseClassRefs]].

	^ self arrowKey: aChar from: view! !

!ObjectExplorer methodsFor: 'menus' stamp: 'AndyKellens 6/11/2010 14:42'!
genericMenu: aMenu 
	"Borrow a menu from my inspector"
	| insideObject menu parentObject |
	currentSelection
		ifNil: [menu := aMenu.
			menu
				add: '*nothing selected*'
				target: self
				selector: #yourself]
		ifNotNil: [insideObject := self object.
			parentObject := self parentObject.
			inspector
				ifNil: [inspector := Inspector new].
			inspector inspect: parentObject;
				 object: insideObject.
			aMenu defaultTarget: inspector.
			inspector fieldListMenu: aMenu.
			aMenu items
				do: [:i | (#(#inspectSelection #exploreSelection #referencesToSelection #defsOfSelection #objectReferencesToSelection #explorePointers) includes: i selector)
						ifTrue: [i target: self]].
			aMenu addLine;
				add: 'monitor changes'
				target: self
				selector: #monitor:
				argument: currentSelection].
	monitorList isEmptyOrNil
		ifFalse: [aMenu addLine;
				add: 'stop monitoring all'
				target: self
				selector: #stopMonitoring].
	^ aMenu! !

!ObjectExplorer methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 3/26/2011 21:57'!
getCurrentSelection

	^currentSelection! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'nk 7/24/2003 09:16'!
getList

	^Array with: (ObjectExplorerWrapper with: rootObject name: 'root' model: self parent: nil)
! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'RAA 6/2/2000 16:23'!
initialExtent

	^300@500! !

!ObjectExplorer methodsFor: 'menus' stamp: 'nk 7/24/2003 10:26'!
inspectSelection
	"Open an Inspector on the current selection"
	self object inspect! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:02'!
monitor: anObjectExplorerWrapper
	"Start stepping and watching the given wrapper for changes."
	anObjectExplorerWrapper ifNil: [ ^self ].
	self world ifNil: [ ^self ].
	self monitorList at: anObjectExplorerWrapper put: anObjectExplorerWrapper asString.
	self world startStepping: self at: Time millisecondClockValue selector: #step arguments: #() stepTime: 200.! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'sd 11/20/2005 21:27'!
monitorList
	^monitorList ifNil: [ monitorList := WeakIdentityKeyDictionary new ].! !

!ObjectExplorer methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 3/26/2011 21:57'!
noteNewSelection: x

	currentSelection := x.
	self changed: #getCurrentSelection.
	currentSelection ifNil: [^self].
	currentSelection sendSettingMessageTo: self.
! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'marcus.denker 11/10/2008 10:04'!
object
	^currentSelection ifNotNil: [ :cs | cs withoutListWrapper ]! !

!ObjectExplorer methodsFor: 'menus' stamp: 'nk 7/24/2003 10:00'!
objectReferencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what currently selected. "
	self systemNavigation
		browseAllObjectReferencesTo: self object
		except: (Array with: self parentObject with: currentSelection with: inspector)
		ifNone: [:obj | self changed: #flash].
! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'eem 5/7/2008 11:17'!
openExplorerFor: anObject
"
ObjectExplorer new openExplorerFor: Smalltalk
"

	| win |
	win := (self explorerFor: anObject) openInWorld.
	Cursor wait showWhile:
		[win submorphs do:
			[:sm|
			(sm respondsTo: #expandRoots) ifTrue:
				[sm expandRoots]]].
	^self
! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'stephaneducasse 9/17/2005 21:51'!
openExplorerFor: anObject withLabel: label 
     "ObjectExplorer new openExplorerFor: Smalltalk withLabel: 'Smalltalk'"

	(self explorerFor: anObject withLabel: label)
openInWorld! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'nk 7/24/2003 10:02'!
parentObject
	currentSelection ifNil: [ ^nil ].
	currentSelection parent ifNil: [ ^rootObject ].
	^currentSelection parent withoutListWrapper! !

!ObjectExplorer methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 3/26/2011 21:57'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!ObjectExplorer methodsFor: 'menus' stamp: 'sd 11/20/2005 21:27'!
referencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what's currently selected."
	| aClass sel |

	(aClass := self parentObject class) isVariable ifTrue: [^ self changed: #flash].
	sel := self selector.
	self systemNavigation browseAllAccessesTo: sel from: aClass! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:01'!
release
	self world ifNotNil: [ self world stopStepping: self selector: #step ].
	super release.! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'JuanVuletich 10/26/2010 14:27'!
rootObject: anObject
	rootObject := anObject! !

!ObjectExplorer methodsFor: 'menus' stamp: 'RAA 9/23/1999 13:19'!
selectedClass
	"Answer the class of the receiver's current selection"

	^self doItReceiver class
! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'marcus.denker 11/10/2008 10:04'!
selector
	^currentSelection ifNotNil: [ :cs | cs selector ]! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/12/2003 17:55'!
shouldGetStepsFrom: aWorld
	^self monitorList notEmpty! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nice 1/5/2010 15:59'!
step
	"If there's anything in my monitor list, see if the strings have changed."
	| changes |
	changes := false.
	self monitorList keysAndValuesDo: [ :k :v | | string |
		k ifNotNil: [
			k refresh.
			(string := k asString) ~= v ifTrue: [ self monitorList at: k put: string. changes := true ].
		]
	].
	changes ifTrue: [ | sel |
		sel := currentSelection.
		self changed: #getList.
		self noteNewSelection: sel.
	].
	self monitorList isEmpty ifTrue: [ ActiveWorld stopStepping: self selector: #step ].! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'sd 11/20/2005 21:27'!
stopMonitoring
	monitorList := nil.
	self world stopStepping: self selector: #step! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 10:11'!
taskbarIcon 
	^ self theme  smallInspectItIcon ! !

!ObjectExplorer methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 3/26/2011 21:58'!
update: aSymbol

	aSymbol == #hierarchicalList ifTrue: [
		^self changed: #getList
	].
	super update: aSymbol! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:01'!
world
	^ActiveWorld! !

ObjectExplorer subclass: #PointerExplorer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!PointerExplorer commentStamp: 'avi 8/21/2004 20:01' prior: 0!
A variant on the ObjectExlorer that works "backwards": like the ObjectExplorer, it shows a tree of objects, but expanding a node won't show the objects which that node references, but rather the objects that reference that node.  Its main use is to track down memory leaks: if you want to know why a particular object is still alive, open a PointerExplorer on it and drill down until you find the root object that's referencing it.  For example, find all the references to the symbol #zot with:

PointerExplorer new openExplorerFor: #zot

For the "name" of the object, the PointerExplorer shows each object's identityHash, to allow the user to identify when two similar objects are identical and notice cycles.!

!PointerExplorer class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/20/2011 15:08'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools class]" 
	registry register: self as: #pointerExplorer
! !

!PointerExplorer methodsFor: 'accessing' stamp: 'ab 8/22/2003 18:51'!
getList
	^Array with: (PointerExplorerWrapper with: rootObject name: rootObject identityHash asString model: self)
! !

ObjectExplorer subclass: #ViewHierarchyExplorer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!ViewHierarchyExplorer commentStamp: 'SW 5/22/2009 12:30' prior: 0!
A ViewHierarchyExplorer is specialized Explorer that works exclusively with morphs.  It provides the user with a tree view of the Morph Composition Hierarchy and allows you to quickly identify a tree item with its corresponding morph by "flashing" the node in question.  This tool facilitates the understanding of complex morph compositions.

A View Hierarchy Explorer can be opened from the Halo Debug Menu > "explore view hierarchy", or simply send #ExploreViewHierarchy to any morph instance.

try..
[World exploreViewHierarchy]
and
[OBPackageBrowser open exploreViewHierarchy]!

!ViewHierarchyExplorer class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 2/20/2011 15:40'!
registerToolsOn: registry

	registry register: self as: #viewHierarchyExplorer  ! !

!ViewHierarchyExplorer methodsFor: 'actions' stamp: 'SW 5/18/2009 15:29'!
browseHierarchyOfSelection

	currentSelection ifNotNil:[currentSelection item browseHierarchy]! !

!ViewHierarchyExplorer methodsFor: 'actions' stamp: 'SW 5/18/2009 15:30'!
exploreSelection

	currentSelection ifNotNil:[currentSelection item explore]! !

!ViewHierarchyExplorer methodsFor: 'actions' stamp: 'SW 5/22/2009 12:05'!
exploreViewHierarchyOfSelection

	currentSelection ifNotNil:[currentSelection item exploreViewHierarchy]! !

!ViewHierarchyExplorer methodsFor: 'actions' stamp: 'SW 5/22/2009 12:18'!
flashSelection

	currentSelection ifNotNil:[currentSelection item flashBounds]! !

!ViewHierarchyExplorer methodsFor: 'accessing' stamp: 'tbn 7/29/2010 22:13'!
genericMenu: aMenu 
	"Borrow a menu from my inspector"
	| menu |
	currentSelection
		ifNil: [menu := aMenu.
			menu
				add: '*nothing selected*'
				target: self
				selector: #yourself]
		ifNotNil: [
			aMenu addList: #(
		('Flash'						flashSelection)

								('Browse Hierarchy'				browseHierarchyOfSelection)

								('Explore View Hierarchy'						exploreViewHierarchyOfSelection)

								('Inspect'						inspectSelection)

								('Explore'						exploreSelection)).

			].
	monitorList isEmptyOrNil
		ifFalse: [aMenu addLine;
				add: 'Stop monitoring all'
				target: self
				selector: #stopMonitoring].
	^ aMenu! !

!ViewHierarchyExplorer methodsFor: 'accessing' stamp: 'SW 5/18/2009 15:15'!
getList

	^Array with: (MorphWithSubmorphsWrapper with: rootObject)
	
! !

!ViewHierarchyExplorer methodsFor: 'actions' stamp: 'SW 5/18/2009 15:30'!
inspectSelection

	currentSelection ifNotNil:[currentSelection item inspect]! !

Model subclass: #ProcessBrowser
	instanceVariableNames: 'selectedProcess selectedContext methodText processList processListIndex stackList stackListIndex selectedClass selectedSelector searchString autoUpdateProcess deferredMessageRecipient lastUpdate startedCPUWatcher'
	classVariableNames: 'SuspendedProcesses WellKnownProcesses'
	poolDictionaries: ''
	category: 'Tools-Process Browser'!

!ProcessBrowser commentStamp: '<historical>' prior: 0!
Change Set:		ProcessBrowser
Date:			14 March 2000
Author:			Ned Konz

email: ned@bike-nomad.com

This is distributed under the Squeak License.

Added 14 March:
	CPUWatcher integration
	automatically start and stop CPUWatcher
	added CPUWatcher to process list menu

Added 29 October:
	MVC version
	2.8, 2.7 compatibility
	rearranged menus
	added pointer inspection and chasing
	added suspend/resume
	recognized more well-known processes
	misc. bug fixes

Added 26 October: highlight pc in source code
Added 27 October: added 'signal semaphore'
added 'inspect receiver', 'explore receiver', 'message tally' to stack list menu
added 'find context', 'next context' to process list menu
added 'change priority' and 'debug' choices to process list menu

27 October mods by Bob Arning:

alters process display in Ned's ProcessBrowser to 
- show process priority
- drop 'a Process in' that appears on each line
- show in priority order
- prettier names for known processes
- fix to Utilities to forget update downloading process when it ends (1 less dead
process)
- correct stack dump for the active process
!

!ProcessBrowser class methodsFor: 'initialization' stamp: 'janniklaval 2/10/2011 12:09'!
cleanUp
 "Remove terminated processes from my suspended list"

       self suspendedProcesses keys do: [:ea |
               ea isTerminated ifTrue: [self suspendedProcesses removeKey: ea]]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 17:09'!
debugProcess: aProcess
	self resumeProcess: aProcess.
	aProcess debugWithTitle: 'Interrupted from the Process Browser'.
! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'stephane.ducasse 8/5/2009 17:06'!
dumpPigStackOn: aStream andClose: aBoolean
	"Must run forked on its own process, so the monitored behavior is not affected too much" 

	| promise tally process depth stack suspendedContext |
	promise := Processor tallyCPUUsageFor: 1 every: 10.
	tally := promise value.
	"WorldState addDeferredUIMessage: [self dumpTallyOnTranscript: tally]."
	aStream nextPutAll: '====Al processes===='; cr.
	self dumpTally: tally on: aStream.
	aStream cr; nextPutAll: '====Process using most CPU===='; cr.
	process := tally sortedCounts first value.
	(100.0 * (tally occurrencesOf: process) / tally size) rounded printOn: aStream.
	aStream
		nextPutAll: ' % ';
		nextPutAll: (process browserPrintStringWith: (ProcessBrowser nameAndRulesFor: process) first);
		cr.
	depth := 20.
	stack := process == Processor activeProcess
		ifTrue: [thisContext stackOfSize: depth]
		ifFalse: [suspendedContext := process suspendedContext.
			suspendedContext
				ifNotNil: [suspendedContext stackOfSize: depth]].
	stack 
		ifNil: [ aStream nextPutAll: 'No context'; cr]
		ifNotNil: [
			stack do: [ :c | 
				c printOn: aStream.
				aStream cr]].
	aBoolean ifTrue: [aStream close]! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'stephane.ducasse 8/5/2009 17:07'!
dumpTally: tally on: aStream
	"tally is from ProcessorScheduler>>tallyCPUUsageFor:
	Dumps lines with percentage of time, hash of process, and a friendly name"

	tally sortedCounts do: [ :assoc | | procName |
		procName := (self nameAndRulesFor: assoc value) first.
		(((assoc key / tally size) * 100.0) roundTo: 1) printOn: aStream.
		aStream
			nextPutAll: '%   ';
			print: assoc value identityHash; space;
			nextPutAll: procName;
			cr.
	]! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'jmv 5/22/2009 12:30'!
dumpTallyOnTranscript: tally
	"tally is from ProcessorScheduler>>tallyCPUUsageFor:
	Dumps lines with percentage of time, hash of process, and a friendly name"

	self dumpTally: tally on: Transcript.
	Transcript flush.! !

!ProcessBrowser class methodsFor: 'initialization' stamp: 'MarcusDenker 5/20/2011 08:32'!
initialize
	"ProcessBrowser initialize"
	SuspendedProcesses ifNil: [ SuspendedProcesses := IdentityDictionary new ].
	self registerWellKnownProcesses! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'JuanVuletich 11/1/2010 10:13'!
isUIProcess: aProcess
	^ aProcess == UIManager default uiProcess! !

!ProcessBrowser class methodsFor: 'world menu' stamp: 'AlainPlantec 2/18/2010 11:20'!
menuCommandOn: aBuilder 
	<worldMenu> 
	(aBuilder item: #'Process Browser')
		parent: #Tools;
		action:[self open]; 
		icon: self taskbarIcon.! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'HenrikSperreJohansen 6/28/2010 12:25'!
nameAndRulesFor: aProcess 
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	^ [aProcess caseOf: WellKnownProcesses
		 otherwise: 
			[(aProcess priority = Processor timingPriority
					and: [aProcess suspendedContext receiver == Delay])
				ifTrue: [{'the timer interrupt watcher'. false. false}]
				ifFalse: [{aProcess suspendedContext asString. true. true}]]]
		ifError: [{aProcess suspendedContext asString. true. true}]! !

!ProcessBrowser class methodsFor: 'instance creation' stamp: 'alain.plantec 6/1/2008 19:03'!
open
	"ProcessBrowser open"
	"Create and schedule a ProcessBrowser."
	Smalltalk garbageCollect.
	^ self new openAsMorph! !

!ProcessBrowser class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:01'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #processBrowser
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'dtl 2/5/2005 09:07'!
registerWellKnownProcess: aBlockForProcess label: nickname allowStop: allowStop allowDebug: allowDebug
	"Add an entry to the registry of well known processes. aBlockForProcess
	evaluates to a known process to be identified by nickname, and allowStop
	and allowDebug are flags controlling allowable actions for this process
	in the browser."

	WellKnownProcesses add: aBlockForProcess->[{nickname . allowStop . allowDebug}]! !

!ProcessBrowser class methodsFor: 'initialization' stamp: 'JuanVuletich 11/1/2010 10:14'!
registerWellKnownProcesses
	"Associate each well-known process with a nickname and two flags: allow-stop, and allow-debug.
	Additional processes may be added to this list as required"

	WellKnownProcesses := OrderedCollection new.
	self
		registerWellKnownProcess: [  ]
		label: 'no process'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Smalltalk lowSpaceWatcherProcess ]
		label: 'the low space watcher'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ WeakArray runningFinalizationProcess ]
		label: 'the WeakArray finalization process'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Processor activeProcess ]
		label: 'the UI process'
		allowStop: false
		allowDebug: true.
	self
		registerWellKnownProcess: [ Processor backgroundProcess ]
		label: 'the idle process'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Sensor interruptWatcherProcess ]
		label: 'the user interrupt watcher'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Sensor eventTicklerProcess ]
		label: 'the event tickler'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ UIManager default uiProcess ]
		label: 'the inactive Morphic UI process'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Smalltalk globals at: #SoundPlayer ifPresent: [ :sp | sp playerProcess ] ]
		label: 'the Sound Player'
		allowStop: false
		allowDebug: false.
	self
		registerWellKnownProcess: [ Smalltalk globals at: #CPUWatcher ifPresent: [ :cw | cw currentWatcherProcess ] ]
		label: 'the CPUWatcher'
		allowStop: false
		allowDebug: false! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
resumeProcess: aProcess
	| priority |
	priority := self suspendedProcesses
				removeKey: aProcess
				ifAbsent: [aProcess priority].
	aProcess priority: priority.
	aProcess suspendedContext ifNotNil: [ aProcess resume ]
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
setProcess: aProcess toPriority: priority
	| oldPriority |
	oldPriority := self suspendedProcesses at: aProcess ifAbsent: [ ].
	oldPriority ifNotNil: [ self suspendedProcesses at: aProcess put: priority ].
	aProcess priority: priority.
	^oldPriority! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
suspendProcess: aProcess
	| priority |
	priority := aProcess priority.
	self suspendedProcesses at: aProcess put: priority.
	"Need to take the priority down below the caller's
	so that it can keep control after signaling the Semaphore"
	(aProcess suspendingList isKindOf: Semaphore)
		ifTrue: [aProcess priority: Processor lowestPriority.
			aProcess suspendingList signal].
	[aProcess suspend]
		on: Error
		do: [:ex | self suspendedProcesses removeKey: aProcess].
	aProcess priority: priority.
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
suspendedProcesses
	"Answer a collection of processes that my instances have suspended.  
	This is so that they don't get garbage collected."
	^ SuspendedProcesses
		ifNil: [SuspendedProcesses := IdentityDictionary new]! !

!ProcessBrowser class methodsFor: 'cpu utilization' stamp: 'nk 3/8/2001 12:49'!
tallyCPUUsageFor: seconds
	"Compute CPU usage using a 10-msec sample for the given number of seconds,
	then dump the usage statistics on the Transcript. The UI is free to continue, meanwhile"
	"ProcessBrowser tallyCPUUsageFor: 10"
	^self tallyCPUUsageFor: seconds every: 10! !

!ProcessBrowser class methodsFor: 'cpu utilization' stamp: 'alain.plantec 6/1/2008 19:06'!
tallyCPUUsageFor: seconds every: msec
	"Compute CPU usage using a msec millisecond sample for the given number of seconds,
	then dump the usage statistics on the Transcript. The UI is free to continue, meanwhile"
	"ProcessBrowser tallyCPUUsageFor: 10 every: 100"

	| promise |
	promise := Processor tallyCPUUsageFor: seconds every: msec.

	[ | tally |
		tally := promise value.
		WorldState addDeferredUIMessage: [ self dumpTallyOnTranscript: tally ].
	] fork.! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 13:25'!
terminateProcess: aProcess
	aProcess ifNotNil: [
		self suspendedProcesses
			removeKey: aProcess
			ifAbsent: [].
		aProcess terminate
	].
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
unregisterWellKnownProcess: aProcess
	"Remove the first registry entry that matches aProcess. Use
	with caution if more than one registry entry may match aProcess."

	"self unregisterWellKnownProcess: Smalltalk lowSpaceWatcherProcess"

	| entry |
	entry := WellKnownProcesses
		detect: [:e | e key value == aProcess]
		ifNone: [^ self].
	WellKnownProcesses remove: entry! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 13:26'!
wasProcessSuspendedByProcessBrowser: aProcess
	^self suspendedProcesses includesKey: aProcess! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 16:49'!
browseContext
	selectedContext
		ifNil: [^ self].
	Browser newOnClass: self selectedClass selector: self selectedSelector! !

!ProcessBrowser methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 20:38'!
buildWith: builder
	"Create a pluggable version of me, answer a window"
	| windowSpec listSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Process Browser'.
	windowSpec children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #processNameList; 
		getIndex: #processListIndex; 
		setIndex: #processListIndex:; 
		menu: #processListMenu:; 
		keyPress: #processListKey:from:;
		frame: (0 @ 0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #stackNameList; 
		getIndex: #stackListIndex; 
		setIndex: #stackListIndex:; 
		menu: #stackListMenu:; 
		keyPress: #stackListKey:from:;
		frame: (0.5 @ 0.0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #selectedMethod; 
		setText: nil; 
		selection: nil; 
		menu: nil;
		frame: (0 @ 0.5 corner: 1 @ 1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'DamienCassou 9/29/2009 13:07'!
changePriority
	| str newPriority nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules third
		ifFalse: [self inform: 'Nope, won''t change priority of ' , nameAndRules first.
			^ self].
	str := UIManager default 
				request: 'New priority' 
		  initialAnswer: selectedProcess priority asString.
	str ifNil: [str := String new].
	newPriority := str asNumber asInteger.
	newPriority
		ifNil: [^ self].
	(newPriority < 1
			or: [newPriority > Processor highestPriority])
		ifTrue: [self inform: 'Bad priority'.
			^ self].
	self class setProcess: selectedProcess toPriority: newPriority.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'sd 11/20/2005 21:27'!
changeStackListTo: aCollection 

        stackList := aCollection.
        self changed: #stackNameList.
        self stackListIndex: 0! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
debugProcess
	| nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules third
		ifFalse: [self inform: 'Nope, won''t debug ' , nameAndRules first.
			^ self].
	self class debugProcess: selectedProcess.! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'JuanVuletich 10/26/2010 14:01'!
deferredMessageRecipient: anObject
	deferredMessageRecipient := anObject! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:28'!
exploreContext
	selectedContext explore! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'AndyKellens 6/11/2010 14:41'!
explorePointers
	| saved |
	selectedProcess ifNil: [ ^ self ].
	saved := selectedProcess.
	[ 
	selectedProcess := nil.
	(Smalltalk globals includesKey: #PointerExplorer)
		ifTrue: [ PointerExplorer new openExplorerFor: saved ]
		ifFalse: [ self inspectPointers ] ] ensure: [ selectedProcess := saved ]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 10/27/2000 09:24'!
exploreProcess
	selectedProcess explore! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:41'!
exploreReceiver
	selectedContext ifNotNil: [ selectedContext receiver explore ]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'DamienCassou 9/29/2009 13:07'!
findContext
	| initialProcessIndex initialStackIndex found |
	initialProcessIndex := self processListIndex.
	initialStackIndex := self stackListIndex.
	searchString := UIManager default 
			request: 'Enter a string to search for in the process stack lists'
	  initialAnswer: searchString.
	searchString isEmptyOrNil
		ifTrue: [^ false].
	self processListIndex: 1.
	self stackListIndex: 1.
	found := self nextContext.
	found
		ifFalse: [self processListIndex: initialProcessIndex.
			self stackListIndex: initialStackIndex].
	^ found! !

!ProcessBrowser methodsFor: 'views' stamp: 'nk 10/28/2000 11:44'!
hasView
	^self dependents isEmptyOrNil not! !

!ProcessBrowser methodsFor: 'initialization' stamp: 'AlainPlantec 12/5/2009 19:46'!
initialize
	super initialize.
	methodText := ''.
	stackListIndex := 0.
	searchString := ''.
	lastUpdate := 0.
	startedCPUWatcher := CPUWatcher cpuWatcherEnabled and: [ self startCPUWatcher ].
	self updateProcessList; processListIndex: 1! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 7/8/2000 20:23'!
inspectContext
	selectedContext inspect! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'AndyKellens 6/11/2010 14:15'!
inspectPointers
	| tc pointers |
	selectedProcess ifNil: [^self].
	tc := thisContext.
	pointers := selectedProcess pointersToExcept: { 
						self processList.
						tc.
						self}.
	pointers isEmpty ifTrue: [^self].
	OrderedCollectionInspector 
		openOn: pointers
		withEvalPane: false
		withLabel: 'Objects pointing to ' , selectedProcess browserPrintString! !

!ProcessBrowser methodsFor: 'process list' stamp: ''!
inspectProcess
	selectedProcess inspect! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:41'!
inspectReceiver
	selectedContext
		ifNotNil: [selectedContext receiver inspect]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 10/28/2000 21:48'!
isAutoUpdating
	^autoUpdateProcess notNil and: [ autoUpdateProcess isSuspended  not ]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 6/18/2003 07:20'!
isAutoUpdatingPaused
	^autoUpdateProcess notNil and: [ autoUpdateProcess isSuspended ]! !

!ProcessBrowser methodsFor: 'initialization' stamp: 'JuanVuletich 10/26/2010 14:05'!
mayBeStartCPUWatcher
	startedCPUWatcher ifTrue: [ self setUpdateCallbackAfter: 7 ].
! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'DamienCassou 9/29/2009 13:07'!
messageTally
	| secString secs |
	secString := UIManager default request: 'Profile for how many seconds?' initialAnswer: '4'.
	secString ifNil: [secString := String new].
	secs := secString asNumber asInteger.
	(secs isNil
			or: [secs isZero])
		ifTrue: [^ self].
	[ TimeProfileBrowser spyOnProcess: selectedProcess forMilliseconds: secs * 1000 ] forkAt: selectedProcess priority + 1.! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 12:13'!
moreStack
	self updateStackList: 2000! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 3/8/2001 13:35'!
nameAndRulesFor: aProcess 
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	aProcess == autoUpdateProcess ifTrue: [ ^{'my auto-update process'. true. true} ].
	^self class nameAndRulesFor: aProcess 
! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/28/2000 20:31'!
nameAndRulesForSelectedProcess
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	^self nameAndRulesFor: selectedProcess! !

!ProcessBrowser methodsFor: 'process list' stamp: 'sd 11/20/2005 21:27'!
nextContext
	| initialProcessIndex initialStackIndex found |
	searchString isEmpty ifTrue: [ ^false ].
	initialProcessIndex := self processListIndex.
	initialStackIndex := self stackListIndex.
	found := false.
	initialProcessIndex
		to: self processList size
		do: [:pi | found
				ifFalse: [self processListIndex: pi.
					self stackNameList
						withIndexDo: [:name :si | (found not
									and: [pi ~= initialProcessIndex
											or: [si > initialStackIndex]])
								ifTrue: [(name includesSubString: searchString)
										ifTrue: [self stackListIndex: si.
											found := true]]]]].
	found
		ifFalse: [self processListIndex: initialProcessIndex.
			self stackListIndex: initialStackIndex].
	^ found! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 10/28/2000 08:19'!
notify: errorString at: location in: aStream 
	"A syntax error happened when I was trying to highlight my pc. 
	Raise a signal so that it can be ignored."
	Warning signal: 'syntax error'! !

!ProcessBrowser methodsFor: 'views' stamp: 'JuanVuletich 10/26/2010 14:06'!
openAsMorph
	"Create a pluggable version of me, open and answer a window"
	^ UIManager default openProcessBrowserAsMorph: self! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 6/18/2003 07:20'!
pauseAutoUpdate
	self isAutoUpdating
		ifTrue: [ autoUpdateProcess suspend ].
	self updateProcessList! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'eem 6/12/2008 12:44'!
pcRange
	"Answer the indices in the source code for the method corresponding to  
	the selected context's program counter value."
	(selectedContext isNil or: [methodText isEmptyOrNil])
		ifTrue: [^ 1 to: 0].
	^selectedContext debuggerMap
		rangeForPC: (selectedContext pc ifNotNil: [:pc| pc] ifNil: [selectedContext method endPC])
		contextIsActiveContext: stackListIndex = 1! !

!ProcessBrowser methodsFor: 'message handling' stamp: 'nk 10/28/2000 20:53'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."
	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'sd 11/20/2005 21:27'!
prettyNameForProcess: aProcess 
	| nameAndRules |
	aProcess ifNil: [ ^'<nil>' ].
	nameAndRules := self nameAndRulesFor: aProcess.
	^ aProcess browserPrintStringWith: nameAndRules first! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
processList
	^ processList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
processListIndex
	^ processListIndex! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
processListIndex: index 
	processListIndex := index.
	selectedProcess := processList
				at: index
				ifAbsent: [].
	self updateStackList.
	self changed: #processListIndex.! !

!ProcessBrowser methodsFor: 'process list' stamp: 'AndyKellens 6/11/2010 14:49'!
processListKey: aKey from: aView 
	^ aKey caseOf: {
		[$i] -> [self inspectProcess].
		[$I] -> [self exploreProcess].
		[$e] -> [self explorePointers].
		[$P] -> [self inspectPointers].
		[$t] -> [self terminateProcess].
		[$r] -> [self resumeProcess].
		[$s] -> [self suspendProcess].
		[$d] -> [self debugProcess].
		[$p] -> [self changePriority].
		[$m] -> [self messageTally].
		[$f] -> [self findContext].
		[$g] -> [self nextContext].
		[$a] -> [self toggleAutoUpdate].
		[$u] -> [self updateProcessList].
		[$S] -> [self signalSemaphore].
		[$k] -> [self moreStack]}
		 otherwise: [self arrowKey: aKey from: aView]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'tbn 7/29/2010 22:11'!
processListMenu: menu
	| pw |
	selectedProcess
		ifNotNil: [ 
			| nameAndRules |
			nameAndRules := self nameAndRulesForSelectedProcess.
			menu
				addList:
					{{'Inspect (i)'.
					#inspectProcess}.
					{'Explore (I)'.
					#exploreProcess}.
					{'Inspect Pointers (P)'.
					#inspectPointers}}.
			(Smalltalk globals includesKey: #PointerExplorer)
				ifTrue: [ menu add: 'Explore pointers (e)' action: #explorePointers ].
			nameAndRules second
				ifTrue: [ 
					menu add: 'Terminate (t)' action: #terminateProcess.
					selectedProcess isSuspended
						ifTrue: [ menu add: 'Resume (r)' action: #resumeProcess ]
						ifFalse: [ menu add: 'Suspend (s)' action: #suspendProcess ] ].
			nameAndRules third
				ifTrue: [ 
					menu
						addList:
							{{'Change priority (p)'.
							#changePriority}.
							{'Debug (d)'.
							#debugProcess}} ].
			menu
				addList:
					{{'Profile messages (m)'.
					#messageTally}}.
			(selectedProcess suspendingList isKindOf: Semaphore)
				ifTrue: [ menu add: 'Signal Semaphore (S)' action: #signalSemaphore ].
			menu add: 'Full stack (k)' action: #moreStack.
			menu addLine ].
	menu
		addList:
			{{'Find context... (f)'.
			#findContext}.
			{'Find again (g)'.
			#nextContext}}.
	menu addLine.
	menu
		add:
			(self isAutoUpdating
				ifTrue: [ 'Turn off auto-update (a)' ]
				ifFalse: [ 'Turn on auto-update (a)' ])
		action: #toggleAutoUpdate.
	menu add: 'Update list (u)' action: #updateProcessList.
	pw := Smalltalk globals at: #CPUWatcher ifAbsent: [  ].
	pw
		ifNotNil: [ 
			menu addLine.
			pw isMonitoring
				ifTrue: [ menu add: 'Stop CPUWatcher' action: #stopCPUWatcher ]
				ifFalse: [ menu add: 'Start CPUWatcher' action: #startCPUWatcher ] ].
	^ menu! !

!ProcessBrowser methodsFor: 'process list' stamp: 'lr 3/14/2010 21:13'!
processNameList
	"since processList is a WeakArray, we have to strengthen the result"

	| pw tally |
	pw := Smalltalk globals at: #CPUWatcher ifAbsent: [  ].
	tally := pw ifNotNil: [ pw current ifNotNil: [ pw current tally ] ].
	^ (processList asOrderedCollection copyWithout: nil)
		collect: [ :each | 
			| percent |
			percent := tally
				ifNotNil: [ (((tally occurrencesOf: each) * 100.0 / tally size roundTo: 1) asString padded: #left to: 2 with: $ ) , '% ' ]
				ifNil: [ '' ].
			percent , (self prettyNameForProcess: each) ]! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 3/8/2001 13:23'!
resumeProcess
	selectedProcess
		ifNil: [^ self].
	self class resumeProcess: selectedProcess.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'md 2/17/2006 09:32'!
selectedClass
	"Answer the class in which the currently selected context's method was  
	found."
	^ selectedClass
		ifNil: [selectedClass := selectedContext receiver
				ifNil: [selectedSelector := selectedContext method selector.
					   selectedContext method methodClass]
				ifNotNil: [selectedContext methodClass]]! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 6/28/2010 12:24'!
selectedMethod
	^ methodText ifNil: [methodText := selectedContext
						ifNil: ['']
						ifNotNil: [| pcRange | 
							methodText := [ selectedContext sourceCode ]
								ifError: [ 'error getting method text' ].
							pcRange := self pcRange.
							methodText asText
								addAttribute: TextColor red
								from: pcRange first
								to: pcRange last;
								
								addAttribute: TextEmphasis bold
								from: pcRange first
								to: pcRange last]]! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'md 2/17/2006 12:07'!
selectedSelector
	"Answer the class in which the currently selected context's method was  
	found."
	^ selectedSelector
		ifNil: [selectedSelector := selectedContext receiver
				ifNil: [selectedClass := selectedContext method methodClass
					   selectedContext method selector]
				ifNotNil: [selectedContext selector]]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'sd 11/20/2005 21:27'!
setUpdateCallbackAfter: seconds

		deferredMessageRecipient ifNotNil: [ | d |
			d := Delay forSeconds: seconds.
			[  d wait.
				d := nil.
				deferredMessageRecipient addDeferredUIMessage: [self updateProcessList]
			] fork
		]! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/29/2000 09:58'!
signalSemaphore
	(selectedProcess suspendingList isKindOf: Semaphore)
		ifFalse: [^ self].
	[selectedProcess suspendingList signal] fork.
	(Delay forMilliseconds: 300) wait.
	"Hate to make the UI wait, but it's convenient..."
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
stackList
	^ stackList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
stackListIndex
	^ stackListIndex! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'eem 6/12/2008 12:41'!
stackListIndex: index 
	stackListIndex := index.
	selectedContext := (stackList notNil
						and: [index > 0]) ifTrue:
							[stackList at: index ifAbsent: []].
	selectedClass := nil.
	selectedSelector := nil.
	methodText := nil.
	self changed: #stackListIndex.
	self changed: #selectedMethod! !

!ProcessBrowser methodsFor: 'views' stamp: 'nk 10/28/2000 16:53'!
stackListKey: aKey from: aView 
	^ aKey caseOf: {
		[$c] -> [self inspectContext].
		[$C] -> [self exploreContext].
		[$i] -> [self inspectReceiver].
		[$I] -> [self exploreReceiver].
		[$b] -> [self browseContext]}
		 otherwise: [self arrowKey: aKey from: aView]! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'tbn 7/29/2010 22:14'!
stackListMenu: aMenu 
	| menu |
	selectedContext
		ifNil: [^ aMenu].
	menu := aMenu
				labels: 'Inspect context (c)
Explore context (C)
Inspect receiver (i)
Explore receiver (I)
Browse (b)'
				lines: #(2 4 )
				selections: #(#inspectContext #exploreContext #inspectReceiver #exploreReceiver #browseContext ).
	^ menu! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 16:18'!
stackNameList
	^ stackList
		ifNil: [#()]
		ifNotNil: [stackList
				collect: [:each | each asString]]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'sd 11/20/2005 21:27'!
startAutoUpdate
	self isAutoUpdatingPaused ifTrue: [ ^autoUpdateProcess resume ].
	self isAutoUpdating
		ifFalse: [| delay | 
			delay := Delay forSeconds: 2.
			autoUpdateProcess := [[self hasView]
						whileTrue: [delay wait.
							deferredMessageRecipient ifNotNil: [
								deferredMessageRecipient addDeferredUIMessage: [self updateProcessList]]
							ifNil: [ self updateProcessList ]].
					autoUpdateProcess := nil] fork].
	self updateProcessList! !

!ProcessBrowser methodsFor: 'initialization' stamp: 'lr 3/14/2010 21:13'!
startCPUWatcher
	"Answers whether I started the CPUWatcher"

	| pw |
	pw := Smalltalk globals at: #CPUWatcher ifAbsent: [ ^ self ].
	pw
		ifNotNil: [ 
			pw isMonitoring
				ifFalse: [ 
					pw startMonitoringPeriod: 5 rate: 100 threshold: 0.85.
					self setUpdateCallbackAfter: 7.
					^ true ] ].
	^ false! !

!ProcessBrowser methodsFor: 'updating' stamp: 'sd 11/20/2005 21:27'!
stopAutoUpdate
	autoUpdateProcess ifNotNil: [
		autoUpdateProcess terminate.
		autoUpdateProcess := nil].
	self updateProcessList! !

!ProcessBrowser methodsFor: 'initialization' stamp: 'lr 3/14/2010 21:13'!
stopCPUWatcher
	| pw |
	pw := Smalltalk globals at: #CPUWatcher ifAbsent: [ ^ self ].
	pw
		ifNotNil: [ 
			pw stopMonitoring.
			self updateProcessList.
			startedCPUWatcher := false	"so a manual restart won't be killed later" ]! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
suspendProcess
	| nameAndRules |
	selectedProcess isSuspended
		ifTrue: [^ self].
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules second
		ifFalse: [self inform: 'Nope, won''t suspend ' , nameAndRules first.
			^ self].
	self class suspendProcess: selectedProcess.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
terminateProcess
	| nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules second
		ifFalse: [self inform: 'Nope, won''t kill ' , nameAndRules first.
			^ self].
	self class terminateProcess: selectedProcess.	
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'nk 10/28/2000 08:36'!
text
	^methodText! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 10/28/2000 21:50'!
toggleAutoUpdate
	self isAutoUpdating
		ifTrue: [ self stopAutoUpdate ]
		ifFalse: [ self startAutoUpdate ].
! !

!ProcessBrowser methodsFor: 'process list' stamp: 'GuillermoPolito 9/4/2010 20:39'!
updateProcessList
	| oldSelectedProcess newIndex now |
	now := Time millisecondClockValue.
	now - lastUpdate < 500
		ifTrue: [^ self].
	"Don't update too fast"
	lastUpdate := now.
	oldSelectedProcess := selectedProcess.
	processList := selectedProcess := selectedSelector := nil.
	Smalltalk garbageCollectMost.
	"lose defunct processes"

	processList := Process allSubInstances
				reject: [:each | each isTerminated].
	processList := processList
				sort: [:a :b | a priority >= b priority].
	processList := WeakArray withAll: processList.
	newIndex := processList
				indexOf: oldSelectedProcess
				ifAbsent: [0].
	self changed: #processNameList.
	self processListIndex: newIndex! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 7/8/2000 20:24'!
updateStackList
	self updateStackList: 20! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'sd 11/20/2005 21:27'!
updateStackList: depth 
	| suspendedContext oldHighlight |
	selectedProcess
		ifNil: [^ self changeStackListTo: nil].
	(stackList notNil and: [ stackListIndex > 0 ])
		ifTrue: [oldHighlight := stackList at: stackListIndex].
	selectedProcess == Processor activeProcess
		ifTrue: [self
				changeStackListTo: (thisContext stackOfSize: depth)]
		ifFalse: [suspendedContext := selectedProcess suspendedContext.
			suspendedContext
				ifNil: [self changeStackListTo: nil]
				ifNotNil: [self
						changeStackListTo: (suspendedContext stackOfSize: depth)]].
	self
		stackListIndex: (stackList
				ifNil: [0]
				ifNotNil: [stackList indexOf: oldHighlight])! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/29/2000 08:56'!
wasProcessSuspendedByProcessBrowser: aProcess
	^self class suspendedProcesses includesKey: aProcess! !

!ProcessBrowser methodsFor: 'initialization' stamp: 'nk 3/14/2001 08:03'!
windowIsClosing
	startedCPUWatcher ifTrue: [ CPUWatcher stopMonitoring ]! !

Model subclass: #StringHolder
	instanceVariableNames: 'contents textMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!StringHolder commentStamp: '<historical>' prior: 0!
I am a kind of Model that includes a piece of text.  In some cases, the text can be edited, and in some the text is a method.

Categories 'code pane menu' and 'message list menu' are messages that may be called by my menus when the text is a method, and when some pane is a list of methods.  Other of my subclasses may ignore these two catagories altogether.!

StringHolder subclass: #CodeHolder
	instanceVariableNames: 'currentCompiledMethod contentsSymbol'
	classVariableNames: 'AnnotationRequests BrowseWithPrettyPrint DecorateBrowserButtons DiffsInChangeList DiffsWithPrettyPrint OptionalButtons ShowAnnotationPane SmartUpdating'
	poolDictionaries: ''
	category: 'Tools-Base'!

!CodeHolder commentStamp: '<historical>' prior: 0!
An ancestor class for all models which can show code.  Eventually, much of the code that currently resides in StringHolder which only applies to code-holding StringHolders might get moved down here.!

CodeHolder subclass: #Browser
	instanceVariableNames: 'systemOrganizer classOrganizer metaClassOrganizer systemCategoryListIndex classListIndex messageCategoryListIndex messageListIndex editSelection metaClassIndicated'
	classVariableNames: 'RecentClasses'
	poolDictionaries: ''
	category: 'Tools-Browser'!

!Browser commentStamp: '<historical>' prior: 0!
I represent a query path into the class descriptions, the software of the system.!

!Browser class methodsFor: 'cleanup' stamp: 'MarcusDenker 4/17/2011 21:46'!
cleanUp

	self flushCaches.! !

!Browser class methodsFor: 'initialization' stamp: 'MarcusDenker 4/17/2011 21:46'!
flushCaches
	RecentClasses := OrderedCollection new.! !

!Browser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:27'!
fullOnClass: aClass 
	"Open a new full browser set to class."
	| brow |
	brow := self new.
	brow setClass: aClass selector: nil.
	^ self 
		openBrowserView: (brow openEditString: nil)
		label: 'System Browser'! !

!Browser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
fullOnClass: aClass selector: aSelector
	"Open a new full browser set to class."

	| brow classToUse |
	classToUse := SystemBrowser default.
	brow := classToUse new.
	brow setClass: aClass selector: aSelector.
	^ classToUse 
		openBrowserView: (brow openEditString: nil)
		label: brow labelString! !

!Browser class methodsFor: 'initialization' stamp: 'MarcusDenker 4/17/2011 21:46'!
initialize
	"Browser initialize"

	self flushCaches.
	self registerInAppRegistry	! !

!Browser class methodsFor: 'instance creation' stamp: 'di 10/18/1999 22:03'!
new

	^super new systemOrganizer: SystemOrganization! !

!Browser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
newOnCategory: aCategory
	"Browse the system category of the given name.  7/13/96 sw"

	"Browser newOnCategory: 'Interface-Browser'"

	| newBrowser catList |
	newBrowser := self new.
	catList := newBrowser systemCategoryList.
	newBrowser systemCategoryListIndex: 
		(catList indexOf: aCategory asSymbol ifAbsent: [^ self inform: 'No such category']).
	^ self 
		openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: 'Classes in category ', aCategory
! !

!Browser class methodsFor: 'instance creation' stamp: 'tk 4/18/1998 16:28'!
newOnClass: aClass 
	"Open a new class browser on this class."
	^ self newOnClass: aClass label: 'Class Browser: ', aClass name! !

!Browser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
newOnClass: aClass label: aLabel
	"Open a new class browser on this class."
	| newBrowser |

	newBrowser := self new.
	newBrowser setClass: aClass selector: nil.
	^ self 
		openBrowserView: (newBrowser openOnClassWithEditString: nil)
		label: aLabel
! !

!Browser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
newOnClass: aClass selector: aSymbol
	"Open a new class browser on this class."
	| newBrowser |

	newBrowser := self new.
	newBrowser setClass: aClass selector: aSymbol.
	^ self 
		openBrowserView: (newBrowser openOnClassWithEditString: nil)
		label: 'Class Browser: ', aClass name
! !

!Browser class methodsFor: 'instance creation' stamp: 'md 3/10/2006 21:46'!
open
	^self openBrowser

! !

!Browser class methodsFor: 'instance creation' stamp: 'jcg 10/29/2003 23:11'!
openBrowser
	"Create and schedule a BrowserView with default browser label. The
	view consists of five subviews, starting with the list view of system
	categories of SystemOrganization. The initial text view part is empty."

	| br |
	br := self new.
	^ self
		openBrowserView: (br openEditString: nil)
		label: br defaultBrowserTitle.

! !

!Browser class methodsFor: 'instance creation' stamp: 'alain.plantec 6/19/2008 09:43'!
openBrowserView: aBrowserView label: aString 
	"Schedule aBrowserView, labelling the view aString."

	(aBrowserView setLabel: aString) openInWorld.
	^ aBrowserView model
! !

!Browser class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:20'!
patchworkUIThemeColor 
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."
	^ Color paleGreen	! !

!Browser class methodsFor: 'initialization' stamp: 'hpt 8/5/2004 19:41'!
registerInAppRegistry
	"Register the receiver in the SystemBrowser AppRegistry"
	SystemBrowser register: self.! !

!Browser class methodsFor: 'instance creation' stamp: 'nk 6/2/2004 12:55'!
systemOrganizer: anOrganizer

	^(super new)
		systemOrganizer: anOrganizer;
		yourself! !

!Browser class methodsFor: 'initialization' stamp: 'AlainPlantec 10/17/2009 16:42'!
unload
	"Unload the receiver from global registries"
	SystemBrowser unregister: self.! !

!Browser methodsFor: 'drag and drop' stamp: 'nk 6/12/2004 17:43'!
acceptDroppingMorph: transferMorph event: evt inMorph: dstListMorph 
	"Here we are fetching informations from the dropped transferMorph 
	and  
	performing the correct action for this drop."
	| srcType success srcBrowser |
	success := false.
	srcType := transferMorph dragTransferType.
	srcBrowser := transferMorph source model.
	srcType == #messageList
		ifTrue: [ | srcClass srcSelector srcCategory |
			srcClass := transferMorph passenger key.
			srcSelector := transferMorph passenger value.
			srcCategory := srcBrowser selectedMessageCategoryName.
			srcCategory
				ifNil: [srcCategory := srcClass organization categoryOfElement: srcSelector].
			success := self
						acceptMethod: srcSelector
						messageCategory: srcCategory
						class: srcClass
						atListMorph: dstListMorph
						internal: self == srcBrowser
						copy: transferMorph shouldCopy].
	srcType == #classList
		ifTrue: [success := self
						changeCategoryForClass: transferMorph passenger
						srcSystemCategory: srcBrowser selectedSystemCategoryName
						atListMorph: dstListMorph
						internal: self == srcBrowser
						copy: transferMorph shouldCopy].
	^ success! !

!Browser methodsFor: 'drag and drop' stamp: 'nice 10/22/2009 15:13'!
acceptMethod: methodSel dstMessageCategory: dstMessageCategorySel srcMessageCategory: srcMessageCategorySel dstClass: dstClass dstClassOrMeta: dstClassOrMeta srcClassOrMeta: srcClassOrMeta internal: internal copySemantic: copyFlag 
	| success hierarchyChange higher checkForOverwrite |
	(success := dstClassOrMeta ~~ nil) ifFalse: [^false].
	checkForOverwrite := dstClassOrMeta includesSelector: methodSel.
	hierarchyChange := (higher := srcClassOrMeta inheritsFrom: dstClassOrMeta) | (dstClassOrMeta inheritsFrom: srcClassOrMeta).
	success := (checkForOverwrite not
				or: [self
						overwriteDialogHierarchyChange: hierarchyChange
						higher: higher
						sourceClassName: srcClassOrMeta name
						destinationClassName: dstClassOrMeta name
						methodSelector: methodSel])
				and: [self
						message: methodSel
						compileInClass: dstClassOrMeta
						fromClass: srcClassOrMeta
						dstMessageCategory: dstMessageCategorySel
						srcMessageCategory: srcMessageCategorySel
						internal: internal
						copySemantic: copyFlag].
	^ success! !

!Browser methodsFor: 'drag and drop' stamp: 'MarcusDenker 3/26/2010 16:49'!
acceptMethod: methodSel messageCategory: srcMessageCategorySel class: srcClassOrMeta atListMorph: dstListMorph internal: internal copy: copyFlag 
	| success dstClassOrMeta dstClass dstMessageCategorySel |
	dstClass := self destinationClassDestinationListMorph: dstListMorph.
	dstClassOrMeta := dstClass
				ifNotNil: [self metaClassIndicated
						ifTrue: [dstClass classSide]
						ifFalse: [dstClass]].
	dstMessageCategorySel := self dstMessageCategoryDstListMorph: dstListMorph.
	success := (dstClassOrMeta notNil
				and: [dstClassOrMeta == srcClassOrMeta])
						ifTrue: ["one class"
							self
								changeMessageCategoryForMethod: methodSel
								dstMessageCategory: dstMessageCategorySel
								srcMessageCategory: srcMessageCategorySel
								insideClassOrMeta: dstClassOrMeta
								internal: internal
								copySemantic: copyFlag]
						ifFalse: ["different classes"
							self
								acceptMethod: methodSel
								dstMessageCategory: dstMessageCategorySel
								srcMessageCategory: srcMessageCategorySel
								dstClass: dstClass
								dstClassOrMeta: dstClassOrMeta
								srcClassOrMeta: srcClassOrMeta
								internal: internal
								copySemantic: copyFlag].
	^ success! !

!Browser methodsFor: 'initialization' stamp: 'GaryChambers 2/24/2011 12:56'!
addAListPane: aListPane to: window at: nominalFractions plus: verticalOffset

	| row switchHeight divider |

	row := AlignmentMorph newColumn
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 1;
		layoutPolicy: ProportionalLayout new.
	switchHeight := StandardFonts buttonFont height + 12.
	self 
		addMorphicSwitchesTo: row 
		at: (
			LayoutFrame 
				fractions: (0@1 corner: 1@1) 
				offsets: (0@(1-(switchHeight))  corner: 0@0)
		).

	divider := BorderedSubpaneDividerMorph forTopEdge.
	divider extent: 4@4; color: Color transparent; borderColor: #raised; borderWidth: 0.
	row 
		addMorph: divider
		fullFrame: (
			LayoutFrame 
				fractions: (0@1 corner: 1@1) 
				offsets: (0@switchHeight negated corner: 0@(1-switchHeight))
		).	

	row 
		addMorph: aListPane
		fullFrame: (
			LayoutFrame 
				fractions: (0@0 corner: 1@1) 
				offsets: (0@0 corner: 0@(switchHeight negated))
		).	

	window 
		addMorph: row
		fullFrame: (
			LayoutFrame 
				fractions: nominalFractions 
				offsets: (0@verticalOffset corner: 0@0)
		).	
	row on: #mouseEnter send: #paneTransition: to: window.
	row on: #mouseLeave send: #paneTransition: to: window.

! !

!Browser methodsFor: 'class functions' stamp: 'StephaneDucasse 12/5/2009 21:25'!
addAllMethodsToCurrentChangeSet
	"Add all the methods in the selected class or metaclass to the current change set.  You ought to know what you're doing before you invoke this!!"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[aClass selectorsDo:
			[:sel |
				ChangeSet current adoptSelector: sel forClass: aClass].
		self changed: #annotation]
! !

!Browser methodsFor: 'message category functions' stamp: 'MarcusDenker 10/28/2010 13:51'!
addCategory
	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| labels reject lines oldIndex newName |
	self okToChange ifFalse: [^ self].
	classListIndex = 0 ifTrue: [^ self].
	labels := OrderedCollection new.
	reject := Set new.
	reject
		addAll: self selectedClassOrMetaClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	self selectedClassOrMetaClass allSuperclasses do: [:cls | | cats |
		cls = Object ifFalse: [
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse: [
				lines add: labels size.
				labels addAll: cats asSortedCollection.
				reject addAll: cats ] ] ].
	
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: 'Add Category'.
	newName ifNil: [ ^self ].
	newName := newName asSymbol.
	
	oldIndex := messageCategoryListIndex.
	self classOrMetaClassOrganizer
		addCategory: newName
		before: (messageCategoryListIndex = 0
				ifTrue: [nil]
				ifFalse: [self selectedMessageCategoryName]).
	self changed: #messageCategoryList.
	self messageCategoryListIndex:
		(oldIndex = 0
			ifTrue: [self classOrMetaClassOrganizer categories size + 1]
			ifFalse: [oldIndex]).
	self changed: #messageCategoryList.
 ! !

!Browser methodsFor: 'initialization' stamp: 'RAA 1/10/2001 11:46'!
addClassAndSwitchesTo: window at: nominalFractions plus: verticalOffset

	^self
		addAListPane: self buildMorphicClassList 
		to: window 
		at: nominalFractions 
		plus: verticalOffset
! !

!Browser methodsFor: 'message functions' stamp: 'sw 1/11/2001 07:22'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built; some menu items are appropriate only for certain kinds of browsers, and this gives a hook for them to be added as approrpiate.  If any is added here, a line should be added first -- browse reimplementors of this message for examples."
! !

!Browser methodsFor: 'construction' stamp: 'AlainPlantec 11/17/2010 11:35'!
addLowerPanesTo: window at: nominalFractions with: editString
	| commentPane |
	super addLowerPanesTo: window at: nominalFractions with: editString.
	commentPane := self buildMorphicCommentPane.
	window addMorph: commentPane fullFrame: commentPane defaultLayoutFrame.
	self changed: #editSelection.! !

!Browser methodsFor: 'user interface' stamp: 'hpt 9/30/2004 20:51'!
addModelItemsToWindowMenu: aMenu
	"Add model-related items to the window menu"
	super addModelItemsToWindowMenu: aMenu.
	SystemBrowser addRegistryMenuItemsTo: aMenu inAccountOf: self.! !

!Browser methodsFor: 'initialization' stamp: 'rr 6/21/2005 13:24'!
addMorphicSwitchesTo: window at: aLayoutFrame

	window 
		addMorph: self buildMorphicSwitches
		fullFrame: aLayoutFrame.

! !

!Browser methodsFor: 'system category functions' stamp: 'DamienCassou 9/29/2009 09:04'!
addSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldIndex newName |
	self okToChange ifFalse: [^ self].
	oldIndex := systemCategoryListIndex.
	newName := self
		request: 'Please type new category name'
		initialAnswer: 'Category-Name'.
	newName isEmptyOrNil
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	systemOrganizer
		addCategory: newName
		before: (systemCategoryListIndex = 0
				ifTrue: [nil]
				ifFalse: [self selectedSystemCategoryName]).
	self systemCategoryListIndex:
		(oldIndex = 0
			ifTrue: [self systemCategoryList size]
			ifFalse: [oldIndex]).
	self changed: #systemCategoryList.! !

!Browser methodsFor: 'traits' stamp: 'lr 3/14/2010 21:13'!
addTrait
	| input trait |
	input := UIManager default request: 'add trait'.
	input isEmptyOrNil
		ifFalse: [ 
			trait := Smalltalk globals classNamed: input.
			(trait isNil or: [ trait isTrait not ])
				ifTrue: [ ^ self inform: 'Input invalid. ' , input , ' does not exist or is not a trait' ].
			self selectedClass addToComposition: trait.
			self contentsChanged ]! !

!Browser methodsFor: 'message category functions' stamp: 'NS 4/7/2004 22:47'!
alphabetizeMessageCategories
	classListIndex = 0 ifTrue: [^ false].
	self okToChange ifFalse: [^ false].
	self classOrMetaClassOrganizer sortCategories.
	self clearUserEditFlag.
	self editClass.
	self classListIndex: classListIndex.
	^ true! !

!Browser methodsFor: 'system category functions' stamp: 'brp 8/4/2003 21:38'!
alphabetizeSystemCategories

	self okToChange ifFalse: [^ false].
	systemOrganizer sortCategories.
	self systemCategoryListIndex: 0.
	self changed: #systemCategoryList.
! !

!Browser methodsFor: 'annotation' stamp: 'sd 11/20/2005 21:26'!
annotation
	"Provide a line of content for an annotation pane, representing information about the method associated with the selected class and selector in the receiver."

	|  aSelector aClass |
	(aClass := self selectedClassOrMetaClass) == nil ifTrue: [^ '------'].
	self editSelection == #editComment ifTrue:
		[^ self annotationForSelector: #Comment ofClass: aClass].

	self editSelection == #editClass ifTrue:
		[^ self annotationForSelector: #Definition ofClass: aClass].
	(aSelector := self selectedMessageName) ifNil: [^ '------'].
	^ self annotationForSelector: aSelector ofClass: aClass! !

!Browser methodsFor: 'metaclass' stamp: 'MarcusDenker 4/29/2011 00:33'!
beStyled

	self textMorph ifNotNil: [:it | it beStyled].! !

!Browser methodsFor: 'metaclass' stamp: 'MarcusDenker 4/29/2011 00:33'!
beUnStyled

	self textMorph ifNotNil: [:it | it beUnStyled].! !

!Browser methodsFor: 'initialization' stamp: 'rww 8/18/2002 09:31'!
browseSelectionInPlace

	"In place code - incomplete"
"	self systemCategoryListIndex: 
		(self systemCategoryList indexOf: self selectedClass category).
	self classListIndex: (self classList indexOf: self selectedClass name)"

	self spawnHierarchy.! !

!Browser methodsFor: 'message category functions' stamp: ''!
buildMessageCategoryBrowser
	"Create and schedule a message category browser for the currently 
	selected message category."

	self buildMessageCategoryBrowserEditString: nil! !

!Browser methodsFor: 'message category functions' stamp: 'IgorStasenko 3/6/2011 18:04'!
buildMessageCategoryBrowserEditString: aString 
	"Create and schedule a message category browser for the currently 
	selected message category. The initial text view contains the characters 
	in aString."

	| newBrowser |
	messageCategoryListIndex ~= 0
		ifTrue: 
			[newBrowser := self new.
			newBrowser systemCategoryListIndex: systemCategoryListIndex.
			newBrowser classListIndex: (newBrowser classList indexOf: self selectedClassName).
			newBrowser metaClassIndicated: metaClassIndicated.
			newBrowser messageCategoryListIndex: messageCategoryListIndex.
			newBrowser messageListIndex: messageListIndex.
			self class openBrowserView: (newBrowser openMessageCatEditString: aString)
				label: 'Message Category Browser (' , 
						newBrowser selectedClassOrMetaClassName , ')']! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/15/2009 09:36'!
buildMorphicClassList

	| myClassList |

	(myClassList := PluggableListMorph new) 
			on: self list: #classList
			selected: #classListIndex changeSelected: #classListIndex:
			menu: #classListMenu:shifted: keystroke: #classListKey:from:.
	myClassList borderWidth: 0.
	myClassList enableDragNDrop: true.
	myClassList doubleClickSelector: #browseSelectionInPlace.
	"For doubleClick to work best disable autoDeselect"
	myClassList autoDeselect: false .
	^myClassList

! !

!Browser methodsFor: 'class comment pane' stamp: 'AlainPlantec 1/7/2010 21:39'!
buildMorphicCommentPane
	"Construct the pane that shows the class comment.
	Respect the setting StandardFonts codeFont."

	| commentPane |
	commentPane := BrowserCommentTextMorph
				on: self
				text: #classCommentText
				accept: #classComment:notifying:
				readSelection: nil
				menu: #codePaneMenu:shifted:.
	commentPane font: StandardFonts codeFont.
	^ commentPane! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/15/2009 09:36'!
buildMorphicMessageCatList

	| myMessageCatList |

	(myMessageCatList := PluggableMessageCategoryListMorph new) 
			on: self list: #messageCategoryList
			selected: #messageCategoryListIndex changeSelected: #messageCategoryListIndex:
			menu: #messageCategoryMenu: 
			keystroke: #arrowKey:from: getRawListSelector: #rawMessageCategoryList.
	myMessageCatList enableDragNDrop: true.
	^myMessageCatList
! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/15/2009 09:36'!
buildMorphicMessageList
	"Build a morphic message list, with #messageList as its list-getter"

	| aListMorph |
	(aListMorph := PluggableListMorph new) 
			on: self list: #messageList
			selected: #messageListIndex changeSelected: #messageListIndex:
			menu: #messageListMenu:shifted:
			keystroke: #messageListKey:from:.
	aListMorph enableDragNDrop: true.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	^aListMorph

! !

!Browser methodsFor: 'initialization' stamp: 'tbn 8/1/2010 14:06'!
buildMorphicSwitches

	| instanceSwitch commentSwitch classSwitch row |

	instanceSwitch := PluggableButtonMorph
		on: self
		getState: #instanceMessagesIndicated
		action: #indicateInstanceMessages.
	instanceSwitch
		label: 'Instance';
		askBeforeChanging: true;
		setBalloonText: 'Display instance side';
		borderWidth: 1;
		borderColor: Color gray.
	commentSwitch := PluggableButtonMorph
		on: self
		getState: #classCommentIndicated
		action: #plusButtonHit.
		
	commentSwitch
		label: '?' asText allBold;
		askBeforeChanging: true;
		setBalloonText: 'Cycle among definition, comment, and hierachy';
		borderWidth: 1;
		borderColor: Color gray.
	classSwitch := PluggableButtonMorph
		on: self
		getState: #classMessagesIndicated
		action: #indicateClassMessages.
	classSwitch
		label: 'Class';
		askBeforeChanging: true;
		setBalloonText: 'Display class side';		
		borderWidth: 1;
		borderColor: Color gray.
	row := AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		cellInset: 0;
		borderWidth: 0;
		layoutInset: 0;
		addMorphBack: instanceSwitch;
		addMorphBack: commentSwitch;
		addMorphBack: classSwitch.

	row color: Color white.
	{instanceSwitch. commentSwitch. classSwitch} do: [:m | 
		m 
			color: Color transparent;
			hResizing: #spaceFill;
			vResizing: #spaceFill.].
	^OverflowRowMorph new
		baseMorph: row;
		height: row minExtent y
! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/15/2009 09:36'!
buildMorphicSystemCatList
	| dragNDropFlag myCatList |
	dragNDropFlag := true.
	(myCatList := PluggableListMorph new) 
			on: self list: #systemCategoryList
			selected: #systemCategoryListIndex changeSelected: #systemCategoryListIndex:
			menu: #systemCategoryMenu: keystroke: #systemCatListKey:from:.
	myCatList enableDragNDrop: dragNDropFlag.
	^myCatList
! !

!Browser methodsFor: 'system category functions' stamp: ''!
buildSystemCategoryBrowser
	"Create and schedule a new system category browser."

	self buildSystemCategoryBrowserEditString: nil! !

!Browser methodsFor: 'system category functions' stamp: 'sd 11/20/2005 21:26'!
buildSystemCategoryBrowserEditString: aString 
	"Create and schedule a new system category browser with initial textual 
	contents set to aString."

	| newBrowser |
	systemCategoryListIndex > 0
		ifTrue: 
			[newBrowser := self class new.
			newBrowser systemCategoryListIndex: systemCategoryListIndex.
			newBrowser setClass: self selectedClassOrMetaClass selector: self selectedMessageName.
			self class openBrowserView: (newBrowser openSystemCatEditString: aString)
				label: 'Classes in category ', newBrowser selectedSystemCategoryName]! !

!Browser methodsFor: 'toolbuilder' stamp: 'tbn 7/6/2010 16:10'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec listSpec textSpec buttonSpec panelSpec max |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.43] ifFalse:[0.5].
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategoryList; 
		getIndex: #systemCategoryListIndex; 
		setIndex: #systemCategoryListIndex:; 
		menu: #systemCategoryMenu:; 
		keyPress: #systemCatListKey:from:;
		frame: (0@0 corner: 0.25@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		menu: #classListMenu:; 
		keyPress: #classListKey:from:;
		frame: (0.25@0 corner: 0.5@(max-0.1)).
	windowSpec children add: listSpec.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec frame: (0.25@(max-0.1) corner: 0.5@max).
	panelSpec children: OrderedCollection new.
	windowSpec children addLast: panelSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'Instance'; 
			state: #instanceMessagesIndicated; 
			action: #indicateInstanceMessages;
			frame: (0@0 corner: 0.4@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: '?'; 
			state: #classCommentIndicated; 
			action: #plusButtonHit;
			frame: (0.4@0 corner: 0.6@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'Class'; 
			state: #classMessagesIndicated; 
			action: #indicateClassMessages;
			frame: (0.6@0 corner: 1@1).
		panelSpec children addLast: buttonSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCategoryList; 
		getIndex: #messageCategoryListIndex; 
		setIndex: #messageCategoryListIndex:; 
		menu: #messageCategoryMenu:; 
		keyPress: #arrowKey:from:;
		frame: (0.5@0 corner: 0.75@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (0.75@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.43 corner: 1@0.5).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.5corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!Browser methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:10'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ true! !

!Browser methodsFor: 'message category list' stamp: 'sd 11/20/2005 21:26'!
categorizeAllUncategorizedMethods
	"Categorize methods by looking in parent classes for a method category."

	| organizer organizers |
	organizer := self classOrMetaClassOrganizer.
	organizers := self selectedClassOrMetaClass withAllSuperclasses collect: [:ea | ea organization].
	(organizer listAtCategoryNamed: ClassOrganizer default) do: [:sel | | found |
		found := (organizers collect: [ :org | org categoryOfElement: sel])
			detect: [:ea | ea ~= ClassOrganizer default and: [ ea ~= nil]]
			ifNone: [].
		found ifNotNil: [organizer classify: sel under: found]].

	self changed: #messageCategoryList! !

!Browser methodsFor: 'message category functions' stamp: 'sd 11/20/2005 21:26'!
categoryOfCurrentMethod
	"Determine the method category associated with the receiver at the current moment, or nil if none"

	| aCategory |
	^ super categoryOfCurrentMethod ifNil:
		[(aCategory := self messageCategoryListSelection) == ClassOrganizer allCategory
					ifTrue:
						[nil]
					ifFalse:
						[aCategory]]! !

!Browser methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:26'!
changeCategoryForClass: class srcSystemCategory: srcSystemCategorySel atListMorph: dstListMorph internal: internal copy: copyFlag 
	"only move semantic"
	| newClassCategory success |
	self flag: #stringSymbolProblem.
	success := copyFlag not ifFalse: [^ false].
	newClassCategory := self dstCategoryDstListMorph: dstListMorph.
	(success := newClassCategory notNil & (newClassCategory ~= class category))
		ifTrue: 
			[class category: newClassCategory.
			self changed: #classList.
			internal ifFalse: [self selectClass: class]].
	^ success! !

!Browser methodsFor: 'message category functions' stamp: 'NS 4/7/2004 22:56'!
changeMessageCategories: aString 
	"The characters in aString represent an edited version of the the message 
	categories for the selected class. Update this information in the system 
	and inform any dependents that the categories have been changed. This 
	message is invoked because the user had issued the categories command 
	and edited the message categories. Then the user issued the accept 
	command."

	self classOrMetaClassOrganizer changeFromString: aString.
	self clearUserEditFlag.
	self editClass.
	self classListIndex: classListIndex.
	^ true! !

!Browser methodsFor: 'drag and drop' stamp: 'nk 4/22/2004 18:00'!
changeMessageCategoryForMethod: methodSel dstMessageCategory: dstMessageCategorySel srcMessageCategory: srcMessageCategorySel insideClassOrMeta: classOrMeta internal: internal copySemantic: copyFlag 
	"Recategorize the method named by methodSel. 
	If the dstMessageCategorySel is the allCategory, then recategorize 
	it from its parents."
	| success messageCategorySel |
	copyFlag
		ifTrue: [^ false].
	"only move semantic"
	messageCategorySel := dstMessageCategorySel
				ifNil: [srcMessageCategorySel].
	(success := messageCategorySel notNil
					and: [messageCategorySel ~= srcMessageCategorySel])
		ifTrue: [success := messageCategorySel == ClassOrganizer allCategory
						ifTrue: [self recategorizeMethodSelector: methodSel]
						ifFalse: [(classOrMeta organization categories includes: messageCategorySel)
								and: [classOrMeta organization
										classify: methodSel
										under: messageCategorySel
										suppressIfDefault: false.
									true]]].
	success
		ifTrue: [self changed: #messageList.
			internal
				ifFalse: [self setSelector: methodSel]].
	^ success! !

!Browser methodsFor: 'system category functions' stamp: 'di 4/12/98 13:21'!
changeSystemCategories: aString 
	"Update the class categories by parsing the argument aString."

	systemOrganizer changeFromString: aString.
	self changed: #systemCategoryList.
	^ true! !

!Browser methodsFor: 'class comment pane' stamp: 'JB 2/12/2010 15:22'!
classComment: aText notifying: aPluggableTextMorph 
	"The user has just entered aText.
	It may be all red (a side-effect of replacing the default comment), so remove the color if it is."

	| theClass cleanedText redRange |
	theClass := self selectedClassOrMetaClass.
	theClass
		ifNotNil: [cleanedText := aText asText.
			redRange := cleanedText rangeOf: TextColor red startingAt: 1.
			redRange size = cleanedText size
				ifTrue: [cleanedText
						removeAttribute: TextColor red
						from: 1
						to: redRange last ].
			theClass comment: aText stamp: Author changeStamp].
	self changed: #classCommentText.
	^ true! !

!Browser methodsFor: 'metaclass' stamp: 'di 1/14/98 12:25'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."

	^ editSelection == #editComment 
! !

!Browser methodsFor: 'class functions' stamp: 'sd 11/20/2005 21:26'!
classCommentText
	"return the text to display for the comment of the currently selected class"
	| theClass |
	theClass := self selectedClassOrMetaClass.
	theClass ifNil: [ ^''].

	^ theClass hasComment
		ifTrue: [  theClass comment  ]
		ifFalse: [ self noCommentNagString ]! !

!Browser methodsFor: 'class functions' stamp: 'eem 5/7/2008 12:04'!
classDefinitionText
	"return the text to display for the definition of the currently selected class"
	| theClass |
	^(theClass := self selectedClassOrMetaClass) ifNil: [''] ifNotNil: [theClass definition]! !

!Browser methodsFor: 'class list' stamp: ''!
classList
	"Answer an array of the class names of the selected category. Answer an 
	empty array if no selection exists."

	systemCategoryListIndex = 0
		ifTrue: [^Array new]
		ifFalse: [^systemOrganizer listAtCategoryNumber: systemCategoryListIndex]! !

!Browser methodsFor: 'class list' stamp: ''!
classListIndex
	"Answer the index of the current class selection."

	^classListIndex! !

!Browser methodsFor: 'class list' stamp: 'StephaneDucasse 2/12/2011 14:09'!
classListIndex: anInteger 
	"Set anInteger to be the index of the current class selection."

	| className |
	self beUnStyled.
	classListIndex := anInteger.
	self setClassOrganizer.
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	self classCommentIndicated
		ifTrue: []
		ifFalse: [self editSelection: (anInteger = 0
					ifTrue: [metaClassIndicated | (systemCategoryListIndex == 0)
						ifTrue: [#none]
						ifFalse: [#newClass]]
					ifFalse: [#editClass])].
	contents := nil.
	self selectedClass isNil
		ifFalse: [className := self selectedClass name.
					(RecentClasses includes: className)
				ifTrue: [RecentClasses remove: className].
			RecentClasses addFirst: className.
			RecentClasses size > 16
				ifTrue: [RecentClasses removeLast]].
	self changed: #classSelectionChanged.
	self changed: #classCommentText.
	self changed: #classListIndex.	"update my selection"
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #relabel.
	self contentsChanged! !

!Browser methodsFor: 'class functions' stamp: 'tbn 7/6/2010 16:05'!
classListMenu: aMenu shifted: shifted
	"Set up the menu to apply to the receiver's class list, honoring the #shifted boolean"

	shifted
		ifTrue:
			[^ self shiftedClassListMenu: aMenu].
	aMenu addList: #(
		-
		('Browse full (b)'			browseMethodFull)
		('Browse hierarchy (h)'		spawnHierarchy)
		('Browse protocol (p)'		browseFullProtocol)
		-
		('FileOut'					fileOutClass)
		-
		('Show hierarchy'			hierarchy)
		('Show definition'			editClass)
		('Show comment'			editComment)
		-
		('Inst var refs...'			browseInstVarRefs)
		('Inst var defs...'			browseInstVarDefs)
		-
		('Class var refs...'			browseClassVarRefs)
		('Class vars'					browseClassVariables)
		('Class refs (N)'				browseClassRefs)
		-
		('Rename class ...'			renameClass)
		('Copy class'				copyClass)
		('Remove class (x)'			removeClass)
		-
		('Find method...'				findMethod)
		('Find method wildcard...'	findMethodWithWildcard)
		-
		('More...'					offerShiftedClassListMenu)).
	^ aMenu
! !

!Browser methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
classListSingleton

	| name |
	name := self selectedClassName.
	^ name ifNil: [Array new]
		ifNotNil: [Array with: name]! !

!Browser methodsFor: 'metaclass' stamp: 'mir 9/25/2008 14:56'!
classMessagesIndicated
	"Answer whether the messages to be presented should come from the metaclass."

	^ self metaClassIndicated and: [self classCommentIndicated not]! !

!Browser methodsFor: 'system category functions' stamp: 'tk 4/2/98 13:43'!
classNotFound

	self changed: #flash.! !

!Browser methodsFor: 'metaclass' stamp: 'StephaneDucasse 2/12/2011 14:15'!
classOrMetaClassOrganizer
	"Answer the class organizer for the metaclass or class, depending on 
	which (instance or class) is indicated."

	^ self metaClassIndicated
		ifTrue: [ metaClassOrganizer]
		ifFalse: [ classOrganizer]! !

!Browser methodsFor: 'code pane' stamp: 'AlainPlantec 1/7/2010 21:06'!
codePaneMenu: aMenu shifted: shifted 
	super codePaneMenu: aMenu shifted: shifted.
	^ aMenu! !

!Browser methodsFor: 'code pane' stamp: 'sd 11/20/2005 21:26'!
compileMessage: aText notifying: aController
	"Compile the code that was accepted by the user, placing the compiled method into an appropriate message category.  Return true if the compilation succeeded, else false."

	| fallBackCategoryIndex fallBackMethodIndex originalSelectorName result |

	self selectedMessageCategoryName ifNil:
			[ self selectOriginalCategoryForCurrentMethod 	
										ifFalse:["Select the '--all--' category"
											self messageCategoryListIndex: 1]]. 


	self selectedMessageCategoryName asSymbol = ClassOrganizer allCategory
		ifTrue:
			[ "User tried to save a method while the ALL category was selected"
			fallBackCategoryIndex := messageCategoryListIndex.
			fallBackMethodIndex := messageListIndex.
			editSelection == #newMessage
				ifTrue:
					[ "Select the 'as yet unclassified' category"
					messageCategoryListIndex := 0.
					(result := self defineMessageFrom: aText notifying: aController)
						ifNil:
							["Compilation failure:  reselect the original category & method"
							messageCategoryListIndex := fallBackCategoryIndex.
							messageListIndex := fallBackMethodIndex]
						ifNotNil:
							[self setSelector: result]]
				ifFalse:
					[originalSelectorName := self selectedMessageName.
					self setOriginalCategoryIndexForCurrentMethod.
					messageListIndex := fallBackMethodIndex := self messageList indexOf: originalSelectorName.			
					(result := self defineMessageFrom: aText notifying: aController)
						ifNotNil:
							[self setSelector: result]
						ifNil:
							[ "Compilation failure:  reselect the original category & method"
							messageCategoryListIndex := fallBackCategoryIndex.
							messageListIndex := fallBackMethodIndex.
							^ result notNil]].
			self changed: #messageCategoryList.
			^ result notNil]
		ifFalse:
			[ "User tried to save a method while the ALL category was NOT selected"
			^ (self defineMessageFrom: aText notifying: aController) notNil]! !

!Browser methodsFor: 'accessing' stamp: 'al 12/6/2005 22:36'!
contents
	"Depending on the current selection, different information is retrieved.
	Answer a string description of that information. This information is the
	method of the currently selected class and message."

	| comment theClass latestCompiledMethod |
	latestCompiledMethod := currentCompiledMethod.
	currentCompiledMethod := nil.

	editSelection == #newTrait
		ifTrue: [^Trait newTemplateIn: self selectedSystemCategoryName].
	editSelection == #none ifTrue: [^ ''].
	editSelection == #editSystemCategories 
		ifTrue: [^ systemOrganizer printString].
	editSelection == #newClass 
		ifTrue: [^ (theClass := self selectedClass)
			ifNil:
				[Class template: self selectedSystemCategoryName]
			ifNotNil:
				[Class templateForSubclassOf: theClass category: self selectedSystemCategoryName]].
	editSelection == #editClass 
		ifTrue: [^self classDefinitionText].
	editSelection == #editComment 
		ifTrue:
			[(theClass := self selectedClass) ifNil: [^ ''].
			comment := theClass comment.
			currentCompiledMethod := theClass organization commentRemoteStr.
			^ comment size = 0
				ifTrue: ['This class has not yet been commented.']
				ifFalse: [comment]].
	editSelection == #hierarchy 
		ifTrue: [
			self selectedClassOrMetaClass isTrait
				ifTrue: [^'']
				ifFalse: [^self selectedClassOrMetaClass printHierarchy]].
	editSelection == #editMessageCategories 
		ifTrue: [^ self classOrMetaClassOrganizer printString].
	editSelection == #newMessage
		ifTrue:
			[^ (theClass := self selectedClassOrMetaClass) 
				ifNil: ['']
				ifNotNil: [theClass sourceCodeTemplate]].
	editSelection == #editMessage
		ifTrue:
			[self showingByteCodes ifTrue: [^ self selectedBytecodes].
			currentCompiledMethod := latestCompiledMethod.
			^ self selectedMessage].

	self error: 'Browser internal error: unknown edit selection.'! !

!Browser methodsFor: 'accessing' stamp: 'JB 2/12/2010 15:22'!
contents: input notifying: aController 
	"The retrieved information has changed and its source must now be
	 updated. The information can be a variety of things, depending on
	 the list selections (such as templates for class or message definition,
	 methods) or the user menu commands (such as definition, comment,
	 hierarchy).  Answer the result of updating the source."

	| aString aText theClass |
	self changed: #annotation.
	aString := input asString.
	aText := input asText.
	editSelection == #newTrait ifTrue: [^self defineTrait: input asString notifying: aController].
	editSelection == #editSystemCategories ifTrue: [^ self changeSystemCategories: aString].
	editSelection == #editClass | (editSelection == #newClass) ifTrue: [^ self defineClass: aString notifying: aController].
	editSelection == #editComment
		ifTrue: 
			[theClass := self selectedClass.
			theClass
				ifNil: 
					[self inform: 'You must select a class
before giving it a comment.'.
					^ false].
			theClass comment: aText stamp: Author changeStamp.
			self changed: #classCommentText.
			^ true].
	editSelection == #hierarchy ifTrue: [^ true].
	editSelection == #editMessageCategories ifTrue: [^ self changeMessageCategories: aString].
	editSelection == #editMessage | (editSelection == #newMessage)
		ifTrue:
			[^ self okayToAccept
				ifFalse:
					[false]
				ifTrue:
					[self compileMessage: aText notifying: aController]].
	editSelection == #none
		ifTrue: 
			[self inform: 'This text cannot be accepted
in this part of the browser.'.
			^ false].
	self error: 'unacceptable accept'! !

!Browser methodsFor: 'accessing' stamp: 'tak 9/25/2008 14:58'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	messageCategoryListIndex > 0 & (messageListIndex = 0)
		ifTrue: [^ 1 to: 500]	"entire empty method template"
		ifFalse: [^ 1 to: 0]		"null selection"! !

!Browser methodsFor: 'class functions' stamp: 'DamienCassou 9/29/2009 09:05'!
copyClass
	| copysName |
	classListIndex = 0
		ifTrue: [^ self].
	self okToChange
		ifFalse: [^ self].
	copysName := self request: 'Please type new class name' initialAnswer: self selectedClass name.
	copysName isEmptyOrNil
		ifTrue: [^ self].
	"Cancel returns ''"
	self selectedClass duplicateClassWithNewName: copysName.
	self classListIndex: 0.
	self changed: #classList! !

!Browser methodsFor: 'accessing' stamp: 'MarcusDenker 10/24/2010 16:59'!
couldBrowseAnyClass
	"Answer whether the receiver is equipped to browse any class. This is in
	support of the system-brower feature that allows the browser to be
	redirected at the selected class name. This implementation is clearly
	ugly, but the feature it enables is handsome enough. 3/1/96 sw"
	self dependents
		detect: [:d | (d respondsTo: #getListSelector)
				and: [d getListSelector == #systemCategoryList]]
		ifNone: [^ false].
	^ true! !

!Browser methodsFor: 'class functions' stamp: 'StephaneDucasse 5/13/2010 11:43'!
createInstVarAccessors
	"Create getters and setters for all inst vars defined at the level of the current class selection, except do NOT clobber or override any selectors already understood by the instances of the selected class"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[aClass instVarNames do: 
			[:aName | | setter newMessage |
				(aClass canUnderstand: aName asSymbol)
					ifFalse:
						[newMessage := aName, '

	^ ', aName.
						aClass compile: newMessage classified: 'accessing' notifying: nil].
				(aClass canUnderstand: (setter := aName, ':') asSymbol)
					ifFalse:
						[newMessage := setter, ' anObject

	', aName, ' := anObject'.
						aClass compile: newMessage classified: 'accessing' notifying: nil]]]! !

!Browser methodsFor: 'initialization' stamp: 'sw 1/13/2000 16:45'!
defaultBrowserTitle
	^ 'System Browser'! !

!Browser methodsFor: 'class functions' stamp: 'MarcusDenker 3/26/2010 16:48'!
defineClass: defString notifying: aController  
	"The receiver's textual content is a request to define a new class. The
	source code is defString. If any errors occur in compilation, notify
	aController."
	| oldClass class newClassName defTokens keywdIx envt |
	oldClass := self selectedClassOrMetaClass.
	defTokens := defString findTokens: Character separators.
	
	((defTokens first = 'Trait' and: [defTokens second = 'named:'])
		or: [defTokens second = 'classTrait'])
		ifTrue: [^self defineTrait: defString notifying: aController].
		
	keywdIx := defTokens findFirst: [:x | x beginsWith: 'category'].
	envt := Smalltalk globals.
	keywdIx := defTokens findFirst: [:x | '*subclass*' match: x].
	newClassName := (defTokens at: keywdIx+1) copyWithoutAll: '#()'.
	((oldClass isNil or: [oldClass theNonMetaClass name asString ~= newClassName])
		and: [envt includesKey: newClassName asSymbol]) ifTrue:
			["Attempting to define new class over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newClassName , ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newClassName size))
				ifFalse: [^ false]].
	"ar 8/29/1999: Use oldClass superclass for defining oldClass
	since oldClass superclass knows the definerClass of oldClass."
	oldClass ifNotNil:[oldClass := oldClass superclass].
	class := oldClass subclassDefinerClass
				evaluate: defString
				notifying: aController
				logged: true.
	(class isKindOf: Behavior)
		ifTrue: [self changed: #systemCategoryList.
				self changed: #classList.
				self clearUserEditFlag.
				self setClass: class selector: nil.
				"self clearUserEditFlag; editClass."
				^ true]
		ifFalse: [^ false]! !

!Browser methodsFor: 'message functions' stamp: 'lr 7/3/2009 20:59'!
defineMessageFrom: aString notifying: aController
	"Compile the expressions in aString. Notify aController if a syntax error occurs. Install the compiled method in the selected class classified under  the currently selected message category name. Answer the selector obtained if compilation succeeds, nil otherwise."
	| selectedMessageName selector category oldMessageList |
	selectedMessageName := self selectedMessageName.
	oldMessageList := self messageList.
	contents := nil.
	selector := self selectedClassOrMetaClass
				compile: aString
				classified: (category := self selectedMessageCategoryName)
				notifying: aController.
	selector == nil ifTrue: [^ nil].
	contents := aString copy.
	selector ~~ selectedMessageName
		ifTrue: 
			[category = ClassOrganizer nullCategory
				ifTrue: [self changed: #classSelectionChanged.
						self changed: #classList.
						self messageCategoryListIndex: 1].
			self setClassOrganizer.  "In case organization not cached"
			(oldMessageList includes: selector)
				ifFalse: [self changed: #messageList].
			self messageListIndex: (self messageList indexOf: selector)].
	^ selector! !

!Browser methodsFor: 'traits' stamp: 'md 3/3/2006 10:58'!
defineTrait: defString notifying: aController  

	| defTokens keywdIx envt oldTrait newTraitName trait |
	oldTrait := self selectedClassOrMetaClass.
	defTokens := defString findTokens: Character separators.
	keywdIx := defTokens findFirst: [:x | x = 'category'].
	envt := self selectedEnvironment.
	keywdIx := defTokens findFirst: [:x | x = 'named:'].
	newTraitName := (defTokens at: keywdIx+1) copyWithoutAll: '#()'.
	((oldTrait isNil or: [oldTrait baseTrait name asString ~= newTraitName])
		and: [envt includesKey: newTraitName asSymbol]) ifTrue:
			["Attempting to define new class/trait over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newTraitName , ' is an existing class/trait in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newTraitName size))
				ifFalse: [^ false]].

	trait := Compiler evaluate: defString notifying: aController logged: true.
	^(trait isKindOf: TraitBehavior)
		ifTrue: [
			self changed: #classList.
			self classListIndex: (self classList indexOf: trait baseTrait name).
			self clearUserEditFlag; editClass.
			true]
		ifFalse: [ false ]
! !

!Browser methodsFor: 'drag and drop' stamp: 'MarcusDenker 3/26/2010 16:48'!
destinationClassDestinationListMorph: dstListMorph
	| dropItem |
	^ dstListMorph getListSelector == #classList
		ifTrue: [ (dropItem := dstListMorph potentialDropItem) ifNotNil: [ Smalltalk globals at: dropItem withBlanksCondensed asSymbol ] ]
		ifFalse: [ dstListMorph model selectedClass ]! !

!Browser methodsFor: 'accessing' stamp: 'nice 3/31/2011 22:34'!
doItReceiver
	"This class's classPool has been jimmied to be the classPool of the class 
	being browsed. A doIt in the code pane will let the user see the value of 
	the class variables."

	^ self selectedClass! !

!Browser methodsFor: 'drag and drop' stamp: 'IgorStasenko 3/6/2011 18:07'!
dragPassengerFor: item inMorph: dragSource 
	| transferType smn |
	(dragSource isKindOf: PluggableListMorph)
		ifFalse: [^nil].
	transferType := self dragTransferTypeForMorph: dragSource.
	transferType == #classList
		ifTrue: [^self selectedClass].
	transferType == #messageList
		ifFalse: [ ^nil ].
	smn := self selectedMessageName ifNil: [ ^nil ].
	(Smalltalk tools messageSet isPseudoSelector: smn) ifTrue: [ ^nil ].

	^ self selectedClassOrMetaClass -> smn.
! !

!Browser methodsFor: 'drag and drop' stamp: 'panda 4/28/2000 16:18'!
dragTransferTypeForMorph: dragSource 
	^(dragSource isKindOf: PluggableListMorph)
		ifTrue: [dragSource getListSelector]! !

!Browser methodsFor: 'drag and drop' stamp: 'ls 6/22/2001 23:21'!
dstCategoryDstListMorph: dstListMorph
	^(dstListMorph getListSelector == #systemCategoryList)
		ifTrue: [dstListMorph potentialDropItem ]
		ifFalse: [self selectedSystemCategoryName]! !

!Browser methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:26'!
dstMessageCategoryDstListMorph: dstListMorph
	| dropItem |
	^dstListMorph getListSelector == #messageCategoryList
		ifTrue: 
			[dropItem := dstListMorph potentialDropItem.
			dropItem ifNotNil: [dropItem asSymbol]]
		ifFalse: [self selectedMessageCategoryName ifNil: [ Categorizer default ]]! !

!Browser methodsFor: 'class functions' stamp: 'nk 2/15/2004 13:23'!
editClass
	"Retrieve the description of the class definition."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	self editSelection: #editClass.
	self changed: #contents.
	self changed: #classCommentText.
! !

!Browser methodsFor: 'class functions' stamp: 'sd 11/20/2005 21:26'!
editComment
	"Retrieve the description of the class comment."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	metaClassIndicated := false.
	self editSelection: #editComment.
	self changed: #classSelectionChanged.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self decorateButtons.
	self contentsChanged
! !

!Browser methodsFor: 'message category functions' stamp: 'nk 2/14/2004 15:06'!
editMessageCategories
	"Indicate to the receiver and its dependents that the message categories of 
	the selected class have been changed."

	self okToChange ifFalse: [^ self].
	classListIndex ~= 0
		ifTrue: 
			[self messageCategoryListIndex: 0.
			self editSelection: #editMessageCategories.
			self changed: #editMessageCategories.
			self contentsChanged]! !

!Browser methodsFor: 'accessing' stamp: ''!
editSelection
	^editSelection! !

!Browser methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:26'!
editSelection: aSelection
	"Set the editSelection as requested."

	editSelection := aSelection.
	self changed: #editSelection.! !

!Browser methodsFor: 'system category functions' stamp: 'nk 2/14/2004 15:09'!
editSystemCategories
	"Retrieve the description of the class categories of the system organizer."

	self okToChange ifFalse: [^ self].
	self systemCategoryListIndex: 0.
	self editSelection: #editSystemCategories.
	self changed: #editSystemCategories.
	self contentsChanged! !

!Browser methodsFor: 'class functions' stamp: 'AlainPlantec 11/25/2010 09:27'!
fileOutClass
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .st."

	Cursor write showWhile:
		[classListIndex ~= 0 ifTrue: [self selectedClass fileOut]]! !

!Browser methodsFor: 'message category functions' stamp: 'tk 4/2/98 13:53'!
fileOutMessageCategories
	"Print a description of the selected message category of the selected class 
	onto an external file."

Cursor write showWhile:
	[messageCategoryListIndex ~= 0
		ifTrue: 
			[self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName]]! !

!Browser methodsFor: 'system category functions' stamp: 'tk 3/31/98 07:52'!
fileOutSystemCategory
	"Print a description of each class in the selected category onto a file 
	whose name is the category name followed by .st."

	systemCategoryListIndex ~= 0
		ifTrue: [systemOrganizer fileOutCategory: self selectedSystemCategoryName]! !

!Browser methodsFor: 'system category functions' stamp: 'AlainPlantec 4/15/2010 10:43'!
findClass
	"Search for a class from a pattern or from the recent list"
	| pattern foundClassOrTrait recentList |

	self okToChange ifFalse: [^ self classNotFound].
	recentList := RecentClasses select: [ :n | Smalltalk globals includesKey: n ].
	recentList size == 0
		ifTrue: [pattern := UIManager default request: 'Class name or fragment?']
		ifFalse: [pattern := UIManager default 
				enterOrRequestFrom: recentList 
				lines: #() 
				title: 'Class name or fragment?'].
	pattern isEmptyOrNil ifTrue: [^ self classNotFound].
	foundClassOrTrait := SystemNavigation default classFromPattern: pattern withCaption: ''.
	foundClassOrTrait ifNil: [^ self classNotFound].
 	self selectCategoryForClass: foundClassOrTrait.
	self selectClass: foundClassOrTrait
! !

!Browser methodsFor: 'class functions' stamp: 'MarcusDenker 3/5/2010 14:30'!
findMethod
	"Pop up a list of the current class's methods, and select the one chosen by the user"

	| aClass selectors reply cat messageCatIndex messageIndex choices |
	self classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	selectors := aClass selectors sort.
	selectors isEmpty ifTrue: [self inform: aClass name, ' has no methods.' translated. ^ self].
	choices := (Array with: 'Enter Wildcard' translated), selectors.
	reply := UIManager default chooseFrom: choices lines: #(1).
	reply = 0 ifTrue: [^self].
	reply = 1
		ifTrue: [
			reply := UIManager default request: 'Enter partial method name:' translated.
			(reply isNil or: [reply isEmpty])
				ifTrue: [^self].
			(reply includes: $*)
				ifFalse: [reply := '*', reply, '*'].
			selectors := selectors select: [:each | reply match: each].
			selectors isEmpty ifTrue: [self inform: aClass name, ' has no matching methods.' translated. ^ self].
			reply := selectors size = 1
				ifTrue: [selectors first]
				ifFalse: [	UIManager default chooseFrom: selectors values: selectors].
			reply isNil ifTrue: [^self]]
		ifFalse: [reply := choices at: reply].

	cat := aClass whichCategoryIncludesSelector: reply.
	messageCatIndex := self messageCategoryList indexOf: cat.
	self messageCategoryListIndex: messageCatIndex.
	messageIndex := (self messageList indexOf: reply).
	self messageListIndex: messageIndex! !

!Browser methodsFor: 'class functions' stamp: 'MarcusDenker 3/5/2010 14:30'!
findMethodWithWildcard
	"Pop up a list of the current class's methods, and select the one chosen by the user"

	| aClass selectors reply cat messageCatIndex messageIndex |
	self classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	selectors := aClass selectors sort.
	selectors isEmpty ifTrue: [self inform: aClass name, ' has no methods.' translated. ^ self].

	reply := UIManager default request: 'Enter partial method name:' translated.
	(reply isNil or: [reply isEmpty])
		ifTrue: [^self].
	(reply includes: $*)
		ifFalse: [reply := '*', reply, '*'].
	selectors := selectors select: [:each | reply match: each].
	selectors isEmpty ifTrue: [self inform: aClass name, ' has no matching methods.'. ^ self].
	reply := selectors size = 1
		ifTrue: [selectors first]
		ifFalse: [UIManager default chooseFrom: selectors values: selectors].
	reply == nil ifTrue: [^ self].

	cat := aClass whichCategoryIncludesSelector: reply.
	messageCatIndex := self messageCategoryList indexOf: cat.
	self messageCategoryListIndex: messageCatIndex.
	messageIndex := (self messageList indexOf: reply).
	self messageListIndex: messageIndex! !

!Browser methodsFor: 'class functions' stamp: 'nk 2/14/2004 15:09'!
hierarchy
	"Display the inheritance hierarchy of the receiver's selected class."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	self editSelection: #hierarchy.
	self changed: #editComment.
	self contentsChanged.
	^ self! !

!Browser methodsFor: 'system category list' stamp: 'tk 5/4/1998 15:46'!
indexIsOne
	"When used as a singleton list, index is always one"
	^ 1! !

!Browser methodsFor: 'system category list' stamp: 'tk 5/4/1998 15:46'!
indexIsOne: value
	"When used as a singleton list, can't change it"

	^ self! !

!Browser methodsFor: 'metaclass' stamp: 'StephaneDucasse 2/12/2011 14:07'!
indicateClassMessages
	"Indicate that the message selection should come from the metaclass 
	messages."

	self beStyled.
	self metaClassIndicated: true! !

!Browser methodsFor: 'metaclass' stamp: 'StephaneDucasse 2/12/2011 14:09'!
indicateInstanceMessages
	"Indicate that the message selection should come from the class (instance) 
	messages."

	self beUnStyled.
	self metaClassIndicated: false! !

!Browser methodsFor: 'message functions' stamp: 'al 4/24/2004 12:48'!
inspectInstances
	"Inspect all instances of the selected class.  1/26/96 sw"

	| myClass |
	((myClass := self selectedClassOrMetaClass) isNil or: [myClass isTrait])
		ifFalse: [myClass theNonMetaClass inspectAllInstances]
! !

!Browser methodsFor: 'message functions' stamp: 'al 4/24/2004 12:48'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses  1/26/96 sw"

	| aClass |
	((aClass := self selectedClassOrMetaClass) isNil or: [aClass isTrait])
		ifFalse: [
			aClass := aClass theNonMetaClass.
			aClass inspectSubInstances].
! !

!Browser methodsFor: 'metaclass' stamp: 'di 1/14/98 13:20'!
instanceMessagesIndicated
	"Answer whether the messages to be presented should come from the 
	class."

	^metaClassIndicated not and: [self classCommentIndicated not]! !

!Browser methodsFor: 'initialization' stamp: 'AlainPlantec 2/25/2011 12:02'!
labelString
	self selectedClass ifNil: [ ^ self defaultBrowserTitle ].
	^ String streamContents: [ :s|
		self selectedClass printOn: s.
		self metaClassIndicated ifTrue: [ s << ' class' ].
		self selectedMessageName ifNotNil: [ :name| s << '>>' << name ]]
! !

!Browser methodsFor: 'class functions' stamp: 'nk 2/14/2004 15:07'!
makeNewSubclass

	self selectedClassOrMetaClass ifNil: [^ self].
	self okToChange ifFalse: [^ self].
	self editSelection: #newClass.
	self contentsChanged! !

!Browser methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:26'!
message: messageSel compileInClass: dstClassOrMeta fromClass: srcClassOrMeta dstMessageCategory: dstMessageCategorySel srcMessageCategory: srcMessageCategorySel internal: internal copySemantic: copyFlag 
	| source messageCategorySel tm success oldOrNoMethod newMethod |
	source := srcClassOrMeta sourceCodeAt: messageSel.
	messageCategorySel := dstMessageCategorySel ifNil: [srcMessageCategorySel].
	self selectClass: dstClassOrMeta theNonMetaClass.
	(self messageCategoryList includes: messageCategorySel)
		ifFalse: ["create message category"
			self classOrMetaClassOrganizer addCategory: messageCategorySel].
	self selectMessageCategoryNamed: messageCategorySel.
	tm := self codeTextMorph.
	tm setText: source.
	tm setSelection: (0 to: 0).
	tm hasUnacceptedEdits: true.
	oldOrNoMethod := srcClassOrMeta compiledMethodAt: messageSel ifAbsent: [].
	tm accept.
	"compilation successful?"
	newMethod := dstClassOrMeta compiledMethodAt: messageSel ifAbsent: [].
	success := newMethod ~~ nil & (newMethod ~~ oldOrNoMethod).
	"	success ifFalse: [TransferMorph allInstances do: [:e | e delete]].            
	 "
	success
		ifTrue: 
			[copyFlag not ifTrue: ["remove old method in move semantic if new exists"
		srcClassOrMeta removeSelector: messageSel].internal
				ifTrue: [self selectClass: srcClassOrMeta]
				ifFalse: [self selectClass: dstClassOrMeta].
			self setSelector: messageSel].
	^ success! !

!Browser methodsFor: 'message category list' stamp: 'sd 11/20/2005 21:26'!
messageCatListSingleton

	| name |
	name := self selectedMessageCategoryName.
	^ name ifNil: [Array new]
		ifNotNil: [Array with: name]! !

!Browser methodsFor: 'message category list' stamp: 'ccn 3/22/1999 17:56'!
messageCategoryList
	"Answer the selected category of messages."

	classListIndex = 0
		ifTrue: [^ Array new]
		ifFalse: [^ (Array with: ClassOrganizer allCategory), self classOrMetaClassOrganizer categories]! !

!Browser methodsFor: 'message category list' stamp: ''!
messageCategoryListIndex
	"Answer the index of the selected message category."

	^messageCategoryListIndex! !

!Browser methodsFor: 'message category list' stamp: 'CamilloBruni 2/14/2011 11:26'!
messageCategoryListIndex: anInteger
	"Set the selected message category to be the one indexed by anInteger."

	messageCategoryListIndex := anInteger.
	messageListIndex := 0.
	self changed: #messageCategorySelectionChanged.
	self changed: #messageCategoryListIndex. "update my selection"
	self changed: #messageList.
	self editSelection: (anInteger > 0
		ifTrue: [
			self beStyled.
			#newMessage]
		ifFalse: [self classListIndex > 0
			ifTrue: [
				self beUnStyled.
				#editClass]
			ifFalse: [
				self beUnStyled.
				#newClass]]).
	contents := nil.
	self contentsChanged.
	self changed: #relabel.! !

!Browser methodsFor: 'message category list' stamp: 'ccn 3/24/1999 11:02'!
messageCategoryListSelection
	"Return the selected category name or nil."

	^ ((self messageCategoryList size = 0 
		or: [self messageCategoryListIndex = 0]) 
		or: [self messageCategoryList size < self messageCategoryListIndex])
			ifTrue: [nil]
			ifFalse: [self messageCategoryList at: (self messageCategoryListIndex max: 1)]! !

!Browser methodsFor: 'message category functions' stamp: 'tbn 7/6/2010 16:12'!
messageCategoryMenu: aMenu
	^ aMenu labels:
'Browse
FileOut
Reorganize
Alphabetize
Remove empty categories
Categorize all uncategorized
New Category...
Rename...
Remove'
		lines: #(3 8)
		selections:
		#(buildMessageCategoryBrowser fileOutMessageCategories
		editMessageCategories alphabetizeMessageCategories removeEmptyCategories
		categorizeAllUncategorizedMethods addCategory renameCategory removeMessageCategory)
! !

!Browser methodsFor: 'message list' stamp: 'adrian-lienhard 5/18/2009 21:10'!
messageList
	"Answer an Array of the message selectors of the currently selected message category, provided that the messageCategoryListIndex is in proper range.  Otherwise, answer an empty Array  If messageCategoryListIndex is found to be larger than the number of categories (it happens!!), it is reset to zero."
	| sel |
	(sel := self messageCategoryListSelection) ifNil: [
		^ self classOrMetaClassOrganizer
			ifNil: [Array new]
			ifNotNil: [self classOrMetaClassOrganizer allMethodSelectors]
	].

	^ sel = ClassOrganizer allCategory ifTrue:  [
		self classOrMetaClassOrganizer
			ifNil: [Array new]
			ifNotNil: [self classOrMetaClassOrganizer allMethodSelectors]]
		ifFalse: [
			(self classOrMetaClassOrganizer listAtCategoryNumber: messageCategoryListIndex - 1)
				ifNil: [messageCategoryListIndex := 0.  Array new]]! !

!Browser methodsFor: 'message list' stamp: ''!
messageListIndex
	"Answer the index of the selected message selector into the currently 
	selected message category."

	^messageListIndex! !

!Browser methodsFor: 'message list' stamp: 'StephaneDucasse 2/22/2011 18:41'!
messageListIndex: anInteger
	"Set the selected message selector to be the one indexed by anInteger."

	messageListIndex := anInteger.
	self beStyled.
	self editSelection: (anInteger > 0
		ifTrue: [#editMessage]
		ifFalse: [self messageCategoryListIndex > 0
			ifTrue: [#newMessage]
			ifFalse: [self classListIndex > 0
				ifTrue: [#editClass]
				ifFalse: [#newClass]]]).
	contents := nil.
	self changed: #messageListIndex. "update my selection"
	self contentsChanged.
	self decorateButtons.
	self changed: #relabel.! !

!Browser methodsFor: 'message functions' stamp: 'tbn 7/6/2010 16:13'!
messageListMenu: aMenu shifted: shifted 
	"Answer the message-list menu"
	shifted
		ifTrue: [^ self shiftedMessageListMenu: aMenu].
	aMenu addList: #(
			('What to show...'			offerWhatToShowMenu)
			('Toggle break on entry'		toggleBreakOnEntry)
			-
			('Browse full (b)' 			browseMethodFull)
			('Browse hierarchy (h)'			classHierarchy)
			('Browse method (O)'			openSingleMessageBrowser)
			('Browse protocol (p)'			browseFullProtocol)
			-
			('FileOut'				fileOutMessage)
			-
			('Senders of... (n)'			browseSendersOfMessages)
			('Implementors of... (m)'		browseMessages)
			('Inheritance (i)'			methodHierarchy)
			('Versions (v)'				browseVersions)
			-
			('Inst var refs...'			browseInstVarRefs)
			('Inst var defs...'			browseInstVarDefs)
			('Class var refs...'			browseClassVarRefs)
			('Class variables'			browseClassVariables)
			('Class refs (N)'			browseClassRefs)
			-
			('Remove method (x)'			removeMessage)
			-
			('More...'				shiftedYellowButtonActivity)).
	^ aMenu! !

!Browser methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
messageListSingleton

	| name |
	name := self selectedMessageName.
	^ name ifNil: [Array new]
		ifNotNil: [Array with: name]! !

!Browser methodsFor: 'metaclass' stamp: 'sr 6/21/2000 17:23'!
metaClassIndicated
	"Answer the boolean flag that indicates which of the method dictionaries, 
	class or metaclass."

	^ metaClassIndicated! !

!Browser methodsFor: 'metaclass' stamp: 'sd 11/20/2005 21:26'!
metaClassIndicated: trueOrFalse 
	"Indicate whether browsing instance or class messages."

	metaClassIndicated := trueOrFalse.
	self setClassOrganizer.
	systemCategoryListIndex > 0 ifTrue:
		[self editSelection: (classListIndex = 0
			ifTrue: [metaClassIndicated
				ifTrue: [#none]
				ifFalse: [#newClass]]
			ifFalse: [#editClass])].
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	contents := nil.
	self changed: #classSelectionChanged.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #contents.
	self changed: #annotation.
	self decorateButtons
! !

!Browser methodsFor: 'initialization' stamp: 'sw 9/22/1999 17:13'!
methodCategoryChanged
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #annotation.
	self messageListIndex: 0! !

!Browser methodsFor: 'traits' stamp: 'al 4/24/2004 11:48'!
newClass
	(self selectedClassOrMetaClass notNil and: 
		[self selectedClassOrMetaClass isTrait]) ifTrue: [self classListIndex: 0].
	self editClass.
	editSelection := #newClass.
	self contentsChanged! !

!Browser methodsFor: 'traits' stamp: 'al 4/24/2004 11:48'!
newTrait
	self classListIndex: 0.
	self editClass.
	editSelection := #newTrait.
	self contentsChanged! !

!Browser methodsFor: 'class comment pane' stamp: 'StephaneDucasse 4/24/2011 18:49'!
noCommentNagString

	^ Text string: 'Unfortunately this class has not been documented yet. The class comment should describe the purpose of the class, its collaborations and its variables. We encourage you to fill up the following template.

Class intention
	I m doing the following ...

Class collaborations
	I usually do not work in isolation but with the help of ...

Class main API
	My main public API is ...My subclasses may want to override such specific hooks ...
Implementation notes
' translated attribute: TextColor red.
		! !

!Browser methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:26'!
noteSelectionIndex: anInteger for: aSymbol
	aSymbol == #systemCategoryList
		ifTrue:
			[systemCategoryListIndex := anInteger].
	aSymbol == #classList
		ifTrue:
			[classListIndex := anInteger].
	aSymbol == #messageCategoryList
		ifTrue:
			[messageCategoryListIndex := anInteger].
	aSymbol == #messageList
		ifTrue:
			[messageListIndex := anInteger].! !

!Browser methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 12:51'!
openAsMorphClassEditing: editString
	"Create a pluggable version a Browser on just a single class."
	^UIManager default openBrowser: self asMorphClassEditing: editString! !

!Browser methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 12:50'!
openAsMorphEditing: editString
	"Create a pluggable version of all the morphs for a Browser in Morphic"
	^UIManager default openBrowser: self asMorphEditing: editString! !

!Browser methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 12:41'!
openAsMorphMessageEditing: editString
	"Create a pluggable version a Browser that shows just one message"
	^UIManager default openBrowser: self asMorphMessageEditing: editString! !

!Browser methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 12:46'!
openAsMorphMsgCatEditing: editString
	"Create a pluggable version a Browser on just a message category."
	^UIManager default openBrowser: self asMorphMsgCatEditing: editString! !

!Browser methodsFor: 'initialization' stamp: 'JuanVuletich 10/21/2010 11:52'!
openAsMorphSysCatEditing: editString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	^UIManager default openBrowser: self  asMorphSysCatEditing: editString
! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 6/10/2008 18:33'!
openEditString: aString
        "Create a pluggable version of all the views for a Browser, including views and controllers."
         ^ self openAsMorphEditing: aString! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 6/10/2008 18:34'!
openMessageCatEditString: aString
        "Create a pluggable version of the views for a Browser that just shows one message category."
         ^ self openAsMorphMsgCatEditing: aString! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/30/2008 10:08'!
openMessageEditString: aString
	"Create a pluggable version of the views for a Browser that just shows one message."
	^ self openAsMorphMessageEditing: aString! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/30/2008 10:13'!
openOnClassWithEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	^ self openAsMorphClassEditing: aString.

! !

!Browser methodsFor: 'initialization' stamp: 'alain.plantec 5/30/2008 10:16'!
openSystemCatEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."

	^ self openAsMorphSysCatEditing: aString! !

!Browser methodsFor: 'drag and drop' stamp: 'alain.plantec 2/6/2009 16:33'!
overwriteDialogHierarchyChange: hierarchyChange higher: higherFlag sourceClassName: srcClassName destinationClassName: dstClassName methodSelector: methodSelector 
	| lf |
	lf := Character cr asString.
	^ UIManager default
				confirm: 'There is a conflict.' translated, ' Overwrite' translated, (hierarchyChange
							ifTrue: [higherFlag
									ifTrue: [' superclass' translated]
									ifFalse: [' subclass' translated]]
							ifFalse: ['']) , ' method' translated, lf , dstClassName , '>>' , methodSelector , lf , 'by ' translated, (hierarchyChange
							ifTrue: ['moving' translated]
							ifFalse: ['copying' translated]) , ' method' translated, lf , srcClassName name , '>>' , methodSelector , ' ?'.
				
! !

!Browser methodsFor: 'class functions' stamp: 'StephaneDucasse 2/12/2011 14:09'!
plusButtonHit
	"Cycle among definition, comment, and hierachy"
	
	self beUnStyled.
	editSelection == #editComment
		ifTrue: [self hierarchy. ^ self].
	editSelection == #hierarchy
		ifTrue: [self editSelection: #editClass.
			classListIndex = 0 ifTrue: [^ self].
			self okToChange ifFalse: [^ self].
			self changed: #editComment.
			self contentsChanged.
			^ self].
	self editComment! !

!Browser methodsFor: 'message category list' stamp: 'sw 10/16/1999 22:56'!
rawMessageCategoryList
	^ classListIndex = 0
		ifTrue: [Array new]
		ifFalse: [self classOrMetaClassOrganizer categories]! !

!Browser methodsFor: 'message category list' stamp: 'nice 1/5/2010 15:59'!
recategorizeMethodSelector: sel 
	"Categorize method named sel by looking in parent classes for a 
	method category. 
	Answer true if recategorized."
	
	self selectedClassOrMetaClass allSuperclasses
		do: [:ea | | thisCat | 
			thisCat := ea organization categoryOfElement: sel.
			(thisCat ~= ClassOrganizer default
					and: [thisCat notNil])
				ifTrue: [self classOrMetaClassOrganizer classify: sel under: thisCat.
					self changed: #messageCategoryList.
					^ true]].
	^ false! !

!Browser methodsFor: 'class list' stamp: 'lr 3/14/2010 21:13'!
recent
	"Let the user select from a list of recently visited classes.  11/96 stp.
	 12/96 di:  use class name, not classes themselves.
	 : dont fall into debugger in empty case"

	| className class recentList |
	recentList := RecentClasses select: [ :n | Smalltalk globals includesKey: n ].
	recentList size == 0
		ifTrue: [ ^ Beeper beep ].
	className := UIManager default chooseFrom: recentList values: recentList.
	className isNil
		ifTrue: [ ^ self ].
	class := Smalltalk globals at: className.
	self selectCategoryForClass: class.
	self classListIndex: (self classList indexOf: class name)! !

!Browser methodsFor: 'message list' stamp: 'sw 12/1/2000 11:17'!
reformulateList
	"If the receiver has a way of reformulating its message list, here is a chance for it to do so"

	super reformulateList.
	self messageListIndex: 0! !

!Browser methodsFor: 'class functions' stamp: 'sw 3/5/2001 18:04'!
removeClass
	"If the user confirms the wish to delete the class, do so"

	super removeClass ifTrue:
		[self classListIndex: 0]! !

!Browser methodsFor: 'message category functions' stamp: 'nk 4/23/2004 09:18'!
removeEmptyCategories
	self okToChange ifFalse: [^ self].
	self selectedClassOrMetaClass organization removeEmptyCategories.
	self changed: #messageCategoryList
! !

!Browser methodsFor: 'message functions' stamp: 'AlainPlantec 1/7/2010 21:40'!
removeMessage
	"If a message is selected, create a Confirmer so the user can verify that  
	the currently selected message should be removed from the system. If 
	so, remove it. "
	| messageName confirmation |
	messageListIndex = 0
		ifTrue: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	confirmation := self systemNavigation   confirmRemovalOf: messageName on: self selectedClassOrMetaClass.
	confirmation == 3
		ifTrue: [^ self].
	(self selectedClassOrMetaClass includesLocalSelector: messageName)
		ifTrue: [self selectedClassOrMetaClass removeSelector: messageName]
		ifFalse: [self removeNonLocalSelector: messageName].
	self messageListIndex: 0.
	self changed: #messageList.
	self setClassOrganizer.
	"In case organization not cached"
	confirmation == 2
		ifTrue: [self systemNavigation browseAllCallsOn: messageName]! !

!Browser methodsFor: 'message category functions' stamp: 'sd 11/20/2005 21:26'!
removeMessageCategory
	"If a message category is selected, create a Confirmer so the user can 
	verify that the currently selected message category should be removed
 	from the system. If so, remove it."

	| messageCategoryName |
	messageCategoryListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	messageCategoryName := self selectedMessageCategoryName.
	(self messageList size = 0
		or: [self confirm: 'Are you sure you want to
remove this method category 
and all its methods?'])
		ifTrue: 
			[self selectedClassOrMetaClass removeCategory: messageCategoryName.
			self messageCategoryListIndex: 0.
			self changed: #classSelectionChanged].
	self changed: #messageCategoryList.
! !

!Browser methodsFor: 'message functions' stamp: 'tk 4/2/98 17:03'!
removeMessageFromBrowser
	"Our list speaks the truth and can't have arbitrary things removed"

	^ self changed: #flash! !

!Browser methodsFor: 'traits' stamp: 'jannik.laval 5/1/2010 16:00'!
removeNonLocalSelector: aSymbol
	| traits isAlias |
	traits := self selectedClassOrMetaClass traitsProvidingSelector: aSymbol.
	isAlias := self selectedClassOrMetaClass isLocalAliasSelector: aSymbol.
	isAlias
		ifTrue: [
			[traits size = 1] assert.
			self selectedClassOrMetaClass removeAlias: aSymbol of: traits first]
		ifFalse: [
			traits do: [:each |
				self selectedClassOrMetaClass addExclusionOf: aSymbol to: each ]]
	! !

!Browser methodsFor: 'system category functions' stamp: 'di 4/12/98 13:55'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	systemCategoryListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(self classList size = 0
		or: [self confirm: 'Are you sure you want to
remove this system category 
and all its classes?'])
		ifTrue: 
		[systemOrganizer removeSystemCategory: self selectedSystemCategoryName.
		self systemCategoryListIndex: 0.
		self changed: #systemCategoryList]! !

!Browser methodsFor: 'message category functions' stamp: 'DamienCassou 9/29/2009 09:05'!
renameCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldIndex oldName newName |
	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(oldIndex := messageCategoryListIndex) = 0 ifTrue: [^ self].
	oldName := self selectedMessageCategoryName.
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmptyOrNil
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	newName = oldName ifTrue: [^ self].
	self classOrMetaClassOrganizer
		renameCategory: oldName
		toBe: newName.
	self classListIndex: classListIndex.
	self messageCategoryListIndex: oldIndex.
	self changed: #messageCategoryList.
! !

!Browser methodsFor: 'class functions' stamp: 'StephaneDucasse 4/28/2010 12:25'!
renameClass
	| oldName newName obs |
	classListIndex = 0
		ifTrue: [ ^ self ].
	self okToChange
		ifFalse: [ ^ self ].
	oldName := self selectedClass name.
	newName := self request: 'Please type new class name' initialAnswer: oldName.
	newName isEmptyOrNil
		ifTrue: [ ^ self ].	"Cancel returns ''"
	newName := newName asSymbol.
	newName = oldName
		ifTrue: [ ^ self ].
	(Smalltalk globals includesKey: newName)
		ifTrue: [ ^ self error: newName , ' already exists' ].
	self selectedClass rename: newName.
	self changed: #classList.
	self classListIndex: ((systemOrganizer listAtCategoryNamed: self selectedSystemCategoryName) indexOf: newName).
	obs := self systemNavigation allReferencesTo: (self selectedClass environment associationAt: newName).
	obs isEmpty
		ifFalse: [ self systemNavigation browseMessageList: obs name: 'Obsolete References to ' , oldName autoSelect: oldName ]! !

!Browser methodsFor: 'system category functions' stamp: 'DamienCassou 9/29/2009 09:05'!
renameSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldIndex oldName newName |
	(oldIndex := systemCategoryListIndex) = 0
		ifTrue: [^ self].  "no selection"
	self okToChange ifFalse: [^ self].
	oldName := self selectedSystemCategoryName.
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmptyOrNil 
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	oldName = newName ifTrue: [^ self].
	systemOrganizer
		renameCategory: oldName
		toBe: newName.
	self systemCategoryListIndex: oldIndex.
	self changed: #systemCategoryList.! !

!Browser methodsFor: 'accessing' stamp: 'rbb 3/1/2005 10:26'!
request: prompt initialAnswer: initialAnswer

	^ UIManager default
		request: prompt
		initialAnswer: initialAnswer
! !

!Browser methodsFor: 'system category list' stamp: 'stp 01/13/2000 12:25'!
selectCategoryForClass: theClass

	self systemCategoryListIndex: (self systemCategoryList indexOf: theClass category)
! !

!Browser methodsFor: 'class list' stamp: 'sr 10/29/1999 20:28'!
selectClass: classNotMeta
	self classListIndex: (self classList indexOf: classNotMeta name)! !

!Browser methodsFor: 'message category list' stamp: 'nk 6/13/2004 06:20'!
selectMessageCategoryNamed: aSymbol 
	"Given aSymbol, select the category with that name.  Do nothing if 
	aSymbol doesn't exist."
	self messageCategoryListIndex: (self messageCategoryList indexOf: aSymbol ifAbsent: [ 1])! !

!Browser methodsFor: 'message category list' stamp: 'sd 11/20/2005 21:26'!
selectOriginalCategoryForCurrentMethod
	"private - Select the message category for the current method. 
	 
	 Note:  This should only be called when somebody tries to save  
	 a method that they are modifying while ALL is selected. 
	 
	 Returns: true on success, false on failure."
	| aSymbol selectorName |
	aSymbol := self categoryOfCurrentMethod.
	selectorName := self selectedMessageName.
	(aSymbol notNil and: [aSymbol ~= ClassOrganizer allCategory])
		ifTrue: 
			[messageCategoryListIndex := (self messageCategoryList indexOf: aSymbol).
			messageListIndex := (self messageList indexOf: selectorName).
			self changed: #messageCategorySelectionChanged.
			self changed: #messageCategoryListIndex.	"update my selection"
			self changed: #messageList.
			self changed: #messageListIndex.
			^ true].
	^ false! !

!Browser methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."

	| name envt |
	(name := self selectedClassName) ifNil: [^ nil].
	(envt := self selectedEnvironment) ifNil: [^ nil].
	^ envt at: name! !

!Browser methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
selectedClassName
	| aClassList |
	"Answer the name of the current class. Answer nil if no selection exists."

	(classListIndex = 0 or: [classListIndex > (aClassList := self classList) size]) ifTrue: [^ nil].
	^ aClassList at: classListIndex! !

!Browser methodsFor: 'metaclass' stamp: 'al 4/24/2004 11:47'!
selectedClassOrMetaClass
	"Answer the selected class/trait or metaclass/classTrait."

	| cls |
	^self metaClassIndicated
		ifTrue: [(cls := self selectedClass) ifNil: [nil] ifNotNil: [cls classSide]]
		ifFalse: [self selectedClass]! !

!Browser methodsFor: 'metaclass' stamp: ''!
selectedClassOrMetaClassName
	"Answer the selected class name or metaclass name."

	^self selectedClassOrMetaClass name! !

!Browser methodsFor: 'system category list' stamp: 'md 3/3/2006 11:02'!
selectedEnvironment
	"Answer the name of the selected system category or nil."

	systemCategoryListIndex = 0 ifTrue: [^nil].
	^ Smalltalk! !

!Browser methodsFor: 'message list' stamp: 'md 2/20/2006 15:01'!
selectedMessage
	"Answer a copy of the source code for the selected message."

	| class selector method |
	contents == nil ifFalse: [^ contents copy].

	self showingDecompile ifTrue:
		[^ self decompiledSourceIntoContents].

	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	method := class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod := method.

	^ contents := (self showingDocumentation
		ifFalse: [ self sourceStringPrettifiedAndDiffed ]
		ifTrue: [ self commentContents ])
			copy asText makeSelectorBoldIn: class! !

!Browser methodsFor: 'message category list' stamp: ''!
selectedMessageCategoryName
	"Answer the name of the selected message category, if any. Answer nil 
	otherwise."

	messageCategoryListIndex = 0 ifTrue: [^nil].
	^self messageCategoryList at: messageCategoryListIndex! !

!Browser methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
selectedMessageName
	"Answer the message selector of the currently selected message, if any. 
	Answer nil otherwise."

	| aList |
	messageListIndex = 0 ifTrue: [^ nil].
	^ (aList := self messageList) size >= messageListIndex
		ifTrue:
			[aList at: messageListIndex]
		ifFalse:
			[nil]! !

!Browser methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
selectedMessageName: aSelector
	"Make the given selector be the selected message name"

	| anIndex |
	anIndex := self messageList indexOf: aSelector.
	anIndex > 0 ifTrue:
		[self messageListIndex: anIndex]! !

!Browser methodsFor: 'system category list' stamp: ''!
selectedSystemCategoryName
	"Answer the name of the selected system category or nil."

	systemCategoryListIndex = 0 ifTrue: [^nil].
	^self systemCategoryList at: systemCategoryListIndex! !

!Browser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
setClass: aBehavior selector: aSymbol
	"Set the state of a new, uninitialized Browser."

	| isMeta aClass messageCatIndex |
	aBehavior ifNil: [^ self].
	(aBehavior isKindOf: Metaclass)
		ifTrue: [
			isMeta := true.
			aClass := aBehavior soleInstance]
		ifFalse: [
			isMeta := false.
			aClass := aBehavior].
	self selectCategoryForClass: aClass.
	self classListIndex: (
		(systemOrganizer listAtCategoryNamed: self selectedSystemCategoryName)
			indexOf: aClass name).
	self metaClassIndicated: isMeta.
	aSymbol ifNil: [^ self].
	messageCatIndex := aBehavior organization numberOfCategoryOfElement: aSymbol.
	self messageCategoryListIndex: (messageCatIndex > 0
		ifTrue: [messageCatIndex + 1]
		ifFalse: [0]).
	messageCatIndex = 0 ifTrue: [^ self].
	self messageListIndex: (
		(aBehavior organization listAtCategoryNumber: messageCatIndex)
			indexOf: aSymbol).! !

!Browser methodsFor: 'metaclass' stamp: 'md 2/18/2006 16:31'!
setClassOrganizer
	"Install whatever organization is appropriate"
	| theClass |
	classOrganizer := nil.
	metaClassOrganizer := nil.
	classListIndex = 0 ifTrue: [^ self].
	theClass := self selectedClass ifNil: [ ^self ].
	classOrganizer := theClass organization.
	metaClassOrganizer := theClass classSide organization.! !

!Browser methodsFor: 'message category list' stamp: 'sd 11/20/2005 21:26'!
setOriginalCategoryIndexForCurrentMethod
	"private - Set the message category index for the currently selected method. 
	 
	 Note:  This should only be called when somebody tries to save  
	 a method that they are modifying while ALL is selected."

	messageCategoryListIndex := self messageCategoryList indexOf: self categoryOfCurrentMethod
	! !

!Browser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
setSelector: aSymbol
	"Make the receiver point at the given selector, in the currently chosen class"

	| aClass messageCatIndex |
	aSymbol ifNil: [^ self].
	(aClass := self selectedClassOrMetaClass) ifNil: [^ self].
	messageCatIndex := aClass organization numberOfCategoryOfElement: aSymbol.
	self messageCategoryListIndex: messageCatIndex + 1.
	messageCatIndex = 0 ifTrue: [^ self].
	self messageListIndex:
			((aClass organization listAtCategoryNumber: messageCatIndex)
					indexOf: aSymbol)! !

!Browser methodsFor: 'class functions' stamp: 'MarcusDenker 4/21/2011 22:27'!
shiftedClassListMenu: aMenu
	"Set up the menu to apply to the receiver's class list when the shift key is down"

	^ aMenu addList: #(
			-
			('Unsent methods'			browseUnusedMethods	'Browse all methods defined by this class that have no senders')
			('Unreferenced inst vars'	showUnreferencedInstVars	'Show a list of all instance variables that are not referenced in methods')
			('Unreferenced class vars'	showUnreferencedClassVars	'Show a list of all class variables that are not referenced in methods')
			('Subclass template'			makeNewSubclass		'Put a template into the code pane for defining of a subclass of this class')
			-
			('Inspect instances'			inspectInstances			'Open an inspector on all the extant instances of this class')
			('Inspect subinstances'		inspectSubInstances		'Open an inspector on all the extant instances of this class and of all of its subclasses')
			-
			('Add all meths to current chgs'		addAllMethodsToCurrentChangeSet
																'Place all the methods defined by this class into the current change set')
			('Create inst var accessors'	createInstVarAccessors	'Compile instance-variable access methods for any instance variables that do not yet have them')
			-
			('More...'					offerUnshiftedClassListMenu	'Return to the standard class-list menu'))! !

!Browser methodsFor: 'message functions' stamp: 'MarcusDenker 4/21/2011 22:27'!
shiftedMessageListMenu: aMenu
	"Fill aMenu with the items appropriate when the shift key is held down"

	aMenu addStayUpItem.
	aMenu addList: #(
		('Toggle diffing (D)'						toggleDiffing)
		('Implementors of sent messages'			browseAllMessages)
		-
		('Local senders of...'						browseLocalSendersOfMessages)
		('Local implementors of...'				browseLocalImplementors)
		-
		('Spawn sub-protocol'					spawnProtocol)
		('Spawn full protocol'					spawnFullProtocol)
		-
		('Inspect instances'						inspectInstances)
		('Inspect subinstances'					inspectSubInstances)).

	self addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(
		-
		('Change category...'					changeCategory)).

	self canShowMultipleMessageCategories ifTrue: [aMenu addList:
		 #(('Show category (C)'						showHomeCategory))].
	aMenu addList: #(
		-
		('Change sets with this method'			findMethodInChangeSets)
		('Revert to previous version'				revertToPreviousVersion)
		('Remove from current change set'		removeFromCurrentChanges)
		('Revert & remove from changes'		revertAndForget)
		('Add to current change set'				adoptMessageInCurrentChangeset)
		('Copy up or copy down...'				copyUpOrCopyDown)
		-
		('More...' 								unshiftedYellowButtonActivity)).
	^ aMenu
! !

!Browser methodsFor: 'message category functions' stamp: 'sd 11/20/2005 21:26'!
showHomeCategory
	"Show the home category of the selected method.  This is only really useful if one is in a tool that supports the showing of categories.  Thus, it's good in browsers and hierarchy browsers but not in message-list browsers"

	| aSelector |
	self okToChange ifTrue:
		[(aSelector := self selectedMessageName) ifNotNil:
			[self selectOriginalCategoryForCurrentMethod.
			self selectedMessageName: aSelector]]! !

!Browser methodsFor: 'initialization' stamp: 'sw 11/8/1999 13:36'!
systemCatSingletonKey: aChar from: aView
	^ self messageListKey: aChar from: aView! !

!Browser methodsFor: 'system category functions' stamp: 'marcus.denker 10/20/2008 21:22'!
systemCatSingletonMenu: aMenu

	^ aMenu labels:
'browse
fileOut
update
rename...
remove' 
	lines: #(2 4)
	selections:
		#(buildSystemCategoryBrowser
		fileOutSystemCategory updateSystemCategories
		renameSystemCategory removeSystemCategory)
! !

!Browser methodsFor: 'system category list' stamp: ''!
systemCategoryList
	"Answer the class categories modelled by the receiver."

	^systemOrganizer categories! !

!Browser methodsFor: 'system category list' stamp: ''!
systemCategoryListIndex
	"Answer the index of the selected class category."

	^systemCategoryListIndex! !

!Browser methodsFor: 'system category list' stamp: 'StephaneDucasse 2/12/2011 14:11'!
systemCategoryListIndex: anInteger 
	"Set the selected system category index to be anInteger. Update all other 
	selections to be deselected."

	self beStyled.
	systemCategoryListIndex := anInteger.
	classListIndex := 0.
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	self editSelection: ( anInteger = 0 ifTrue: [#none] ifFalse: [#newClass]).
	metaClassIndicated := false.
	self setClassOrganizer.
	contents := nil.
	self changed: #systemCategorySelectionChanged.
	self changed: #systemCategoryListIndex.	"update my selection"
	self changed: #classList.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #relabel.
	self contentsChanged! !

!Browser methodsFor: 'system category functions' stamp: 'tbn 7/6/2010 16:09'!
systemCategoryMenu: aMenu
		
	^ aMenu labels:
'Find Class... (f)
Browse
FileOut
Reorganize
Alphabetize
Update
Add Category...
Rename...
Remove' 
	lines: #(2 4 6 8)
	selections:
		#(findClass buildSystemCategoryBrowser
		fileOutSystemCategory
		editSystemCategories alphabetizeSystemCategories updateSystemCategories
		addSystemCategory renameSystemCategory removeSystemCategory )! !

!Browser methodsFor: 'system category list' stamp: 'sd 11/20/2005 21:26'!
systemCategorySingleton

	| cat |
	cat := self selectedSystemCategoryName.
	^ cat ifNil: [Array new]
		ifNotNil: [Array with: cat]! !

!Browser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
systemOrganizer: aSystemOrganizer
	"Initialize the receiver as a perspective on the system organizer, 
	aSystemOrganizer. Typically there is only one--the system variable 
	SystemOrganization."
	
	contents := nil.
	systemOrganizer := aSystemOrganizer.
	systemCategoryListIndex := 0.
	classListIndex := 0.
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	metaClassIndicated := false.
	self setClassOrganizer.
	self editSelection: #none.! !

!Browser methodsFor: 'breakpoints' stamp: 'marcus.denker 10/9/2008 20:32'!
toggleBreakOnEntry
	"Install or uninstall a halt-on-entry breakpoint"
	super toggleBreakOnEntry.
	self changed: #messageList
		! !

!Browser methodsFor: 'system category functions' stamp: 'di 4/12/98 13:17'!
updateSystemCategories
	"The class categories were changed in another browser. The receiver must 
	reorganize its lists based on these changes."

	self okToChange ifFalse: [^ self].
	self changed: #systemCategoryList! !

!Browser methodsFor: 'copying' stamp: 'sd 11/20/2005 21:26'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
"systemOrganizer := systemOrganizer. 	clone has the old value. we share it"
"classOrganizer := classOrganizer		clone has the old value. we share it"
"metaClassOrganizer 	:= metaClassOrganizer	clone has the old value. we share it"
systemCategoryListIndex := systemCategoryListIndex veryDeepCopyWith: deepCopier.
classListIndex := classListIndex veryDeepCopyWith: deepCopier.
messageCategoryListIndex := messageCategoryListIndex veryDeepCopyWith: deepCopier.
messageListIndex := messageListIndex veryDeepCopyWith: deepCopier.
editSelection := editSelection veryDeepCopyWith: deepCopier.
metaClassIndicated := metaClassIndicated veryDeepCopyWith: deepCopier.
! !

!Browser methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:26'!
wantsDroppedMorph: transferMorph event: anEvent inMorph: destinationLM 
	"We are only interested in TransferMorphs as wrappers for             
	informations. If their content is really interesting for us, will             
	determined later in >>acceptDroppingMorph:event:."

	| srcType dstType |

	"only want drops on lists (not, for example, on pluggable texts)"
	(destinationLM isKindOf: PluggableListMorph) ifFalse: [^ false].

	srcType := transferMorph dragTransferType.
	dstType := destinationLM getListSelector.

	(srcType == #messageList
		and: [dstType == #messageCategoryList or: [dstType == #classList]])
		ifTrue: [^true].
	(srcType == #classList
		and: [dstType == #systemCategoryList])
		ifTrue: [^true].
"			[
			srcLS == #messageList ifTrue: [^ dstLS == #messageList | (dstLS == #messageCategoryList) | (dstLS == #classList)].
			srcLS == #classList ifTrue: [^ dstLS == #classList | (dstLS == #systemCategoryList)]].
"
	^ false! !

Browser subclass: #FileContentsBrowser
	instanceVariableNames: 'packages infoString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-File Contents Browser'!

!FileContentsBrowser commentStamp: '<historical>' prior: 0!
I am a class browser view on a fileout (either a source file (.st) or change set (.cs)). I do not actually load the code into to the system, nor do I alter the classes in the image. Use me to vet code in a comfortable way before loading it into your image.

From a FileList, I can be invoked by selecting a source file and selecting the "browse code" menu item from the yellow button menu.

I use PseudoClass, PseudoClassOrganizers, and PseudoMetaclass to model the class structure of the source file.!

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 1/5/2010 15:59'!
browseCompressedCodeStream: aStandardFileStream 
	"Browse the selected file in fileIn format."
	| unzipped |
	[ | zipped |zipped := GZipReadStream on: aStandardFileStream.
	unzipped := MultiByteBinaryOrTextStream with: zipped contents asString]
		ensure: [aStandardFileStream close].
	unzipped reset.
	self browseStream: unzipped named: aStandardFileStream name! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nb 6/17/2003 12:25'!
browseFile: aFilename
	"Open a file contents browser on a file of the given name"

	aFilename ifNil: [^ Beeper beep].
	self browseFiles: (Array with: aFilename)! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 1/5/2010 15:59'!
browseFiles: fileList

	| browser |
	Cursor wait showWhile: [ | packageDict organizer |
		packageDict := Dictionary new.
		organizer := SystemOrganizer defaultList: Array new.
		fileList do: [:fileName | | package |
			package := FilePackage fromFileNamed: fileName.
			packageDict 
				at: package packageName 
				put: package.
			organizer 
				classifyAll: package classes keys 
				under: package packageName].
		(browser := self systemOrganizer: organizer)
			packages: packageDict].
	self
		openBrowserView: browser createViews
		label: 'File Contents Browser'.
! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'IgorStasenko 3/6/2011 18:12'!
browseStream: aStream

	aStream setConverterForCode.
	self browseStream: aStream named: aStream name! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 1/5/2010 15:59'!
browseStream: aStream named: aString

	| browser |
	Cursor wait showWhile: [ | package packageDict organizer |
		packageDict := Dictionary new.
		browser := self new.
		organizer := SystemOrganizer defaultList: Array new.
		package := (FilePackage new fullName: aString; fileInFrom: aStream).
		packageDict 
			at: package packageName 
			put: package.
		organizer 
			classifyAll: package classes keys 
			under: package packageName.
		(browser := self systemOrganizer: organizer)
			packages: packageDict].
	self
		openBrowserView: browser createViews
		label: 'File Contents Browser'.
! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'nk 6/12/2004 11:41'!
fileReaderServicesForDirectory: aDirectory
	^{ self serviceBrowseCodeFiles }! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'nk 2/17/2004 19:18'!
fileReaderServicesForFile: fullName suffix: suffix

	((FileStream isSourceFileSuffix: suffix) or: [ suffix = '*' ])
		ifTrue: [ ^Array with: self serviceBrowseCode].

	^(fullName endsWith: 'cs.gz')
		ifTrue: [ Array with: self serviceBrowseCompressedCode ]
		ifFalse: [#()]
! !

!FileContentsBrowser class methodsFor: 'initialization' stamp: 'GabrielOmarCotelli 6/4/2009 20:36'!
initialize

	FileServices registerFileReader: self! !

!FileContentsBrowser class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color tan
! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'DamienCassou 9/29/2009 09:11'!
selectAndBrowseFile: aFileList
	"When no file are selected you can ask to browse several of them"

	| selectionPattern files |
	selectionPattern := UIManager default request:'What files?' initialAnswer: '*.cs;*.st'.
	selectionPattern ifNil: [selectionPattern := String new].
	files := (aFileList directory fileNamesMatching: selectionPattern) 
				collect: [:each | aFileList directory fullNameFor: each].
	self browseFiles: files.


! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'tbn 8/11/2010 10:11'!
serviceBrowseCode
	"Answer the service of opening a file-contents browser"

	^ (SimpleServiceEntry
		provider: self 
		label: 'Code-file browser'
		selector: #browseStream:
		description: 'Open a "file-contents browser" on this file, allowing you to view and selectively load its code'
		buttonLabel: 'code')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'tbn 8/11/2010 10:11'!
serviceBrowseCodeFiles

	^  (SimpleServiceEntry 
		provider: self
		label: 'Browse code files' 
		selector: #selectAndBrowseFile:)
		argumentGetter: [ :fileList | fileList ];
		yourself! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'tbn 8/11/2010 10:12'!
serviceBrowseCompressedCode
	"Answer a service for opening a changelist browser on a file"

	^ (SimpleServiceEntry 
		provider: self 
		label: 'Code-file browser'
		selector: #browseCompressedCodeStream:
		description: 'Open a "file-contents browser" on this file, allowing you to view and selectively load its code'
		buttonLabel: 'Code')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'md 11/23/2004 13:34'!
services
	"Answer potential file services associated with this class"

	^ {self serviceBrowseCode}.! !

!FileContentsBrowser class methodsFor: 'initialization' stamp: 'GabrielOmarCotelli 6/4/2009 20:37'!
unload

	FileServices unregisterFileReader: self ! !

!FileContentsBrowser methodsFor: 'creation' stamp: 'md 2/24/2006 15:46'!
addLowerPanesTo: window at: nominalFractions with: editString

	| verticalOffset row codePane infoPane infoHeight divider |

	row := AlignmentMorph newColumn
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 1;
		borderColor: Color black;
		layoutPolicy: ProportionalLayout new.

	codePane := MorphicTextEditor default on: self text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	infoPane := PluggableTextMorph on: self text: #infoViewContents accept: nil
			readSelection: nil menu: nil.
	infoPane askBeforeDiscardingEdits: false.
	verticalOffset := 0.

">>not with this browser--- at least not yet ---
	innerFractions := 0@0 corner: 1@0.
	verticalOffset := self addOptionalAnnotationsTo: row at: innerFractions plus: verticalOffset.
	verticalOffset := self addOptionalButtonsTo: row  at: innerFractions plus: verticalOffset.
<<<<"

	infoHeight := 20.
	row 
		addMorph: (codePane borderWidth: 0)
		fullFrame: (
			LayoutFrame 
				fractions: (0@0 corner: 1@1) 
				offsets: (0@verticalOffset corner: 0@infoHeight negated)
		).
	divider := BorderedSubpaneDividerMorph forTopEdge.
	divider extent: 4@4; color: Color transparent; borderColor: #raised; borderWidth: 2.
	row 
		addMorph: divider
		fullFrame: (
			LayoutFrame 
				fractions: (0@1 corner: 1@1) 
				offsets: (0@infoHeight negated corner: 0@(1-infoHeight))
		).
	row 
		addMorph: (infoPane borderWidth: 0; hideScrollBarsIndefinitely)
		fullFrame: (
			LayoutFrame 
				fractions: (0@1 corner: 1@1) 
				offsets: (0@(1-infoHeight) corner: 0@0)
		).
	window 
		addMorph: row
		frame: nominalFractions.

	row on: #mouseEnter send: #paneTransition: to: window.
	row on: #mouseLeave send: #paneTransition: to: window.

! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'IgorStasenko 3/6/2011 18:51'!
browseMethodFull
	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[Smalltalk tools browse: myClass realClass selector: self selectedMessageName]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'bkv 8/13/2003 23:59'!
browseSenders
	"Create and schedule a message set browser on all senders of the 
	currently selected message selector. Do nothing if no message is selected."

	messageListIndex ~= 0 
		ifTrue: [self systemNavigation browseAllCallsOn: self selectedMessageName]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'IgorStasenko 3/6/2011 18:51'!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."
	| class selector |
	(selector := self selectedMessageName) ifNotNil:
		[class := self selectedClassOrMetaClass.
		(class exists and: [class realClass includesSelector: selector]) ifTrue:
			[Smalltalk tools versionBrowser
				browseVersionsOf: (class realClass compiledMethodAt: selector)
				class: class realClass theNonMetaClass
				meta: class realClass isMeta
				category: self selectedMessageCategoryName
				selector: selector]]! !

!FileContentsBrowser methodsFor: 'toolbuilder' stamp: 'StephaneDucasse 3/12/2011 13:27'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec listSpec textSpec buttonSpec panelSpec max |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.43] ifFalse:[0.5].
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategoryList; 
		getIndex: #systemCategoryListIndex; 
		setIndex: #systemCategoryListIndex:;  
		keyPress: #packageListKey:from:;
		frame: (0@0 corner: 0.25@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		menu: #classListMenu:; 
		keyPress: #classListKey:from:;
		frame: (0.25@0 corner: 0.5@(max-0.1)).
	windowSpec children add: listSpec.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec frame: (0.25@(max-0.1) corner: 0.5@max).
	panelSpec children: OrderedCollection new.
	windowSpec children addLast: panelSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'instance'; 
			state: #instanceMessagesIndicated; 
			action: #indicateInstanceMessages;
			frame: (0@0 corner: 0.4@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: '?'; 
			state: #classCommentIndicated; 
			action: #plusButtonHit;
			frame: (0.4@0 corner: 0.6@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'class'; 
			state: #classMessagesIndicated; 
			action: #indicateClassMessages;
			frame: (0.6@0 corner: 1@1).
		panelSpec children addLast: buttonSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCategoryList; 
		getIndex: #messageCategoryListIndex; 
		setIndex: #messageCategoryListIndex:; 
		menu: #messageCategoryMenu:; 
		keyPress: #arrowKey:from:;
		frame: (0.5@0 corner: 0.75@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (0.75@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.43 corner: 1@0.5).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.5corner: 1@0.92).
	windowSpec children add: textSpec.

	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: self;
		getText: #infoViewContents; 
		frame: (0@0.92corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!FileContentsBrowser methodsFor: 'other' stamp: ''!
changeMessageCategories: aString 
	"The characters in aString represent an edited version of the the message 
	categories for the selected class. Update this information in the system 
	and inform any dependents that the categories have been changed. This 
	message is invoked because the user had issued the categories command 
	and edited the message categories. Then the user issued the accept 
	command."

	self classOrMetaClassOrganizer changeFromString: aString.
	self unlock.
	self editClass.
	self classListIndex: classListIndex.
	^ true! !

!FileContentsBrowser methodsFor: 'class list' stamp: ''!
classList
	"Answer an array of the class names of the selected category. Answer an 
	empty array if no selection exists."

	(systemCategoryListIndex = 0 or:[self selectedPackage isNil])
		ifTrue: [^Array new]
		ifFalse: [^self selectedPackage classes keys asSortedCollection].! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 5/6/2000 18:48'!
classListKey: aChar from: view
	aChar == $b ifTrue: [^ self browseMethodFull].
	aChar == $N ifTrue: [^ self browseClassRefs].
	self packageListKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'tbn 7/6/2010 16:23'!
classListMenu: aMenu

	^ aMenu 
		labels:
'Definition
Comment
Browse full (b)
Class refs (N)
FileIn
FileOut
Rename...
Remove
Remove existing'
		lines: #(2 4 6 8)
		selections: #(editClass editComment browseMethodFull browseClassRefs fileInClass fileOutClass renameClass removeClass removeUnmodifiedCategories) 

! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'tpr 3/11/2001 21:26'!
classListMenu: aMenu shifted: ignored
	"Answer the class list menu, ignoring the state of the shift key in this case"

	^ self classListMenu: aMenu! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
contents
	self updateInfoView.
	(editSelection == #newClass and:[self selectedPackage notNil])
		ifTrue: [^self selectedPackage packageInfo].
	editSelection == #editClass
		ifTrue:[^self modifiedClassDefinition].
	^super contents! !

!FileContentsBrowser methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
contents: input notifying: aController 
	"The retrieved information has changed and its source must now be 
	updated. The information can be a variety of things, depending on the 
	list selections (such as templates for class or message definition, methods) 
	or the user menu commands (such as definition, comment, hierarchy). 
	Answer the result of updating the source."

	| aString aText theClass |
	aString := input asString.
	aText := input asText.

	editSelection == #editComment 
		ifTrue: [theClass := self selectedClass.
				theClass ifNil: [self inform: 'You must select a class
before giving it a comment.'.
				^ false].
				theClass comment: aText. ^ true].
	editSelection == #editMessageCategories 
		ifTrue: [^ self changeMessageCategories: aString].

	self inform:'You cannot change the current selection'.
	^false
! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'sw 11/13/2001 09:12'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane.  For the file-contents browser, the choices are restricted to source and the two diffing options"

	^ self sourceAndDiffsQuintsOnly! !

!FileContentsBrowser methodsFor: 'creation' stamp: 'alain.plantec 5/30/2008 13:03'!
createViews
	contentsSymbol := self defaultDiffsSymbol.
	"#showDiffs or #prettyDiffs"
	^ self openAsMorph! !

!FileContentsBrowser methodsFor: 'initialization' stamp: 'dew 9/15/2001 16:19'!
defaultBrowserTitle
	^ 'File Contents Browser'! !

!FileContentsBrowser methodsFor: 'other' stamp: 'sd 11/20/2005 21:27'!
didCodeChangeElsewhere
	"Determine whether the code for the currently selected method and class has been changed somewhere else."

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNil: [^ false].

	(aClass isKindOf: PseudoClass) ifTrue: [^ false]. "class not installed"
	^super didCodeChangeElsewhere! !

!FileContentsBrowser methodsFor: 'infoview' stamp: 'sma 5/6/2000 19:19'!
extraInfo
	^ (self
		methodDiffFor: (self selectedClassOrMetaClass sourceCodeAt: self selectedMessageName)
		class: self selectedClass
		selector: self selectedMessageName
		meta: self metaClassIndicated) unembellished
			ifTrue: [' - identical']
			ifFalse: [' - modified']! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: ''!
fileInClass
	Cursor read showWhile:[
		self selectedClass fileIn.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 6/16/1998 17:14'!
fileInMessage
	
	self selectedMessageName ifNil: [^self].
	Cursor read showWhile: [
		self selectedClassOrMetaClass fileInMethod: self selectedMessageName.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 2/3/1999 18:46'!
fileInMessageCategories
	Cursor read showWhile:[
		self selectedClassOrMetaClass fileInCategory: self selectedMessageCategoryName.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 5/13/1998 12:50'!
fileInPackage
	Cursor read showWhile:[
		self selectedPackage fileIn.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'ar 9/27/2005 20:07'!
fileIntoNewChangeSet
	| p ff |
	(p := self selectedPackage) ifNil: [^ Beeper beep].
	ff := FileStream readOnlyFileNamed: p fullPackageName.
	ChangeSet newChangesFromStream: ff named: p packageName! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: ''!
fileOutClass
	Cursor write showWhile:[
		self selectedClass fileOut.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 6/16/1998 17:14'!
fileOutMessage

	self selectedMessageName ifNil: [^self].
	Cursor write showWhile: [
		self selectedClassOrMetaClass fileOutMethod: self selectedMessageName].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 2/3/1999 18:46'!
fileOutMessageCategories
	Cursor write showWhile:[
		self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName.
	].! !

!FileContentsBrowser methodsFor: 'filein/fileout' stamp: 'wod 5/13/1998 14:19'!
fileOutPackage
	Cursor write showWhile:[
		self selectedPackage fileOut.
	].! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'DamienCassou 9/23/2009 08:37'!
findClass
	| pattern foundClass classNames index foundPackage |
	self okToChange ifFalse: [^ self classNotFound].
	pattern := (UIManager default request: 'Class Name?') asLowercase.
	pattern isEmptyOrNil ifTrue: [^ self].
	classNames := Set new.
	self packages do:[:p| classNames addAll: p classes keys].
	classNames := classNames asArray select: 
		[:n | (n asLowercase indexOfSubCollection: pattern startingAt: 1) > 0].
	classNames isEmpty ifTrue: [^ self].
	index := classNames size == 1
				ifTrue:	[1]
				ifFalse:	[(UIManager default chooseFrom: classNames lines: #())].
	index = 0 ifTrue: [^ self].
	foundPackage := nil.
	foundClass := nil.
	self packages do:[:p| 
		(p classes includesKey: (classNames at: index)) ifTrue:[
			foundClass := p classes at: (classNames at: index).
			foundPackage := p]].
	foundClass isNil ifTrue:[^self].
 	self systemCategoryListIndex: (self systemCategoryList indexOf: foundPackage packageName asSymbol).
	self classListIndex: (self classList indexOf: foundClass name). ! !

!FileContentsBrowser methodsFor: 'infoview' stamp: 'lr 3/14/2010 21:13'!
infoViewContents
	"Answer the string to show in the info view"

	| theClass stamp exists |
	editSelection == #newClass
		ifTrue: [ ^ self packageInfo: self selectedPackage ].
	self selectedClass isNil
		ifTrue: [ ^ '' ].
	theClass := Smalltalk globals at: self selectedClass name asSymbol ifAbsent: [  ].
	editSelection == #editClass
		ifTrue: [ 
			^ theClass notNil
				ifTrue: [ 'Class exists already in the system' translated ]
				ifFalse: [ 'New class' translated ] ].
	editSelection == #editMessage
		ifFalse: [ ^ '' ].
	(theClass notNil and: [ self metaClassIndicated ])
		ifTrue: [ theClass := theClass class ].
	stamp := self selectedClassOrMetaClass stampAt: self selectedMessageName.
	exists := theClass notNil and: [ theClass includesSelector: self selectedMessageName ].
	^ stamp = 'methodWasRemoved'
		ifTrue: [ 
			exists
				ifTrue: [ 'Existing method removed  by this change-set' translated ]
				ifFalse: [ 'Removal request for a method that is not present in this image' translated ] ]
		ifFalse: [ 
			stamp , '  '
				,
					(exists
						ifTrue: [ 'Method already exists' translated , self extraInfo ]
						ifFalse: [ 'New method' translated ]) ]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'sw 10/1/2001 11:16'!
labelString
	"Answer the string for the window title"

	^ 'File Contents Browser ', (self selectedSystemCategoryName ifNil: [''])! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'tbn 7/6/2010 16:23'!
messageCategoryMenu: aMenu

	^ aMenu 
		labels:
'FileIn
FileOut
Reorganize
Add item...
Rename...
Remove
Remove existing'
		lines: #(2 3 6)
		selections: #(fileInMessageCategories fileOutMessageCategories editMessageCategories addCategory renameCategory removeMessageCategory removeUnmodifiedMethods)! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 5/6/2000 18:50'!
messageListKey: aChar from: view
	aChar == $b ifTrue: [^ self browseMethodFull].
	super messageListKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'tbn 7/6/2010 16:24'!
messageListMenu: aMenu

	^ aMenu 
		labels:
'FileIn
FileOut
Senders (n)
Implementors (m)
Method inheritance (h)
Versions (v)
Remove'
		lines: #(2 6)
		selections: #(fileInMessage fileOutMessage
browseSenders browseImplementors methodHierarchy browseVersions
removeMessage).! !

!FileContentsBrowser methodsFor: 'diffs' stamp: 'lr 3/14/2010 21:13'!
methodDiffFor: aString class: aPseudoClass selector: selector meta: meta
	"Answer the diff between the current copy of the given class/selector/meta for the string provided"

	| theClass source |
	theClass := Smalltalk globals at: aPseudoClass name ifAbsent: [ ^ aString copy ].
	meta
		ifTrue: [ theClass := theClass class ].
	(theClass includesSelector: selector)
		ifFalse: [ ^ aString copy ].
	source := theClass sourceCodeAt: selector.
	^ Cursor wait
		showWhile: [ 
			TextDiffBuilder
				buildDisplayPatchFrom: source
				to: aString
				inClass: theClass
				prettyDiffs: self showingPrettyDiffs ]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'sma 2/6/2000 12:27'!
methodHierarchy
	(self selectedClassOrMetaClass isNil or:
		[self selectedClassOrMetaClass hasDefinition])
			ifFalse: [super methodHierarchy]! !

!FileContentsBrowser methodsFor: 'diffs' stamp: 'lr 3/14/2010 21:13'!
modifiedClassDefinition
	| pClass rClass old new diff |
	pClass := self selectedClassOrMetaClass.
	pClass hasDefinition
		ifFalse: [ ^ pClass definition ].
	rClass := Smalltalk globals at: self selectedClass name asSymbol ifAbsent: [ nil ].
	rClass isNil
		ifTrue: [ ^ pClass definition ].
	self metaClassIndicated
		ifTrue: [ rClass := rClass class ].
	old := rClass definition.
	new := pClass definition.
	Cursor wait showWhile: [ diff := ClassDiffBuilder buildDisplayPatchFrom: old to: new ].
	^ diff! !

!FileContentsBrowser methodsFor: 'creation' stamp: 'StephaneDucasse 3/12/2011 13:27'!
openAsMorph
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	| window aListExtent next mySingletonList |
	window := (SystemWindow labelled: 'later') model: self.
	self packages size = 1
		ifTrue: [
			aListExtent := 0.333333 @ 0.34.
			self systemCategoryListIndex: 1.
			mySingletonList := PluggableListMorph on: self list: #systemCategorySingleton
					selected: #indexIsOne changeSelected: #indexIsOne:
					menu: nil
					keystroke: #packageListKey:from:.
			mySingletonList hideScrollBarsIndefinitely.
			window addMorph: mySingletonList frame: (0@0 extent: 1.0@0.06).
			next := 0@0.06]
		ifFalse: [
			aListExtent := 0.25 @ 0.4.
			window addMorph: (PluggableListMorph on: self list: #systemCategoryList
					selected: #systemCategoryListIndex changeSelected: #systemCategoryListIndex:
					menu: nil
					keystroke: #packageListKey:from:)
				frame: (0@0 extent: aListExtent).
			next := aListExtent x @ 0].

	self addClassAndSwitchesTo: window at: (next extent: aListExtent) plus: 0.

	next := next + (aListExtent x @ 0).
	window addMorph: (PluggableListMorph on: self list: #messageCategoryList
			selected: #messageCategoryListIndex changeSelected: #messageCategoryListIndex:
			menu: #messageCategoryMenu:)
		frame: (next extent: aListExtent).
	next := next + (aListExtent x @ 0).
	window addMorph: (PluggableListMorph on: self list: #messageList
			selected: #messageListIndex changeSelected: #messageListIndex:
			menu: #messageListMenu:
			keystroke: #messageListKey:from:)
		frame: (next extent: aListExtent).

	self addLowerPanesTo: window at: (0@0.4 corner: 1@1) with: nil.
	^ window
! !

!FileContentsBrowser methodsFor: 'infoview' stamp: 'lr 3/14/2010 21:13'!
packageInfo: p
	| nClasses newClasses oldClasses |
	p isNil
		ifTrue: [ ^ '' ].
	nClasses := newClasses := oldClasses := 0.
	p classes
		do: [ :cls | 
			nClasses := nClasses + 1.
			(Smalltalk globals includesKey: cls name asSymbol)
				ifTrue: [ oldClasses := oldClasses + 1 ]
				ifFalse: [ newClasses := newClasses + 1 ] ].
	^ nClasses printString , ' classes (' , newClasses printString , ' new / ' , oldClasses printString , ' modified)'! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 2/6/2000 12:05'!
packageListKey: aChar from: view
	aChar == $f ifTrue: [^ self findClass].
	self arrowKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
packages
	^packages! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
packages: aDictionary
	packages := aDictionary.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'sd 11/20/2005 21:27'!
removeClass
	| class |
	classListIndex = 0 ifTrue: [^ self].
	class := self selectedClass.
	(self confirm:'Are you certain that you
want to delete the class ', class name, '?') ifFalse:[^self].
	self selectedPackage removeClass: class.
	self classListIndex: 0.
	self changed: #classList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'sd 11/20/2005 21:27'!
removeMessage
	| messageName |
	messageListIndex = 0
		ifTrue: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	(self selectedClass confirmRemovalOf: messageName)
		ifFalse: [^ false].
	self selectedClassOrMetaClass removeMethod: self selectedMessageName.
	self messageListIndex: 0.
	self setClassOrganizer.
	"In case organization not cached"
	self changed: #messageList! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'sd 11/20/2005 21:27'!
removeMessageCategory
	"If a message category is selected, create a Confirmer so the user can 
	verify that the currently selected message category should be removed
 	from the system. If so, remove it."

	| messageCategoryName |
	messageCategoryListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	messageCategoryName := self selectedMessageCategoryName.
	(self messageList size = 0
		or: [self confirm: 'Are you sure you want to
remove this method category 
and all its methods?']) ifFalse: [^ self].
	self selectedClassOrMetaClass removeCategory: messageCategoryName.
	self messageCategoryListIndex: 0.
	self changed: #messageCategoryList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'wod 5/24/1998 20:52'!
removePackage
	systemCategoryListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(self confirm: 'Are you sure you want to
remove this package 
and all its classes?') ifFalse:[^self].
	(systemOrganizer listAtCategoryNamed: self selectedSystemCategoryName) do:[:el|
		systemOrganizer removeElement: el].
	self packages removeKey: self selectedPackage packageName.
	systemOrganizer removeCategory: self selectedSystemCategoryName.
	self systemCategoryListIndex: 0.
	self changed: #systemCategoryList! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'sd 11/20/2005 21:27'!
removeUnmodifiedCategories
	| theClass |
	self okToChange ifFalse: [^self].
	theClass := self selectedClass.
	theClass isNil ifTrue: [^self].
	Cursor wait showWhile:
		[theClass removeUnmodifiedMethods: theClass selectors.
		theClass metaClass removeUnmodifiedMethods: theClass metaClass selectors].
	self messageCategoryListIndex: 0.
	self changed: #messageCategoryList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'wod 5/24/1998 20:37'!
removeUnmodifiedClasses
	| packageList |
	self okToChange ifFalse:[^self].
	packageList := self selectedPackage isNil
						ifTrue:[self packages] 
						ifFalse:[Array with: self selectedPackage].
	packageList do:[:package|
		package classes copy do:[:theClass|
			Cursor wait showWhile:[
				theClass removeAllUnmodified.
			].
			theClass hasChanges ifFalse:[
				package removeClass: theClass.
			].
		]].
	self classListIndex: 0.
	self changed: #classList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'wod 2/3/1999 18:47'!
removeUnmodifiedMethods
	| theClass cat |
	self okToChange ifFalse:[^self].
	theClass := self selectedClassOrMetaClass.
	theClass isNil ifTrue:[^self].
	cat := self selectedMessageCategoryName.
	cat isNil ifTrue:[^self].
	Cursor wait showWhile:[
		theClass removeUnmodifiedMethods: (theClass organization listAtCategoryNamed: cat).
	].
	self messageListIndex: 0.
	self changed: #messageList.! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'DamienCassou 9/29/2009 09:11'!
renameClass
	| oldName newName |
	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	oldName := self selectedClass name.
	newName := (self request: 'Please type new class name'
						initialAnswer: oldName) asSymbol.
	(newName isEmptyOrNil or:[newName = oldName]) ifTrue: [^ self].
	(self selectedPackage classes includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists in the package'].
	systemOrganizer classify: newName under: self selectedSystemCategoryName.
	systemOrganizer removeElement: oldName.
	self selectedPackage renameClass: self selectedClass to: newName.
	self changed: #classList.
	self classListIndex: ((systemOrganizer listAtCategoryNamed: self selectedSystemCategoryName) indexOf: newName).
! !

!FileContentsBrowser methodsFor: 'edit pane' stamp: 'md 2/13/2006 14:36'!
selectedBytecodes
	"Compile the source code for the selected message selector and extract and return
	the bytecode listing."
	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	contents := class sourceCodeAt: selector.
	contents := Compiler new
					parse: contents
					in: class
					notifying: nil.
	contents := contents generate.
	^ contents symbolic asText! !

!FileContentsBrowser methodsFor: 'class list' stamp: ''!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."

	self selectedClassName == nil ifTrue: [^nil].
	^self selectedPackage classAt: self selectedClassName! !

!FileContentsBrowser methodsFor: 'metaclass' stamp: 'sd 11/20/2005 21:27'!
selectedClassOrMetaClass
	"Answer the selected class or metaclass."

	| cls |
	self metaClassIndicated
		ifTrue: [^ (cls := self selectedClass) ifNotNil: [cls metaClass]]
		ifFalse: [^ self selectedClass]! !

!FileContentsBrowser methodsFor: 'edit pane' stamp: 'AlainPlantec 11/27/2009 09:24'!
selectedMessage
	"Answer a copy of the source code for the selected message selector."
 
	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	contents := class sourceCodeAt: selector.
	self browseWithPrettyPrint 
		ifTrue: 
			[contents := class prettyPrinterClass 
						format: contents
						in: class
						notifying: nil].
	self showingAnyKindOfDiffs 
		ifTrue: 
			[contents := self 
						methodDiffFor: contents
						class: self selectedClass
						selector: self selectedMessageName
						meta: self metaClassIndicated].
	^contents asText makeSelectorBoldIn: class! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
selectedPackage
	| cat |
	cat := self selectedSystemCategoryName.
	cat isNil ifTrue:[^nil].
	^self packages at: cat asString ifAbsent:[nil]! !

!FileContentsBrowser methodsFor: 'metaclass' stamp: 'sd 11/20/2005 21:27'!
setClassOrganizer
	"Install whatever organization is appropriate"
	| theClass |
	classOrganizer := nil.
	metaClassOrganizer := nil.
	classListIndex = 0 ifTrue: [^ self].
	classOrganizer := (theClass := self selectedClass) organization.
	metaClassOrganizer := theClass metaClass organization.
! !

!FileContentsBrowser methodsFor: 'infoview' stamp: 'alain.plantec 5/30/2008 13:04'!
updateInfoView

	self changed: #infoViewContents! !

Browser subclass: #HierarchyBrowser
	instanceVariableNames: 'classList centralClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

HierarchyBrowser subclass: #ClassListBrowser
	instanceVariableNames: 'defaultTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ClassListBrowser commentStamp: '<historical>' prior: 0!
A ClassListBrowser displays the code for an arbitrary list of classes.

ClassListBrowser example1.  "all classes that have the string 'Pluggable' in their names"
ClassListBrowser example2.  "all classes whose names start with the letter S"
ClassListBrowser example3.  "all variable classes"
ClassListBrowser example4.  "all classes with more than 100 methods"
ClassListBrowser example5.  "all classes that lack class comments"
ClassListBrowser example6.  "all classes that have class instance variables"

ClassListBrowser new initForClassesNamed: #(Browser Boolean) title: 'Browser and Boolean!!'.
!

!ClassListBrowser class methodsFor: 'instance creation' stamp: 'sd 4/17/2003 21:21'!
browseClassesSatisfying: classBlock title: aTitle
	"Put up a ClassListBrowser showing all classes that satisfy the classBlock."

	self new
		initForClassesNamed:
			(self systemNavigation allClasses select:
					[:c | (classBlock value: c) == true]
				thenCollect:
					[:c | c name])
		title:
			aTitle! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 16:01'!
example1
	"Put up a ClassListBrowser that shows all classes that have the string 'Pluggable' in their names"

	self browseClassesSatisfying: [:cl | cl name includesSubString: 'Pluggable'] title: 'Pluggables'

"ClassListBrowser example1"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sd 4/17/2003 21:21'!
example2
	"Put up a ClassListBrowser that shows all classes whose names start with 
	the letter S"

	self new
		initForClassesNamed: (self systemNavigation allClasses
				collect: [:c | c name]
				thenSelect: [:aName | aName first == $S])
		title: 'All classes starting with S'
	"ClassListBrowser example2"! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 16:03'!
example3
	"Put up a ClassListBrowser that shows all Variable classes"

	self browseClassesSatisfying:  [:c | c isVariable] title: 'All Variable classes'

"ClassListBrowser example3"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 16:04'!
example4
	"Put up a ClassListBrowser that shows all classes implementing more than 100 methods"

	self browseClassesSatisfying:
		[:c | (c selectors size + c class selectors size) > 100] title: 'Classes with more than 100 methods'

"ClassListBrowser example4"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 14:32'!
example5
	"Put up a ClassListBrowser that shows all classes that lack class comments"

	self
		browseClassesSatisfying: 
			[:c | c organization classComment isEmptyOrNil] 
		title: 'Classes lacking class comments'

"ClassListBrowser example5"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 14:33'!
example6
	"Put up a ClassListBrowser that shows all classes that have class instance variables"

	self
		browseClassesSatisfying: 
			[:c | c class instVarNames size > 0]
		title:
			'Classes that define class-side instance variables'

"ClassListBrowser example6"! !

!ClassListBrowser class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #classListBrowser
! !

!ClassListBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
initForClassesNamed: nameList title: aTitle
	"Initialize the receiver for the class-name-list and title provided"

	self systemOrganizer: SystemOrganization.
	metaClassIndicated := false.
	defaultTitle := aTitle.
	classList := nameList copy.
	self class openBrowserView:  (self openSystemCatEditString: nil)
		label: aTitle

	"ClassListBrowser new initForClassesNamed: #(Browser CategoryViewer) title: 'Frogs'"! !

!ClassListBrowser methodsFor: 'title' stamp: 'sw 7/18/2002 22:43'!
labelString
	"Answer the label strilng to use on the browser"

	^ defaultTitle ifNil: [super labelString]! !

!HierarchyBrowser class methodsFor: 'instance creation' stamp: 'IgorStasenko 3/6/2011 18:54'!
newFor: aClass
	"Open a new HierarchyBrowser on the given class"
	|  newBrowser |
	newBrowser := self new initHierarchyForClass: aClass.
	Smalltalk tools browser 
		openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: newBrowser labelString

"HierarchyBrowser newFor: Boolean"! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'sd 11/20/2005 21:27'!
assureSelectionsShow
	"This is a workaround for the fact that a hierarchy browser, when launched, often does not show the selected class"

	| saveCatIndex saveMsgIndex |
	saveCatIndex := messageCategoryListIndex.
	saveMsgIndex := messageListIndex.
	self classListIndex: classListIndex.
	self messageCategoryListIndex: saveCatIndex.
	self messageListIndex: saveMsgIndex! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'tk 4/7/98 13:53'!
buildClassBrowserEditString: aString 
	"Create and schedule a new class browser for the current selection, if one 
	exists, with initial textual contents set to aString."

	self spawnHierarchy! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'sw 5/8/2000 01:28'!
changed: sym
	sym == #classList ifTrue: [self updateAfterClassChange].
	super changed: sym! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'lr 3/14/2010 21:13'!
classList
	classList := classList select: [ :each | Smalltalk globals includesKey: each withBlanksTrimmed asSymbol ].
	^ classList! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
classListIndex: newIndex
	"Cause system organization to reflect appropriate category"
	| newClassName ind |
	newIndex ~= 0 ifTrue:
		[newClassName := (classList at: newIndex) copyWithout: $ .
		systemCategoryListIndex :=
			systemOrganizer numberOfCategoryOfElement: newClassName].
	ind := super classListIndex: newIndex.
	self changed: #systemCategorySingleton.
	^ ind! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'dew 9/15/2001 16:19'!
defaultBrowserTitle
	^ 'Hierarchy Browser'! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'nice 1/5/2010 15:59'!
initHierarchyForClass: aClassOrMetaClass
	| index nonMetaClass tab |
	centralClass := aClassOrMetaClass.
	nonMetaClass := aClassOrMetaClass theNonMetaClass.
	self systemOrganizer: SystemOrganization.
	metaClassIndicated := aClassOrMetaClass isMeta.
	classList := OrderedCollection new.
	tab := ''.
	nonMetaClass allSuperclasses reverseDo: 
		[:aClass | 
		classList add: tab , aClass name.
		tab := tab , '  '].
	index := classList size + 1.
	nonMetaClass allSubclassesWithLevelDo:
		[:aClass :level | | stab |
		stab := ''.  1 to: level do: [:i | stab := stab , '  '].
		classList add: tab , stab , aClass name]
	 	startingLevel: 0.
	self classListIndex: index! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'tk 4/5/98 10:29'!
openEditString: aString
	"Create a pluggable version of all the views for a HierarchyBrowser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."

	^ self openSystemCatEditString: aString! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'tk 4/3/98 11:22'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	self inform: 'Use a normal Browser, in which you can see 
the entire category you are trying to remove.'! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
selectClass: classNotMeta
	| name |
	name := classNotMeta name.
	self classListIndex: (self classList findFirst:
			[:each | (each endsWith: name)
					and: [each size = name size
							or: [(each at: each size - name size) isSeparator]]])! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
selectedClassName
	"Answer the name of the class currently selected.   di
	  bug fix for the case where name cannot be found -- return nil rather than halt"

	| aName |
	aName := super selectedClassName.
	^ aName == nil
		ifTrue:
			[aName]
		ifFalse:
			[(aName copyWithout: $ ) asSymbol]! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'sw 11/8/1999 13:35'!
systemCatSingletonKey: aChar from: aView
	^ self systemCatListKey: aChar from: aView! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'tbn 7/6/2010 16:25'!
systemCatSingletonMenu: aMenu

	^ aMenu labels:
'Find class... (f)
Browse
FileOut
Update
Rename...
Remove' 
	lines: #(1 4)
	selections:
		#(findClass buildSystemCategoryBrowser
		 fileOutSystemCategory updateSystemCategories
		 renameSystemCategory removeSystemCategory )
! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
systemCategorySingleton

	| cls |
	cls := self selectedClass.
	^ cls ifNil: [Array new]
		ifNotNil: [Array with: cls category]! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'rhi 12/2/2001 21:32'!
updateAfterClassChange
	"It is possible that some the classes comprising the hierarchy have changed, so reinitialize the entire browser."

	(centralClass notNil and: [centralClass isObsolete not])
		ifTrue: [self initHierarchyForClass: centralClass]! !

Browser subclass: #MessageSet
	instanceVariableNames: 'messageList autoSelectString growable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!MessageSet commentStamp: '<historical>' prior: 0!
I represent a query path of the retrieval result of making a query about methods in the system. The result is a set of methods, denoted by a message selector and the class in which the method was found. As a StringHolder, the string I represent is the source code of the currently selected method. I am typically viewed in a Message Set Browser consisting of a MessageListView and a BrowserCodeView.!

MessageSet subclass: #ChangedMessageSet
	instanceVariableNames: 'changeSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ChangedMessageSet commentStamp: '<historical>' prior: 0!
A ChangedMessageSet is a message set associated with a change-set; it bears an entry for every method added or changed in the change set, as well as for every class-comment of which the change-set bears a note.!

!ChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 1/7/2011 17:42'!
openFor: aChangeSet
	"Open up a ChangedMessageSet browser on the given change set; this is a conventional message-list browser whose message-list consists of all the methods in aChangeSet.  After any method submission, the message list is refigured, making it plausibly dynamic"

	| messageSet |

	messageSet := aChangeSet changedMessageList select: [ :each | each isValid].
	self 
		openMessageList: messageSet 
		name: 'Methods in Change Set ', aChangeSet name
		autoSelect: nil
		changeSet: aChangeSet! !

!ChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/30/2008 11:20'!
openMessageList: messageList name: labelString autoSelect: autoSelectString changeSet: aChangeSet
	| messageSet |

	messageSet := self messageList: messageList.
	messageSet changeSet: aChangeSet.
	messageSet autoSelectString: autoSelectString.
	self openAsMorph: messageSet name: labelString! !

!ChangedMessageSet class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #changedMessageSet
! !

!ChangedMessageSet methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
changeSet: aChangeSet
	changeSet := aChangeSet! !

!ChangedMessageSet methodsFor: 'acceptance' stamp: 'md 2/20/2006 18:42'!
contents: aString notifying: aController
	"Accept the string as new source for the current method, and make certain the annotation pane gets invalidated"

	| existingSelector existingClass superResult newSelector |
	existingSelector := self selectedMessageName.
	existingClass := self selectedClassOrMetaClass.

	superResult := super contents: aString notifying: aController.
	superResult ifTrue:  "succeeded"
		[newSelector := existingClass parserClass new parseSelector: aString.
		newSelector ~= existingSelector
			ifTrue:   "Selector changed -- maybe an addition"
				[self reformulateList.
				self changed: #messageList.
				self messageList doWithIndex:
					[:aMethodReference :anIndex |
						(aMethodReference actualClass == existingClass and:
									[aMethodReference methodSymbol == newSelector])
							ifTrue:
								[self messageListIndex: anIndex]]]].
	^ superResult! !

!ChangedMessageSet methodsFor: 'message list' stamp: 'sw 1/28/2001 20:59'!
growable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!ChangedMessageSet methodsFor: 'reformulation' stamp: 'StephaneDucasse 1/7/2011 17:42'!
reformulateList
	"Reformulate the message list of the receiver"

	self initializeMessageList: 
		(changeSet changedMessageList select: [:each | each isValid])
! !

MessageSet subclass: #MessageNames
	instanceVariableNames: 'searchString selectorList selectorListIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!MessageNames class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
methodBrowserSearchingFor: searchString
	"Answer an method-browser window whose search-string is initially as indicated"

	| aWindow |
	aWindow := self new inMorphicWindowWithInitialSearchString: searchString.
	aWindow applyModelExtent.
	^ aWindow! !

!MessageNames class methodsFor: 'instance creation' stamp: 'JuanVuletich 10/22/2010 09:50'!
openMessageNames
	"Open a new instance of the receiver in the active world"

	self new openAsMorphNamed: 'Message Names' inWorld: self currentWorld

	"MessageNames openMessageNames"
! !

!MessageNames class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:09'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color fromRgbTriplet: #(0.645 1.0 0.452) ! !

!MessageNames class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 

	registry register: self as: #messageNames
! !

!MessageNames methodsFor: 'search' stamp: 'LucFabresse 10/1/2010 16:08'!
computeSelectorListFromSearchString
	"Compute selector list from search string"
	
	self searchString: (searchString asString copyWithout: Character arrowUp).
	selectorList := Cursor wait showWhile: [
					(SystemNavigation default allImplementedMessages select: [:each | each  includesSubstring: searchString caseSensitive: false]) asArray sort: [:x :y | x asLowercase <= y asLowercase]].
	^ selectorList! !

!MessageNames methodsFor: 'message list menu' stamp: 'sd 11/20/2005 21:27'!
copyName
	"Copy the current selector to the clipboard"

	| selector |
	(selector := self selectorList at: selectorListIndex ifAbsent: [nil]) ifNotNil:
		[Clipboard clipboardText: selector asString asText]! !

!MessageNames methodsFor: 'search' stamp: 'GaryChambers 7/5/2010 13:28'!
doSearchFrom: aTextMorph
	"The user hit the Search button -- treat it as a synonym for the user having hit the Return or Enter (or cmd-s) in the type-in pane"

	aTextMorph
		accept;
		changed! !

!MessageNames methodsFor: 'initialization' stamp: 'sw 7/28/2001 02:16'!
inMorphicWindowLabeled: labelString
	"Answer a morphic window with the given label that can display the receiver"
"MessageNames openMessageNames"

	^ self inMorphicWindowWithInitialSearchString: nil! !

!MessageNames methodsFor: 'initialization' stamp: 'JuanVuletich 10/22/2010 09:13'!
inMorphicWindowWithInitialSearchString: initialString
	"Answer a morphic window with the given initial search string, nil if none"

"MessageNames openMessageNames"

	^UIManager default messageNames: self inMorphicWindowWithInitialSearchString: initialString! !

!MessageNames methodsFor: 'selector list' stamp: 'sd 11/20/2005 21:27'!
messageList
	"Answer the receiver's message list, computing it if necessary. The way 
	to force a recomputation is to set the messageList to nil"
	messageList
		ifNil: [messageList := selectorListIndex == 0
						ifTrue: [#()]
						ifFalse: [self systemNavigation
								allImplementorsOf: (selectorList at: selectorListIndex)].
			self
				messageListIndex: (messageList size > 0
						ifTrue: [1]
						ifFalse: [0])].
	^ messageList! !

!MessageNames methodsFor: 'search' stamp: 'GaryChambers 7/5/2010 12:57'!
searchString
	"Answer the current searchString, initializing it if need be"

	^ searchString ifNil: [searchString := '']! !

!MessageNames methodsFor: 'accessing' stamp: 'LucFabresse 10/1/2010 15:56'!
searchString: aStringToSearch
	searchString := aStringToSearch! !

!MessageNames methodsFor: 'search' stamp: 'JuanVuletich 10/22/2010 10:12'!
searchString: aString from: aTextMorph
	"Take what the user typed and find all selectors containing it"

	aTextMorph selectAll.
	searchString := aString asString copyWithout: $ .
	self containingWindow ifNotNil: [ :w | w setLabel: ('Message names containing "{1}"' translated format: {searchString asLowercase}) ].
	selectorList := nil.
	self changed: #selectorList.
	self changed: #messageList.
	^true! !

!MessageNames methodsFor: 'selection' stamp: 'MarcusDenker 8/15/2010 10:57'!
selectedMessageName
	^selectorListIndex = 0 ifFalse: [selectorList at: selectorListIndex]! !

!MessageNames methodsFor: 'selection' stamp: 'sw 7/24/2001 01:46'!
selection
	"Answer the item in the list that is currently selected, or nil if no selection is present"

	^ self messageList at: messageListIndex ifAbsent: [nil]! !

!MessageNames methodsFor: 'selector list' stamp: 'sd 11/20/2005 21:27'!
selectorList
	"Answer the selectorList"

	selectorList ifNil:
		[self computeSelectorListFromSearchString.
		selectorListIndex :=  selectorList size > 0
			ifTrue:	[1]
			ifFalse: [0].
		messageList := nil].
	^ selectorList! !

!MessageNames methodsFor: 'selector list' stamp: 'sw 7/24/2001 01:55'!
selectorListIndex
	"Answer the selectorListIndex"

	^ selectorListIndex! !

!MessageNames methodsFor: 'selector list' stamp: 'AlainPlantec 11/13/2009 22:59'!
selectorListIndex: anInteger 
	"Set the selectorListIndex as specified, and propagate consequences"

	selectorListIndex := anInteger.
	selectorListIndex = 0
		ifTrue: [^ self].
	messageList := nil.
	self changed: #selectorListIndex.
	self changed: #messageList.
	self changed: #annotation.! !

!MessageNames methodsFor: 'initialization' stamp: 'sw 7/24/2001 01:35'!
selectorListKey: aChar from: view
	"Respond to a Command key in the message-list pane."

	aChar == $n ifTrue: [^ self browseSenders].
	aChar == $c ifTrue: [^ self copyName].
	aChar == $b ifTrue: [^ self browseMethodFull].
! !

!MessageNames methodsFor: 'initialization' stamp: 'tbn 7/29/2010 21:56'!
selectorListMenu: aMenu
	"Answer the menu associated with the selectorList"

	aMenu addList: {
		{'Senders (n)' translated. #browseSenders. 'Browse senders of the chosen selector' translated}.
		{'Copy selector to clipboard' translated. #copyName. 'Copy the chosen selector to the clipboard, for subsequent pasting elsewhere' translated}.
		#-.
		{'Show only implemented selectors' translated. #showOnlyImplementedSelectors. 'Remove from the selector-list all symbols that do not represent implemented methods' translated}}.

	^ aMenu! !

!MessageNames methodsFor: 'selector list' stamp: 'sw 7/24/2001 01:47'!
selectorListMenuTitle
	"Answer the title to supply for the menu belonging to the selector-list pane"

	^ 'Click on any item in the list
to see all implementors of it'! !

!MessageNames methodsFor: 'search' stamp: 'sd 11/20/2005 21:27'!
showOnlyImplementedSelectors
	"Caution -- can be slow!! Filter my selector list down such that it only  
	shows selectors that are actually implemented somewhere in the system."
	self okToChange
		ifTrue: [Cursor wait
				showWhile: [selectorList := self systemNavigation allSelectorsWithAnyImplementorsIn: selectorList.
					self changed: #selectorList.
					self changed: #messageList]]! !

!MessageSet class methodsFor: 'utilities' stamp: 'sw 6/6/2001 15:09'!
isPseudoSelector: aSelector
	"Answer whether the given selector is a special marker"

	^ #(Comment Definition Hierarchy) includes: aSelector! !

!MessageSet class methodsFor: 'instance creation' stamp: ''!
messageList: anArray 
	"Answer an instance of me with message list anArray."

	^self new initializeMessageList: anArray! !

!MessageSet class methodsFor: 'instance creation' stamp: 'alain.plantec 5/30/2008 13:44'!
open: aMessageSet name: aString 
	"Create a standard system view for the messageSet, aMessageSet, whose label is aString."
	^ self openAsMorph: aMessageSet name: aString! !

!MessageSet class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 10:41'!
openAsMorph: aMessageSet name: labelString 
	"Create a SystemWindow aMessageSet, with the label labelString, in a Morphic project"
	^ self openAsMorph: aMessageSet name: labelString inWorld: self currentWorld! !

!MessageSet class methodsFor: 'instance creation' stamp: 'RAA 1/10/2001 11:07'!
openAsMorph: aMessageSet name: labelString inWorld: aWorld

	^aMessageSet openAsMorphNamed: labelString inWorld: aWorld
! !

!MessageSet class methodsFor: 'instance creation' stamp: ''!
openMessageList: anArray name: aString 
	"Create a standard system view for the message set on the list, anArray. 
	The label of the view is aString."

	self open: (self messageList: anArray) name: aString! !

!MessageSet class methodsFor: 'instance creation' stamp: 'alain.plantec 5/30/2008 13:45'!
openMessageList: messageList name: labelString autoSelect: autoSelectString
	"Open a system view for a MessageSet on messageList. 
	 1/24/96 sw: the there-are-no msg now supplied by my sender"

	| messageSet |
	messageSet := self messageList: messageList.
	messageSet autoSelectString: autoSelectString.
	^ self openAsMorph: messageSet name: labelString! !

!MessageSet class methodsFor: 'utilities' stamp: 'JohanBrichau 1/21/2011 11:11'!
parse: methodRef toClassAndSelector: csBlock
	"Decode strings of the form <className> [class] <selectorName>."

	| tuple cl |
	self flag: #mref.	"compatibility with pre-MethodReference lists"
	methodRef ifNil: [ ^ csBlock value: nil value: nil ].
	(methodRef isKindOf: SourceReference)
		ifTrue: [ ^ methodRef setClassAndSelectorIn: csBlock ].
	methodRef isEmpty
		ifTrue: [ ^ csBlock value: nil value: nil ].
	tuple := methodRef asString findTokens: ' .'.
	cl := Smalltalk globals at: tuple first asSymbol ifAbsent: [ ^ csBlock value: nil value: nil ].
	(tuple size = 2 or: [ tuple size > 2 and: [ (tuple at: 2) ~= 'class' ] ])
		ifTrue: [ ^ csBlock value: cl value: (tuple at: 2) asSymbol ]
		ifFalse: [ ^ csBlock value: cl class value: (tuple at: 3) asSymbol ]! !

!MessageSet class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:09'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightBlue
! !

!MessageSet class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 

	registry register: self as: #messageSet
! !

!MessageSet methodsFor: 'message list' stamp: 'tbn 7/6/2010 16:25'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built.  Add items specific to MessageSet"

	self growable ifTrue:
		[aMenu addList: #(
			-
			('Remove from this browser'		removeMessageFromBrowser)
			('Filter message list...'			filterMessageList)
			)].
	aMenu add: 'Sort by date' action: #sortByDate! !

!MessageSet methodsFor: 'message list' stamp: 'tk 5/1/2001 18:14'!
addItem: classAndMethod
	"Append a classAndMethod string to the list.  Select the new item."

	"Do some checks on the input?"
	self okToChange ifFalse: [^ self].
	messageList add: classAndMethod.
	self changed: #messageList.
	self messageListIndex: messageList size.! !

!MessageSet methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
adjustWindowTitleAfterFiltering
	"Set the title of the receiver's window, if any, to reflect the just-completed filtering"

	| aWindow existingLabel newLabel |

	(aWindow := self containingWindow) ifNil: [^ self].
	(existingLabel := aWindow label) isEmptyOrNil ifTrue: [^ self].
	(((existingLabel size < 3) or: [existingLabel last ~~ $]]) or: [(existingLabel at: (existingLabel size - 1)) isDigit not]) ifTrue: [^ self].
	existingLabel size to: 1 by: -1 do:
		[:anIndex | ((existingLabel at: anIndex) == $[) ifTrue:
			[newLabel := (existingLabel copyFrom: 1 to: anIndex),
				'Filtered: ',
				messageList size printString,
				']'.
			^ aWindow setLabel: newLabel]]
			

! !

!MessageSet methodsFor: 'private' stamp: ''!
autoSelectString
	"Return the string to be highlighted when making new selections"
	^ autoSelectString! !

!MessageSet methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
autoSelectString: aString
	"Set the string to be highlighted when making new selections"
	autoSelectString := aString! !

!MessageSet methodsFor: 'private' stamp: 'alain.plantec 5/15/2009 09:38'!
buildMorphicMessageList
	"Build my message-list object in morphic"

	| aListMorph |
	aListMorph := PluggableListMorph new.
	aListMorph
		on: self list: #messageList
		selected: #messageListIndex changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	aListMorph enableDragNDrop: true.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	^ aListMorph

! !

!MessageSet methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:10'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ false! !

!MessageSet methodsFor: 'metaclass' stamp: 'nk 4/29/2004 12:20'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."

	^ editSelection == #editComment or: [ self selectedMessageName == #Comment ]! !

!MessageSet methodsFor: 'contents' stamp: 'sd 11/20/2005 21:27'!
contents
	"Answer the contents of the receiver"

	^ contents == nil
		ifTrue: [currentCompiledMethod := nil. '']
		ifFalse: [messageListIndex = 0 
			ifTrue: [currentCompiledMethod := nil. contents]
			ifFalse: [self showingByteCodes
				ifTrue: [self selectedBytecodes]
				ifFalse: [self selectedMessage]]]! !

!MessageSet methodsFor: 'private' stamp: 'JB 2/12/2010 15:22'!
contents: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Answer false if the compilation fails. Otherwise, if the compilation 
	created a new method, deselect the current selection. Then answer true."

	| category selector class oldSelector |
	self okayToAccept ifFalse: [^ false].
	self setClassAndSelectorIn: [:c :os | class := c.  oldSelector := os].
	class ifNil: [^ false].
	(oldSelector ~~ nil and: [oldSelector first isUppercase]) ifTrue:
		[oldSelector = #Comment ifTrue:
			[class comment: aString stamp: Author changeStamp.
			self changed: #annotation.
 			self clearUserEditFlag.
			^ false].
		oldSelector = #Definition ifTrue:
			["self defineClass: aString notifying: aController."
			class subclassDefinerClass
				evaluate: aString
				notifying: aController
				logged: true.
			self clearUserEditFlag.
 			^ false].
		oldSelector = #Hierarchy ifTrue:
			[self inform: 'To change the hierarchy, edit the class definitions'. 
			^ false]].
	"Normal method accept"
	category := class organization categoryOfElement: oldSelector.
	selector := class compile: aString
				classified: category
				notifying: aController.
	selector == nil ifTrue: [^ false].
	self noteAcceptanceOfCodeFor: selector.
	selector == oldSelector ifFalse:
		[self reformulateListNoting: selector].
	contents := aString copy.
	self changed: #annotation.
	^ true! !

!MessageSet methodsFor: 'message functions' stamp: 'sd 11/20/2005 21:27'!
deleteFromMessageList: aMessage
	"Delete the given message from the receiver's message list"

	messageList := messageList copyWithout: aMessage! !

!MessageSet methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:27'!
dragPassengerFor: item inMorph: dragSource 
	| transferType |
	transferType := self dragTransferTypeForMorph: dragSource.
	transferType == #messageList
		ifTrue: [^self selectedClassOrMetaClass->(item contents findTokens: ' ') second asSymbol].
	transferType == #classList
		ifTrue: [^self selectedClass].
	^nil! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 11/20/2005 21:27'!
filterFrom: aBlock
	"Filter the receiver's list down to only those items that satisfy aBlock, which takes a class an a selector as its arguments."

	| newList |
	newList := messageList select:
		[:anElement |
			self class parse: anElement toClassAndSelector: [ :cls :sel | 
				(self class isPseudoSelector: sel) not and: [  aBlock value: cls value: sel ]]].
	self setFilteredList: newList! !

!MessageSet methodsFor: 'filtering' stamp: 'tbn 7/29/2010 22:10'!
filterMessageList
	"Allow the user to refine the list of messages."

	| aMenu evt |
	messageList size <= 1 ifTrue: [^ self inform: 'this is not a propitious filtering situation'].

	"would like to get the evt coming in but thwarted by the setInvokingView: circumlocution"
	evt := self currentWorld activeHand lastEvent.
	aMenu := MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Filter by only showing...'.
	aMenu addStayUpItem.

	aMenu addList: #(
		('Unsent messages'						filterToUnsentMessages		'Filter to show only messages that have no senders')
		-
		('Messages that send...'					filterToSendersOf			'Filter to show only messages that send a selector I specify')
		('Messages that do not send...'			filterToNotSendersOf		'Filter to show only messages that do not send a selector I specify')
		-
		('Messages whose selector is...'			filterToImplementorsOf		'Filter to show only messages with a given selector I specify')
		('Messages whose selector is NOT...'		filterToNotImplementorsOf	'Filter to show only messages whose selector is NOT a seletor I specify')
		-
		('Messages in current change set'		filterToCurrentChangeSet	'Filter to show only messages that are in the current change set')
		('Messages not in current change set'	filterToNotCurrentChangeSet	'Filter to show only messages that are not in the current change set')
		-
		('Messages in any change set'			filterToAnyChangeSet		'Filter to show only messages that occur in at least one change set')
		('Messages not in any change set'		filterToNotAnyChangeSet		'Filter to show only messages that do not occur in any change set in the system')
		-
		('Messages authored by me'				filterToCurrentAuthor		'Filter to show only messages whose authoring stamp has my initials')
		('Messages not authored by me'			filterToNotCurrentAuthor	'Filter to show only messages whose authoring stamp does not have my initials')
		-
		('Messages logged in .changes file'		filterToMessagesInChangesFile	'Filter to show only messages whose latest source code is logged in the .changes file')
		('Messages only in .sources file'			filterToMessagesInSourcesFile	'Filter to show only messages whose latest source code is logged in the .sources file')
		-
		('Messages with prior versions'			filterToMessagesWithPriorVersions	'Filter to show only messages that have at least one prior version')
		('Messages without prior versions'		filterToMessagesWithoutPriorVersions	'Filter to show only messages that have no prior versions')
		-
		('Uncommented messages' filterToUncommentedMethods 'Filter to show only messages that do not have comments at the beginning')
		('Commented messages' filterToCommentedMethods 'Filter to show only messages that have comments at the beginning')
		-
		('Messages that...'						filterToMessagesThat			'Let me type in a block taking a class and a selector, which will specify yea or nay concerning which elements should remain in the list')
			).

	aMenu popUpEvent: evt hand lastEvent in: evt hand world.! !

!MessageSet methodsFor: 'filtering' stamp: 'StephaneDucasse 3/31/2010 17:10'!
filterToAnyChangeSet
	"Filter down only to messages present in ANY change set"

	self filterFrom:
		[:aClass :aSelector |
			ChangeSet doesAnyChangeSetHaveClass: aClass andSelector: aSelector]
! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 10/20/2009 20:58'!
filterToCommentedMethods
	"Filter the receiver's list down to only those items which have comments"

	self filterFrom:
		[:aClass :aSelector |
			(aClass includesSelector: aSelector) and:
						[(aClass firstPrecodeCommentFor: aSelector) isEmptyOrNil not]]! !

!MessageSet methodsFor: 'filtering' stamp: 'MarcusDenker 2/20/2010 03:16'!
filterToCurrentAuthor
	"Filter down only to messages with my full name as most recent author"

	| myFullName |
	(myFullName := Author fullNamePerSe) ifNil: [^ self inform: 'No author full name set in this image'].
	self filterFrom:
		[:aClass :aSelector | | aMethod aTimeStamp |
			(aClass notNil and: [aSelector notNil]) and:			
				[aMethod := aClass compiledMethodAt: aSelector ifAbsent: [nil].
				aMethod notNil and:
					[(aTimeStamp := aMethod timeStamp) notNil and:
						[aTimeStamp beginsWith: myFullName]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 5/23/2003 14:38'!
filterToCurrentChangeSet
	"Filter the receiver's list down to only those items in the current change set"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(ChangeSet current atSelector: aSelector class: aClass) ~~ #none]]! !

!MessageSet methodsFor: 'filtering' stamp: 'rbb 3/1/2005 11:00'!
filterToImplementorsOf
	"Filter the receiver's list down to only those items with a given selector"

	| aFragment inputWithBlanksTrimmed |

	aFragment := UIManager default request: 'type selector:' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector |
					aSelector == aSymbol]]! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 1/5/2010 15:59'!
filterToMessagesInChangesFile
	"Filter down only to messages whose source code risides in the Changes file.  This allows one to ignore long-standing methods that live in the .sources file."

	
	self filterFrom:
		[:aClass :aSelector | | cm |
			aClass notNil and: [aSelector notNil and:
				[(self class isPseudoSelector: aSelector) not and:
					[(cm := aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil and:
					[cm fileIndex ~~ 1]]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 1/5/2010 15:59'!
filterToMessagesInSourcesFile
	"Filter down only to messages whose source code resides in the .sources file."

	
	self filterFrom: [:aClass :aSelector | | cm |
		(aClass notNil and: [aSelector notNil]) and:
			[(self class isPseudoSelector: aSelector) not and:
				[(cm := aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil and:
					[cm fileIndex == 1]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'rbb 3/1/2005 11:00'!
filterToMessagesThat
	"Allow the user to type in a block which will be"

	| reply |
	reply := UIManager default
		multiLineRequest: 'Type your block here'
		centerAt: Sensor cursorPoint
		initialAnswer: '[:aClass :aSelector |
	
	]'
		answerHeight: 200.
	reply isEmptyOrNil ifTrue: [^ self].
	self filterFrom: (Compiler evaluate: reply)
! !

!MessageSet methodsFor: 'filtering' stamp: 'sw 8/12/2001 22:25'!
filterToMessagesWithPriorVersions
	"Filter down only to messages which have at least one prior version"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(self class isPseudoSelector: aSelector) not and:
					[(VersionsBrowser versionCountForSelector: aSelector class: aClass) > 1]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sw 6/6/2001 15:12'!
filterToMessagesWithoutPriorVersions
	"Filter down only to messages which have no prior version stored"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(self class isPseudoSelector: aSelector) not and:
					[(VersionsBrowser versionCountForSelector: aSelector class: aClass) <= 1]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'StephaneDucasse 3/31/2010 17:10'!
filterToNotAnyChangeSet
	"Filter down only to messages present in NO change set"

	self filterFrom:
		[:aClass :aSelector |
			(ChangeSet doesAnyChangeSetHaveClass: aClass andSelector: aSelector) not]
! !

!MessageSet methodsFor: 'filtering' stamp: 'MarcusDenker 2/20/2010 03:16'!
filterToNotCurrentAuthor
	"Filter down only to messages not stamped with my initials"

	| myFullName |
	(myFullName := Author fullNamePerSe) ifNil: [^ self inform: 'No author full name set in this image'].
	self filterFrom:
		[:aClass :aSelector | | aMethod aTimeStamp |
			(aClass notNil and: [aSelector notNil]) and:			
				[aMethod := aClass compiledMethodAt: aSelector ifAbsent: [nil].
				aMethod notNil and:
					[(aTimeStamp := aMethod timeStamp) isNil or:
						[(aTimeStamp beginsWith: myFullName) not]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 5/23/2003 14:38'!
filterToNotCurrentChangeSet
	"Filter the receiver's list down to only those items not in the current change set"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(ChangeSet current atSelector: aSelector class: aClass) == #none]]! !

!MessageSet methodsFor: 'filtering' stamp: 'rbb 3/1/2005 11:00'!
filterToNotImplementorsOf
	"Filter the receiver's list down to only those items whose selector is NOT one solicited from the user."

	| aFragment inputWithBlanksTrimmed |

	aFragment := UIManager default request: 'type selector: ' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector |
					aSelector ~~ aSymbol]]! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 1/5/2010 15:59'!
filterToNotSendersOf
	"Filter the receiver's list down to only those items which do not send a given selector"
	| aFragment inputWithBlanksTrimmed |
	aFragment := UIManager default 
		request: 'type selector:'
		initialAnswer: ''.
	aFragment isEmptyOrNil ifTrue: [ ^ self ].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol 
		hasInterned: inputWithBlanksTrimmed
		ifTrue: 
			[ :aSymbol | 
			self filterFrom: 
				[ :aClass :aSelector | | aMethod | 
				(aMethod := aClass compiledMethodAt: aSelector) isNil or: [ (aMethod refersToLiteral: aSymbol) not ] ] ]! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 1/5/2010 15:59'!
filterToSendersOf
	"Filter the receiver's list down to only those items which send a given selector"
	| aFragment inputWithBlanksTrimmed |
	aFragment := UIManager default 
		request: 'type selector:'
		initialAnswer: ''.
	aFragment isEmptyOrNil ifTrue: [ ^ self ].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol 
		hasInterned: inputWithBlanksTrimmed
		ifTrue: 
			[ :aSymbol | 
			self filterFrom: 
				[ :aClass :aSelector | | aMethod | 
				(aMethod := aClass compiledMethodAt: aSelector) notNil and: [ aMethod refersToLiteral: aSymbol ] ] ]! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 10/20/2009 20:59'!
filterToUncommentedMethods
	"Filter the receiver's list down to only those items which lack comments"

	self filterFrom:
		[:aClass :aSelector |
			(aClass includesSelector: aSelector) and:
						[(aClass firstPrecodeCommentFor: aSelector) isEmptyOrNil]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 4/29/2003 12:24'!
filterToUnsentMessages
	"Filter the receiver's list down to only those items which have no  
	senders"
	self
		filterFrom: [:aClass :aSelector | (self systemNavigation allCallsOn: aSelector) isEmpty]! !

!MessageSet methodsFor: 'message list' stamp: 'sw 1/28/2001 20:56'!
growable
	"Answer whether the receiver is capable of growing/shrinking dynamically"

	^ growable ~~ false! !

!MessageSet methodsFor: 'message list' stamp: 'sd 11/20/2005 21:27'!
growable: aBoolean
	"Give or take away the growable trait; when a message set is growable, methods submitted within it will be added to its message list"

	growable := aBoolean! !

!MessageSet methodsFor: 'private' stamp: 'JuanVuletich 10/18/2010 09:50'!
inMorphicWindowLabeled: labelString
	"Answer a morphic window with the given label that can display the receiver"

	^UIManager default messageSet: self inMorphicWindowLabeled: labelString! !

!MessageSet methodsFor: 'private' stamp: 'al 9/21/2008 20:11'!
initialExtent
	^ 500@500! !

!MessageSet methodsFor: 'private' stamp: 'MarcusDenker 4/28/2011 15:42'!
initializeMessageList: anArray
	"Initialize my messageList from the given list of MethodReference or string objects."

	
	messageList := anArray asOrderedCollection.
	messageListIndex := messageList isEmpty ifTrue: [0] ifFalse: [1].
	contents := ''! !

!MessageSet methodsFor: 'message list' stamp: ''!
messageList
	"Answer the current list of messages."

	^messageList! !

!MessageSet methodsFor: 'message list' stamp: 'AlainPlantec 11/13/2009 22:53'!
messageListIndex: anInteger 
	"Set the index of the selected item to be anInteger."

	messageListIndex := anInteger.
	contents := 
		messageListIndex ~= 0
			ifTrue: [self selectedMessage]
			ifFalse: [''].
	self changed: #messageListIndex.	 "update my selection"
	self editSelection: #editMessage.
	self changed: #annotation.
	self contentsChanged.
	(messageListIndex ~= 0 and: [autoSelectString notNil])
		ifTrue: [self changed: #autoSelect].
	self decorateButtons
! !

!MessageSet methodsFor: 'class list' stamp: ''!
metaClassIndicated
	"Answer the boolean flag that indicates whether
	this is a class method."

	^ self selectedClassOrMetaClass isMeta! !

!MessageSet methodsFor: 'message functions' stamp: 'sw 2/24/1999 18:31'!
methodCategoryChanged
	self changed: #annotation! !

!MessageSet methodsFor: 'private' stamp: 'yo 7/30/2004 16:36'!
openAsMorphNamed: labelString inWorld: aWorld
	"Open the receiver in a morphic window in the given world"

	(self inMorphicWindowLabeled: labelString) openInWorld: aWorld.
	self messageListIndex: 1.
! !

!MessageSet methodsFor: 'message functions' stamp: 'sw 12/1/2000 11:54'!
reformulateList
	"The receiver's messageList has been changed; rebuild it"

	super reformulateList.
	self initializeMessageList: messageList.
	self changed: #messageList.
	self changed: #messageListIndex.
	self contentsChanged
! !

!MessageSet methodsFor: 'message functions' stamp: 'sd 11/20/2005 21:27'!
removeMessage
	"Remove the selected message from the system. 1/15/96 sw"
	| messageName confirmation |
	messageListIndex = 0
		ifTrue: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	confirmation := self systemNavigation  confirmRemovalOf: messageName on: self selectedClassOrMetaClass.
	confirmation == 3
		ifTrue: [^ self].
	self selectedClassOrMetaClass removeSelector: messageName.
	self deleteFromMessageList: self selection.
	self reformulateList.
	confirmation == 2
		ifTrue: [self systemNavigation browseAllCallsOn: messageName]! !

!MessageSet methodsFor: 'message functions' stamp: 'sw 1/12/2001 00:19'!
removeMessageFromBrowser
	"Remove the selected message from the browser."

	messageListIndex = 0 ifTrue: [^ self].
	self deleteFromMessageList: self selection.
	self reformulateList.
	self adjustWindowTitleAfterFiltering
! !

!MessageSet methodsFor: 'class list' stamp: 'sd 11/20/2005 21:27'!
selectedClass 
	"Return the base class for the current selection.  1/17/96 sw fixed up so that it doesn't fall into a debugger in a msg browser that has no message selected"

	| aClass |
	^ (aClass := self selectedClassOrMetaClass) == nil
		ifTrue:
			[nil]
		ifFalse:
			[aClass theNonMetaClass]! !

!MessageSet methodsFor: 'class list' stamp: 'sd 11/20/2005 21:27'!
selectedClassName
	"Answer the name of class of the currently selected message. Answer nil if no selection 
	exists."

	| cls |
	(cls := self selectedClass) ifNil: [^ nil].
	^ cls name! !

!MessageSet methodsFor: 'class list' stamp: ''!
selectedClassOrMetaClass
	"Answer the currently selected class (or metaclass)."
	messageListIndex = 0 ifTrue: [^nil].
	self setClassAndSelectorIn: [:c :s | ^c]! !

!MessageSet methodsFor: 'contents' stamp: 'nice 1/5/2010 15:59'!
selectedMessage
	"Answer the source method for the currently selected message."

	
	self setClassAndSelectorIn: [:class :selector | | source | 
		class ifNil: [^ 'Class vanished'].
		selector first isUppercase ifTrue:
			[selector == #Comment ifTrue:
				[currentCompiledMethod := class organization commentRemoteStr.
				^ class comment].
			selector == #Definition ifTrue:
				[^ class definition].
			selector == #Hierarchy ifTrue: [^ class printHierarchy]].
		source := class sourceMethodAt: selector ifAbsent:
			[currentCompiledMethod := nil.
			^ 'Missing'].

		self showingDecompile ifTrue: [^ self decompiledSourceIntoContents].

		currentCompiledMethod := class compiledMethodAt: selector ifAbsent: [nil].
		self showingDocumentation ifTrue: [^ self commentContents].

	source := self sourceStringPrettifiedAndDiffed.
	^ source asText makeSelectorBoldIn: class]! !

!MessageSet methodsFor: 'class list' stamp: ''!
selectedMessageCategoryName 
	"Answer the name of the selected message category or nil."
	messageListIndex = 0 ifTrue: [^ nil].
	^ self selectedClassOrMetaClass organization categoryOfElement: self selectedMessageName! !

!MessageSet methodsFor: 'message list' stamp: 'sma 3/3/2000 11:17'!
selectedMessageName
	"Answer the name of the currently selected message."
	"wod 6/16/1998: answer nil if none are selected."

	messageListIndex = 0 ifTrue: [^ nil].
	^ self setClassAndSelectorIn: [:class :selector | ^ selector]! !

!MessageSet methodsFor: 'private' stamp: 'sw 12/28/2000 14:28'!
selection
	"Answer the item in the list that is currently selected, or nil if no selection is present"

	^ messageList at: messageListIndex ifAbsent: [nil]! !

!MessageSet methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/6/2011 16:09'!
setClassAndSelectorIn: csBlock
	| sel |
	"Decode strings of the form <className> [class] <selectorName>."

	sel := self selection.
	"MethodReference should be remove"
	^((sel isKindOf: MethodReference) or:[sel isKindOf: SourcedMethodReference]) ifTrue: [
		sel setClassAndSelectorIn: csBlock
	] ifFalse: [
		self class parse: sel toClassAndSelector: csBlock
	]! !

!MessageSet methodsFor: 'contents' stamp: 'sd 11/20/2005 21:27'!
setContentsToForceRefetch
	"Set the receiver's contents such that on the next update the contents will be formulated afresh.  This is a critical and obscure difference between Browsers on the one hand and MessageSets on the other, and has over the years been the source of much confusion and much difficulty.  By centralizing the different handling here, we don't need so many idiosyncratic overrides in MessageSet any more"

	contents := ''! !

!MessageSet methodsFor: 'private' stamp: 'sw 1/11/2001 09:18'!
setFilteredList: newList
	"Establish newList as the new list if appropriate, and adjust the window title accordingly; if the new list is of the same size as the old, warn and do nothing"

	newList size == 0
		ifTrue:
			[^ self inform: 'Nothing would be left in the list if you did that'].
	newList size == messageList size
		ifTrue:
			[^ self inform: 'That leaves the list unchanged'].
	self initializeMessageList: newList.
	self adjustWindowTitleAfterFiltering! !

!MessageSet methodsFor: 'message list' stamp: 'nice 1/5/2010 15:59'!
sortByDate
	"Sort the message-list by date of time-stamp"

	| assocs inOrder |
	assocs := messageList collect:
		[:aRef | | aCompiledMethod aDate |
			aDate := aRef methodSymbol == #Comment
				ifTrue:
					[aRef actualClass organization dateCommentLastSubmitted]
				ifFalse:
					[aCompiledMethod := aRef actualClass compiledMethodAt: aRef methodSymbol ifAbsent: [nil].
					aCompiledMethod ifNotNil: [aCompiledMethod dateMethodLastSubmitted]].
			aRef -> (aDate ifNil: [Date fromString: '01/01/1996'])].  "The dawn of Squeak history"
	inOrder := assocs asSortedCollection:
		[:a :b | a value < b value].

	messageList := inOrder asArray collect: [:assoc | assoc key].
	self changed: #messageList! !

MessageSet subclass: #ProtocolBrowser
	instanceVariableNames: 'selectedClass selectedSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ProtocolBrowser commentStamp: '<historical>' prior: 0!
An instance of ProtocolBrowser shows the methods a class understands--inherited or implemented at this level--as a "flattened" list.!

!ProtocolBrowser class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 5/6/2011 16:10'!
openFullProtocolForClass: aClass 
	"Create and schedule a browser for the entire protocol of the class."
	"ProtocolBrowser openFullProtocolForClass: ProtocolBrowser."
	| aPBrowser label |
	aPBrowser := self new on: aClass.
	label := 'Entire protocol of: ', aClass name.
	self open: aPBrowser name: label! !

!ProtocolBrowser class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 5/6/2011 16:10'!
openSubProtocolForClass: aClass 
	"Create and schedule a browser for the entire protocol of the class."
	"ProtocolBrowser openSubProtocolForClass: ProtocolBrowser."
	| aPBrowser label |
	aPBrowser := self new onSubProtocolOf: aClass.
	label := 'Sub-protocol of: ', aClass name.
	self open: aPBrowser name: label! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:33'!
getList
	"Answer the receiver's message list."
	^ messageList! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'sw 1/28/2001 21:01'!
growable
	"Answer whether the receiver is subject to manual additions and deletions"

	^ false! !

!ProtocolBrowser methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/6/2010 22:27'!
initListFrom: selectorCollection highlighting: aClass 
	"Make up the messageList with items from aClass in boldface."
	

	messageList := OrderedCollection new.
	selectorCollection do: [ :selector | | defClass item |  
		defClass := aClass whichClassIncludesSelector: selector.
		item := selector, '     (' , defClass name , ')'.
		defClass == aClass ifTrue: [item := item asText allBold].
		messageList add: (
			SourcedMethodReference new
				setClass: defClass 
				methodSymbol: selector 
				stringVersion: item
		)
	].
	selectedClass := aClass.! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:33'!
list
	"Answer the receiver's message list."
	^ messageList! !

!ProtocolBrowser methodsFor: 'private' stamp: 'di 7/13/97 16:26'!
on: aClass 
	"Initialize with the entire protocol for the class, aClass."
	self initListFrom: aClass allSelectors asSortedCollection
		highlighting: aClass! !

!ProtocolBrowser methodsFor: 'private' stamp: 'di 11/26/1999 19:39'!
onSubProtocolOf: aClass 
	"Initialize with the entire protocol for the class, aClass,
		but excluding those inherited from Object."
	| selectors |
	selectors := Set new.
	aClass withAllSuperclasses do:
		[:each | (each == Object or: [each == ProtoObject]) 
			ifFalse: [selectors addAll: each selectors]].
	self initListFrom: selectors asSortedCollection
		highlighting: aClass! !

!ProtocolBrowser methodsFor: 'class list' stamp: 'nk 4/10/2001 08:16'!
selectedClassOrMetaClass
	^selectedClass! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
selector
	"Answer the receiver's selected selector."
	^ selectedSelector! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
selector: aString
	"Set the currently selected message selector to be aString."
	selectedSelector := aString.
	self changed: #selector! !

!ProtocolBrowser methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/6/2010 22:39'!
setClassAndSelectorIn: csBlock
	"Decode strings of the form    <selectorName> (<className> [class])"

	| i classAndSelString selString sel |

	sel := self selection ifNil: [^ csBlock value: nil value: nil].
	"MethodReference should be remove"
	((sel isKindOf: MethodReference) or: [sel isKindOf: SourcedMethodReference]) ifTrue: [
		sel setClassAndSelectorIn: csBlock
	] ifFalse: [
		selString := sel asString.
		i := selString indexOf: $(.
		"Rearrange to  <className> [class] <selectorName> , and use MessageSet"
		classAndSelString := (selString copyFrom: i + 1 to: selString size - 1) , ' ' ,
							(selString copyFrom: 1 to: i - 1) withoutTrailingBlanks.
		MessageSet parse: classAndSelString toClassAndSelector: csBlock.
	].
! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
setSelector: aString
	"Set the currently selected message selector to be aString."
	selectedSelector := aString! !

MessageSet subclass: #RecentMessageSet
	instanceVariableNames: ''
	classVariableNames: 'NumberOfRecentSubmissionsToStore RecentSubmissions'
	poolDictionaries: ''
	category: 'Tools-Browser'!

!RecentMessageSet commentStamp: 'StephaneDucasse 1/12/2011 17:25' prior: 0!
RecentMessageSet is a message set that shows the most recently-submitted methods, in chronological order.

DEPRECATED use the new RecentMessageList browser.!

!RecentMessageSet class methodsFor: 'utilities' stamp: 'JB 2/19/2010 16:40'!
assureMostRecentSubmissionExists
	"Make certain that the most recent submission exists"

	[RecentSubmissions size > 0 and:
		[RecentSubmissions last isValid not]] whileTrue:
			[RecentSubmissions removeLast].! !

!RecentMessageSet class methodsFor: 'public' stamp: 'StephaneDucasse 2/20/2010 23:36'!
browseRecentSubmissions
	"Open up a browser on the most recent methods submitted in the image.  5/96 sw."
	"RecentMessageSet browseRecentSubmissions"

	| recentMessages |
	self recentMethodSubmissions size isZero ifTrue:
		[^ self inform: 'There are no recent submissions'].
	recentMessages := RecentSubmissions copy reversed.
	self 
		openMessageList: recentMessages 
		name: 'Recent submissions -- youngest first ' 
		autoSelect: nil! !

!RecentMessageSet class methodsFor: 'queries' stamp: 'BenjaminVanRyseghem 1/8/2011 15:37'!
event: anEvent
	"Hook for SystemChangeNotifier"

	^self! !

!RecentMessageSet class methodsFor: 'initialization' stamp: 'StephaneDucasse 2/20/2010 23:39'!
initialize
	
	RecentSubmissions := OrderedCollection new.
	! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'AlainPlantec 4/11/2010 12:45'!
menuCommandOn: aBuilder
		<worldMenu> 
		(aBuilder item: #'Recent Submissions')
			action: [self browseRecentSubmissions];
			parent: #Tools;
			help: 'Browse your last methods submissions';
			icon: self taskbarIcon 
! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'StephaneDucasse 2/28/2010 11:48'!
mostRecentlySubmittedMessage
	"Answer the most recently submitted method that is still extant"

	self assureMostRecentSubmissionExists.
	(RecentSubmissions isEmpty) ifTrue: [^ nil]
	ifFalse: [^ RecentSubmissions last].! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'StephaneDucasse 1/7/2011 19:07'!
noteMethodSubmission: selectorName forClass: class

	| submission |
	self recentMethodSubmissions.	"ensure it is valid"
	class wantsChangeSetLogging ifFalse: [^ self].
	self purgeRecentSubmissionsOfMissingMethods.
	submission := class name asString, ' ', selectorName.
	RecentSubmissions removeAllSuchThat: [ :each | each stringVersion = submission ].
	RecentSubmissions size >= self numberOfRecentSubmissionsToStore 
		ifTrue: [RecentSubmissions removeFirst].
	RecentSubmissions addLast: 
		(SourcedMethodReference new
			setClass: class 
			methodSymbol: selectorName 
			stringVersion: submission) 
! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'StephaneDucasse 2/20/2010 23:38'!
numberOfRecentSubmissionsToStore
	"Answer how many methods back the 'recent method submissions' history should store"

	^ NumberOfRecentSubmissionsToStore ifNil: [NumberOfRecentSubmissionsToStore := 150]! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'JB 2/19/2010 16:40'!
numberOfRecentSubmissionsToStore: aNumber
	"Set the number of Recent Submissions to store"

	NumberOfRecentSubmissionsToStore := aNumber! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'JB 2/19/2010 16:55'!
purgeFromRecentSubmissions: aMethodReference
	"Purge any reference found in RecentSubmissions to the method supplied"

	RecentSubmissions := RecentSubmissions select:
		[:aSubmission |
			self setClassAndSelectorFrom: aSubmission in:
				[:aClass :aSelector | (aClass ~~ aMethodReference actualClass) or: [aSelector ~~ aMethodReference methodSymbol]]]! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'MarcusDenker 3/21/2010 20:16'!
purgeRecentSubmissionsOfMissingMethods
	"RecentMessageSet purgeRecentSubmissionsOfMissingMethods"
	
	RecentSubmissions := RecentSubmissions select:
		[:aSubmission | | keep | 
			self setClassAndSelectorFrom: aSubmission in:
				[:aClass :aSelector |
					keep := aClass notNil
						and: [aSelector == #Comment or: [(aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil]]].
			keep]! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'JB 2/19/2010 16:39'!
recentMethodSubmissions
	"Answer the list of recent method submissions, in order.  5/16/96 sw"

	RecentSubmissions ifNil: [RecentSubmissions := OrderedCollection new].
	^ RecentSubmissions! !

!RecentMessageSet class methodsFor: 'public' stamp: 'StephaneDucasse 2/20/2010 23:36'!
recentSubmissionsWindow
	"Answer a SystemWindow holding recent submissions"
	"RecentMessageSet recentSubmissionsWindow openInHand"
	
	| recentMessages messageSet |
	recentMessages := RecentSubmissions copy reversed.
	messageSet := self messageList: recentMessages.
	messageSet autoSelectString: nil.
	^ (messageSet inMorphicWindowLabeled: 'Recent submissions -- youngest first') applyModelExtent

	

! !

!RecentMessageSet class methodsFor: 'instance creation' stamp: 'JB 2/19/2010 16:56'!
revertLastMethodSubmission
	| changeRecords lastSubmission theClass theSelector |
	"If the most recent method submission was a method change, revert
	that change, and if it was a submission of a brand-new method, 
	remove that method."

	RecentSubmissions isEmptyOrNil ifTrue: [^ Beeper beep].
	lastSubmission := RecentSubmissions last.
	theClass := lastSubmission actualClass ifNil: [^ Beeper beep].
	theSelector := lastSubmission methodSymbol.
	changeRecords := theClass changeRecordsAt: theSelector.
	changeRecords isEmptyOrNil ifTrue: [^ Beeper beep].
	changeRecords size == 1
		ifTrue:
			["method has no prior version, so reverting in this case means removing"
			theClass removeSelector: theSelector]
		ifFalse:
			[changeRecords second fileIn].

"RecentMessageSet revertLastMethodSubmission"! !

!RecentMessageSet class methodsFor: 'utilities' stamp: 'lr 3/14/2010 21:13'!
setClassAndSelectorFrom: messageIDString in: csBlock
	"Decode strings of the form <className> [class] <selectorName>.   If <className> does not exist as a class, use nil for the class in the block"

	| aStream aClass maybeClass sel |
	(messageIDString isKindOf: MethodReference)
		ifTrue: [ ^ messageIDString setClassAndSelectorIn: csBlock ].
	aStream := messageIDString readStream.
	aClass := Smalltalk globals at: (aStream upTo: $ ) asSymbol ifAbsent: [ nil ].
	maybeClass := aStream upTo: $ .
	sel := aStream upTo: $ .
	maybeClass = 'class' & (sel size ~= 0)
		ifTrue: [ aClass ifNil: [ csBlock value: nil value: sel asSymbol ] ifNotNil: [ csBlock value: aClass class value: sel asSymbol ] ]
		ifFalse: [ 
			csBlock value: aClass value: maybeClass asSymbol	"
Utilities setClassAndSelectorFrom: 'Utilities class oppositeModeTo:' in: [:aClass :aSelector | Transcript cr; show: 'Class = ', aClass name printString, ' selector = ', aSelector printString].

Utilities setClassAndSelectorFrom: 'MessageSet setClassAndSelectorIn:' in: [:aClass :aSelector | Transcript cr; show: 'Class = ', aClass name printString, ' selector = ', aSelector printString].
" ]! !

!RecentMessageSet class methodsFor: 'initialization' stamp: 'JB 2/19/2010 16:42'!
startUp
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:.! !

!RecentMessageSet methodsFor: 'message functions' stamp: 'StephaneDucasse 2/20/2010 23:27'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built.  Overridden here to defeat the presence of the items that add or change order, since RecentMessageSet defines methods & order explicitly based on external criteria"

	aMenu add: 'Set size of recent history...' action: #setRecentHistorySize! !

!RecentMessageSet methodsFor: 'contents' stamp: 'sd 11/20/2005 21:27'!
contents: c notifying: n
	| result |
	result := super contents: c notifying: n.
	result == true ifTrue:
		[self reformulateList].
	^ result! !

!RecentMessageSet methodsFor: 'update' stamp: 'sw 1/28/2001 20:59'!
growable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!RecentMessageSet methodsFor: 'message functions' stamp: 'tbn 7/6/2010 16:28'!
messageListMenu: aMenu shifted: shifted
	"Answer the message-list menu"

	shifted ifTrue: [^ self shiftedMessageListMenu: aMenu].
	aMenu addList:#(
			('What to show...'						offerWhatToShowMenu)
			-
			('Browse full (b)' 						browseMethodFull)
			('Browse hierarchy (h)'					classHierarchy)
			('Browse method (O)'					openSingleMessageBrowser)
			('Browse protocol (p)'					browseFullProtocol)
			-
			('FileOut (o)'							fileOutMessage)
			('Copy selector (c)'						copySelector)
			-
			('Senders of... (n)'						browseSendersOfMessages)
			('Implementors of... (m)'					browseMessages)
			('Inheritance (i)'						methodHierarchy)
			('Versions (v)'							browseVersions)
			-
			('Inst var refs...'						browseInstVarRefs)
			('Inst var defs...'						browseInstVarDefs)
			('Class var refs...'						browseClassVarRefs)
			('Class variables'						browseClassVariables)
			('Class refs (N)'							browseClassRefs)
			-
			('Remove method (x)'					removeMessage)
			('Remove from RecentSubmissions'		removeFromRecentSubmissions)
			-
			('More...'								shiftedYellowButtonActivity)).
	^ aMenu! !

!RecentMessageSet methodsFor: 'update' stamp: 'StephaneDucasse 2/20/2010 23:34'!
reformulateList
	| myList |
	"Reformulate the receiver's list.  Exclude methods now deleted"
	myList := self class recentMethodSubmissions reversed select: [ :each | each isValid].
	self initializeMessageList: myList.
	self messageListIndex: (messageList size min: messageListIndex).	
	self changed: #messageList.
	self changed: #messageListIndex! !

!RecentMessageSet methodsFor: 'message functions' stamp: 'BenjaminVanRyseghem 11/6/2010 22:27'!
removeFromRecentSubmissions
	"Remove the currently-selected method from the RecentSubmissions list"

	| aClass methodSym |
	((aClass := self selectedClassOrMetaClass) notNil and: [(methodSym := self selectedMessageName) notNil])
		ifTrue: 
			[self class purgeFromRecentSubmissions: (SourcedMethodReference new setClass: aClass methodSymbol: methodSym).
			self reformulateList]! !

!RecentMessageSet methodsFor: 'message list' stamp: 'JB 2/19/2010 18:09'!
setRecentHistorySize
	"Let the user specify the recent history size"

	| aReply aNumber |
	aReply := UIManager default request: 'How many recent methods
should be maintained?' initialAnswer: self class numberOfRecentSubmissionsToStore asString.
	aReply isEmptyOrNil ifFalse:
		[aNumber := aReply asNumber rounded.
		(aNumber > 1 and: [aNumber <= 1000])
			ifTrue:
				[self class numberOfRecentSubmissionsToStore: aNumber.
				self inform: 'Okay, ', aNumber asString, ' is the new size of the recent method history']
			ifFalse:
				[self inform: 'Sorry, must be a number between 2 & 1000']]
			! !

!RecentMessageSet methodsFor: 'update' stamp: 'pavel.krivanek 2/24/2010 14:05'!
updateListsAndCodeIn: aWindow

	| recent |
	
	self canDiscardEdits ifFalse: [^ self].

	recent := self class mostRecentlySubmittedMessage.

	(((messageList isEmpty ) not and: [recent notNil]) and: [messageList first == recent])
		ifFalse:	[self reformulateList]
		ifTrue: [self updateCodePaneIfNeeded]! !

MessageSet subclass: #TimeProfileBrowser
	instanceVariableNames: 'selectedClass selectedSelector block tally'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!TimeProfileBrowser commentStamp: '<historical>' prior: 0!
A TimeProfileBrowser is a browser visualizing the runtime profile of an executed Smalltalk block.  It is useful for finding performance bottlenecks in code. When optimizing code it can
be hard to know what methods actually constitute the bulk of the execution time. Is it a few
methods that take very long time to execute or is it perhaps a single method that gets executed a thousand times?

The block is first spied on using a MessageTally instance (which has even more funtionality than used by the TimeProfileBrowser) which samples the block during it's execution and collects the amount of time approximately spent in the methods executed. Then the methods are shown in the browser with their relative execution time in percent.

Example:
TimeProfileBrowser onBlock: [20 timesRepeat:  [Transcript show: 100 factorial printString]]
!

!TimeProfileBrowser class methodsFor: 'documentation' stamp: 'CedrickBeler 11/13/2009 18:13'!
helpMessage

	^'"Choose a method to see its implementation
	
	
****General information****

Process(priorityNumber) aNumber : means the priority is priorityNumber for process number aNumber.

Tree: 		total time spent in each method (including called methods).

Leaves: 	time spent in each method but without the time spent in methods called.

Memory: 	information about the memory evolution.
	free represents the memory space that was used during the execution of the profiled expression.

GC''s:		information about garbage collection
				incr, tenures and root table are for expert only.
				An important remark is that if incr time should be less than 10%.
				Incr value greater than 25% really denotes a problem"'! !

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
onBlock: block
	"Open a profile browser on the given block, thereby running the block and 
	 collecting the message tally."
	"TimeProfileBrowser onBlock: [20 timesRepeat: 
			[Transcript show: 100 factorial printString]]"

	| inst result |
	inst := self new.
	result := inst runBlock: block.
	self open: inst name: 'Time Profile'.
	^ result! !

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'nk 3/8/2004 12:46'!
spyOn: block
	"Open a profile browser on the given block, thereby running the block and 
	 collecting the message tally."
	"TimeProfileBrowser spyOn:  [20 timesRepeat: 
			[Transcript show: 100 factorial printString]]"

	^self onBlock: block! !

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
spyOnProcess: aProcess forMilliseconds: msecDuration 
	"Run aProcess for msecDuration milliseconds, then open a TimeProfileBrowser on the results."

	"| p |  
	p := [100000 timesRepeat: [3.14159 printString]] fork.  
	(Delay forMilliseconds: 100) wait.  
	TimeProfileBrowser spyOnProcess: p forMilliseconds: 1000"

	| inst |
	inst := self new.
	inst runProcess: aProcess forMilliseconds: msecDuration pollingEvery: MessageTally defaultPollPeriod.
	self open: inst name: (String streamContents: [ :s | s nextPutAll: 'Time Profile for '; print: msecDuration; nextPutAll: ' msec' ]).
	^ inst! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
initializeMessageList: anArray
	messageList := anArray.
	messageListIndex := 0.
	contents := ''! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'nk 3/8/2004 13:22'!
messageListKey: aChar from: view 
	"Respond to a Command key. Cmd-D means re-run block."

	aChar == $d ifTrue: [^Cursor execute showWhile: [ block value ]].
	^super messageListKey: aChar from: view! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'stp 05/08/1999 15:27'!
messageListMenu: aMenu shifted: shifted
	"Add a menu to the inherited one."

	| menu |
	menu := super messageListMenu: aMenu shifted: shifted.
"	menu addItem: (0)."
	^menu! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'nk 3/8/2004 12:51'!
runBlock: aBlock
	^self runBlock: aBlock pollingEvery: MessageTally defaultPollPeriod! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'jmv 9/24/2009 16:12'!
runBlock: aBlock pollingEvery: pollPeriod 
	| stream list result |
	block := MessageSend 
				receiver: self
				selector: #runBlock:pollingEvery:
				arguments: { 
						aBlock.
						pollPeriod}.	"so we can re-run it"
	tally := MessageTally new.
	tally
		reportOtherProcesses: false;
		maxClassNameSize: 1000;
		maxClassPlusSelectorSize: 1000;
		maxTabs: 100.
	result := tally spyEvery: pollPeriod on: aBlock.
	stream := ReadWriteStream 
				with: (String streamContents: 
							[:s | 
							tally
								report: s;
								close]).
	stream reset.
	list := OrderedCollection new.
	[stream atEnd] whileFalse: [list add: stream nextLine].
	self initializeMessageList: list.
	self changed: #messageList.
	self changed: #messageListIndex.
	^result! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'jmv 9/24/2009 16:13'!
runProcess: aProcess forMilliseconds: msecDuration pollingEvery: pollPeriod 
	| stream list result |
	block := MessageSend 
				receiver: self
				selector: #runProcess:forMilliseconds:pollingEvery: 
				arguments: { 
						aProcess.
						msecDuration.
						pollPeriod}.	"so we can re-run it"
	tally := MessageTally new.
	tally
		reportOtherProcesses: false;
		maxClassNameSize: 1000;
		maxClassPlusSelectorSize: 1000;
		maxTabs: 100.
	result := tally 
				spyEvery: pollPeriod
				onProcess: aProcess
				forMilliseconds: msecDuration.
	stream := ReadWriteStream 
				with: (String streamContents: 
							[:s | 
							tally
								report: s;
								close]).
	stream reset.
	list := OrderedCollection new.
	[stream atEnd] whileFalse: [list add: stream nextLine].
	self initializeMessageList: list.
	self changed: #messageList.
	self changed: #messageListIndex.
	^result! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
selectedClass
	"Answer the receiver's 'selectedClass'."

	^selectedClass! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
selectedClass: anObject
	"Set the receiver's instance variable 'selectedClass' to be anObject."

	selectedClass := anObject! !

!TimeProfileBrowser methodsFor: 'message list' stamp: 'nice 1/5/2010 15:59'!
selectedMessage
	"Answer the source method for the currently selected message." 

	
	self setClassAndSelectorIn: 
			[:class :selector | | source | 
			source := class sourceMethodAt: selector ifAbsent: [^'Missing'].
			self browseWithPrettyPrint 
				ifTrue: 
					[source := class prettyPrinterClass 
								format: source
								in: class
								notifying: nil].
			self selectedClass: class.
			self selectedSelector: selector.
			^source asText makeSelectorBoldIn: class].

	^self class helpMessage! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
selectedSelector
	"Answer the receiver's 'selectedSelector'."

	^selectedSelector! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
selectedSelector: anObject
	"Set the receiver's instance variable 'selectedSelector' to be anObject."

	selectedSelector := anObject! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'MarcusDenker 1/23/2011 09:07'!
setClassAndSelectorIn: csBlock 
	"Decode strings of the form    <selectorName> (<className> [class])  "
	
	self flag: #mref.	"fix for faster references to methods"
	
	[ | parens sel string class strm | string := self selection asString.
	string first == $* ifTrue: [ ^ contents := nil ].	"Ignore lines starting with *"
	parens := string includes: $(.	"Does it have open-paren?"
	strm := string readStream.
	parens 
		ifTrue: 
			[ strm skipTo: $(.	"easy case"
			class := strm upTo: $).
			strm next: 2.]
		ifFalse: 
			[ strm position: (string findString: ' class>>').
			strm position > 0 
				ifFalse: [ strm position: (string findLast: [ :ch | ch == $  ]) ]
				ifTrue: 
					[ "find the next to last space character"
					| subString |
					subString := strm contents 
						copyFrom: 1
						to: (string findLast: [ :ch | ch == $  ]) - 1.
					strm position: (subString findLast: [ :ch | ch == $  ]) ].
			"ifFalse: [strm position: (string findLast: [ :ch | ch == $ ])."
			class := strm upTo: $>.
			strm next.].
	sel := strm upToEnd. 
	^ MessageSet 
		parse: class , ' ' , sel
		toClassAndSelector: csBlock ] 
		on: Error
		do: [ :ex | ^ contents := nil ]! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
tally
	"Answer the receiver's 'tally'."

	^tally! !

CodeHolder subclass: #ChangeList
	instanceVariableNames: 'changeList list listIndex listSelections file lostMethodPointer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeList commentStamp: 'StephaneDucasse 7/23/2010 21:17' prior: 0!
A ChangeList represents a list of changed methods that reside on a file in fileOut format.  The classes and methods in my list are not necessarily in this image!!  Used as the model when changes are recovered.

It holds three lists:
	changeList - a list of ChangeRecords
	list - a list of one-line printable headers
	listSelections - a list of Booleans (true = selected, false = not selected) multiple OK.
	listIndex 
Items that are removed (removeDoits, remove an item) are removed from all three lists.
Most recently clicked item is the one showing in the bottom pane.!

!ChangeList class methodsFor: 'filein/out' stamp: 'IgorStasenko 3/6/2011 18:14'!
browseChangesFile: fullName
	"Browse the selected file in fileIn format."

	fullName
		ifNotNil:
			[self browseStream: (FileStream readOnlyFileNamed:  fullName)]
		ifNil:
			[Beeper beep]! !

!ChangeList class methodsFor: 'filein/out' stamp: 'IgorStasenko 3/6/2011 18:14'!
browseCompressedChangesFile: fullName 
	"Browse the selected file in fileIn format."

	| unzipped stream |
	fullName ifNil: [^Beeper beep].
	stream := FileStream readOnlyFileNamed: fullName.
	[ | zipped |
	stream converter: Latin1TextConverter new.
	zipped := GZipReadStream on: stream.
	unzipped := zipped contents asString]
		ensure: [stream close].
	stream := (MultiByteBinaryOrTextStream with: unzipped) reset.
	self browseStream: stream! !

!ChangeList class methodsFor: 'public access' stamp: 'di 1/18/2001 15:30'!
browseFile: fileName    "ChangeList browseFile: 'AutoDeclareFix.st'"
	"Opens a changeList on the file named fileName"

	^ self browseStream: (FileStream readOnlyFileNamed: fileName)! !

!ChangeList class methodsFor: 'public access' stamp: 'tak 9/25/2008 16:25'!
browseRecent: charCount 
	"ChangeList browseRecent: 5000"
	"Opens a changeList on the end of the changes log file"
	"The core was moved to browserRecent:on:."
	^ self browseRecent: charCount on: (SourceFiles at: 2) ! !

!ChangeList class methodsFor: 'public access' stamp: 'sd 11/20/2005 21:28'!
browseRecent: charCount on: origChangesFile 
	"Opens a changeList on the end of the specified changes log file"
	| changeList end changesFile |
	changesFile := origChangesFile readOnlyCopy.
	changesFile setConverterForCode.
	end := changesFile size.
	Cursor read
		showWhile: [changeList := self new
						scanFile: changesFile
						from: (0 max: end - charCount)
						to: end].
	changesFile close.
	self
		open: changeList
		name: 'Recent changes'
		multiSelect: true! !

!ChangeList class methodsFor: 'public access' stamp: 'StephaneDucasse 3/17/2010 20:53'!
browseRecentLog
	"ChangeList browseRecentLog"
	"Prompt with a menu of how far back to go to browse the current image's changes log file"
	^ self
		browseRecentLogOn: (SourceFiles at: 2)
		startingFrom: Smalltalk lastQuitLogPosition! !

!ChangeList class methodsFor: 'public access' stamp: 'sd 11/20/2005 21:28'!
browseRecentLogOn: origChangesFile 
	"figure out where the last snapshot or quit was, then browse the recent entries."

	| end done block pos chunk changesFile positions prevBlock |
	changesFile := origChangesFile readOnlyCopy.
	positions := SortedCollection new.
	end := changesFile size.
	prevBlock := end.
	block := end - 1024 max: 0.
	done := false.
	[done
		or: [positions size > 0]]
		whileFalse: [changesFile position: block.
			"ignore first fragment"
			changesFile nextChunk.
			[changesFile position < prevBlock]
				whileTrue: [pos := changesFile position.
					chunk := changesFile nextChunk.
					((chunk indexOfSubCollection: '----' startingAt: 1) = 1) ifTrue: [
						({ '----QUIT'. '----SNAPSHOT' } anySatisfy: [ :str |
							chunk beginsWith: str ])
								ifTrue: [positions add: pos]]].
			block = 0
				ifTrue: [done := true]
				ifFalse: [prevBlock := block.
					block := block - 1024 max: 0]].
	changesFile close.
	positions isEmpty
		ifTrue: [self inform: 'File ' , changesFile name , ' does not appear to be a changes file']
		ifFalse: [self browseRecentLogOn: origChangesFile startingFrom: positions last]! !

!ChangeList class methodsFor: 'public access' stamp: 'alain.plantec 2/6/2009 16:37'!
browseRecentLogOn: origChangesFile startingFrom: initialPos 
	"Prompt with a menu of how far back to go when browsing a changes file."

	| end banners positions pos chunk i changesFile |
	changesFile := origChangesFile readOnlyCopy.
	banners := OrderedCollection new.
	positions := OrderedCollection new.
	end := changesFile size.
	changesFile setConverterForCode.
	pos := initialPos.
	[pos = 0
		or: [banners size > 20]]
		whileFalse: [changesFile position: pos.
			chunk := changesFile nextChunk.
			i := chunk indexOfSubCollection: 'priorSource: ' startingAt: 1.
			i > 0
				ifTrue: [positions addLast: pos.
					banners
						addLast: (chunk copyFrom: 5 to: i - 2).
					pos := Number
								readFrom: (chunk copyFrom: i + 13 to: chunk size)]
				ifFalse: [pos := 0]].
	changesFile close.
	banners size == 0 ifTrue: [^ self inform: 
'this image has never been saved
since changes were compressed' translated].
	pos := UIManager default chooseFrom:  banners values: positions title: 'Browse as far back as...' translated.
	pos isNil ifTrue: [^ self].
	self browseRecent: end - pos on: origChangesFile! !

!ChangeList class methodsFor: 'public access' stamp: 'nb 6/17/2003 12:25'!
browseRecentLogOnPath: fullName 
	"figure out where the last snapshot or quit was, then browse the recent  entries."

	fullName
		ifNotNil:
			[self browseRecentLogOn: (FileStream readOnlyFileNamed: fullName)]
		ifNil:
			[Beeper beep]
	! !

!ChangeList class methodsFor: 'public access' stamp: 'sd 11/20/2005 21:28'!
browseStream: changesFile
	"Opens a changeList on a fileStream"
	| changeList charCount |
	changesFile readOnly.
	changesFile setConverterForCode.
	charCount := changesFile size.
	charCount > 1000000 ifTrue:
		[(self confirm: 'The file ', changesFile name , '
is really long (' , charCount printString , ' characters).
Would you prefer to view only the last million characters?')
			ifTrue: [charCount := 1000000]].
	"changesFile setEncoderForSourceCodeNamed: changesFile name."
	Cursor read showWhile:
		[changeList := self new
			scanFile: changesFile from: changesFile size-charCount to: changesFile size].
	changesFile close.
	self open: changeList name: changesFile localName , ' log' multiSelect: true! !

!ChangeList class methodsFor: 'filein/out' stamp: 'sd 11/20/2005 21:28'!
fileReaderServicesForFile: fullName suffix: suffix
	| services |
	services := OrderedCollection new.
	(FileStream isSourceFileSuffix: suffix) | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseChangeFile ].
	(suffix = 'changes') | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseDotChangesFile ].
	(fullName asLowercase endsWith: '.cs.gz') | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseCompressedChangeFile ].
	^services! !

!ChangeList class methodsFor: 'initialize-release' stamp: 'GabrielOmarCotelli 6/4/2009 20:38'!
initialize

	FileServices registerFileReader: self! !

!ChangeList class methodsFor: 'instance creation' stamp: 'alain.plantec 5/30/2008 10:41'!
open: aChangeList name: aString multiSelect: multiSelect
	"Create a standard system view for the messageSet, whose label is aString.
	The listView may be either single or multiple selection type"
	^ self openAsMorph: aChangeList name: aString multiSelect: multiSelect.
! !

!ChangeList class methodsFor: 'instance creation' stamp: 'RAA 1/11/2001 08:20'!
openAsMorph: aChangeList name: labelString multiSelect: multiSelect
	"Open a morphic view for the messageSet, whose label is labelString.
	The listView may be either single or multiple selection type"

	^aChangeList openAsMorphName: labelString multiSelect: multiSelect
! !

!ChangeList class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightBlue
! !

!ChangeList class methodsFor: 'tool registry' stamp: 'IgorStasenko 2/20/2011 14:41'!
registerToolsOn: registry

	registry register: self as: #changeList ! !

!ChangeList class methodsFor: 'filein/out' stamp: 'tbn 8/11/2010 10:11'!
serviceBrowseChangeFile
	"Answer a service for opening a changelist browser on a file"

	^ (SimpleServiceEntry 
		provider: self 
		label: 'changelist browser'
		selector: #browseStream:
		description: 'Open a changelist tool on this file'
		buttonLabel: 'Changes')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!ChangeList class methodsFor: 'filein/out' stamp: 'tbn 8/11/2010 10:11'!
serviceBrowseCompressedChangeFile
	"Answer a service for opening a changelist browser on a file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Changelist browser'
		selector: #browseCompressedChangesFile:
		description: 'Open a changelist tool on this file'
		buttonLabel: 'Changes'! !

!ChangeList class methodsFor: 'filein/out' stamp: 'tbn 8/11/2010 10:11'!
serviceBrowseDotChangesFile
	"Answer a service for opening a changelist browser on the tail end of a .changes file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Recent changes in file'
		selector: #browseRecentLogOnPath:
		description: 'Open a changelist tool on recent changes in file'
		buttonLabel: 'Recent changes'! !

!ChangeList class methodsFor: 'filein/out' stamp: 'nk 12/13/2002 12:04'!
services
	"Answer potential file services associated with this class"

	^ { self serviceBrowseChangeFile. 
		self serviceBrowseDotChangesFile.
		self serviceBrowseCompressedChangeFile }! !

!ChangeList class methodsFor: 'initialization' stamp: 'GabrielOmarCotelli 6/4/2009 20:38'!
unload

	FileServices unregisterFileReader: self ! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sd 11/20/2005 21:26'!
addItem: item text: text
	| cr |
	cr := Character cr.
	changeList addLast: item.
	list addLast: (text collect: [:x | x = cr ifTrue: [$/] ifFalse: [x]])! !

!ChangeList methodsFor: 'viewing access' stamp: 'sd 11/20/2005 21:26'!
annotation
	"Answer the string to be shown in an annotation pane.  Make plain that the annotation is associated with the current in-image version of the code, not of the selected disk-based version, and if the corresponding method is missing from the in-image version, mention that fact."

	| annot aChange aClass |

	annot := super annotation.
	annot asString = '------' ifTrue: [^ annot].

	^ ((aChange := self currentChange) notNil and: [aChange methodSelector notNil])
		ifFalse:
			[annot]
		ifTrue:
			[((aClass := aChange methodClass) isNil or: [(aClass includesSelector: aChange methodSelector) not])
				ifTrue:
					[aChange methodClassName, ' >> ', aChange methodSelector, ' is not present in the current image.']
				ifFalse:
					['current version: ', annot]]! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 21:09'!
browseAllVersionsOfSelections
	"Opens a Versions browser on all the currently selected methods, showing each alongside all of their historical versions."
	
	|  oldSelection aList |
	oldSelection := self listIndex.
	aList := OrderedCollection new.
	Cursor read showWhile: [
		1 to: changeList size do: [:i |
			(listSelections at: i) ifTrue: [
				listIndex := i.
				self browseVersions.
				aList add: i. ]]].
	listIndex := oldSelection.

	aList size == 0 ifTrue: [^ self inform: 'no selected methods have in-memory counterparts'].
! !

!ChangeList methodsFor: 'menu actions' stamp: 'IgorStasenko 3/6/2011 18:13'!
browseCurrentVersionsOfSelections
	"Opens a message-list browser on the current in-memory versions of all methods that are currently seleted"
	
	| aList |

	aList := OrderedCollection new.
	Cursor read showWhile: [
		1 to: changeList size do: [:i |
			(listSelections at: i) ifTrue: [
				| aClass aChange |
				aChange := changeList at: i.
				(aClass := aChange methodClass) notNil
					and: [aChange isMethodDefinedInImage
								ifTrue: [
									aList add: ( SourcedMethodReference new
													setClass: aClass  
													methodSymbol: aChange methodSelector)]
						]]]].

	aList size == 0 ifTrue: [^ self inform: 'no selected methods have in-memory counterparts'].
	Smalltalk tools messageSet 
		openMessageList: aList 
		name: 'Current versions of selected methods in ', file localName
! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:00'!
browseVersions
	| change class browser |
	listIndex = 0
		ifTrue: [^ nil ].
	change := changeList at: listIndex.
	change classIncludesSelector		
			ifFalse: [ ^nil ].
	browser := super browseVersions.
	browser ifNotNil: [ browser addedChangeRecord: change ].
	^browser! !

!ChangeList methodsFor: 'menu actions' stamp: 'BenjaminVanRyseghem 1/29/2011 13:24'!
buildMorphicCodePaneWith: editString

	| codePane |
	codePane := (AcceptableCleanTextMorph
		on: self
		text: #contents 
		accept: #contents:
		readSelection: #contentsSelection 
		menu: #codePaneMenu:shifted:)
			beStyled;
			yourself.
	codePane font: StandardFonts codeFont.
	editString ifNotNil: [
		codePane editString: editString.
		codePane hasUnacceptedEdits: true ].
	^codePane
! !

!ChangeList methodsFor: 'toolbuilder' stamp: 'ar 2/12/2005 13:50'!
buildWith: builder
	^self buildWith: builder multiSelect: true! !

!ChangeList methodsFor: 'toolbuilder' stamp: 'ar 2/12/2005 14:46'!
buildWith: builder multiSelect: multiSelect 
	"Open a morphic view for the messageSet, whose label is labelString. 
	The listView may be either single or multiple selection type"

	| windowSpec max listSpec panelSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.33] ifFalse:[0.4].

	multiSelect ifTrue:[
		listSpec := builder pluggableMultiSelectionListSpec new.
		listSpec getSelectionList: #listSelectionAt:.
		listSpec setSelectionList: #listSelectionAt:put:.
	] ifFalse:[
		listSpec := builder pluggableListSpec new.
	].

	listSpec 
		model: self;
		list: #list; 
		getIndex: #listIndex; 
		setIndex: #toggleListIndex:; 
		menu: (self showsVersions ifTrue: [#versionsMenu:] ifFalse: [#changeListMenu:]); 
		keyPress: #changeListKey:from:;
		frame: (0@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.33 corner: 1@0.4).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:;
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.4corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!ChangeList methodsFor: 'accessing' stamp: ''!
changeList
	^ changeList! !

!ChangeList methodsFor: 'initialization-release' stamp: 'tbn 7/6/2010 16:46'!
changeListButtonSpecs

	^#(
		('Select all' 			selectAll				'select all entries')
		('Deselect all'		deselectAll			'deselect all entries')
		('Select conflicts'	selectAllConflicts	'select all methods that occur in any change set')
		('File in selections' 	fileInSelections		'file in all selected entries')
		)! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:00'!
changeListKey: aChar from: view
	"Respond to a Command key in the list pane."

	aChar == $D ifTrue: [ ^self toggleDiffing ].
	aChar == $a ifTrue: [ ^self selectAll ].
	^ self arrowKey: aChar from: view! !

!ChangeList methodsFor: 'menu actions' stamp: 'tbn 7/29/2010 22:02'!
changeListMenu: aMenu
	"Fill aMenu up so that it comprises the primary changelist-browser menu"

	aMenu addTitle: 'change list'.
	aMenu addStayUpItemSpecial.

	aMenu addList: #(

	('FileIn selections'							fileInSelections						'Import the selected items into the image')
	('FileOut selections...	'						fileOutSelections						'create a new file containing the selected items')
	-
	('Compare to current'						compareToCurrentVersion			'Open a separate window which shows the text differences between the on-file version and the in-image version.' )
	('Toggle diffing (D)'							toggleDiffing						'Start or stop showing diffs in the code pane.')
	-
	('Select conflicts with any changeset'		selectAllConflicts					'Select methods in the file which also occur in any change-set in the system')
	('Select conflicts with current changeset'	selectConflicts						'Select methods in the file which also occur in the current change-set')
	('Select conflicts with...'						selectConflictsWith					'Allows you to designate a file or change-set against which to check for code conflicts.')
	-
	('Select unchanged definitions'				selectUnchangedDefinitions			'Select class definitions, class comments and methods in the file whose in-image versions are the same as their in-file counterparts' )
	('Select unchanged methods'					selectUnchangedMethods				'Select methods in the file whose in-image versions are the same as their in-file counterparts' )
	('Select new methods'						selectNewMethods					'Select methods in the file that do not current occur in the image')
	('Select methods for this class'				selectMethodsForThisClass			'Select all methods in the file that belong to the currently-selected class')
	('Select methods for classes in image'			selectMethodsForExtantClasses		'Select all methods in the file that belong to a class that exists in the image')
	('Select changes with contents matching'		selectContentsMatching				'Select all changes in the file whose text includes a pattern')

	-
	('Select all (a)'								selectAll								'Select all the items in the list')
	('Deselect all'								deselectAll							'Deselect all the items in the list')
	('Invert selections'							invertSelections						'Select every item that is not currently selected, and deselect every item that *is* currently selected')
	-
	('Browse all versions of single selection'			browseVersions		'Open a version browser showing the versions of the currently selected method')
	('Browse all versions of selections'			browseAllVersionsOfSelections		'Open a version browser showing all the versions of all the selected methods')
	('Browse current versions of selections'		browseCurrentVersionsOfSelections	'Open a message-list browser showing the current (in-image) counterparts of the selected methods')
	('Destroy current methods of selections'		destroyCurrentCodeOfSelections		'Remove (*destroy*) the in-image counterparts of all selected methods')
	-
	('Remove doIts'								removeDoIts							'Remove all items that are doIts rather than methods')
	('Remove older versions'						removeOlderMethodVersions			'Remove all but the most recent versions of methods in the list')
	('Remove up-to-date versions'				removeExistingMethodVersions		'Remove all items whose code is the same as the counterpart in-image code')
	('Remove selected items'						removeSelections					'Remove the selected items from the change-list')
	('Remove unselected items'					removeNonSelections					'Remove all the items not currently selected from the change-list')).

	^ aMenu

! !

!ChangeList methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:26'!
changes: changeRecords file: aFile
	file := aFile.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	changeRecords do: [:each |
		(each respondsTo: #methodClass)
			ifFalse: [self addItem: ChangeRecord new text: each asString]
			ifTrue:
				[self addItem: each text: ('method: ' , each methodClass name , (each isMetaClassChange ifTrue: [' class '] ifFalse: [' '])
					, each methodSelector
					, '; ' , each stamp)]].
	listSelections := Array new: list size withAll: false! !

!ChangeList methodsFor: 'menu actions' stamp: 'FernandoOlivero 3/30/2011 16:27'!
compareToCurrentVersion
	"If the current selection corresponds to a method in the system, then spawn a window showing the diffs as text"

	| change class s1 s2 |
	listIndex = 0
		ifTrue: [^ self].
	change := changeList at: listIndex.
	((class := change methodClass) notNil
			and: [class includesSelector: change methodSelector])
		ifTrue: [s1 := (class sourceCodeAt: change methodSelector) asString.
			s2 := change string.
			s1 = s2
				ifTrue: [ self inform: 'Exact Match']
				ifFalse:[ UIManager default
					openComparisionFrom: s2
					to: s1
					belongingTo: class
					from: change
					labeled: (class compiledMethodAt: change methodSelector) timeStamp 
					inWindowLabeled: 'Comparison to Current Version' ]	]					
		ifFalse: [self flash]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 9/5/2001 13:52'!
contents
	"Answer the contents string, obeying diffing directives if needed"

	^ self showingAnyKindOfDiffs
		ifFalse:
			[self undiffedContents]
		ifTrue:
			[self showsVersions
				ifTrue:
					[self diffedVersionContents]
				ifFalse:
					[self contentsDiffedFromCurrent]]! !

!ChangeList methodsFor: 'viewing access' stamp: 'tk 4/10/1998 09:25'!
contents: aString
	listIndex = 0 ifTrue: [self changed: #flash. ^ false].
	lostMethodPointer ifNotNil: [^ self restoreDeletedMethod].
	self okToChange "means not dirty" ifFalse: ["is dirty"
		self inform: 'This is a view of a method on a file.\Please cancel your changes.  You may\accept, but only when the method is untouched.' withCRs.  ^ false].
		"Can't accept changes here.  Method text must be unchanged!!"
	(changeList at: listIndex) fileIn.
	^ true! !

!ChangeList methodsFor: 'viewing access' stamp: 'HenrikSperreJohansen 8/4/2010 12:15'!
contentsDiffedFromCurrent
	"Answer the contents diffed forward from current (in-memory) method version"

	| aChange aClass |
	listIndex = 0
		ifTrue: [^ ''].
	aChange := changeList at: listIndex.
	 (aChange type == #method
	 and: [(aClass := aChange methodClass) notNil
	 and: [aClass includesSelector: aChange methodSelector]]) ifTrue:
		[^TextDiffBuilder
			buildDisplayPatchFrom: (aClass sourceCodeAt: aChange methodSelector)
			to: aChange text
			inClass: aClass
			prettyDiffs: self showingPrettyDiffs].

	aChange type == #doIt ifTrue:
		[| tokens |
		 tokens := Scanner new scanTokens: aChange string.
		 ((tokens select:
				[:substr| #(subclass: variableByteSubclass: variableWordSubclass:
							instanceVariableNames: classVariableNames: ) includes: substr])
					asSet size >= 3
		  and: [(aClass := Smalltalk globals at: tokens third ifAbsent: []) notNil
		  and: [aClass isBehavior]]) ifTrue:
			[^TextDiffBuilder buildDisplayPatchFrom: aClass definition to: aChange string].

		(tokens size = 4
		 and: [tokens second == #class
		 and: [tokens third == #instanceVariableNames:
		 and: [(aClass := Smalltalk globals at: tokens first ifAbsent: []) notNil
		 and: [aClass isBehavior]]]]) ifTrue:
			[^TextDiffBuilder buildDisplayPatchFrom: aClass class definition to: aChange string]].

	(aChange type == #classComment
	and: [(aClass := aChange commentClass) notNil]) ifTrue:
		[^TextDiffBuilder buildDisplayPatchFrom: aClass comment asString to: aChange string].

	^(changeList at: listIndex) text! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 11/13/2001 09:12'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane"

	^ self sourceAndDiffsQuintsOnly! !

!ChangeList methodsFor: 'accessing' stamp: 'ls 5/12/1999 07:55'!
currentChange
	"return the current change being viewed, or nil if none"
	listIndex = 0 ifTrue: [ ^nil ].
	^changeList at: listIndex! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
deselectAll 
	"Deselect all items in the list pane, and clear the code pane"

	listIndex := 0.
	listSelections atAllPut: false.
	self changed: #allSelections.
	self contentsChanged! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:01'!
destroyCurrentCodeOfSelections
	"Actually remove from the system any in-memory methods with class and selector identical to items current selected.  This may seem rather arcane but believe me it has its great uses, when trying to split out code.  To use effectively, first file out a change set that you wish to split off.  Then open a ChangeList browser on that fileout.  Now look through the methods, and select any of them which you want to remove completely from the system, then issue this command.  For those methods where you have made changes to pre-existing versions, of course, you won't want to remove them from the system, so use this mechanism with care!!"

	|  aClass aChange aList |
	aList := OrderedCollection new.
	1 to: changeList size do:
		[:index |
			(listSelections at: index) ifTrue:
				[aChange := changeList at: index.
				aChange isMethodDefinedInImage
						ifTrue: [aList add: {aClass. aChange methodSelector}]]].
	aList size > 0 ifTrue:
		[(self confirm: 'Warning!! This will actually remove ', aList size printString,  ' method(s) from the system!!') ifFalse: [^ self]].
	aList do:
		[:aPair | Transcript cr; show: 'Removed: ', aPair first printString, '.', aPair second.
			aPair first removeSelector: aPair second]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sd 11/20/2005 21:26'!
diffedVersionContents
	"Answer diffed version contents, maybe pretty maybe not"

	| change class earlier later |
	(listIndex = 0
			or: [changeList size < listIndex])
		ifTrue: [^ ''].
	change := changeList at: listIndex.
	later := change text.
	class := change methodClass.
	(listIndex == changeList size or: [class == nil])
		ifTrue: [^ later].

	earlier := (changeList at: listIndex + 1) text.

	^ TextDiffBuilder buildDisplayPatchFrom: earlier to: later inClass: class prettyDiffs: self showingPrettyDiffs! !

!ChangeList methodsFor: 'accessing' stamp: ''!
file
	^file! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
fileInSelections 
	| any |
	any := false.
	listSelections with: changeList do: 
		[:selected :item | selected ifTrue: [any := true. item fileIn]].
	any ifFalse:
		[self inform: 'nothing selected, so nothing done']! !

!ChangeList methodsFor: 'menu actions' stamp: 'PeterHugossonMiller 9/3/2009 00:17'!
fileOutSelections 
	| fileName internalStream |
	fileName := UIManager default request: 'Enter the base of file name' initialAnswer: 'Filename'.
	internalStream := (String new: 1000) writeStream.
	internalStream header; timeStamp.
	listSelections with: changeList do: 
		[:selected :item | selected ifTrue: [item fileOutOn: internalStream]].

	FileStream writeSourceCodeFrom: internalStream baseName: fileName isSt: true
! !

!ChangeList methodsFor: 'initialization-release' stamp: 'StephaneDucasse 7/23/2010 21:09'!
initialize
	"Initialize a blank ChangeList.  Set the contentsSymbol to reflect whether diffs will initally be shown or not"

	contentsSymbol := CodeHolder diffsInChangeList
							ifTrue: [self defaultDiffsSymbol]
							ifFalse: [#source].
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	super initialize! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
invertSelections
	"Invert the selectedness of each item in the changelist"

	listSelections := listSelections collect: [ :ea | ea not].
	listIndex := 0.
	self changed: #allSelections.
	self contentsChanged! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
list
	^ list! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
listIndex
	^ listIndex! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
listSelectionAt: index
	^ listSelections at: index! !

!ChangeList methodsFor: 'viewing access' stamp: 'di 1/13/1999 14:59'!
listSelectionAt: index put: value

	^ listSelections at: index put: value! !

!ChangeList methodsFor: 'initialization-release' stamp: 'JuanVuletich 10/26/2010 15:00'!
openAsMorphName: labelString multiSelect: multiSelect 
	"Open a morphic view for the messageSet, whose label is labelString. 
	The listView may be either single or multiple selection type"

	^UIManager default openChangeList: self asMorphName: labelString multiSelect: multiSelect
! !

!ChangeList methodsFor: 'initialization-release' stamp: 'nice 4/16/2009 09:39'!
optionalButtonRow
	"Answer a row of buttons to occur in a tool pane"

	| aRow |
	aRow := AlignmentMorph newRow.
	aRow hResizing: #spaceFill.
	aRow clipSubmorphs: true.
	aRow layoutInset: 2@2; cellInset: 3.
	aRow wrapCentering: #center; cellPositioning: #leftCenter.
	self changeListButtonSpecs do:
		[:triplet | | aButton |
			aButton := PluggableButtonMorph
				on: self
				getState: nil
				action: triplet second.
			aButton
				hResizing: #spaceFill;
				vResizing: #spaceFill;
				label: triplet first asString;
				askBeforeChanging: true;
				onColor: Color white offColor: Color white.

			aRow addMorphBack: aButton.
			aButton setBalloonText: triplet third].
	aRow addMorphBack: self regularDiffButton.
	self wantsPrettyDiffOption ifTrue:
		[aRow addMorphBack: self prettyDiffButton].
	^ aRow! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:12'!
refreshNewList: newChangeList andList: newList

	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list


! !

!ChangeList methodsFor: 'menu actions' stamp: 'HenrikSperreJohansen 9/12/2010 11:01'!
removeDoIts
	"Remove doits from the receiver, other than initializes. 1/26/96 sw"

	self removeRecordsWhere: 
		[:record :displayString | 
			record type = #doIt and: [(displayString endsWith: 'initialize') not]]! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:15'!
removeExistingMethodVersions
	"Remove all up to date version of entries from the receiver"
	
	| newChangeList newList |
	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	changeList with: list do: [ :chRec :strNstamp | 
			| str keep sel cls | 
			keep := true.
			(cls := chRec methodClass) ifNotNil:
				[ str := chRec string.
				sel := cls parserClass new parseSelector: str.
				keep := (cls sourceCodeAt: sel ifAbsent: ['']) asString ~= str].
			keep ifTrue:
					[ newChangeList add: chRec.
					newList add: strNstamp]].
	self refreshNewList: newChangeList andList: newList! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:12'!
removeNonSelections
	"Remove the unselected items from the receiver."

	| newChangeList newList |
	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	1 to: changeList size do:
		[ :i | (listSelections at: i)
			ifTrue:
				[ newChangeList add: (changeList at: i).
				newList add: (list at: i) ]].
	newChangeList size == 0 
		ifTrue: [^ self inform: 'That would remove everything.
Why would you want to do that?'].
	
	self refreshNewList: newChangeList andList: newList.
	
	! !

!ChangeList methodsFor: 'menu actions' stamp: 'HenrikSperreJohansen 9/12/2010 10:54'!
removeOlderMethodVersions
	"Remove older versions of entries from the receiver."
	| equivalent toRemove|

	equivalent := Dictionary new.
	toRemove := Set new.
	changeList with: list do:
		[:change :displayString | 
			(equivalent at: (displayString copyUpTo: $;) ifAbsentPut: [OrderedCollection new.]) add: change ].
	equivalent do: #removeLast.
	equivalent do: [:each | toRemove addAll: each].
	self removeRecordsWhere: [:change | toRemove includes: change]! !

!ChangeList methodsFor: 'menu actions' stamp: 'HenrikSperreJohansen 9/12/2010 10:37'!
removeRecordsWhere: aBlock
	"Remove records for which the block returns true."

	| newChangeList newList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	changeList with: list do:
		[:record :textualDescription |
		(aBlock cull: record cull: textualDescription)
					ifFalse:
						[newChangeList add: record.
						newList add: textualDescription]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list.! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:16'!
removeSelections
	"Remove the selected items from the receiver."

	| newChangeList newList |
	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.
	1 to: changeList size do:
		[:i | (listSelections at: i) ifFalse:
			[newChangeList add: (changeList at: i).
			newList add: (list at: i)]].
	self refreshNewList: newChangeList andList: newList

	! !

!ChangeList methodsFor: 'viewing access' stamp: 'Igor.Stasenko 12/20/2009 18:54'!
restoreDeletedMethod
	"If lostMethodPointer is not nil, then this is a version browser for a method that has been removed.  In this case we want to establish a sourceCode link to prior versions.  We do this by installing a dummy method with the correct source code pointer prior to installing this version."
	| dummyMethod class selector |
	dummyMethod := CompiledMethod toReturnSelfTrailerBytes: 
		(CompiledMethodTrailer new sourcePointer: lostMethodPointer).
	class := (changeList at: listIndex) methodClass.
	selector := (changeList at: listIndex) methodSelector.
	class addSelectorSilently: selector withMethod: dummyMethod.
	(changeList at: listIndex) fileIn.
	"IF for some reason, the dummy remains, remove it, but (N.B.!!) we might not get control back if the compile (fileIn above) fails."
	(class compiledMethodAt: selector) == dummyMethod
		ifTrue: [class basicRemoveSelector: selector].
	^ true! !

!ChangeList methodsFor: 'scanning' stamp: 'eem 7/17/2008 15:50'!
scanCategory  
	"Scan anything that involves more than one chunk; method name is historical only"

	| itemPosition item tokens stamp anIndex |
	itemPosition := file position.
	item := file nextChunk.

	((item includesSubString: 'commentStamp:')
	or: [(item includesSubString: 'methodsFor:')
	or: [item endsWith: 'reorganize']]) ifFalse:
		["Maybe a preamble, but not one we recognize; bail out with the preamble trick"
		^ self addItem: (ChangeRecord new file: file position: itemPosition type: #preamble)
				 text: ('preamble: ' , item contractTo: 50)].

	tokens := Scanner new scanTokens: item.
	tokens size >= 3 ifTrue:
		[stamp := ''.
		anIndex := tokens indexOf: #stamp: ifAbsent: [nil].
		anIndex ifNotNil: [stamp := tokens at: (anIndex + 1)].

		tokens second == #methodsFor:
			ifTrue: [^ self scanCategory: tokens third class: tokens first
							meta: false stamp: stamp].
		tokens third == #methodsFor:
			ifTrue: [^ self scanCategory: tokens fourth class: tokens first
							meta: true stamp: stamp]].

	tokens second == #commentStamp:
		ifTrue:
			[stamp := tokens third.
			self addItem:
					(ChangeRecord new file: file position: file position type: #classComment
									class: tokens first category: nil meta: false stamp: stamp)
					text: 'class comment for ' , tokens first, 
						  (stamp isEmpty ifTrue: [''] ifFalse: ['; ' , stamp]).
			file nextChunk.
			^ file skipStyleChunk].

	self assert: tokens last == #reorganize.
	self addItem:
		(ChangeRecord new
			file: file position: file position type: #reorganize
			class: tokens first category: nil meta: false stamp: stamp)
		text: 'organization for ' , tokens first, (tokens second == #class ifTrue: [' class'] ifFalse: ['']).
	file nextChunk! !

!ChangeList methodsFor: 'scanning' stamp: 'StephaneDucasse 7/23/2010 21:04'!
scanCategory: category class: class meta: meta stamp: stamp
	| itemPosition method selector |
	[itemPosition := file position.
	method := file nextChunk.
	file skipStyleChunk.
	method size > 0]						"done when double terminators"
		whileTrue:
		[self addItem: (ChangeRecord new file: file position: itemPosition type: #method
							class: class category: category meta: meta stamp: stamp)
			text: 'method: ' , class , (meta ifTrue: [' class '] ifFalse: [' '])
				, ((selector := (Smalltalk globals at: class ifAbsent: [Object]) parserClass new parseSelector: method) isNil
					ifTrue: ['unparsableSelector']
					ifFalse: [selector])
				, (stamp isEmpty ifTrue: [''] ifFalse: ['; ' , stamp])]! !

!ChangeList methodsFor: 'scanning' stamp: 'nice 1/5/2010 15:59'!
scanFile: aFile from: startPosition to: stopPosition
	
	file := aFile.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	file position: startPosition.
'Scanning ', aFile localName, '...'
	displayProgressAt: Sensor cursorPoint
	from: startPosition to: stopPosition
	during: [:bar | | item prevChar itemPosition |
	[file position < stopPosition]
		whileTrue:
		[bar value: file position.
		[file atEnd not and: [file peek isSeparator]]
				whileTrue: [prevChar := file next].
		(file peekFor: $!!)
		ifTrue:
			[(prevChar = Character cr or: [prevChar = Character lf])
				ifTrue: [self scanCategory]]
		ifFalse:
			[itemPosition := file position.
			item := file nextChunk.
			file skipStyleChunk.
			item size > 0 ifTrue:
				[self addItem: (ChangeRecord new file: file position: itemPosition type: #doIt)
					text: 'do it: ' , (item contractTo: 50)]]]].
	listSelections := Array new: list size withAll: false! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
selectAll
	listIndex := 0.
	listSelections atAllPut: true.
	self changed: #allSelections! !

!ChangeList methodsFor: 'filter streaming' stamp: 'StephaneDucasse 7/23/2010 21:08'!
selectAllConflicts
	"Selects all method definitions in the receiver which are also in any existing change set in the system.  This makes no statement about whether the content of the methods differ, only whether there is a change represented."

	Cursor read showWhile: 
		[ | aChange aClass |
		1 to: changeList size do:
			[:i | aChange := changeList at: i.
			listSelections at: i put:
				(aChange type = #method
				and: [(aClass := aChange methodClass) notNil
				and: [ChangeSet doesAnyChangeSetHaveClass: aClass andSelector:  aChange methodSelector]])]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/5/2010 15:59'!
selectConflicts
	"Selects all method definitions for which there is ALSO an entry in changes"
	
	Cursor read showWhile: 
	[ | class change |1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [(ChangeSet current atSelector: change methodSelector
						class: class) ~~ #none]])]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/5/2010 15:59'!
selectConflicts: changeSetOrList
	"Selects all method definitions for which there is ALSO an entry in the specified changeSet or changList"
	| systemChanges |
	Cursor read showWhile: 
	[ | change class |(changeSetOrList isKindOf: ChangeSet) ifTrue: [
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [(changeSetOrList atSelector: change methodSelector
						class: class) ~~ #none]])]]
	ifFalse: ["a ChangeList"
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [changeSetOrList list includes: (list at: i)]])]]
	].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'IgorStasenko 3/6/2011 18:18'!
selectConflictsWith
	"Selects all method definitions for which there is ALSO an entry in the specified changeSet or changList chosen by the user."
	| aStream all index |
	aStream := (String new: 200) writeStream.
	(all := ChangeSet allChangeSets copy) do:
		[:sel | aStream nextPutAll: (sel name contractTo: 40); cr].
	self allSubInstancesDo:
		[:sel | aStream nextPutAll: (sel file name); cr.
			all addLast: sel].
	aStream skip: -1.
	index := (UIManager default chooseFrom: (aStream contents substrings)).
	index > 0 ifTrue: [
		self selectConflicts: (all at: index)].
! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:36'!
selectContentsMatching
	| pattern |
	pattern := UIManager default request: 'pattern to match'.
	pattern isEmpty ifTrue: [^self].
	^Cursor execute showWhile:
		[self selectSuchThat: ((pattern includesAnyOf: '?*')
								ifTrue: [ [:change | pattern match: change string]]
								ifFalse: [ [:change | change string includesSubString: pattern]])]! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:37'!
selectMethodsForExtantClasses
	"Select methods for classes which are in the current image"
	
	^self selectSuchThat:
		[ :change | Smalltalk globals hasClassNamed: change methodClassName]! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:37'!
selectMethodsForThisClass
	| name |
	self currentChange ifNil: [ ^self ].
	name := self currentChange methodClassName.
	name ifNil: [ ^self ].
	^self selectSuchThat: [ :change | change methodClassName = name ].! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:42'!
selectNewMethods
	"Selects all method definitions for which there is no counterpart method in the current image"

	Cursor read showWhile: 
		[ | change class |
		1 to: changeList size do:
			[:i | change := changeList at: i.
			listSelections at: i put: change isMethodNotDefinedInImage]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:44'!
selectSuchThat
	"query the user for a selection criterium.  NB: the UI for invoking this from a changelist browser is currently commented out; to reenfranchise it, you'll need to mild editing to ChangeList method #changeListMenu:"
	
	| code block |
	code := UIManager default request: 'selection criteria for a change named aChangeRecord?\For instance, ''aChangeRecord category = ''System-Network''''' withCRs.
	code isEmptyOrNil ifTrue: [^ self ].
	block := Compiler evaluate: '[:aChangeRecord | ', code, ']'.
	self selectSuchThat: block! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
selectSuchThat: aBlock
	"select all changes for which block returns true"
	listSelections := changeList collect: [ :change | aBlock value: change ].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 11/14/2010 22:51'!
selectUnchangedDefinitions
	"Selects all recognizable definitions for which there is already a definition in the current image, whose source is exactly the same."
	| change class tokens |
	Cursor read showWhile: 
	[1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put: false.

		(change type = #method
		 and: [(class := change methodClass) notNil
		 and: [class includesSelector: change methodSelector]]) ifTrue:
			[listSelections
				at: i
				put: change string withBlanksCondensed
					= (class sourceCodeAt: change methodSelector) asString withBlanksCondensed].

		(change type == #classComment
		and: [(class := change commentClass) notNil]) ifTrue:
			[listSelections at: i put: change string = class comment asString].

		change type == #doIt ifTrue:
			[tokens := Scanner new scanTokens: change string.

			 ((tokens select:
				[:substr| #(subclass: variableSubclass: variableByteSubclass: variableWordSubclass:
							instanceVariableNames: classVariableNames: ) includes: substr])
					asSet size >= 3
			 and: [(class := Smalltalk globals at: tokens third ifAbsent: []) notNil
			 and: [class isBehavior]]) ifTrue:
				[listSelections
					at: i
					put: change string withBlanksCondensed
						= class definition withBlanksCondensed].

			(tokens size = 4
			 and: [tokens second == #class
			 and: [tokens third == #instanceVariableNames:
			 and: [(class := Smalltalk globals at: tokens first ifAbsent: []) notNil
			 and: [class isBehavior]]]]) ifTrue:
				[listSelections
					at: i
					put: change string withBlanksCondensed
						= class class definition withBlanksCondensed].

			(tokens size = 3
			 and: [tokens second == #removeSelector:
			 and: [(class := Smalltalk at: tokens first ifAbsent: []) isNil
			 	or: [class isBehavior and: [(class includesSelector: tokens third) not]]]]) ifTrue:
				[listSelections at: i put: true].

			(tokens size = 4
			 and: [tokens second == #class
			 and: [tokens third == #removeSelector:
			 and: [(class := Smalltalk at: tokens first ifAbsent: []) isNil
			 	or: [class isBehavior and: [(class class includesSelector: tokens fourth) not]]]]]) ifTrue:
				[listSelections at: i put: true]]]].

	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'StephaneDucasse 7/23/2010 22:50'!
selectUnchangedMethods
	"Selects all method definitions for which there is already a method in the current image, whose source is exactly the same."
	
	Cursor read showWhile: 
	[ | class change |1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put: change isUnchangedMethod]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'viewing access' stamp: 'nk 2/26/2004 13:50'!
selectedClass
	^(self selectedClassOrMetaClass ifNil: [ ^nil ]) theNonMetaClass ! !

!ChangeList methodsFor: 'viewing access' stamp: 'sd 11/20/2005 21:26'!
selectedClassOrMetaClass
	| c |
	^ (c := self currentChange) ifNotNil: [c methodClass]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sd 11/20/2005 21:26'!
selectedMessageName
	| c |
	^ (c := self currentChange) ifNotNil: [c methodSelector]! !

!ChangeList methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:26'!
setLostMethodPointer: sourcePointer
	lostMethodPointer := sourcePointer! !

!ChangeList methodsFor: 'accessing' stamp: 'sw 10/19/1999 15:11'!
showsVersions
	^ false! !

!ChangeList methodsFor: 'viewing access' stamp: 'AlainPlantec 11/13/2009 23:16'!
toggleListIndex: newListIndex

	listIndex ~= 0 ifTrue: [listSelections at: listIndex put: false].
	newListIndex ~= 0 ifTrue: [listSelections at: newListIndex put: true].
	listIndex := newListIndex.
	self changed: #listIndex.
	self contentsChanged! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/25/1999 14:45'!
undiffedContents
	^ listIndex = 0
		ifTrue: ['']
		ifFalse: [(changeList at: listIndex) text]! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sw 8/15/2002 22:34'!
wantsPrettyDiffOption
	"Answer whether pretty-diffs are meaningful for this tool"

	^ true! !

ChangeList subclass: #VersionsBrowser
	instanceVariableNames: 'classOfMethod selectorOfMethod addedChangeRecord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!VersionsBrowser commentStamp: 'nk 11/25/2003 10:04' prior: 0!
VersionsBrowser shows all the versions of a particular method, and lets you compare them, revert to selected versions, and so on.!

VersionsBrowser subclass: #ClassCommentVersionsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ClassCommentVersionsBrowser commentStamp: 'StephaneDucasse 1/7/2011 17:48' prior: 0!
A tool to read and browse class comment versions.!

!ClassCommentVersionsBrowser class methodsFor: 'instance creation' stamp: 'nice 1/5/2010 15:59'!
browseCommentOf: class
	
	Cursor read showWhile:
		[ | changeList |changeList := self new scanVersionsOf: class.
	 	 changeList ifNil: [^ self inform: 'No versions available'].
		 self open: changeList name: 'Recent versions of ',class name,'''s comments' multiSelect: false ]
! !

!ClassCommentVersionsBrowser class methodsFor: 'utilities' stamp: 'sd 11/20/2005 21:28'!
commentRecordsOf: aClass
	"Return a list of ChangeRecords for all versions of the method at selector. Source code can be retrieved by sending string to any one.  Return nil if the method is absent."

	| aList |
	aList := self new
			scanVersionsOf: aClass.
	^ aList ifNotNil: [aList changeList]! !

!ClassCommentVersionsBrowser class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color fromRgbTriplet: #(0.769 0.653 1.0)

! !

!ClassCommentVersionsBrowser class methodsFor: 'utilities' stamp: 'sd 11/20/2005 21:28'!
timeStampFor: aSelector class: aClass reverseOrdinal: anInteger
	"Answer the time stamp corresponding to some version of the given method, nil if none.  The reverseOrdinal parameter is interpreted as:  1 = current version; 2 = last-but-one version, etc."
	
	| aChangeList |
	aChangeList :=  self new scanVersionsOf: aClass.
	^ aChangeList ifNil: [nil] ifNotNil:
		[aChangeList list size >= anInteger
			ifTrue:
				[(aChangeList changeList at: anInteger) stamp]
			ifFalse:
				[nil]]! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sw 8/17/2002 21:57'!
classCommentIndicated
	"Answer whether the receiver is pointed at a class comment"

	^ true! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'sd 11/20/2005 21:26'!
compareToCurrentVersion
	"If the current selection corresponds to a method in the system, then spawn a window showing the diffs as text"

	| change s1 s2 |
	listIndex = 0
		ifTrue: [^ self].
	change := changeList at: listIndex.
	s1 := classOfMethod organization classComment.
	s2 := change string.
	s1 = s2
		ifTrue: [^ self inform: 'Exact Match'].
			(StringHolder new
				textContents: (TextDiffBuilder buildDisplayPatchFrom: s1 to: s2 inClass: classOfMethod  prettyDiffs: self showingPrettyDiffs))
				openLabel: 'Comparison to Current Version'! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'tbn 7/6/2010 16:41'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane"

	^ #(
(source			togglePlainSource 		showingPlainSourceString	'Source'			'the textual source code as writen')
(showDiffs		toggleRegularDiffing	showingRegularDiffsString	'ShowDiffs'		'the textual source diffed from its prior version'))! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'sd 11/20/2005 21:26'!
diffedVersionContents
	"Answer diffed version contents, maybe pretty maybe not"

	| change class earlier later |
	(listIndex = 0
			or: [changeList size < listIndex])
		ifTrue: [^ ''].
	change := changeList at: listIndex.
	later := change text.
	class := self selectedClass.
	(listIndex == changeList size or: [class == nil])
		ifTrue: [^ later].

	earlier := (changeList at: listIndex + 1) text.

	^ TextDiffBuilder buildDisplayPatchFrom: earlier to: later inClass: class prettyDiffs: self showingPrettyDiffs! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'asm 8/13/2002 21:02'!
offerVersionsHelp
	(StringHolder new contents: self versionsHelpString)
		openLabel: 'Class Comment Versions Browsers'! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sd 11/20/2005 21:26'!
priorSourceOrNil
	"If the currently-selected method has a previous version, return its source, else return nil"
	| aClass aSelector  changeRecords |
	(aClass := self selectedClass) ifNil: [^ nil].
	(aSelector := self selectedMessageName) ifNil: [^ nil].
	changeRecords :=  self class commentRecordsOf: self selectedClass.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [^ nil].
	^ (changeRecords at: 2) string 
! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'sd 11/20/2005 21:26'!
reformulateList

     classOfMethod organization classComment ifNil: [^ self].

	self scanVersionsOf: classOfMethod.
	self changed: #list. "for benefit of mvc"
	listIndex := 1.
	self changed: #listIndex.
	self contentsChanged! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'sd 11/20/2005 21:26'!
scanVersionsOf: class 
	"Scan for all past versions of the class comment of the given class"

	| oldCommentRemoteStr sourceFilesCopy position prevPos stamp preamble tokens prevFileIndex |

	classOfMethod := class.
	oldCommentRemoteStr := class  organization commentRemoteStr.
	currentCompiledMethod := oldCommentRemoteStr.
	selectorOfMethod := #Comment.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	oldCommentRemoteStr ifNil:[^ nil] ifNotNil: [oldCommentRemoteStr sourcePointer].

	sourceFilesCopy := SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	position := oldCommentRemoteStr position.
	file := sourceFilesCopy at: oldCommentRemoteStr sourceFileNumber.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  " Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [preamble := file nextChunk].

		prevPos := nil.
		stamp := ''.
		(preamble findString: 'commentStamp:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble.
				(tokens at: tokens size-3) = #commentStamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size-2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]]
			ifFalse: ["The stamp get lost, maybe after a condenseChanges"
					stamp := '<historical>'].
 		self addItem:
				(ChangeRecord new file: file position: position type: #classComment
						class: class name category: nil meta: class stamp: stamp)
			text: stamp , ' ' , class name , ' class comment'. 
		prevPos = 0 ifTrue:[prevPos := nil].
		position := prevPos.
		prevPos notNil 
					ifTrue:[file := sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections := Array new: list size withAll: false! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'asm 8/13/2002 20:59'!
selectedClass
	"Answer the class currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'sd 11/20/2005 21:26'!
updateListsAndCodeIn: aWindow
	| aComment |
	aComment := classOfMethod organization commentRemoteStr.
	aComment == currentCompiledMethod
		ifFalse:
			["Do not attempt to formulate if there is no source pointer.
			It probably means it has been recompiled, but the source hasn't been written
			(as during a display of the 'save text simply?' confirmation)."
			aComment last ~= 0 ifTrue: [self reformulateList]].
	^ true
! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'StephaneDucasse 1/7/2011 17:51'!
versionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"

	aMenu title: 'Versions'.
	aMenu addStayUpItemSpecial.
	^ aMenu addList: #(

		('Compare to current'		compareToCurrentVersion		'Compare selected version to the current version')
		('Revert to selected version'	fileInSelections					'Resubmit the selected version, so that it becomes the current version')
		('Remove from changes'		removeMethodFromChanges		'Remove this method from the current change set, if present')
		-
		('Toggle diffing (D)'			toggleDiffing					'Toggle whether or not diffs should be shown here')
		('Update list'				reformulateList					'Reformulate the list of versions, in case it somehow got out of synch with reality')
		-
		('Help...'					offerVersionsHelp				'Provide an explanation of the use of this tool'))
! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sw 8/15/2002 22:35'!
wantsPrettyDiffOption
	"Answer whether pretty-diffs are meaningful for this tool"

	^ false! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sd 3/30/2005 21:53'!
browseVersionsForClass: aClass selector: aSelector
	self
		browseVersionsOf: (aClass compiledMethodAt: aSelector)
		class: aClass
		meta: aClass isMeta
		category: (aClass organization categoryOfElement: aSelector)
		selector: aSelector! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'di 1/11/2000 12:45'!
browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector 
	^ self browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector lostMethodPointer: nil! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector lostMethodPointer: sourcePointer 
	| changeList browser |
	Cursor read showWhile:
		[changeList := (browser := self new)
			scanVersionsOf: method class: class meta: meta
			category: msgCategory selector: selector].
	changeList ifNil: [ self inform: 'No versions available'. ^nil ].

	sourcePointer ifNotNil:
		[changeList setLostMethodPointer: sourcePointer].

	self open: changeList name: 'Recent versions of ' ,
selector multiSelect: false.

	^browser! !

!VersionsBrowser class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:10'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color fromRgbTriplet: #(0.869 0.753 1.0)
! !

!VersionsBrowser class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:03'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #versionBrowser
! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
timeStampFor: aSelector class: aClass reverseOrdinal: anInteger
	"Answer the time stamp corresponding to some version of the given method, nil if none.  The reverseOrdinal parameter is interpreted as:  1 = current version; 2 = last-but-one version, etc."
	
	| method aChangeList |
	method := aClass compiledMethodAt: aSelector ifAbsent: [^ nil].
	aChangeList := self new
			scanVersionsOf: method class: aClass meta: aClass isMeta
			category: nil selector: aSelector.
	^ aChangeList ifNil: [nil] ifNotNil:
		[aChangeList list size >= anInteger
			ifTrue:
				[(aChangeList changeList at: anInteger) stamp]
			ifFalse:
				[nil]]! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
versionCountForSelector: aSelector class: aClass
	"Answer the number of versions known to the system for the given class and method, including the current version.  A result of greater than one means that there is at least one superseded version.  Answer zero if no logged version can be obtained."
	
	| method aChangeList |
	method := aClass compiledMethodAt: aSelector ifAbsent: [^ 0].
	aChangeList := self new
			scanVersionsOf: method class: aClass meta: aClass isMeta
			category: nil selector: aSelector.
	^ aChangeList ifNil: [0] ifNotNil: [aChangeList list size]! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 2/27/2001 08:17'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"Add an annotation detailing the prior versions count.  Specially handled here for the case of a selector no longer in the system, whose prior versions are seen in a versions browser -- in this case, the inherited version of this method will not work."

	(aClass includesSelector: aSelector) ifTrue:
		[^ super addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream].

	aStream nextPutAll: 
		((changeList size > 0
			ifTrue:
				[changeList size == 1
					ifTrue:
						['Deleted - one prior version']
					ifFalse:
						['Deleted - ', changeList size printString, ' prior versions']]
			ifFalse:
				['surprisingly, no prior versions']), self annotationSeparator)! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'nk 1/7/2004 10:10'!
addedChangeRecord
	^addedChangeRecord! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'sd 11/20/2005 21:27'!
addedChangeRecord: aChangeRecord
	addedChangeRecord := aChangeRecord.
	self reformulateList.! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'tbn 7/6/2010 16:55'!
changeListButtonSpecs

	^#(
		('Compare to current'
		compareToCurrentVersion
		'opens a separate window which shows the text differences between the selected version and the current version')

		('Revert'
		fileInSelections
		'reverts the method to the version selected')

		('Remove from changes'
		removeMethodFromChanges
		'remove this method from the current change set')

		('Help'
		offerVersionsHelp
		'further explanation about use of Versions browsers')
		)! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 2/27/2001 08:46'!
changeListKey: aChar from: view
	"Respond to a Command key in the list pane. of the versions browser"

	^ self messageListKey: aChar from: view! !

!VersionsBrowser methodsFor: 'menu' stamp: 'FernandoOlivero 3/30/2011 16:27'!
compareToOtherVersion
	"Prompt the user for a reference version, then spawn a window 
	showing the diffs between the older and the newer of the current 
	version and the reference version as text."

	| change1 change2 s1 s2 |
	change1 := changeList at: listIndex ifAbsent: [^self].
	change2 := UIManager default
				chooseFrom: (list copyWithoutIndex: listIndex)
				values: (changeList copyWithoutIndex: listIndex).
	change2 ifNil: [^self].
	s1 := change1 string.
	s2 := change2 string.
	s1 = s2 ifTrue: [^self inform: 'Exact Match' translated].
	UIManager default
		openComparisionFrom: s1
		to: s2
		belongingTo: change1 methodClass 
		from: change2
		labeled: change2 stamp 
		inWindowLabeled: (('Comparison from {1} to {2}' translated) format: {change1 stamp. change2 stamp}).

! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 10/12/1999 17:51'!
fileInSelections 
	super fileInSelections.
	self reformulateList! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sd 11/20/2005 21:27'!
findOriginalChangeSet
	| changeSet |
	self currentChange ifNil: [^ self].
	changeSet := self currentChange originalChangeSetForSelector: self selectedMessageName.
	changeSet = #sources ifTrue:
		[^ self inform: 'This version is in the .sources file.'].
	changeSet ifNil:
		[^ self inform: 'This version was not found in any changeset nor in the .sources file.'].
	(ChangeSorter new myChangeSet: changeSet) open! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 10/12/1999 22:49'!
offerVersionsHelp
	(StringHolder new contents: self versionsHelpString)
		openLabel: 'Versions Browsers'! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'sd 11/20/2005 21:27'!
reformulateList
	| aMethod |
	"Some uncertainty about how to deal with lost methods here"
	aMethod := classOfMethod compiledMethodAt: selectorOfMethod ifAbsent: [^ self].
	
	self scanVersionsOf: aMethod class: classOfMethod theNonMetaClass meta: classOfMethod isMeta category: (classOfMethod whichCategoryIncludesSelector: selectorOfMethod) selector: selectorOfMethod.
	self changed: #list. "for benefit of mvc"
	listIndex := 1.
	self changed: #listIndex.
	self contentsChanged
! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sd 5/23/2003 14:50'!
removeMethodFromChanges
	"Remove my method from the current change set"

	ChangeSet current removeSelectorChanges: selectorOfMethod class: classOfMethod.
	self changed: #annotation
! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'marcus.denker 11/10/2008 10:04'!
scanVersionsOf: method class: class meta: meta category: category selector: selector
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp cat |
	selectorOfMethod := selector.
	currentCompiledMethod := method.
	classOfMethod := meta ifTrue: [class class] ifFalse: [class].
	cat := category ifNil: [''].
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	self addedChangeRecord ifNotNil: [ :change |
		self addItem: change text: ('{1} (in {2})' translated format: { change stamp. change fileName }) ].
	listIndex := 0.
	position := method filePosition.
	sourceFilesCopy := SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	method fileIndex == 0 ifTrue: [^ nil].
	file := sourceFilesCopy at: method fileIndex.

	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  "Skip back to before the preamble"
		preamble := method getPreambleFrom: file at: (0 max: position - 3).

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos := nil.
		stamp := ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble]
			ifFalse: [tokens := Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size-2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos := tokens at: tokens size-2.
						prevFileIndex := tokens last].
				cat := tokens at: tokens size-4.
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size].
				cat := tokens at: tokens size-2].
 		self addItem:
				(ChangeRecord new file: file position: position type: #method
						class: class name category: category meta: meta stamp: stamp)
			text: stamp , ' ' , class name , (meta ifTrue: [' class '] ifFalse: [' ']) , selector, ' {', cat, '}'.
		position := prevPos.
		prevPos notNil ifTrue:
			[file := sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections := Array new: list size withAll: false! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 07:03'!
selectedClass
	"Answer the class currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod theNonMetaClass! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 06:26'!
selectedClassOrMetaClass
	"Answer the class or metaclass currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 06:10'!
selectedMessageName
	"Answer the message name currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ selectorOfMethod! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 10/19/1999 15:04'!
showsVersions
	^ true! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'sd 11/20/2005 21:27'!
updateListsAndCodeIn: aWindow
	| aMethod |
	aMethod := classOfMethod compiledMethodAt: selectorOfMethod ifAbsent: [^ false].
	aMethod == currentCompiledMethod
		ifFalse:
			["Do not attempt to formulate if there is no source pointer.
			It probably means it has been recompiled, but the source hasn't been written
			(as during a display of the 'save text simply?' confirmation)."
			aMethod last ~= 0 ifTrue: [self reformulateList]].
	^ true
! !

!VersionsBrowser methodsFor: 'menu' stamp: 'nk 11/25/2003 10:19'!
versionsHelpString
	^ 'Each entry in the list pane represents a version of the source code for the same method; the topmost entry is the current version, the next entry is the next most recent, etc.

To revert to an earlier version, select it (in the list pane) and then do any of the following:
  *  Choose "revert to this version" from the list pane menu.
  *  Hit the "revert" button,
  *  Type ENTER in the code pane
  *  Type cmd-s (alt-s) in the code pane.

The code pane shows the source for the selected version.  If "diffing" is in effect, then differences betwen the selected version and the version before it are pointed out in the pane.  Turn diffing on and off by choosing "toggle diffing" from the list pane menu, or hitting the "diffs" button, or hitting cmd-D when the cursor is over the list pane.

To get a comparison between the selected version and the current version, choose "compare to current" from the list pane menu or hit the "compare to current" button.  (This is meaningless if the current version is selected, and is unnecessary if you''re interested in diffs from between the current version and the next-most-recent version, since the standard in-pane "diff" feature will give you that.)

You can also compare the selected version with any other version using the "compare to version..." menu choice.

If further versions of the method in question have been submitted elsewhere since you launched a particular Versions Browser, it will still stay nicely up-to-date if you''re in Morphic and have asked that smart updating be maintained; if you''re in mvc or in morphic but with smart-updating turned off, a versions browser is only brought up to date when you activate its window (and when you issue "revert" from within it, of course,) and you can also use the "update list" command to make certain the versions list is up to date.

Hit the "remove from changes" button, or choose the corresponding command in the list pane menu, to have the method in question deleted from the current change set.  This is useful if you''ve put debugging code into a method, and now want to strip it out and cleanse your current change set of all memory of the excursion.

Note:  the annotation pane in versions browsers shows information about the *current* version of the method in the image, not about the selected version.'! !

!VersionsBrowser methodsFor: 'menu' stamp: 'tbn 7/29/2010 21:57'!
versionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"

	aMenu title: 'Versions' translated.
	aMenu addStayUpItemSpecial.

	listIndex > 0 ifTrue:[
		(list size > 1 ) ifTrue: [ aMenu addTranslatedList: #(
			('Compare to current'		compareToCurrentVersion		'Compare selected version to the current version')
			('Compare to version...'	compareToOtherVersion		'Compare selected version to another selected version'))].
		"Note: Revert to selected should be visible for lists of length one for having the ability to revert to an accidentally deleted method"
		 aMenu addTranslatedList: #(
			('Revert to selected version'	fileInSelections					'Resubmit the selected version, so that it becomes the current version') )].

	aMenu addTranslatedList: #(
		('Remove from changes'		removeMethodFromChanges	'Remove this method from the current change set, if present')
		('Edit current method (O)'	openSingleMessageBrowser		'Open a single-message browser on the current version of this method')		
		('Find original change set'	findOriginalChangeSet			'Locate the changeset which originally contained this version')
		-
		('Toggle diffing (D)'			toggleDiffing					'Toggle whether or not diffs should be shown here')
		('Update list'				reformulateList					'Reformulate the list of versions, in case it somehow got out of synch with reality')
		-
		('Senders (n)'				browseSenders					'Browse all senders of this selector')
		('Implementors (m)'			browseImplementors			'Browse all implementors of this selector')
		-
		('Help...'					offerVersionsHelp				'Provide an explanation of the use of this tool')).
											
	^aMenu! !

CodeHolder subclass: #ChangeSorter
	instanceVariableNames: 'parent myChangeSet currentClassName currentSelector priorChangeSetList changeSetCategory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeSorter commentStamp: '<historical>' prior: 0!
I display a ChangeSet.  Two of me are in a DualChangeSorter.!

ChangeSorter subclass: #ChangeSetBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeSetBrowser commentStamp: '<historical>' prior: 0!
A tool allowing you to browse the methods of a single change set.!

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'tbn 7/6/2010 16:35'!
addModelItemsToWindowMenu: aMenu
	"Add model-related items to the given window menu"

	| oldTarget |
	oldTarget := aMenu defaultTarget.
	aMenu defaultTarget: self.
	aMenu addLine.
	aMenu add: 'Rename change set' action: #rename.
	aMenu add: 'Make changes go to me' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'File out' action: #fileOut.
	aMenu add: 'Browse methods' action: #browseChangeSet.
	aMenu addLine.
	myChangeSet hasPreamble
		ifTrue:
			[aMenu add: 'Edit preamble' action: #addPreamble.
			aMenu add: 'Remove preamble' action: #removePreamble]
		ifFalse:
			[aMenu add: 'Add preamble' action: #addPreamble].

	myChangeSet hasPostscript
		ifTrue:
			[aMenu add: 'Edit postscript...' action: #editPostscript.
			aMenu add: 'Remove postscript' action: #removePostscript]
		ifFalse:
			[aMenu add: 'Add postscript...' action: #editPostscript].
	aMenu addLine.
	
	aMenu add: 'Destroy change set' action: #remove.
	aMenu addLine.
	aMenu addLine.
	aMenu add: 'What to show...' target: self action: #offerWhatToShowMenu.
	aMenu addLine.
	aMenu add: 'More...' action: #offerShiftedChangeSetMenu.
	aMenu defaultTarget: oldTarget.

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'sw 3/12/2001 14:07'!
offerUnshiftedChangeSetMenu
	"The user chose 'more' from the shifted window menu; go back to the regular window menu"

	self containingWindow ifNotNil: [self containingWindow offerWindowMenu] ! !

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
openAsMorphIn: window rect: rect
	"Add a set of changeSetBrowser views to the given top view offset by the given amount"

	| aHeight |
	contents := ''.
	aHeight := 0.25.
	self addDependent: window.		"so it will get changed: #relabel"


	window addMorph: (PluggableListMorphByItem on: self
				list: #classList
				selected: #currentClassName
				changeSelected: #currentClassName:
				menu: #classListMenu:shifted:
				keystroke: #classListKey:from:)
		frame: (((0.0@0 extent: 0.5 @ aHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
				list: #messageList
				selected: #currentSelector
				changeSelected: #currentSelector:
				menu: #messageMenu:shifted:
				keystroke: #messageListKey:from:)
		frame: (((0.5@0 extent: 0.5 @ aHeight)
			scaleBy: rect extent) translateBy: rect origin).

	 self addLowerPanesTo: window
		at: (((0@aHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin)
		with: nil! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'StephaneDucasse 1/16/2010 11:11'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	aMenu title: 'Change set (shifted)'.
	aMenu addStayUpItemSpecial.
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.

	aMenu addLine.
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.


	Author fullNamePerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Author fullName, '"'.

		aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods any of whose previous authoring stamps do not start with "', Author fullName, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.

	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'sw 3/9/2001 15:02'!
wantsOptionalButtons
	"Sure, why not?"

	^ true! !

!ChangeSorter class methodsFor: 'browse' stamp: 'IgorStasenko 3/6/2011 18:22'!
browseChangeSetsWithClass: class selector: selector
	"Put up a menu comprising a list of change sets that hold changes for the given class and selector.  If the user selects one, open a single change-sorter onto it"

	| hits index |
	hits := ChangeSet allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ self inform: class name, '.', selector , '
is not in any change set'].
	index := hits size == 1
		ifTrue:	[1]
		ifFalse:	[(UIManager default chooseFrom: (hits collect: [:cs | cs name])
					lines: #())].
	index = 0 ifTrue: [^ self].
	(Smalltalk tools changeSorter new myChangeSet: (hits at: index)) open.
! !

!ChangeSorter class methodsFor: 'browse' stamp: 'StephaneDucasse 3/23/2010 21:20'!
browseChangeSetsWithSelector: aSelector
	"Put up a list of all change sets that contain an addition, deletion, or change of any method with the given selector"

	| hits index |
	hits := ChangeSet allChangeSets select: 
		[:cs | cs hasAnyChangeForSelector: aSelector].
	hits isEmpty ifTrue: [^ self inform: aSelector , '
is not in any change set'].
	index := hits size == 1
		ifTrue:	[1]
		ifFalse:	[(UIManager default chooseFrom: (hits collect: [:cs | cs name])
					lines: #())].
	index = 0 ifTrue: [^ self].
	(ChangeSetBrowser new myChangeSet: (hits at: index)) open

"ChangeSorter browseChangeSetsWithSelector: #clearPenTrails"
! !

!ChangeSorter class methodsFor: 'class initialization' stamp: 'StephaneDucasse 9/9/2010 09:56'!
initialize
	"ChangeSorter initialize"

	FileServices registerFileReader: self.
! !

!ChangeSorter class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:56'!
open
	^ self new morphicWindow openInWorld! !

!ChangeSorter class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightBlue
! !

!ChangeSorter class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #changeSorter
! !

!ChangeSorter class methodsFor: 'browse' stamp: 'StephaneDucasse 3/30/2010 23:04'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come all numbered updates.
		Next come all remaining changesets
	In a ChangeSorter, they will appear in the reversed order."

	"ChangeSorter reorderChangeSets"
	
	ChangeSet reorderChangeSets.
	SystemWindow wakeUpTopWindowUponStartup! !

!ChangeSorter class methodsFor: 'initialization' stamp: 'AlainPlantec 10/17/2009 16:42'!
unload
	"Unload the receiver from global registries"

	self environment at: #FileList ifPresent: [:cl |
	cl unregisterFileReader: self].
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sd 11/20/2005 21:26'!
addPreamble
	myChangeSet assurePreambleExists.
	self okToChange ifTrue:
		[currentClassName := nil.
		currentSelector := nil.
		self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'annotation' stamp: 'sw 2/22/2001 10:35'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"Add an annotation detailing the prior versions count.  Specially handled here for the case of a selector no longer in the system, whose prior version is pointed to by the lost-method pointer in the change held on to by the changeset"

	(aClass includesSelector: aSelector) ifTrue:
		[^ super addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream].
	aStream nextPutAll:
		((myChangeSet methodInfoFromRemoval: {aClass name. aSelector})
			ifNil:
				['no prior versions']
			ifNotNil:
				['version(s) retrievable here']), self annotationSeparator! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'IgorStasenko 3/6/2011 18:21'!
browseChangeSet
	"Open a message list browser on the new and changed methods in the current change set"

	Smalltalk tools changedMessageSet openFor: myChangeSet

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'IgorStasenko 3/6/2011 18:23'!
browseMethodConflicts
	"Check to see if any other change set also holds changes to any methods in the selected change set; if so, open a browser on all such."

	| aList |
	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			(ChangeSet allChangeSetsWithClass: aClass selector: aSelector) size > 1]
		ifNone: [^ self inform: 'No other change set has changes
for any method in this change set.'].
	
	Smalltalk tools messageSet 
		openMessageList: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in other change sets (', aList size printString, ')'
	! !

!ChangeSorter methodsFor: 'message list' stamp: 'Igor.Stasenko 12/20/2009 18:54'!
browseVersions
	"Create and schedule a changelist browser on the versions of the 
	selected message."
	| class selector method category pair sourcePointer |

	(selector := self selectedMessageName) ifNil: [^ self].
	class := self selectedClassOrMetaClass.
	(class includesSelector: selector)
		ifTrue: [method := class compiledMethodAt: selector.
				category := class whichCategoryIncludesSelector: selector.
				sourcePointer := nil]
		ifFalse: [pair := myChangeSet methodInfoFromRemoval: {class name. selector}.
				pair ifNil: [^ nil].
				sourcePointer := pair first.
				method := CompiledMethod toReturnSelfTrailerBytes:
					(CompiledMethodTrailer new sourcePointer: sourcePointer).
				category := pair last].
	VersionsBrowser
		browseVersionsOf: method
		class: self selectedClass meta: class isMeta
		category: category selector: selector
		lostMethodPointer: sourcePointer.
! !

!ChangeSorter methodsFor: 'toolbuilder' stamp: 'StephaneDucasse 2/13/2010 22:35'!
buildWith: builder
	"ToolBuilder open: ChangeSorter."
	
	|  windowSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec label: 'Change Sorter'.
	windowSpec model: self.
	windowSpec children: OrderedCollection new.
	self buildWith: builder in: windowSpec rect: (0@0 extent: 1@1).
	^builder build: windowSpec! !

!ChangeSorter methodsFor: 'toolbuilder' stamp: 'StephaneDucasse 2/13/2010 21:46'!
buildWith: builder in: window rect: rect
	| csListHeight msgListHeight csMsgListHeight listSpec textSpec |
	contents := ''.
	csListHeight := 0.25.
	msgListHeight := 0.25.
	csMsgListHeight := csListHeight + msgListHeight.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #changeSetList; 
		getSelected: #currentChangeSet; 
		setSelected: #showChangeSetNamed:; 
		menu: #changeSetMenu:shifted:; 
		keyPress: #changeSetListKey:from:;
		autoDeselect: false;
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getSelected: #currentClassName; 
		setSelected: #currentClassName:; 
		menu: #classListMenu:shifted:; 
		keyPress: #classListKey:from:;
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getSelected: #currentSelector;
		setSelected: #currentSelector:; 
		menu: #messageMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (((0@csListHeight extent: 1@msgListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin).
	window children add: textSpec.
	^window! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/29/1998 08:22'!
changeSet
	^ myChangeSet! !

!ChangeSorter methodsFor: 'access' stamp: 'sw 1/27/2000 11:19'!
changeSetCurrentlyDisplayed
	^ myChangeSet! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 22:57'!
changeSetList
	"Answer a list of ChangeSet names to be shown in the change sorter."

	|names| 
	names := (ChangeSet allChangeSets collect: [:each | each name]) reversed.
	names = 0 ifTrue: 
		[ ChangeSet assuredChangeSetNamed: 'New Changes'.
		names := #('New Changes')].
	^ names
	! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 17:21'!
changeSetListKey: aChar from: view
	"Respond to a Command key.  I am a model with a listView that has a list of changeSets."

	aChar == $b ifTrue: [^ self browseChangeSet].
	aChar == $B ifTrue: [^ self openChangeSetBrowser].
	aChar == $c ifTrue: [^ self copyAllToOther].
	aChar == $D ifTrue: [^ self toggleDiffing]. 
	aChar == $f ifTrue: [^ self findChangeSet].
	aChar == $m ifTrue: [^ self newCurrent].
	aChar == $n ifTrue: [^ self newSet].
	aChar == $o ifTrue: [^ self fileOut].
	aChar == $p ifTrue: [^ self addPreamble].
	aChar == $r ifTrue: [^ self rename].
	aChar == $x ifTrue: [^ self remove].
	aChar == $- ifTrue: [^ self subtractOtherSide].
	^ self messageListKey: aChar from: view! !

!ChangeSorter methodsFor: 'menu' stamp: 'MarcusDenker 4/16/2011 08:28'!
changeSetMenu: aMenu shifted: isShifted 
	"Set up aMenu to hold commands for the change-set-list pane.  This could be for a single or double changeSorter"

	isShifted ifTrue: [^ self shiftedChangeSetMenu: aMenu].
	aMenu title: 'Change Set'.
	aMenu addStayUpItemSpecial.
	aMenu add: 'Make changes go to me (m)' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'New change set... (n)' action: #newSet.
	aMenu add: 'Find...(f)' action: #findChangeSet.
	aMenu add: 'Select change set...' action: #chooseChangeSet.
	aMenu addLine.
	aMenu add: 'Rename change set (r)' action: #rename.
	aMenu add: 'File out (o)' action: #fileOut.
	aMenu add: 'Browse methods (b)' action: #browseChangeSet.
	aMenu add: 'Browse change set (B)' action: #openChangeSetBrowser.
	aMenu addLine.
	parent
		ifNotNil: 
			[aMenu add: 'Copy all to other side (c)' action: #copyAllToOther.
			aMenu add: 'Submerge into other side' action: #submergeIntoOtherSide.
			aMenu add: 'Subtract other side (-)' action: #subtractOtherSide.
			aMenu addLine].
	myChangeSet hasPreamble
		ifTrue: 
			[aMenu add: 'Edit preamble (p)' action: #addPreamble.
			aMenu add: 'Remove preamble' action: #removePreamble]
		ifFalse: [aMenu add: 'Add preamble (p)' action: #addPreamble].
	myChangeSet hasPostscript
		ifTrue: 
			[aMenu add: 'Edit postscript...' action: #editPostscript.
			aMenu add: 'Remove postscript' action: #removePostscript]
		ifFalse: [aMenu add: 'Add postscript...' action: #editPostscript].
	aMenu addLine.

	aMenu add: 'Destroy change set (x)' action: #remove.
	aMenu addLine.
	aMenu add: 'More...' action: #offerShiftedChangeSetMenu.
	^ aMenu! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 10/30/2000 10:48'!
checkForAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship"

	myChangeSet checkForAlienAuthorship

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'nk 3/30/2002 08:56'!
checkForAnyAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship, even historically"

	myChangeSet checkForAnyAlienAuthorship

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 3/29/2001 12:47'!
checkForUnclassifiedMethods
	"Open a message list browser on all methods in the current change set that have not been categorized"

	myChangeSet checkForUnclassifiedMethods

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 7/18/2002 17:58'!
checkForUncommentedClasses
	"Open a class list browser on classes in the change set that lack class comments"

	myChangeSet checkForUncommentedClasses! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 10/30/2000 10:39'!
checkForUncommentedMethods
	"Open a message list browser on all uncommented methods in the current change set"

	myChangeSet checkForUncommentedMethods

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 10/27/1999 14:20'!
checkForUnsentMessages
	"Open a message list browser on all unsent messages in the current change set"

	myChangeSet checkForUnsentMessages

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 7/8/1999 13:36'!
checkThatSidesDiffer: escapeBlock
	"If the change sets on both sides of the dual sorter are the same, put up an error message and escape via escapeBlock, else proceed happily"

	(myChangeSet == (parent other: self) changeSet)
		ifTrue:
			[self inform: 
'This command requires that the
change sets selected on the two
sides of the change sorter *not*
be the same.'.
			^ escapeBlock value]
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 16:09'!
chooseChangeSet
	"Present the user with an alphabetical list of change set names, and let her choose one"

	| changeSetsSortedAlphabetically chosen |
	self okToChange ifFalse: [^ self].

	changeSetsSortedAlphabetically := self changeSetList asSortedCollection:
		[:a :b | a asLowercase withoutLeadingDigits < b asLowercase withoutLeadingDigits].

	chosen := UIManager default 
					chooseFrom: changeSetsSortedAlphabetically 
					values: changeSetsSortedAlphabetically.
	chosen ifNil: [^ self].
	self showChangeSet: (ChangeSet named: chosen)! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/29/2001 15:19'!
classList
	"Computed.  View should try to preserve selections, even though index changes"

	^ myChangeSet ifNotNil: [myChangeSet changedClassNames] ifNil: [OrderedCollection new]
! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/5/2001 18:24'!
classListKey: aChar from: view
	"Respond to a Command key in the class-list pane."

	aChar == $x ifTrue: [^ self removeClass].
	aChar == $d ifTrue: [^ self forgetClass]. 

	^ self messageListKey: aChar from: view "picks up b,h,p"! !

!ChangeSorter methodsFor: 'menu' stamp: 'MarcusDenker 4/21/2011 22:27'!
classListMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the class list"

	aMenu title: 'Class list'.
	aMenu addStayUpItemSpecial.
	(parent notNil and: [shifted not])
		ifTrue: [aMenu addList: #( "These two only apply to dual change sorters"
			('Copy class chgs to other side'			copyClassToOther)	
			('Move class chgs to other side'			moveClassToOther))].

	aMenu addList: (shifted
		ifFalse: [#(
			-
			('Delete class from change set (d)'		forgetClass)
			('Remove class from system (x)'			removeClass)
			-
			('Browse full (b)'						browseMethodFull)
			('Browse hierarchy (h)'					spawnHierarchy)
			('Browse protocol (p)'					browseFullProtocol)
			-
			('FileOut'								fileOutClass)
			-
			('Inst var refs...'						browseInstVarRefs)
			('Inst var defs...'						browseInstVarDefs)
			('Class var refs...'						browseClassVarRefs)
			('Class vars'								browseClassVariables)
			('Class refs (N)'							browseClassRefs)
			-
			('More...'								offerShiftedClassListMenu))]

		ifTrue: [#(
			-
			('Unsent methods'						browseUnusedMethods)
			('Unreferenced inst vars'				showUnreferencedInstVars)
			('Unreferenced class vars'				showUnreferencedClassVars)
			-
			('Inspect instances'						inspectInstances)
			('Inspect subinstances'					inspectSubInstances)
			-
			('More...'								offerUnshiftedClassListMenu ))]).
	^ aMenu! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sd 11/20/2005 21:26'!
clearChangeSet
	"Clear out the current change set, after getting a confirmation."
	| message |

	self okToChange ifFalse: [^ self].
	myChangeSet isEmpty ifFalse:
		[message := 'Are you certain that you want to\forget all the changes in this set?' withCRs.
		(self confirm: message) ifFalse: [^ self]].
	myChangeSet clear.
	self changed: #classList.
	self changed: #messageList.
	self setContents.
	self contentsChanged.
! !

!ChangeSorter methodsFor: 'code pane' stamp: 'sd 11/20/2005 21:26'!
contents: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Create an error if the category of the selected message is unknown. 
	Answer false if the compilation fails. Otherwise, if the compilation 
	created a new method, deselect the current selection. Then answer true."
	| category selector class oldSelector |

	(class := self selectedClassOrMetaClass) ifNil:
		[(myChangeSet preambleString == nil or: [aString size == 0]) ifTrue: [ ^ false].
		(aString count: [:char | char == $"]) odd 
			ifTrue: [self inform: 'unmatched double quotes in preamble']
			ifFalse: [(Scanner new scanTokens: aString) size > 0 ifTrue: [
				self inform: 'Part of the preamble is not within double-quotes.
To put a double-quote inside a comment, type two double-quotes in a row.
(Ignore this warning if you are including a doIt in the preamble.)']].
		myChangeSet preambleString: aString.
		self currentSelector: nil.  "forces update with no 'unsubmitted chgs' feedback"
		^ true].
	oldSelector := self selectedMessageName.
	category := class organization categoryOfElement: oldSelector.
	selector := class compile: aString
				classified: category
				notifying: aController.
	selector ifNil: [^ false].
	(self messageList includes: selector)
		ifTrue: [self currentSelector: selector]
		ifFalse: [self currentSelector: oldSelector].
	self update.
	^ true! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sd 11/20/2005 21:26'!
copyAllToOther
	"Copy this entire change set into the one on the other side"
	| companionSorter |
	self checkThatSidesDiffer: [^ self].
	(companionSorter := parent other: self) changeSetCurrentlyDisplayed assimilateAllChangesFoundIn: myChangeSet.
	companionSorter changed: #classList.	"Later the changeSet itself will notice..."
	companionSorter changed: #messageList! !

!ChangeSorter methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
copyClassToOther
	"Place these changes in the other changeSet also"

	| otherSorter otherChangeSet |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ Beeper beep].
	currentClassName ifNil: [^ Beeper beep].
	otherSorter := parent other: self.
	otherChangeSet := otherSorter changeSet.

	otherChangeSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	otherSorter showChangeSet: otherChangeSet.! !

!ChangeSorter methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
copyMethodToOther
	"Place this change in the other changeSet also"
	| other cls sel |
	self checkThatSidesDiffer: [^ self].
	currentSelector ifNotNil:
		[other := (parent other: self) changeSet.
		cls := self selectedClassOrMetaClass.
		sel := currentSelector asSymbol.

		other absorbMethod: sel class: cls from: myChangeSet.
		(parent other: self) showChangeSet: other]
! !

!ChangeSorter methodsFor: 'access' stamp: 'StephaneDucasse 2/13/2010 21:46'!
currentChangeSet
	^ myChangeSet name! !

!ChangeSorter methodsFor: 'class list' stamp: 'tk 4/24/1998 09:14'!
currentClassName

	^ currentClassName! !

!ChangeSorter methodsFor: 'class list' stamp: 'AlainPlantec 11/13/2009 23:16'!
currentClassName: aString

	currentClassName := aString.
	currentSelector := nil.	"fix by wod"
	self changed: #currentClassName.
	self changed: #messageList.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'message list' stamp: 'tk 4/24/1998 09:15'!
currentSelector

	^ currentSelector! !

!ChangeSorter methodsFor: 'message list' stamp: 'AlainPlantec 11/13/2009 23:16'!
currentSelector: messageName

	currentSelector := messageName.
	self changed: #currentSelector.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'tk 4/28/1998 08:06'!
editPostscript
	"Allow the user to edit the receiver's change-set's postscript -- in a separate window"

	myChangeSet editPostscript! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'tk 4/28/1998 08:06'!
editPreamble
	"Allow the user to edit the receiver's change-set's preamble -- in a separate window."

	myChangeSet editPreamble! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'janniklaval 6/11/2010 13:15'!
fileIntoNewChangeSet
	"Obtain a file designation from the user, and file its contents into a  
	new change set whose name is a function of the filename. Show the  
	new set and leave the current changeSet unaltered."
	| aNewChangeSet stream file |
	self okToChange
		ifFalse: [^ self].
	ChangeSet promptForDefaultChangeSetDirectoryIfNecessary.
	file := UIManager default chooseFileMatching: nil.
	file
		ifNil: [^ self].
	stream := FileStream fileNamed: file.
	aNewChangeSet := ChangeSet
				newChangesFromStream: stream
				named: (FileDirectory localNameFor: stream name).
	aNewChangeSet
		ifNotNil: [self showChangeSet: aNewChangeSet]! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'tk 6/10/1999 12:44'!
fileOut
	"File out the current change set."

	myChangeSet fileOut.
	parent modelWakeUp.	"notice object conversion methods created"
! !

!ChangeSorter methodsFor: 'class list' stamp: 'StephaneDucasse 4/1/2010 09:48'!
fileOutClass
	"this is a hack!!!! makes a new change set, called the class name, adds author initials to try to make a unique change set name, files it out and removes it. kfr 16 june 2000" 
	| aSet |
	"File out the selected class set."
     aSet := ChangeSet newChangeSet: currentClassName.
	aSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	aSet fileOut.
	ChangeSet removeChangeSet: aSet.
	parent modelWakeUp.	"notice object conversion methods created"

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/3/2010 13:45'!
findChangeSet 
	"Search for a changeSet by name.  Pop up a menu of all changeSets whose name contains the string entered by the user.  If only one matches, then the pop-up menu is bypassed"
	| index pattern candidates nameList |
	self okToChange ifFalse: [^ self].
	pattern := UIManager default request: 'ChangeSet name or fragment?'.
	pattern isEmptyOrNil ifTrue: [^ self].
	nameList := self changeSetList asSet.
	candidates := ChangeSet allChangeSets select:
			[:c | (nameList includes: c name) and: 
				[c name includesSubstring: pattern caseSensitive: false]].
	candidates size = 0 ifTrue: [^ Beeper beep].
	candidates size = 1 ifTrue:
		[^ self showChangeSet: candidates first].
	index := UIManager default chooseFrom: (candidates collect: [:each | each name]).
	index = 0 ifFalse: [self showChangeSet: (candidates at: index)].
! !

!ChangeSorter methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
forget
	"Drop this method from the changeSet"

	self okToChange ifFalse: [^ self].
	currentSelector ifNotNil: [
		myChangeSet removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
		currentSelector := nil.
		self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
forgetClass
	"Remove all mention of this class from the changeSet"

	self okToChange ifFalse: [^ self].
	currentClassName ifNotNil: [
		myChangeSet removeClassChanges: currentClassName.
		currentClassName := nil.
		currentSelector := nil.
		self showChangeSet: myChangeSet].
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 1/10/1999 01:01'!
inspectChangeSet
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet inspectWithLabel: 'Change set: ', myChangeSet name

! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/30/1998 13:37'!
label
	^ self labelString! !

!ChangeSorter methodsFor: 'access' stamp: 'StephaneDucasse 3/29/2010 17:30'!
labelString
	"The label for my entire window.  The large button that displays my name is gotten via mainButtonName"

	^ String streamContents:
		[:aStream |
			aStream nextPutAll: (ChangeSet current == myChangeSet
				ifTrue: ['Changes go to "', myChangeSet name, '"']
				ifFalse: ['ChangeSet: ', myChangeSet name]).]! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 2/17/1999 11:05'!
lookForSlips
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet lookForSlips

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'tk 4/24/1998 13:10'!
mainButtonName

	^ myChangeSet name! !

!ChangeSorter methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
messageList 

	| probe newSelectors |
	currentClassName ifNil: [^ #()].
	probe := (currentClassName endsWith: ' class')
		ifTrue: [currentClassName]
		ifFalse: [currentClassName asSymbol].
	newSelectors := myChangeSet selectorsInClass: probe.
	(newSelectors includes: currentSelector) ifFalse: [currentSelector := nil].
	^ newSelectors asSortedCollection
! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/5/2001 18:30'!
messageListKey: aChar from: view
	"Respond to a Command key in the message-list pane."

	aChar == $d ifTrue: [^ self forget].
	super messageListKey: aChar from: view! !

!ChangeSorter methodsFor: 'menu' stamp: 'sw 3/9/2001 14:27'!
messageListMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	^ self messageMenu: aMenu shifted: shifted! !

!ChangeSorter methodsFor: 'menu' stamp: 'tbn 7/29/2010 22:04'!
messageMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	shifted ifTrue: [^ self shiftedMessageMenu: aMenu].

	aMenu title: 'message list'.
	aMenu addStayUpItemSpecial.

	parent ifNotNil:
		[aMenu addList: #(
			('Copy method to other side'			copyMethodToOther)
			('Move method to other side'			moveMethodToOther))].

	aMenu addList: #(
			('Delete method from changeSet (d)'	forget)
			-
			('Remove method from system (x)'	removeMessage)
				-
			('Browse full (b)'					browseMethodFull)
			('Browse hierarchy (h)'				spawnHierarchy)
			('Browse method (O)'				openSingleMessageBrowser)
			('Browse protocol (p)'				browseFullProtocol)
			-
			('FileOut'							fileOutMessage)
			-
			('Senders of... (n)'					browseSendersOfMessages)
			('Implementors of... (m)'				browseMessages)
			('Inheritance (i)'					methodHierarchy)
			('Versions (v)'						browseVersions)
			-
			('More...'							shiftedYellowButtonActivity)).
	^ aMenu
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'IgorStasenko 3/6/2011 18:21'!
methodConflictsWithOtherSide
	"Check to see if the change set on the other side shares any methods with the selected change set; if so, open a browser on all such."

	| aList other |

	self checkThatSidesDiffer: [^ self].
	other := (parent other: self) changeSet.
	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			aClass notNil and: [(other methodChangesAtClass: aClass name) includesKey: aSelector]
		]
		ifNone:  [^ self inform: 'There are no methods that appear
both in this change set and
in the one on the other side.'].
	
	Smalltalk tools messageSet 
		openMessageList: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in ', other name,' (', aList size printString, ')'
	! !

!ChangeSorter methodsFor: 'access' stamp: 'sma 11/11/2000 23:28'!
modelWakeUp
	"A window with me as model is being entered.
	Make sure I am up-to-date with the changeSets."

	self canDiscardEdits ifTrue: [self update]! !

!ChangeSorter methodsFor: 'opening' stamp: 'StephaneDucasse 5/1/2011 17:22'!
morphicWindow
	"Create a morphic widget so that we can interact with the receiver."
	
	|  window |
	myChangeSet ifNil: [self myChangeSet: ChangeSet current]. 
	window := (SystemWindow labelled: self labelString) model: self.
	self openAsMorphIn: window rect: (0@0 extent: 1@1).
	^ window
! !

!ChangeSorter methodsFor: 'class list' stamp: 'nb 6/17/2003 12:25'!
moveClassToOther
	"Place class changes in the other changeSet and remove them from this one"

	self checkThatSidesDiffer: [^ self].
	(self okToChange and: [currentClassName notNil]) ifFalse: [^ Beeper beep].

	self copyClassToOther.
	self forgetClass! !

!ChangeSorter methodsFor: 'message list' stamp: 'nk 7/30/2004 17:58'!
moveMethodToOther
	"Place this change in the other changeSet and remove it from this side"

	| other cls sel |
	self checkThatSidesDiffer: [^self].
	self okToChange ifFalse: [^Beeper beep].
	currentSelector ifNotNil: 
			[other := (parent other: self) changeSet.
			other == myChangeSet ifTrue: [^Beeper  beep].
			cls := self selectedClassOrMetaClass.
			sel := currentSelector asSymbol.
			other 
				absorbMethod: sel
				class: cls
				from: myChangeSet.
			(parent other: self) showChangeSet: other.
			self forget	"removes the method from this side"]! !

!ChangeSorter methodsFor: 'access' stamp: 'sd 11/20/2005 21:26'!
myChangeSet: anObject
	myChangeSet := anObject! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sd 5/23/2003 15:15'!
newCurrent
	"make my change set be the current one that changes go into"

	ChangeSet  newChanges: myChangeSet.
	self update.  "Because list of changes in a category may thus have changed"
	self changed: #relabel.! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/30/2010 22:45'!
newSet
	"Create a new changeSet and show it., making it the current one.  Reject name if already in use."

	| aSet |
	self okToChange ifFalse: [^ self].
	aSet := ChangeSet newChangeSet.
	aSet ifNotNil:
		[self update.
		self showChangeSet: aSet.
		self changed: #relabel]! !

!ChangeSorter methodsFor: 'menu' stamp: 'sw 2/27/2001 21:55'!
offerShiftedChangeSetMenu
	"Offer the shifted version of the change set menu"

	self offerMenuFrom: #changeSetMenu:shifted: shifted: true! !

!ChangeSorter methodsFor: 'menu' stamp: 'sw 3/6/2001 14:41'!
offerUnshiftedChangeSetMenu
	"Offer the unshifted version of the change set menu"

	self offerMenuFrom: #changeSetMenu:shifted: shifted: false! !

!ChangeSorter methodsFor: 'may be obsolete' stamp: 'alain.plantec 5/30/2008 11:08'!
open
	"ChangeSorter new open"
	^ self openAsMorph! !

!ChangeSorter methodsFor: 'may be obsolete' stamp: 'JuanVuletich 10/21/2010 17:07'!
openAsMorph
	"ChangeSorter new openAsMorph"
	^ UIManager default openChangeSorterAsMorph: self! !

!ChangeSorter methodsFor: 'may be obsolete' stamp: 'StephaneDucasse 2/13/2010 21:46'!
openAsMorphIn: window rect: rect
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 0.5@0."

	| csListHeight msgListHeight csMsgListHeight |
	contents := ''.
	csListHeight := 0.25.
	msgListHeight := 0.25.
	csMsgListHeight := csListHeight + msgListHeight.
	self addDependent: window.		"so it will get changed: #relabel"
	
"The method SystemWindow>>addMorph:fullFrame: checks scrollBarsOnRight, then adds the morph at the back if true, otherwise it is added in front. But flopout hScrollbars needs the crrentSelector pane to be behind the upper ones in the draw order. Hence the value of scrollBarsOnRight affects the order in which the lowerpanes are added."
	window theme settings scrollBarsOnRight ifFalse:
		[window addMorph: (PluggableListMorphByItem on: self
					list: #messageList
					selected: #currentSelector
					changeSelected: #currentSelector:
					menu: #messageMenu:shifted:
					keystroke: #messageListKey:from:)
			frame: (((0@csListHeight extent: 1@msgListHeight)
				scaleBy: rect extent) translateBy: rect origin)].

	window addMorph: ((PluggableListMorphByItem on: self
				list: #changeSetList
				selected: #currentChangeSet
				changeSelected: #showChangeSetNamed:
				menu: #changeSetMenu:shifted:
				keystroke: #changeSetListKey:from:)
			autoDeselect: false)
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
				list: #classList
				selected: #currentClassName
				changeSelected: #currentClassName:
				menu: #classListMenu:shifted:
				keystroke: #classListKey:from:)
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window theme settings scrollBarsOnRight ifTrue:
		[window addMorph: (PluggableListMorphByItem on: self
					list: #messageList
					selected: #currentSelector
					changeSelected: #currentSelector:
					menu: #messageMenu:shifted:
					keystroke: #messageListKey:from:)
			frame: (((0@csListHeight extent: 1@msgListHeight)
				scaleBy: rect extent) translateBy: rect origin)].

	 self addLowerPanesTo: window
		at: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin)
		with: nil.
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'alain.plantec 5/30/2008 11:10'!
openChangeSetBrowser
	"Open a ChangeSet browser on the current change set"

	(ChangeSetBrowser new myChangeSet: myChangeSet) openAsMorph! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/24/1998 08:42'!
parent
	^ parent! !

!ChangeSorter methodsFor: 'access' stamp: 'sd 11/20/2005 21:26'!
parent: anObject
	parent := anObject! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 15:50'!
promoteToTopChangeSet
	"Move the selected change-set to the top of the list"

	ChangeSet promoteToTop: myChangeSet.
	(parent ifNil: [self]) modelWakeUp! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'di 6/14/1998 12:00'!
remove
	"Completely destroy my change set.  Check if it's OK first"

	self okToChange ifFalse: [^ self].
	self removePrompting: true.
	self update! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 17:35'!
removeContainedInClassCategories
	| matchExpression |
	myChangeSet removePreamble.
	matchExpression :=  UIManager default request: 'Enter class category name (wildcard is ok)' initialAnswer: 'System-*'. 
	matchExpression ifNil: [^ self].
	(Smalltalk globals organization categories
		select: [:each | matchExpression match: each])
		do: [:eachCat | 
			| classNames | 
			classNames := Smalltalk globals organization listAtCategoryNamed: eachCat.
			classNames
				do: [:eachClassName | 
					myChangeSet removeClassChanges: eachClassName.
					myChangeSet removeClassChanges: eachClassName , ' class'].
			self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
removeFromCurrentChanges
	"Redisplay after removal in case we are viewing the current changeSet"

	super removeFromCurrentChanges.
	currentSelector := nil.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'message list' stamp: 'sd 11/20/2005 21:26'!
removeMessage
	"Remove the selected msg from the system. Real work done by the 
	parent, a ChangeSorter"
	| confirmation sel |
	self okToChange
		ifFalse: [^ self].
	currentSelector
		ifNotNil: [confirmation := self systemNavigation   confirmRemovalOf: (sel := self selectedMessageName) on: self selectedClassOrMetaClass.
			confirmation == 3
				ifTrue: [^ self].
			self selectedClassOrMetaClass removeSelector: sel.
			self update.
			confirmation == 2
				ifTrue: [self systemNavigation browseAllCallsOn: sel]]! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 6/29/1999 20:53'!
removePostscript
	(myChangeSet hasPostscript and: [myChangeSet postscriptHasDependents]) ifTrue:
		[^ self inform:
'Cannot remove the postscript right
now because there is at least one
window open on that postscript.
Close that window and try again.'].

	myChangeSet removePostscript.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 3/5/1999 19:32'!
removePreamble
	myChangeSet removePreamble.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/23/2010 22:33'!
removePrompting: doPrompt
	"Completely destroy my change set.  Check if it's OK first, and if doPrompt is true, get the user to confirm his intentions first."

	| message aName changeSetNumber msg |
	aName := myChangeSet name.
	myChangeSet okayToRemove ifFalse: [^ self]. "forms current changes for some project"
	(myChangeSet isEmpty or: [doPrompt not]) ifFalse:
		[message := 'Are you certain that you want to 
remove (destroy) the change set
named  "', aName, '" ?'.
		(self confirm: message) ifFalse: [^ self]].

	doPrompt ifTrue:
		[msg := myChangeSet hasPreamble
			ifTrue:
				[myChangeSet hasPostscript
					ifTrue:
						['a preamble and a postscript']
					ifFalse:
						['a preamble']]
			ifFalse:
				[myChangeSet hasPostscript
					ifTrue:
						['a postscript']
					ifFalse:
						['']].
		msg isEmpty ifFalse:
			[(self confirm: 
'Caution!!  This change set has
', msg, ' which will be
lost if you destroy the change set.
Do you really want to go ahead with this?') ifFalse: [^ self]]].

	"Go ahead and remove the change set"
	changeSetNumber := myChangeSet name initialIntegerOrNil.
	changeSetNumber ifNotNil: [SystemVersion current unregisterUpdate: changeSetNumber].
	ChangeSet removeChangeSet: myChangeSet.
	self showChangeSet: ChangeSet current.! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/29/2010 16:09'!
rename
	"Store a new name string into the selected ChangeSet.  reject duplicate name; allow user to back out"

	| newName |
	newName := UIManager default request: 'New name for this change set'
						initialAnswer: myChangeSet name.
	(newName = myChangeSet name or: [newName isEmptyOrNil]) ifTrue:
			[^ Beeper beep].

	(ChangeSet named: newName) ifNotNil:
			[^ UIManager default inform: 'Sorry that name is already used'].

	myChangeSet name: newName.
	self update.
	self changed: #mainButtonName.
	self changed: #relabel.! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/31/2010 16:41'!
reorderChangeSets
	"apply a standard reordering -- let the class handle this"

	^ self class reorderChangeSets! !

!ChangeSorter methodsFor: 'class list' stamp: 'sd 11/20/2005 21:26'!
selectedClass
	"Answer the currently-selected class.  If there is no selection, or if the selection refers to a class no longer extant, return nil"
	| c |
	^ currentClassName ifNotNil: [(c := self selectedClassOrMetaClass)
		ifNotNil: [c theNonMetaClass]]! !

!ChangeSorter methodsFor: 'traits' stamp: 'lr 3/14/2010 21:13'!
selectedClassOrMetaClass
	"Careful, the class may have been removed!!"

	| cName tName |
	currentClassName ifNil: [ ^ nil ].
	(currentClassName endsWith: ' class')
		ifTrue: [ 
			cName := (currentClassName copyFrom: 1 to: currentClassName size - 6) asSymbol.
			^ (Smalltalk globals at: cName ifAbsent: [ ^ nil ]) class ].
	(currentClassName endsWith: ' classTrait')
		ifTrue: [ 
			tName := (currentClassName copyFrom: 1 to: currentClassName size - 11) asSymbol.
			^ (Smalltalk globals at: tName ifAbsent: [ ^ nil ]) classTrait ].
	cName := currentClassName asSymbol.
	^ Smalltalk globals at: cName ifAbsent: [ nil ]! !

!ChangeSorter methodsFor: 'message list' stamp: 'jm 5/4/1998 07:32'!
selectedMessageName

	currentSelector ifNil: [^ nil].
	^ currentSelector asSymbol! !

!ChangeSorter methodsFor: 'code pane' stamp: 'PeterHugossonMiller 9/3/2009 00:52'!
setContents
	"return the source code that shows in the bottom pane"
	| sel class strm changeType |
	self clearUserEditFlag.
	currentClassName
		ifNil: [^ contents := myChangeSet preambleString
						ifNil: ['']].
	class := self selectedClassOrMetaClass.
	(sel := currentSelector) == nil
		ifTrue: [strm := (String new: 100) writeStream.
			(myChangeSet classChangeAt: currentClassName)
				do: [:each | 
					each = #remove
						ifTrue: [strm nextPutAll: 'Entire class was removed.';
								 cr].
					each = #addedThenRemoved
						ifTrue: [strm nextPutAll: 'Class was added then removed.'].
					each = #rename
						ifTrue: [strm nextPutAll: 'Class name was changed.';
								 cr].
					each = #add
						ifTrue: [strm nextPutAll: 'Class definition was added.';
								 cr].
					each = #change
						ifTrue: [strm nextPutAll: 'Class definition was changed.';
								 cr].
					each = #reorganize
						ifTrue: [strm nextPutAll: 'Class organization was changed.';
								 cr].
					each = #comment
						ifTrue: [strm nextPutAll: 'New class comment.';
								 cr]].
			^ contents := strm contents]
		ifFalse: [changeType := myChangeSet atSelector: (sel := sel asSymbol) class: class.
			changeType == #remove
				ifTrue: [^ contents := 'Method has been removed (see versions)'].
			changeType == #addedThenRemoved
				ifTrue: [^ contents := 'Added then removed (see versions)']. 
			class
				ifNil: [^ contents := 'Method was added, but cannot be found!!'].
			(class includesSelector: sel)
				ifFalse: [^ contents := 'Method was added, but cannot be found!!'].
			contents := class sourceCodeAt: sel.
			(#(#prettyPrint #prettyDiffs ) includes: contentsSymbol)
				ifTrue: [contents := class prettyPrinterClass
								format: contents
								in: class
								notifying: nil].
			self showingAnyKindOfDiffs
				ifTrue: [contents := self diffFromPriorSourceFor: contents].
			^ contents := contents asText makeSelectorBoldIn: class]! !

!ChangeSorter methodsFor: 'menu' stamp: 'StephaneDucasse 3/29/2010 22:58'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	aMenu title: 'Change set (shifted)'.
	aMenu addStayUpItemSpecial.

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.
	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' action: #methodConflictsWithOtherSide.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'].
	aMenu addLine.

	"CHECKS SECTION"
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.

	Author fullNamePerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Author fullName, '"'.

	aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
	aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Author fullName, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'promote to top of list' action: #promoteToTopChangeSet.
	aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	aMenu add: 'remove contained in class categories...' action: #removeContainedInClassCategories.
	aMenu balloonTextForLastItem: ' Drops any changes in given class categories'.

	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.

	aMenu addLine.
	aMenu add: 'file into new...' action: #fileIntoNewChangeSet.
	aMenu balloonTextForLastItem: 
'Load a fileout from disk and place its changes into a new change set (seldom needed -- much better to do this from a file-list browser these days.)'.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'.

	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSorter methodsFor: 'message list' stamp: 'MarcusDenker 4/21/2011 22:27'!
shiftedMessageMenu: aMenu
	"Arm the menu so that it holds items appropriate to the message-list while the shift key is down.  Answer the menu."

	^ aMenu addList: #(
		-
		('Toggle diffing (D)'					toggleDiffing)
		('Implementors of sent messages'		browseAllMessages)
		('Change category...'				changeCategory)
			-
		('Inspect instances'					inspectInstances)
		('Inspect subinstances'				inspectSubInstances)
		-
		('Change sets with this method'		findMethodInChangeSets)
		('Revert to previous version'			revertToPreviousVersion)
		('Revert & remove from changes'	revertAndForget)
		-
		('More...'							unshiftedYellowButtonActivity))! !

!ChangeSorter methodsFor: 'access' stamp: 'StephaneDucasse 2/13/2010 21:46'!
showChangeSet: chgSet

	myChangeSet == chgSet ifFalse: [
		myChangeSet := chgSet.
		currentClassName := nil.
		currentSelector := nil].
	self changed: #relabel.
	self changed: #currentChangeSet.	"new -- list of sets"
	self changed: #mainButtonName.	"old, button"
	self changed: #classList.
	self changed: #messageList.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'access' stamp: 'StephaneDucasse 3/29/2010 16:10'!
showChangeSetNamed: aName

	self showChangeSet: (ChangeSet named: aName) ! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sd 11/20/2005 21:26'!
submergeIntoOtherSide
	"Copy the contents of the receiver to the other side, then remove the receiver -- all after checking that all is well."
	| other message nextToView i all |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ self].
	other := (parent other: self) changeSet.
	other == myChangeSet ifTrue: [^ self inform: 'Both sides are the same!!'].
	myChangeSet isEmpty ifTrue: [^ self inform: 'Nothing to copy.  To remove,
simply choose "remove".'].

	myChangeSet okayToRemove ifFalse: [^ self].
	message := 'Please confirm:  copy all changes
in "', myChangeSet name, '" into "', other name, '"
and then destroy the change set
named "', myChangeSet name, '"?'.
 
	(self confirm: message) ifFalse: [^ self].

	(myChangeSet hasPreamble or: [myChangeSet hasPostscript]) ifTrue:
		[(self confirm: 
'Caution!!  This change set has a preamble or
a postscript or both.  If you submerge it into
the other side, these will be lost.
Do you really want to go ahead with this?') ifFalse: [^ self]].

	other assimilateAllChangesFoundIn: myChangeSet.
	all := ChangeSet allChangeSets.
	nextToView := ((all includes: myChangeSet)
		and: [(i := all indexOf: myChangeSet) < all size])
		ifTrue: [all at: i+1]
		ifFalse: [other].

	self removePrompting: false.
	self showChangeSet: nextToView.
	parent modelWakeUp.
! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'sw 7/8/1999 12:32'!
subtractOtherSide
	"Subtract the changes found on the other side from the requesting side."
	self checkThatSidesDiffer: [^ self].
	myChangeSet forgetAllChangesFoundIn: ((parent other: self) changeSet).
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'code pane' stamp: 'sw 11/13/2001 07:34'!
toggleDiffing
	"Toggle whether diffs should be shown in the code pane"

	self okToChange ifTrue:
		[super toggleDiffing.
		self changed: #contents.
		self update]

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'di 5/12/2000 15:03'!
trimHistory
	"Drop non-essential history (rename, reorg, method removals) from newly-added classes."

	myChangeSet trimHistory

! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'di 6/21/1998 13:02'!
update
	"recompute all of my panes"

	self updateIfNecessary.
	parent ifNotNil: [(parent other: self) updateIfNecessary]! !

!ChangeSorter methodsFor: 'changeset menu' stamp: 'StephaneDucasse 3/30/2010 22:51'!
updateIfNecessary
	"Recompute all of my panes."

	| newList |
	self okToChange ifFalse: [^ self].
	myChangeSet ifNil: [^ self].  "Has been known to happen though shouldn't"
	myChangeSet isMoribund ifTrue:
		[self changed: #changeSetList.
		^ self showChangeSet: ChangeSet current].

	newList := self changeSetList.

	(priorChangeSetList == nil or: [priorChangeSetList ~= newList])
		ifTrue:
			[priorChangeSetList := newList.
			self changed: #changeSetList].
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'creation' stamp: 'sd 11/20/2005 21:26'!
veryDeepFixupWith: deepCopier

	super veryDeepFixupWith: deepCopier.
	parent := deepCopier references at: parent ifAbsent: [parent].
	self updateIfNecessary! !

!ChangeSorter methodsFor: 'creation' stamp: 'sd 11/20/2005 21:26'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared."

super veryDeepInner: deepCopier.
"parent := parent.		Weakly copied"
"myChangeSet := myChangeSet.		Weakly copied"
currentClassName := currentClassName veryDeepCopyWith: deepCopier.
"currentSelector := currentSelector.		Symbol"
priorChangeSetList := priorChangeSetList veryDeepCopyWith: deepCopier.
changeSetCategory := changeSetCategory.

! !

!ChangeSorter methodsFor: 'code pane' stamp: 'JW 2/2/2001 21:41'!
wantsOptionalButtons
	"No optional buttons for ChangeSorter"
	^false! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/21/2009 22:20'!
annotationRequests
	"Answer a list of symbol characterizing all the available kinds of annotations; 
	Each symbol represents the info type:
		#timeStamp	: 
			The time stamp of the last submission of the method.
		#firstComment: 
			The first comment in the method, if any.
		#masterComment: 
			The comment at the beginning of the supermost implementor 
			of the method if any.
		#documentation: 
			Comment at beginning of the method or, if it has none, 
			comment at the beginning of a superclass's implementation of the method.
		#messageCategory: 
			Which method category the method lies in.
		#sendersCount: 
			A report of how many senders there of the message.
		#implementorsCount: 
			A report of how many implementors there are of the message.
		#recentChangeSet	: 
			The most recent change set bearing the method.
		#allChangeSets	: 
			A list of all change sets bearing the method.
		#priorVersionsCount: 
			A report of how many previous versions there are of the method.
		#priorTimeStamp: 
			The time stamp of the penultimate submission of the method, if any.
		"
	 ^ AnnotationRequests ifNil: [AnnotationRequests := self defaultAnnotationInfo]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/21/2009 22:21'!
annotationRequests: aList
	"see annotationRequests comment"
	 AnnotationRequests := aList! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 11/24/2009 17:18'!
browseWithPrettyPrint
	^ BrowseWithPrettyPrint ifNil: [BrowseWithPrettyPrint := false]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 11/24/2009 17:19'!
browseWithPrettyPrint: aBoolean
	BrowseWithPrettyPrint := aBoolean! !

!CodeHolder class methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 12/1/2009 22:55'!
decorateBrowserButtons
	^ DecorateBrowserButtons ifNil: [DecorateBrowserButtons := false]! !

!CodeHolder class methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 12/1/2009 22:55'!
decorateBrowserButtons: aBoolean
	DecorateBrowserButtons := aBoolean! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/21/2009 22:21'!
defaultAnnotationInfo
	"see annotationRequests comment"
	^ #(timeStamp messageCategory sendersCount implementorsCount allChangeSets)! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 21:50'!
diffsInChangeList
	^ DiffsInChangeList ifNil: [DiffsInChangeList := true]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 21:51'!
diffsInChangeList: aBoolean
	DiffsInChangeList := aBoolean! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 21:59'!
diffsWithPrettyPrint
	^ DiffsWithPrettyPrint ifNil: [DiffsWithPrettyPrint := false]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 21:58'!
diffsWithPrettyPrint: aBoolean
	DiffsWithPrettyPrint := aBoolean! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 22:45'!
optionalButtons
	^ OptionalButtons ifNil: [OptionalButtons := true]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 22:45'!
optionalButtons: aBoolean
	OptionalButtons := aBoolean! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 1/7/2010 21:49'!
showAnnotationPane
	"Answer whether the receiver, seen in some browser window, 
	would like to have the so-called  annotationpane included.
	see also annotationRequests comment"

	^ ShowAnnotationPane ifNil: [ShowAnnotationPane := false]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 11/24/2009 17:01'!
showAnnotationPane: aBoolean
	ShowAnnotationPane := aBoolean! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 22:34'!
smartUpdating
	^ SmartUpdating ifNil: [SmartUpdating := true]! !

!CodeHolder class methodsFor: 'settings' stamp: 'AlainPlantec 12/1/2009 22:34'!
smartUpdating: aBoolean
	SmartUpdating := aBoolean! !

!CodeHolder methodsFor: 'what to show' stamp: 'alain.plantec 5/30/2008 11:22'!
addContentsTogglesTo: aMenu 
	"Add updating menu toggles governing contents to aMenu."
	self contentsSymbolQuints
		do: [:aQuint | aQuint == #-
				ifTrue: [aMenu addLine]
				ifFalse: [aMenu
						addUpdating: aQuint third
						target: self
						action: aQuint second.
					aMenu balloonTextForLastItem: aQuint fifth]]! !

!CodeHolder methodsFor: 'construction' stamp: 'BenjaminVanRyseghem 1/29/2011 13:33'!
addLowerPanesTo: window at: nominalFractions with: editString

	| verticalOffset row innerFractions |
	self textMorph: (self buildMorphicCodePaneWith: editString).


	row := AlignmentMorph newColumn
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 1;
		borderColor: Color black;
		layoutPolicy: ProportionalLayout new.

	verticalOffset := 0.
	innerFractions := 0@0 corner: 1@0.
	verticalOffset := self addOptionalAnnotationsTo: row at: innerFractions plus: verticalOffset.
	verticalOffset := self addOptionalButtonsTo: row  at: innerFractions plus: verticalOffset.

	row 
		addMorph: (self textMorph borderWidth: 0)
		fullFrame: (
			LayoutFrame 
				fractions: (innerFractions withBottom: 1) 
				offsets: (0@verticalOffset corner: 0@0)
		).
	window 
		addMorph: row
		frame: nominalFractions.

	row on: #mouseEnter send: #paneTransition: to: window.
	row on: #mouseLeave send: #paneTransition: to: window.! !

!CodeHolder methodsFor: 'tiles' stamp: 'tbn 7/6/2010 16:36'!
addModelItemsToWindowMenu: aMenu
	"Add model-related item to the window menu"

	super addModelItemsToWindowMenu: aMenu. 
	aMenu addLine.
	aMenu add: 'What to show...' translated target: self action: #offerWhatToShowMenu! !

!CodeHolder methodsFor: 'annotation' stamp: 'AlainPlantec 1/7/2010 21:44'!
addOptionalAnnotationsTo: window at: fractions plus: verticalOffset
	"Add an annotation pane to the window if desired, and return the incoming verticalOffset plus the height of the added pane, if any"

	| aTextMorph divider delta  |
	self showAnnotationPane ifFalse: [^ verticalOffset].
	aTextMorph := CodeAnnotationMorph on: self.
	divider := BorderedSubpaneDividerMorph forBottomEdge.
	divider extent: 4@4; color: Color transparent; borderColor: #raised; borderWidth: 2.
	delta := StandardFonts defaultFont height + 2.
	window 
		addMorph: aTextMorph
		fullFrame: (LayoutFrame 
				fractions: fractions 
				offsets: (0@verticalOffset corner: 0@(verticalOffset + delta))).
	window 
		addMorph: divider
		fullFrame: (LayoutFrame 
				fractions: fractions 
				offsets: (0@(verticalOffset + delta - 2) corner: 0@(verticalOffset + delta))).
	^ verticalOffset + delta! !

!CodeHolder methodsFor: 'annotation' stamp: 'sd 11/20/2005 21:27'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"add an annotation detailing the prior versions count"
	| versionsCount |

	versionsCount := VersionsBrowser versionCountForSelector: aSelector class: aClass.
	aStream nextPutAll: 
				((versionsCount > 1
					ifTrue:
						[versionsCount == 2 ifTrue:
							['1 prior version']
							ifFalse:
								[versionsCount printString, ' prior versions']]
					ifFalse:
						['no prior versions']), self annotationSeparator)! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 5/23/2003 14:35'!
adoptMessageInCurrentChangeset
	"Add the receiver's method to the current change set if not already there"

	self setClassAndSelectorIn: [:cl :sel |
		cl ifNotNil:
			[ChangeSet current adoptSelector: sel forClass: cl.
			self changed: #annotation]]
! !

!CodeHolder methodsFor: 'annotation' stamp: 'sd 11/20/2005 21:27'!
annotation
	"Provide a line of content for an annotation pane, representing information about the method associated with the selected class and selector in the receiver."

	|  aSelector aClass |

	((aSelector := self selectedMessageName) == nil or: [(aClass := self selectedClassOrMetaClass) == nil])
		ifTrue: [^ '------'].
	^ self annotationForSelector: aSelector ofClass: aClass! !

!CodeHolder methodsFor: 'annotation' stamp: 'sd 11/20/2005 21:27'!
annotationForClassCommentFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the clas comment of the given class."

	| aStamp nonMeta |
	aStamp :=  (nonMeta := aClass theNonMetaClass) organization commentStamp.
	^ aStamp
		ifNil:
			[nonMeta name, ' has no class comment']
		ifNotNil:
			['class comment for ', nonMeta name,
				(aStamp = '<historical>'
					ifFalse:
						[' - ', aStamp]
					ifTrue:
						[''])]! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 8/26/2002 10:19'!
annotationForClassDefinitionFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the class definition of the given class."

	^ 'Class definition for ', aClass name! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 8/26/2002 10:19'!
annotationForHierarchyFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the hierarchy of the given class."

	^ 'Hierarchy for ', aClass name! !

!CodeHolder methodsFor: 'annotation' stamp: 'StephaneDucasse 3/29/2010 15:53'!
annotationForSelector: aSelector ofClass: aClass 
	"Provide a line of content for an annotation pane, representing  
	information about the given selector and class"
	| separator aStream requestList |
	aSelector == #Comment
		ifTrue: [^ self annotationForClassCommentFor: aClass].
	aSelector == #Definition
		ifTrue: [^ self annotationForClassDefinitionFor: aClass].
	aSelector == #Hierarchy
		ifTrue: [^ self annotationForHierarchyFor: aClass].
	aStream := ReadWriteStream on: ''.
	requestList := self annotationRequests.
	separator := requestList size > 1
				ifTrue: [self annotationSeparator]
				ifFalse: [''].
	requestList
		do: [:aRequest | | sendersCount aComment implementorsCount aString aList stamp aCategory | 
			aRequest == #firstComment
				ifTrue: [aComment := aClass firstCommentAt: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #masterComment
				ifTrue: [aComment := aClass supermostPrecodeCommentFor: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #documentation
				ifTrue: [aComment := aClass precodeCommentOrInheritedCommentFor: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #timeStamp
				ifTrue: [stamp := self timeStamp.
					aStream
						nextPutAll: (stamp size > 0
								ifTrue: [stamp , separator]
								ifFalse: ['no timeStamp' , separator])].
			aRequest == #messageCategory
				ifTrue: [aCategory := aClass organization categoryOfElement: aSelector.
					aCategory
						ifNotNil: ["woud be nil for a method no longer present,  
							e.g. in a recent-submissions browser"
							aStream nextPutAll: aCategory , separator]].
			aRequest == #sendersCount
				ifTrue: [sendersCount := (self systemNavigation allCallsOn: aSelector) size.
					sendersCount := sendersCount == 1
								ifTrue: ['1 sender']
								ifFalse: [sendersCount printString , ' senders'].
					aStream nextPutAll: sendersCount , separator].
			aRequest == #implementorsCount
				ifTrue: [implementorsCount := self systemNavigation numberOfImplementorsOf: aSelector.
					implementorsCount := implementorsCount == 1
								ifTrue: ['1 implementor']
								ifFalse: [implementorsCount printString , ' implementors'].
					aStream nextPutAll: implementorsCount , separator].
			aRequest == #priorVersionsCount
				ifTrue: [self
						addPriorVersionsCountForSelector: aSelector
						ofClass: aClass
						to: aStream].
			aRequest == #priorTimeStamp
				ifTrue: [stamp := VersionsBrowser
								timeStampFor: aSelector
								class: aClass
								reverseOrdinal: 2.
					stamp
						ifNotNil: [aStream nextPutAll: 'prior time stamp: ' , stamp , separator]].
			aRequest == #recentChangeSet
				ifTrue: [aString := ChangeSet mostRecentChangeSetWithChangeForClass: aClass selector: aSelector.
					aString size > 0
						ifTrue: [aStream nextPutAll: aString , separator]].
			aRequest == #allChangeSets
				ifTrue: [aList := ChangeSet allChangeSetsWithClass: aClass selector: aSelector.
					aList size > 0
						ifTrue: [aList size = 1
								ifTrue: [aStream nextPutAll: 'only in change set ']
								ifFalse: [aStream nextPutAll: 'in change sets: '].
							aList
								do: [:aChangeSet | aStream nextPutAll: aChangeSet name , ' ']]
						ifFalse: [aStream nextPutAll: 'in no change set'].
					aStream nextPutAll: separator]].
	^ aStream contents! !

!CodeHolder methodsFor: 'annotation' stamp: 'AlainPlantec 12/21/2009 22:23'!
annotationRequests
	^ self class annotationRequests! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 2/22/2001 10:02'!
annotationSeparator
	"Answer the separator to be used between annotations"

	^ '  '! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 11/20/2005 21:27'!
browseImplementors
	"Create and schedule a message set browser on all implementors of the currently selected message selector. Do nothing if no message is selected."

	| aMessageName |
	(aMessageName := self selectedMessageName) ifNotNil: 
		[self systemNavigation browseAllImplementorsOf: aMessageName]! !

!CodeHolder methodsFor: 'commands' stamp: 'nk 6/26/2003 21:43'!
browseSenders
	"Create and schedule a message set browser on all senders of the currently selected message selector.  Of there is no message currently selected, offer a type-in"

	self sendQuery: #browseAllCallsOn: to: self systemNavigation! !

!CodeHolder methodsFor: 'setting' stamp: 'AlainPlantec 11/27/2009 09:24'!
browseWithPrettyPrint
	^ self class browseWithPrettyPrint ! !

!CodeHolder methodsFor: 'construction' stamp: 'sd 11/20/2005 21:27'!
buildClassBrowserEditString: aString 
	"Create and schedule a new class browser for the current selection, with initial textual contents set to aString.  This is used specifically in spawning where a class is established but a method-category is not."

	| newBrowser  |
	newBrowser := Browser new.
	newBrowser setClass: self selectedClassOrMetaClass selector: nil.
	newBrowser editSelection: #newMessage.
	Browser openBrowserView: (newBrowser openOnClassWithEditString: aString)
			label: 'Class Browser: ', self selectedClassOrMetaClass name
! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'ar 2/12/2005 14:32'!
buildCodeProvenanceButtonWith: builder
	| buttonSpec |
	buttonSpec := builder pluggableActionButtonSpec new.
	buttonSpec model: self.
	buttonSpec label: #codePaneProvenanceString.
	buttonSpec action: #offerWhatToShowMenu.
	buttonSpec help: 'Governs what view is shown in the code pane.  Click here to change the view'.
	^buttonSpec! !

!CodeHolder methodsFor: 'construction' stamp: 'StephaneDucasse 2/14/2011 12:41'!
buildMorphicCodePaneWith: editString
	"Construct the pane that shows the code.
	Respect StandardFonts codeFont."

	| codePane |
	codePane := self buildTextMorph..
	editString
		ifNotNil: [codePane editString: editString.
			codePane hasUnacceptedEdits: true].
	^ codePane! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'nice 1/5/2010 15:59'!
buildOptionalButtonsWith: builder

	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self optionalButtonPairs do:[:spec| | buttonSpec |
		buttonSpec := builder pluggableActionButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		spec second == #methodHierarchy ifTrue:[
			buttonSpec color: #inheritanceButtonColor.
		]. 
		spec size > 2 ifTrue:[buttonSpec help: spec third].
		panelSpec children add: buttonSpec.
	].
	"What to show"
	panelSpec children add: (self buildCodeProvenanceButtonWith: builder).

	panelSpec layout: #horizontal. "buttons"
	^panelSpec! !

!CodeHolder methodsFor: 'builder' stamp: 'StephaneDucasse 2/14/2011 12:41'!
buildTextMorph 

	^(MorphicTextEditor default
				on: self
				text: #contents
				accept: #contents:notifying:
				readSelection: #contentsSelection
				menu: #codePaneMenu:shifted:)
			font: StandardFonts codeFont;
			beStyled;
			yourself! !

!CodeHolder methodsFor: 'controls' stamp: 'gm 2/16/2003 20:37'!
buttonWithSelector: aSelector 
	"If receiver has a control button with the given action selector answer it, else answer nil.  morphic only at this point"

	| aWindow aPane |
	((aWindow := self containingWindow) isSystemWindow) 
		ifFalse: [^nil].
	(aPane := aWindow submorphNamed: 'buttonPane') ifNil: [^nil].
	^aPane submorphThat: 
			[:m | 
			(m isKindOf: PluggableButtonMorph) and: [m actionSelector == aSelector]]
		ifNone: [^nil]! !

!CodeHolder methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:19'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ false! !

!CodeHolder methodsFor: 'categories' stamp: 'nice 1/5/2010 15:59'!
categoryFromUserWithPrompt: aPrompt for: aClass
	"self new categoryFromUserWithPrompt: 'testing' for: SystemDictionary"

	| labels myCategories reject lines newName menuIndex | 
	labels := OrderedCollection new.
	labels addAll: (myCategories := aClass organization categories asSortedCollection:
		[:a :b | a asLowercase < b asLowercase]).
	reject := myCategories asSet.
	reject
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection with: 1 with: (myCategories size + 1).

	aClass allSuperclasses do:
		[:cls | | cats |
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse:
				[lines add: labels size.
				labels addAll: (cats asSortedCollection:
					[:a :b | a asLowercase < b asLowercase]).
				reject addAll: cats]].

	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: aPrompt.
	^ newName ifNotNil: [newName asSymbol]! !

!CodeHolder methodsFor: 'categories' stamp: 'sd 11/20/2005 21:27'!
categoryOfCurrentMethod
	"Answer the category that owns the current method.  If unable to determine a category, answer nil."

	| aClass aSelector |
	^ (aClass := self selectedClassOrMetaClass) 
		ifNotNil: [(aSelector := self selectedMessageName) 
			            ifNotNil: [aClass whichCategoryIncludesSelector: aSelector]]! !

!CodeHolder methodsFor: 'categories' stamp: 'sd 11/20/2005 21:26'!
changeCategory
	"Present a menu of the categories of messages for the current class, 
	and let the user choose a new category for the current message"

	| aClass aSelector |
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[(aSelector := self selectedMessageName) ifNotNil:
			[(self letUserReclassify: aSelector in: aClass) ifTrue:
				["ChangeSet current reorganizeClass: aClass."
				"Decided on further review that the above, when present, could cause more
                    unexpected harm than good"
				self methodCategoryChanged]]]! !

!CodeHolder methodsFor: 'controls' stamp: 'gvc 1/20/2009 15:34'!
codePaneProvenanceIndex
	"Answer the selected code provenance index."
	
	^((self contentsSymbolQuints select: [:e | e ~= #-]) collect: [:e |
		e first]) indexOf: self contentsSymbol ifAbsent: [0]! !

!CodeHolder methodsFor: 'controls' stamp: 'gvc 1/20/2009 15:33'!
codePaneProvenanceIndex: anInteger
	"Set the code provenance to the item with the given index."
	
	self perform: ((self contentsSymbolQuints select: [:e | e ~= #-]) at: anInteger) second! !

!CodeHolder methodsFor: 'controls' stamp: 'gvc 1/20/2009 15:31'!
codePaneProvenanceList
	"Answer a list of the display strings for code provenance."
	
	^(self contentsSymbolQuints select: [:e | e ~= #-]) collect: [:e |
		e fourth]! !

!CodeHolder methodsFor: 'controls' stamp: 'sd 11/20/2005 21:26'!
codePaneProvenanceString
	"Answer a string that reports on code-pane-provenance"

	| symsAndWordings |
	(symsAndWordings := self contentsSymbolQuints) do:
		[:aQuad |
			contentsSymbol == aQuad first ifTrue: [^ aQuad fourth]].
	^ symsAndWordings first fourth "default to plain source, for example if nil as initially"! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'marcus.denker 8/17/2008 21:02'!
color

	| flags aColor |
	flags := 0.
	self isThisAnOverride ifTrue: [ flags := flags bitOr: 4 ].
	currentCompiledMethod sendsToSuper ifTrue: [ flags := flags bitOr: 2 ].
	self isThereAnOverride ifTrue: [ flags := flags bitOr: 1 ].
	aColor := {
		Color transparent.
		Color tan lighter.
		Color green muchLighter.
		Color blue muchLighter.
		Color red muchLighter.	"has super but doesn't call it"
		(Color r: 0.94 g: 0.823 b: 0.673).	"has sub; has super but doesn't call it"
		Color green muchLighter.
		Color blue muchLighter.
	} at: flags + 1.

	^aColor! !

!CodeHolder methodsFor: 'contents' stamp: 'sd 11/20/2005 21:27'!
commentContents
	"documentation for the selected method"

	| poss aClass aSelector |
	^ (poss := (aClass := self selectedClassOrMetaClass)
						ifNil:
							['----']
						ifNotNil:
							[(aSelector := self selectedMessageName)
								ifNil:
									['---']
								ifNotNil:
									[(aClass precodeCommentOrInheritedCommentFor: aSelector)", String cr, String cr, self timeStamp"
"which however misses comments that are between the temps  declaration and the body of the method; those are picked up by aClass commentOrInheritedCommentFor: aSelector but that method will get false positives from comments *anywhere* in the method source"]])
		isEmptyOrNil
			ifTrue:
				[aSelector
					ifNotNil:
						[((aClass methodHeaderFor: aSelector), '

Has no comment') asText makeSelectorBoldIn: aClass]
					ifNil:
						['Hamna']]
			ifFalse:	[aSelector
				ifNotNil: [((aClass methodHeaderFor: aSelector), '

', poss) asText makeSelectorBoldIn: aClass]
				ifNil: [poss]]! !

!CodeHolder methodsFor: 'contents' stamp: 'di 10/1/2001 22:25'!
contents
	"Answer the source code or documentation for the selected method"

	self showingByteCodes ifTrue:
		[^ self selectedBytecodes].

	self showingDocumentation ifTrue:
		[^ self commentContents].

	^ self selectedMessage! !

!CodeHolder methodsFor: 'contents' stamp: 'AlainPlantec 11/24/2009 17:20'!
contentsSymbol
	"Answer a symbol indicating what kind of content should be shown for the method; for normal showing of source code, this symbol is #source.  A nil value in the contentsSymbol slot will be set to #source by this method"

	^ contentsSymbol ifNil:
		[contentsSymbol := self browseWithPrettyPrint
								ifTrue:
									[#prettyPrint]
								ifFalse:
									[#source]]! !

!CodeHolder methodsFor: 'contents' stamp: 'gvc 6/21/2010 11:42'!
contentsSymbol: aSymbol
	"Set the contentsSymbol as indicated.  #source means to show source code, #comment means to show the first comment found in the source code"

	contentsSymbol := aSymbol.
	self contentsSymbolChanged! !

!CodeHolder methodsFor: 'contents' stamp: 'gvc 6/21/2010 11:42'!
contentsSymbolChanged
	"Inform any dependents of a change in the contents symbol."

	self
		changed: #showingAnyKindOfDiffs;
		changed: #showingBytecodes;
		changed: #showingDecompile;
		changed: #showingDiffs;
		changed: #showingDocumentation;
		changed: #showingPlainSource;
		changed: #showingPrettyDiffs;
		changed: #showingPrettyPrint;
		changed: #showingRegularDiffs! !

!CodeHolder methodsFor: 'controls' stamp: 'tbn 7/29/2010 22:17'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane
		first element:	the contentsSymbol used
		second element:	the selector to call when this item is chosen.
		third element:	the selector to call to obtain the wording of the menu item.
		fourth element:	the wording to represent this view
		fifth element:	balloon help
	A hypen indicates a need for a seperator line in a menu of such choices"

	^ #(
(source			togglePlainSource 			showingPlainSourceString	'Source'		'The textual source code as writen')
(documentation	toggleShowDocumentation	showingDocumentationString	'Documentation'		'The first comment in the method')
-
(prettyPrint		togglePrettyPrint 			prettyPrintString			'PrettyPrint'			'The method source presented in a standard text format')
-
(showDiffs		toggleRegularDiffing		showingRegularDiffsString	'ShowDiffs'				'The textual source diffed from its prior version')
(prettyDiffs		togglePrettyDiffing			showingPrettyDiffsString	'PrettyDiffs'		'Formatted textual source diffed from formatted form of prior version')
-
(decompile		toggleDecompile				showingDecompileString		'Decompile'			'Source code decompiled from byteCodes')
(byteCodes		toggleShowingByteCodes		showingByteCodesString		'ByteCodes'			'The bytecodes that comprise the compiled method'))! !

!CodeHolder methodsFor: 'commands' stamp: 'StephaneDucasse 2/25/2011 18:40'!
copyUpOrCopyDown
	"Used to copy down code from a superclass to a subclass or vice-versa in one easy step, if you know what you're doing.  Prompt the user for which class to copy down or copy up to, then spawn a fresh browser for that class, with the existing code planted in it, and with the existing method category also established."

	| aClass aSelector allClasses implementors aMenu |
	((aClass := self selectedClassOrMetaClass) isNil or: [(aSelector := self selectedMessageName) == nil]) 
		ifTrue:	[^ Beeper beep].

	allClasses := self systemNavigation hierarchyOfClassesSurrounding: aClass.
	implementors := self systemNavigation hierarchyOfImplementorsOf: aSelector forClass: aClass.
	aMenu := UIManager default
				 newMenuIn: self for: self.
	aMenu title: 
aClass name, '.', aSelector, '
Choose where to insert a copy of this method
(blue = current, black = available, red = other implementors'.
	allClasses do:
		[:cl | | aColor |
			aColor := cl == aClass
				ifTrue:	[#blue]
				ifFalse:
					[(implementors includes: cl)
						ifTrue:	[#red]
						ifFalse:	[#black]].
			(aColor == #red)
				ifFalse:
					[aMenu add: cl name selector: #spawnToClass: argument: cl]
				ifTrue:
					[aMenu add: cl name selector: #spawnToCollidingClass: argument: cl].
			aMenu lastItem color: (Color colorFrom: aColor)].
	aMenu popUpInWorld! !

!CodeHolder methodsFor: 'message list' stamp: 'md 2/22/2006 16:10'!
decompiledSourceIntoContents
	"Obtain a source string by decompiling the method's code, and place 
	that source string into my contents. Also return the string.
	Get temps from source file if shift key is pressed."
	
	|  class |
	class := self selectedClassOrMetaClass.
	"Was method deleted while in another project?"
	currentCompiledMethod := (class compiledMethodAt: self selectedMessageName ifAbsent: [^ '']).

	contents := (Sensor leftShiftDown not) 
		ifTrue: [currentCompiledMethod decompileWithTemps]
		ifFalse: [currentCompiledMethod decompile].
	contents := contents decompileString asText makeSelectorBoldIn: class.
	^ contents copy! !

!CodeHolder methodsFor: 'controls' stamp: 'ar 2/12/2005 14:28'!
decorateButtons
	"Change screen feedback for any buttons in the UI of the receiver that may wish it.  Initially, it is only the Inheritance button that is decorated, but one can imagine others."
	self changed: #inheritanceButtonColor.
	self decorateForInheritance ! !

!CodeHolder methodsFor: 'controls' stamp: 'AlainPlantec 12/1/2009 22:56'!
decorateForInheritance
	"Check to see if the currently-viewed method has a super send or an override, and if so, change screen feedback, unless the #decorateBrowserButtons says not to."

	| aButton |
	(aButton := self inheritanceButton) ifNil: [^ self].

	((currentCompiledMethod isKindOf: CompiledMethod) and: [self class decorateBrowserButtons])
		ifFalse: [^aButton offColor: Color transparent].

	"This table duplicates the old logic, but adds two new colors for the cases where there is a superclass definition, but this method doesn't call it."

	aButton offColor: self color! !

!CodeHolder methodsFor: 'diffs' stamp: 'AlainPlantec 12/1/2009 22:03'!
defaultDiffsSymbol
	"Answer the code symbol to use when generically switching to diffing"

	^ self class diffsWithPrettyPrint 
		ifTrue:
			[#prettyDiffs]
		ifFalse:
			[#showDiffs]! !

!CodeHolder methodsFor: 'self-updating' stamp: 'nk 4/29/2004 12:25'!
didCodeChangeElsewhere
	"Determine whether the code for the currently selected method and class has been changed somewhere else."
	| aClass aSelector aCompiledMethod |
	currentCompiledMethod ifNil: [^ false].

	(aClass := self selectedClassOrMetaClass) ifNil: [^ false].

	(aSelector := self selectedMessageName) ifNil: [^ false].

	self classCommentIndicated
		ifTrue: [^ currentCompiledMethod ~~ aClass organization commentRemoteStr].

	^ (aCompiledMethod := aClass compiledMethodAt: aSelector ifAbsent: [^ false]) ~~ currentCompiledMethod
		and: [aCompiledMethod last ~= 0 "either not yet installed"
				or: [ currentCompiledMethod last = 0 "or these methods don't have source pointers"]]! !

!CodeHolder methodsFor: 'diffs' stamp: 'sd 11/20/2005 21:27'!
diffFromPriorSourceFor: sourceCode 
	"If there is a prior version of source for the selected method, return a diff, else just return the source code"

	| prior |
	^ (prior := self priorSourceOrNil)
		ifNil: [sourceCode]
		ifNotNil: [TextDiffBuilder buildDisplayPatchFrom: prior to: sourceCode inClass: self selectedClass prettyDiffs: self showingPrettyDiffs]! !

!CodeHolder methodsFor: 'misc' stamp: 'nk 4/10/2001 07:52'!
getSelectorAndSendQuery: querySelector to: queryPerformer
	"Obtain a selector relevant to the current context, and then send the querySelector to the queryPerformer with the selector obtained as its argument.  If no message is currently selected, then obtain a method name from a user type-in"

	self getSelectorAndSendQuery: querySelector to: queryPerformer with: { }.
! !

!CodeHolder methodsFor: 'misc' stamp: 'StephaneDucasse 10/15/2009 18:01'!
getSelectorAndSendQuery: querySelector to: queryPerformer with: queryArgs
	"Obtain a selector relevant to the current context, and then send the querySelector to the queryPerformer with the selector obtained and queryArgs as its arguments.  If no message is currently selected, then obtain a method name from a user type-in"

	| strm array |
	strm := (array := Array new: queryArgs size + 1) writeStream.
	strm nextPut: nil.
	strm nextPutAll: queryArgs.

	self selectedMessageName ifNil: [ | selector |
		selector := UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
		selector ifNil: [selector := String new].
		selector := selector copyWithout: Character space.
		^ selector isEmptyOrNil ifFalse: [
			(Symbol hasInterned: selector
				ifTrue: [ :aSymbol |
					array at: 1 put: aSymbol.
					queryPerformer perform: querySelector withArguments: array])
				ifFalse: [ self inform: 'no such selector']
		]
	].

	self selectMessageAndEvaluate: [:selector |
		array at: 1 put: selector.
		queryPerformer perform: querySelector withArguments: array
	]! !

!CodeHolder methodsFor: 'controls' stamp: 'sw 1/25/2001 14:44'!
inheritanceButton
	"If receiver has an Inheritance button, answer it, else answer nil.  morphic only at this point"

	^ self buttonWithSelector: #methodHierarchy! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'AlainPlantec 12/1/2009 22:56'!
inheritanceButtonColor
	"Check to see if the currently-viewed method has a super send or an override, and if so, change screen feedback, unless the #decorateBrowserButtons says not to."

	((currentCompiledMethod isKindOf: CompiledMethod) and: [self class decorateBrowserButtons])
		ifFalse: [^Color transparent].

	"This table duplicates the old logic, but adds two new colors for the cases where there is a superclass definition, but this method doesn't call it."

	^ self color

! !

!CodeHolder methodsFor: 'misc' stamp: 'StephaneDucasse 1/13/2010 14:22'!
initialExtent

	^ 700@500! !

!CodeHolder methodsFor: 'misc' stamp: 'md 2/24/2006 15:28'!
isThereAnOverride
	"Answer whether any subclass of my selected class implements my 
	selected selector"
	| aName aClass |
	aName := self selectedMessageName ifNil: [^ false].
	aClass := self selectedClassOrMetaClass ifNil: [^ false].
	aClass allSubclassesDo: [ :cls | (cls includesSelector: aName) ifTrue: [ ^true ]].
	^ false! !

!CodeHolder methodsFor: 'misc' stamp: 'md 2/24/2006 15:28'!
isThisAnOverride
	"Answer whether any superclass of my selected class implements my selected selector"
	| aName aClass |
	aName := self selectedMessageName ifNil: [^ false].
	aClass := self selectedClassOrMetaClass ifNil: [^ false].
	aClass allSuperclassesDo: [ :cls | (cls includesSelector: aName) ifTrue: [ ^true ]].
	^ false! !

!CodeHolder methodsFor: 'categories' stamp: 'sd 11/20/2005 21:27'!
letUserReclassify: anElement in: aClass
	"Put up a list of categories and solicit one from the user.  
	Answer true if user indeed made a change, else false"
	

	| currentCat newCat |
	currentCat := aClass organization categoryOfElement: anElement.
	newCat := self 
				categoryFromUserWithPrompt: 'choose category (currently "', currentCat, '")' 
				for: aClass.
	(newCat ~~ nil and: [newCat ~= currentCat])
		ifTrue:
			[aClass organization classify: anElement under: newCat suppressIfDefault: false.
			^ true]
		ifFalse:
			[^ false]! !

!CodeHolder methodsFor: 'message list menu' stamp: 'sd 11/20/2005 21:27'!
messageListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also
	have a listView that has a list of methods.  The view knows how to get
	the list and selection."

	| sel class |
	aChar == $D ifTrue: [^ self toggleDiffing].

	sel := self selectedMessageName.
	aChar == $m ifTrue:  "These next two put up a type in if no message selected"
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation].
	aChar == $n ifTrue: 
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation].

	"The following require a class selection"
	(class := self selectedClassOrMetaClass) ifNil: [^ self arrowKey: aChar from: view].
	aChar == $b ifTrue: [^ Browser fullOnClass: class selector: sel].
	aChar == $N ifTrue: [^ self browseClassRefs].
	aChar == $i ifTrue: [^ self methodHierarchy].
	aChar == $h ifTrue: [^ self classHierarchy].
	aChar == $p ifTrue: [^ self browseFullProtocol].

	"The following require a method selection"
	sel ifNotNil: 
		[aChar == $o ifTrue: [^ self fileOutMessage].
		aChar == $c ifTrue: [^ self copySelector].
		aChar == $v ifTrue: [^ self browseVersions].
		aChar == $O ifTrue: [^ self openSingleMessageBrowser].
		aChar == $x ifTrue: [^ self removeMessage].
		aChar == $d ifTrue: [^ self removeMessageFromBrowser].

		(aChar == $C and: [self canShowMultipleMessageCategories])
			ifTrue: [^ self showHomeCategory]].

	^ self arrowKey: aChar from: view! !

!CodeHolder methodsFor: 'categories' stamp: 'sw 9/27/1999 14:11'!
methodCategoryChanged
	self changed: #annotation! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 9/27/2001 01:26'!
modelWakeUpIn: aWindow
	"The window has been activated.  Respond to possible changes that may have taken place while it was inactive"

	self updateListsAndCodeIn: aWindow.
	self decorateButtons.
	self refreshAnnotation.

	super modelWakeUpIn: aWindow! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 2/27/2001 12:14'!
offerShiftedClassListMenu
	"Offer the shifted class-list menu."

	^ self offerMenuFrom: #classListMenu:shifted: shifted: true! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 2/27/2001 12:15'!
offerUnshiftedClassListMenu
	"Offer the shifted class-list menu."

	^ self offerMenuFrom: #classListMenu:shifted: shifted: false! !

!CodeHolder methodsFor: 'what to show' stamp: 'JuanVuletich 10/26/2010 18:19'!
offerWhatToShowMenu
	"Offer a menu governing what to show"
	| aMenu |
	aMenu := UIManager default menuClass new defaultTarget: self.
	aMenu addTitle: 'What to show' translated.
	aMenu addStayUpItem.
	self addContentsTogglesTo: aMenu.
	aMenu popUpInWorld! !

!CodeHolder methodsFor: 'misc' stamp: 'alain.plantec 2/6/2009 16:46'!
okayToAccept
	"Answer whether it is okay to accept the receiver's input"

	self showingDocumentation ifTrue:
		[self inform: 
'Sorry, for the moment you can
only submit changes here when
you are showing source.  Later, you
will be able to edit the isolated comment
here and save it back, but only if you
implement it!!.' translated.
		^ false].

	self showingAnyKindOfDiffs ifFalse:
		[^ true]. 
	^ self confirm: 
'Caution!!
You are "showing diffs" here, so 
there is a danger that some of the text in the
code pane is contaminated by the "diff" display' translated
! !

!CodeHolder methodsFor: 'controls' stamp: 'tbn 7/29/2010 22:18'!
optionalButtonPairs
	"Answer a tuple (formerly pairs) defining buttons, in the format:
			button label
			selector to send
			help message"

	| aList |

	aList := #(
	('Browse'			browseMethodFull			'View this method in a browser')
	('Senders' 			browseSendersOfMessages	'Browse senders of...')
	('Implementors'		browseMessages				'Browse implementors of...')
	('Versions'			browseVersions				'Browse versions')), 

	(self class decorateBrowserButtons
		ifTrue:
			[{#('Inheritance'		methodHierarchy 'Browse method inheritance
green: sends to super
tan: has override(s)
mauve: both of the above
pink: is an override but doesn''t call super
pinkish tan: has override(s), also is an override but doesn''t call super' )}]
		ifFalse:
			[{#('Inheritance'		methodHierarchy			'Browse method inheritance')}]),

	#(
	('Hierarchy'		classHierarchy				'Browse class hierarchy')
	('Inst vars'			browseInstVarRefs			'Inst var refs...')
	('Class vars'			browseClassVarRefs			'Class var refs...')).

	^ aList! !

!CodeHolder methodsFor: 'diffs' stamp: 'FernandoOlivero 4/12/2011 09:43'!
prettyDiffButton
	"Return a checkbox that lets the user decide whether prettyDiffs should be shown or not"
		
	^self theme builder
		newCheckboxFor: self
		getSelected: #showingPrettyDiffs
		setSelected: #togglePrettyDiffing
		label: 'prettyDiffs' translated
		help: 'If checked, then pretty-printed code differences from the previous version, if any, will be shown.' translated

! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 16:36'!
prettyPrintString
	"Answer whether the receiver is showing pretty-print"

	^ ((contentsSymbol == #prettyPrint)
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'prettyPrint'! !

!CodeHolder methodsFor: 'misc' stamp: 'sd 11/20/2005 21:27'!
priorSourceOrNil
	"If the currently-selected method has a previous version, return its source, else return nil"
	| aClass aSelector  changeRecords |
	(aClass := self selectedClassOrMetaClass) ifNil: [^ nil].
	(aSelector := self selectedMessageName) ifNil: [^ nil].
	changeRecords := aClass changeRecordsAt: aSelector.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [^ nil].
	^ (changeRecords at: 2) string 
! !

!CodeHolder methodsFor: 'misc' stamp: 'MarcusDenker 11/26/2009 09:56'!
refreshAnnotation
	"If the receiver has an annotation pane that does not bear unaccepted edits, refresh it"

	(self dependents detect: [:m | (m isKindOf: PluggableTextMorph) and: [m getTextSelector == #annotation]] ifNone: [nil]) ifNotNil:
		[:aPane | aPane hasUnacceptedEdits ifFalse:
			[aPane update: #annotation]]! !

!CodeHolder methodsFor: 'misc' stamp: 'stephane.ducasse 10/26/2008 15:13'!
refusesToAcceptCode
	"Answer whether receiver, given its current contentsSymbol, could accept code happily if asked to"

	^ (#(byteCodes documentation) includes: self contentsSymbol)! !

!CodeHolder methodsFor: 'diffs' stamp: 'FernandoOlivero 4/12/2011 09:43'!
regularDiffButton
	"Return a checkbox that lets the user decide whether regular diffs should be shown or not"

	^self theme builder
		newCheckboxFor: self
		getSelected: #showingRegularDiffs
		setSelected: #toggleRegularDiffing
		label: 'diffs' translated
		help: 'If checked, then code differences from the previous version, if any, will be shown.' translated
! !

!CodeHolder methodsFor: 'misc' stamp: 'sd 11/20/2005 21:27'!
releaseCachedState
	"Can always be found again.  Don't write on a file."
	currentCompiledMethod := nil.! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 11/20/2005 21:27'!
removeClass
	"Remove the selected class from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."

	| message  className classToRemove result |
	self okToChange ifFalse: [^ false].
	classToRemove := self selectedClassOrMetaClass ifNil: [Beeper beep. ^ false].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you
want to REMOVE the class ', className, '
from the system?'.
	(result := self confirm: message)
		ifTrue: 
			[classToRemove subclasses size > 0
				ifTrue: [(self confirm: 'class has subclasses: ' , message)
					ifFalse: [^ false]].
			classToRemove removeFromSystem.
			self changed: #classList.
			true].
	^ result! !

!CodeHolder methodsFor: 'message list' stamp: 'sw 8/16/2002 23:23'!
selectedBytecodes
	"Answer text to show in a code pane when in showing-byte-codes mode"

	^ (self selectedClassOrMetaClass compiledMethodAt: self selectedMessageName ifAbsent: [^ '' asText]) symbolic asText! !

!CodeHolder methodsFor: 'message list' stamp: 'md 2/20/2006 15:02'!
selectedMessage
	"Answer a copy of the source code for the selected message.  This generic version is probably actually never reached, since every subclass probably reimplements and does not send to super.  In time, ideally, most, or all, reimplementors would vanish and all would defer instead to a universal version right here.  Everything in good time."

	| class selector method |
	contents ifNotNil: [^ contents copy].

	self showingDecompile ifTrue:[^ self decompiledSourceIntoContents].

	class := self selectedClassOrMetaClass.
	(class isNil or: [(selector := self selectedMessageName) isNil]) ifTrue: [^ ''].
	method := class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod := method.

	^ contents := (self showComment
		ifFalse: [self sourceStringPrettifiedAndDiffed]
		ifTrue:	[ self commentContents])
			copy asText makeSelectorBoldIn: class! !

!CodeHolder methodsFor: 'categories' stamp: 'sw 3/22/2000 23:04'!
selectedMessageCategoryName
	"Answer the name of the message category of the message of the currently selected context."

	^ self selectedClass organization categoryOfElement: self selectedMessageName! !

!CodeHolder methodsFor: 'misc' stamp: 'rbb 3/1/2005 10:31'!
sendQuery: querySelector to: queryPerformer
	"Apply a query to the primary selector associated with the current context.  If no such selection exists, obtain one by user type-in. Then send querySelector to queryPerformer with the selector as its argument."

	| aSelector aString |
	aSelector := self selectedMessageName ifNil:
		[aString :=UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
		^ aString isEmptyOrNil ifFalse:
			[(Symbol hasInterned: aString ifTrue:
				[:aSymbol | queryPerformer perform: querySelector with: aSymbol])
				ifFalse:
					[self inform: 'no such selector']]].

	queryPerformer perform: querySelector with: aSelector! !

!CodeHolder methodsFor: 'misc' stamp: 'sd 11/20/2005 21:27'!
setClassAndSelectorIn: csBlock
	"Evaluate csBlock with my selected class and and selector as its arguments; provide nil arguments if I don't have a method currently selected"

	| aName |
	(aName := self selectedMessageName)
		ifNil:
			[csBlock value: nil value: nil]
		ifNotNil:
			[csBlock value: self selectedClassOrMetaClass value: aName]
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sd 11/20/2005 21:27'!
setContentsToForceRefetch
	"Set the receiver's contents such that on the next update the contents will be formulated afresh.  This is a critical and obscure difference between Browsers on the one hand and MessageSets on the other, and has over the years been the source of much confusion and much difficulty.  By centralizing the different handling here, we don't need so many idiosyncratic overrides in MessageSet any more"

	contents := nil! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/6/2001 15:18'!
shiftedYellowButtonActivity
	"Offer the shifted selector-list menu"

	^ self offerMenuFrom: #messageListMenu:shifted: shifted: true! !

!CodeHolder methodsFor: 'setting' stamp: 'AlainPlantec 11/24/2009 16:59'!
showAnnotationPane
	^ self class showAnnotationPane! !

!CodeHolder methodsFor: 'what to show' stamp: 'gvc 6/21/2010 11:52'!
showByteCodes: aBoolean
	"Get into or out of bytecode-showoing mode"

	self okToChange ifFalse: [^ self changed: #flash].
	aBoolean
		ifTrue:
			[self contentsSymbol: #byteCodes]
		ifFalse:
			[contentsSymbol == #byteCodes ifTrue: [self contentsSymbol: #source]].
	self contentsChanged! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 11:32'!
showComment
	"Answer whether the receiver should show documentation rather than, say, source code"

	^ self contentsSymbol == #documentation
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 09:14'!
showDecompile: aBoolean
	"Set the decompile toggle as indicated"

	self contentsSymbol: (aBoolean ifFalse: [#source] ifTrue: [#decompile])! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:49'!
showDiffs
	"Answer whether the receiver is showing diffs of source code.  The preferred protocol here is #showingRegularDiffs, but this message is still sent by some preexisting buttons so is retained."

	^ contentsSymbol == #showDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'gvc 6/21/2010 11:51'!
showDiffs: aBoolean
	"Set whether I'm showing diffs as indicated"

	self showingAnyKindOfDiffs
		ifFalse:
			[aBoolean ifTrue:
				[self contentsSymbol: self defaultDiffsSymbol]]
		ifTrue:
			[aBoolean ifFalse:
				[self contentsSymbol: #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 12:25'!
showDocumentation: aBoolean
	"Set the showDocumentation toggle as indicated"

	self contentsSymbol: (aBoolean ifFalse: [#source] ifTrue: [#documentation])! !

!CodeHolder methodsFor: 'diffs' stamp: 'gvc 6/21/2010 11:52'!
showPrettyDiffs: aBoolean
	"Set whether I'm showing pretty diffs as indicated"

	self showingPrettyDiffs
		ifFalse:
			[aBoolean ifTrue:
				[self contentsSymbol: #prettyDiffs]]
		ifTrue:
			[aBoolean ifFalse:
				[self contentsSymbol: #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'diffs' stamp: 'gvc 6/21/2010 11:52'!
showRegularDiffs: aBoolean
	"Set whether I'm showing regular diffs as indicated"

	self showingRegularDiffs
		ifFalse:
			[aBoolean ifTrue:
				[self contentsSymbol: #showDiffs]]
		ifTrue:
			[aBoolean ifFalse:
				[self contentsSymbol: #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'traits' stamp: 'alain.plantec 2/6/2009 16:49'!
showUnreferencedClassVars
	"Search for all class variables known to the selected class, and put up a 
	list of those that have no references anywhere in the system. The 
	search includes superclasses, so that you don't need to navigate your 
	way to the class that defines each class variable in order to determine 
	whether it is unreferenced"
	| cls aList aReport |
	((cls := self selectedClass) isNil or: [cls isTrait]) ifTrue: [^ self].
	aList := self systemNavigation allUnreferencedClassVariablesOf: cls.
	aList size == 0
		ifTrue: [^ self inform: 'There are no unreferenced
class variables in
' , cls name].
	aReport := String
				streamContents: [:aStream | 
					aStream nextPutAll: 'Unreferenced class variable(s) in ' translated, cls name;
						 cr.
					aList
						do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	self inform: aReport! !

!CodeHolder methodsFor: 'traits' stamp: 'alain.plantec 2/6/2009 16:51'!
showUnreferencedInstVars
	"Search for all instance variables known to the selected class, and put up a list of those that have no references anywhere in the system.  The search includes superclasses, so that you don't need to navigate your way to the class that defines each inst variable in order to determine whether it is unreferenced"

	| cls aList aReport |
	((cls := self selectedClassOrMetaClass) isNil or: [cls isTrait]) ifTrue: [^ self].
	aList := cls allUnreferencedInstanceVariables.
	aList size == 0 ifTrue: [^ self inform: 'There are no unreferenced
instance variables in
', cls name].
	aReport := String streamContents:
		[:aStream |
			aStream nextPutAll: 'Unreferenced instance variable(s) in ' translated, cls name; cr.
			aList do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	self inform: aReport! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 9/5/2001 13:32'!
showingAnyKindOfDiffs
	"Answer whether the receiver is currently set to show any kind of diffs"

	^ #(showDiffs prettyDiffs) includes: contentsSymbol! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 18:05'!
showingByteCodes
	"Answer whether the receiver is showing bytecodes"

	^ contentsSymbol == #byteCodes! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 18:28'!
showingByteCodesString
	"Answer whether the receiver is showing bytecodes"

	^ (self showingByteCodes
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'byteCodes'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 06:52'!
showingDecompile
	"Answer whether the receiver should show decompile rather than, say, source code"

	^ self contentsSymbol == #decompile
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 06:50'!
showingDecompileString
	"Answer a string characerizing whether decompilation is showing"

	^ (self showingDecompile
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'decompile'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 12:12'!
showingDocumentation
	"Answer whether the receiver should show documentation rather than, say, source code"

	^ self contentsSymbol == #documentation
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 20:05'!
showingDocumentationString
	"Answer a string characerizing whether documentation is showing"

	^ (self showingDocumentation
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'documentation'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 19:43'!
showingPlainSource
	"Answer whether the receiver is showing plain source"

	^ contentsSymbol == #source! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 09:31'!
showingPlainSourceString
	"Answer a string telling whether the receiver is showing plain source"

	^ (self showingPlainSource
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'source'! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/19/2001 00:07'!
showingPrettyDiffs
	"Answer whether the receiver is showing pretty diffs of source code"

	^ contentsSymbol == #prettyDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/22/2001 16:41'!
showingPrettyDiffsString
	"Answer a string representing whether I'm showing pretty diffs"

	^ (self showingPrettyDiffs
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'prettyDiffs'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 18:36'!
showingPrettyPrint
	"Answer whether the receiver is showing pretty-print"

	^ contentsSymbol == #prettyPrint! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:07'!
showingRegularDiffs
	"Answer whether the receiver is showing regular diffs of source code"

	^ contentsSymbol == #showDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:43'!
showingRegularDiffsString
	"Answer a string representing whether I'm showing regular diffs"

	^ (self showingRegularDiffs
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'showDiffs'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 11:48'!
showingSource
	"Answer whether the receiver is currently showing source code"

	^ self contentsSymbol == #source
! !

!CodeHolder methodsFor: 'controls' stamp: 'tbn 7/6/2010 16:22'!
sourceAndDiffsQuintsOnly
	"Answer a list of quintuplets representing information on the alternative views available in the code pane for the case where the only plausible choices are showing source or either of the two kinds of diffs"

	^ #(
(source			togglePlainSource 		showingPlainSourceString	'Source'			'the textual source code as writen')
(showDiffs		toggleRegularDiffing	showingRegularDiffsString	'ShowDiffs'		'the textual source diffed from its prior version')
(prettyDiffs		togglePrettyDiffing		showingPrettyDiffsString	'PrettyDiffs'		'formatted textual source diffed from formatted form of prior version'))! !

!CodeHolder methodsFor: 'message list' stamp: 'alain.plantec 5/18/2009 15:44'!
sourceStringPrettifiedAndDiffed
	"Answer a copy of the source code for the selected message, transformed by diffing and pretty-printing exigencies"

	| class selector sourceString |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	(class isNil or: [selector isNil]) ifTrue: [^'missing'].
	sourceString := class ultimateSourceCodeAt: selector ifAbsent: [^'error'].
	self validateMessageSource: sourceString forSelector: selector.
	(#(#prettyPrint #prettyDiffs) 
		includes: contentsSymbol) 
			ifTrue: 
				[sourceString := class prettyPrinterClass 
							format: sourceString
							in: class
							notifying: nil].
	self showingAnyKindOfDiffs 
		ifTrue: [sourceString := self diffFromPriorSourceFor: sourceString].
	^sourceString! !

!CodeHolder methodsFor: 'commands' stamp: 'BenjaminVanRyseghem 5/6/2011 16:13'!
spawnFullProtocol
	"Create and schedule a new protocol browser on the currently selected class or meta."

	self spawnProtocol! !

!CodeHolder methodsFor: 'traits' stamp: 'alain.plantec 5/30/2008 11:27'!
spawnHierarchy
	"Create and schedule a new hierarchy browser on the currently selected
	class or meta."
	| newBrowser aSymbol aBehavior messageCatIndex selectedClassOrMetaClass |
	(selectedClassOrMetaClass := self selectedClassOrMetaClass)
		ifNil: [^ self].
	selectedClassOrMetaClass isTrait
		ifTrue: [^ self].
	newBrowser := HierarchyBrowser new initHierarchyForClass: selectedClassOrMetaClass.
	((aSymbol := self selectedMessageName) notNil
			and: [(MessageSet isPseudoSelector: aSymbol) not])
		ifTrue: [aBehavior := selectedClassOrMetaClass.
			messageCatIndex := aBehavior organization numberOfCategoryOfElement: aSymbol.
			newBrowser messageCategoryListIndex: messageCatIndex + 1.
			newBrowser
				messageListIndex: ((aBehavior organization listAtCategoryNumber: messageCatIndex)
						indexOf: aSymbol)].
	Browser
		openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: newBrowser labelString.
	newBrowser assureSelectionsShow! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 11/20/2005 21:27'!
spawnProtocol
	| aClassOrMetaclass |
	"Create and schedule a new protocol browser on the currently selected class or meta."
	(aClassOrMetaclass := self selectedClassOrMetaClass) ifNotNil:
       	[ProtocolBrowser openSubProtocolForClass: aClassOrMetaclass]! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 11/20/2005 21:27'!
spawnToClass: aClass
	"Used to copy down code from a superclass to a subclass in one easy step, if you know what you're doing.  Spawns a new message-category browser for the indicated class, populating it with the source code seen in the current tool."

	| aCategory newBrowser org |	
	(aCategory := self categoryOfCurrentMethod)
		ifNil:
			[self buildClassBrowserEditString: self contents]
		ifNotNil:
			[((org := aClass organization) categories includes: aCategory)
				ifFalse:	[org addCategory: aCategory].
			newBrowser := Browser new setClass: aClass selector: nil.
			newBrowser selectMessageCategoryNamed: aCategory.
			Browser openBrowserView: (newBrowser openMessageCatEditString: self contents)
		label: 'category "', aCategory, '" in ', 
				newBrowser selectedClassOrMetaClassName]! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/20/2001 15:11'!
spawnToCollidingClass: aClass
	"Potentially used to copy down code from a superclass to a subclass in one easy step, in the case where the given class already has its own version of code, which would consequently be clobbered if the spawned code were accepted."

	self inform: 'That would be destructive of
some pre-existing code already in that
class for this selector.  For the moment,
we will not let you do this to yourself.'! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 10/19/1999 08:37'!
stepIn: aSystemWindow
	self updateListsAndCodeIn: aSystemWindow! !

!CodeHolder methodsFor: 'categories & search pane' stamp: 'sd 11/20/2005 21:27'!
textPaneWithSelector: aSelector
	"If, among my window's paneMorphs, there is a text pane defined with aSelector as its retriever, answer it, else answer nil"

	| aWindow |
	^ (aWindow := self containingWindow) ifNotNil:
		[aWindow paneMorphSatisfying:
			[:aMorph | (aMorph isKindOf: PluggableTextMorph) and:
				[aMorph getTextSelector == aSelector]]]! !

!CodeHolder methodsFor: 'breakpoints' stamp: 'marcus.denker 10/9/2008 20:31'!
toggleBreakOnEntry
	"Install or uninstall a halt-on-entry breakpoint"

	| selectedMethod |
	self selectedClassOrMetaClass isNil ifTrue:[^self].
	selectedMethod := self selectedClassOrMetaClass >> self selectedMessageName.
	selectedMethod hasBreakpoint
		ifTrue:
			[BreakpointManager unInstall: selectedMethod]
		ifFalse:
			[BreakpointManager 
				installInClass: self selectedClassOrMetaClass
				selector: self selectedMessageName].! !

!CodeHolder methodsFor: 'what to show' stamp: 'marcus.denker 9/20/2008 20:31'!
toggleDecompile
	"Toggle the setting of the showingDecompile flag, unless there are unsubmitted edits that the user declines to discard"

	| wasShowing |
	self okToChange ifTrue:
		[wasShowing := self showingDecompile.
		self showDecompile: wasShowing not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'marcus.denker 9/20/2008 20:31'!
toggleDiffing
	"Toggle whether diffs should be shown in the code pane.  If any kind of diffs were being shown, stop showing diffs.  If no kind of diffs were being shown, start showing whatever kind of diffs are called for by default."

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingAnyKindOfDiffs.
		self showDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'gvc 6/21/2010 11:52'!
togglePlainSource
	"Toggle whether plain source shown in the code pane"
	
	| wasShowingPlainSource |
	self okToChange ifTrue:
		[wasShowingPlainSource := self showingPlainSource.
		wasShowingPlainSource
			ifTrue:
				[self showDocumentation: true]
			ifFalse:
				[self contentsSymbol: #source].
		self setContentsToForceRefetch.
		self changed: #contents]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'marcus.denker 9/20/2008 20:31'!
togglePrettyDiffing
	"Toggle whether pretty-diffing should be shown in the code pane"

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingPrettyDiffs.
		self showPrettyDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'gvc 6/21/2010 11:52'!
togglePrettyPrint
	"Toggle whether pretty-print is in effectin the code pane"

	self okToChange ifTrue:
		[self showingPrettyPrint
			ifTrue:
				[self contentsSymbol: #source]
			ifFalse:
				[self contentsSymbol: #prettyPrint].
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'marcus.denker 9/20/2008 20:31'!
toggleRegularDiffing
	"Toggle whether regular-diffing should be shown in the code pane"

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingRegularDiffs.
		self showRegularDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'what to show' stamp: 'marcus.denker 9/20/2008 20:32'!
toggleShowDocumentation
	"Toggle the setting of the showingDocumentation flag, unless there are unsubmitted edits that the user declines to discard"

	| wasShowing |
	self okToChange ifTrue:
		[wasShowing := self showingDocumentation.
		self showDocumentation: wasShowing not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'what to show' stamp: 'marcus.denker 9/20/2008 20:32'!
toggleShowingByteCodes
	"Toggle whether the receiver is showing bytecodes"

	self showByteCodes: self showingByteCodes not.
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/6/2001 15:19'!
unshiftedYellowButtonActivity
	"Offer the unshifted shifted selector-list menu"

	^ self offerMenuFrom: #messageListMenu:shifted: shifted: false! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 2/14/2001 15:34'!
updateCodePaneIfNeeded
	"If the code for the currently selected method has changed underneath me, then update the contents of my code pane unless it holds unaccepted edits"

	self didCodeChangeElsewhere
		ifTrue:
			[self hasUnacceptedEdits
				ifFalse:
					[self setContentsToForceRefetch.
					self contentsChanged]
				ifTrue:
					[self changed: #codeChangedElsewhere]]! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 10/19/1999 14:14'!
updateListsAndCodeIn: aWindow
	super updateListsAndCodeIn: aWindow.
	self updateCodePaneIfNeeded! !

!CodeHolder methodsFor: 'misc' stamp: 'rbb 3/1/2005 10:31'!
useSelector: incomingSelector orGetSelectorAndSendQuery: querySelector to: queryPerformer
	"If incomingSelector is not nil, use it, else obtain a selector from user type-in.   Using the determined selector, send the query to the performer provided."

	| aSelector |
	incomingSelector
		ifNotNil:
			[queryPerformer perform: querySelector with: incomingSelector]
		ifNil:
			[aSelector :=UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
			aSelector isEmptyOrNil ifFalse:
				[(Symbol hasInterned: aSelector ifTrue:
					[:aSymbol | queryPerformer perform: querySelector with: aSymbol])
					ifFalse:
						[self inform: 'no such selector']]]! !

!CodeHolder methodsFor: 'message list' stamp: 'StephaneDucasse 3/17/2010 20:53'!
validateMessageSource: sourceString forSelector: aSelector
	"Check whether there is evidence that method source is invalid"

	| sourcesName |
	(self selectedClass parserClass new parseSelector: sourceString asString) = aSelector
		ifFalse: [sourcesName := FileDirectory localNameFor: Smalltalk sourcesName.
			self inform: 'There may be a problem with your sources file!!

The source code for every method should (usually) start with the
method selector but this is not the case with this method!! You may
proceed with caution but it is recommended that you get a new source file.

This can happen if you download the "' , sourcesName  , '" file, 
or the ".changes" file you use, as TEXT. It must be transfered 
in BINARY mode, even if it looks like a text file, 
to preserve the CR line ends.

Mac users: This may have been caused by Stuffit Expander. 
To prevent the files above to be converted to Mac line ends 
when they are expanded, do this: Start the program, then 
from Preferences... in the File menu, choose the Cross 
Platform panel, then select "Never" and press OK. 
Then expand the compressed archive again.

(Occasionally, the source code for a method may legitimately
start with a non-alphabetic character -- for example, Behavior
method #formalHeaderPartsFor:.  In such rare cases, you can
happily disregard this warning.)'].! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:24'!
wantsDiffFeedback
	"Answer whether the receiver is showing diffs of source code"

	^ self showingAnyKindOfDiffs! !

!CodeHolder methodsFor: 'setting' stamp: 'AlainPlantec 1/7/2010 21:48'!
wantsOptionalButtons
	"Answer whether the receiver, seen in some browser window, would like to have the so-called optional button pane included.  By default, various browsers defer to the optionalButtons class accessor -- but individual subclasses can insist to the contrary."

	^ self class optionalButtons! !

!CodeHolder methodsFor: 'self-updating' stamp: 'AlainPlantec 12/1/2009 22:36'!
wantsStepsIn: aWindow
	^ self class smartUpdating! !

CodeHolder subclass: #Debugger
	instanceVariableNames: 'interruptedProcess interruptedController contextStack contextStackTop contextStackIndex contextStackList receiverInspector contextVariablesInspector externalInterrupt proceedValue selectingPC savedCursor errorWasInUIProcess labelString'
	classVariableNames: 'AlwaysOpenFullDebugger ContextStackKeystrokes ErrorRecursion ExtraDebuggerButtons LogDebuggerStackToFile LogFileName RestartAlsoProceeds'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!Debugger commentStamp: '<historical>' prior: 0!
I represent the machine state at the time of an interrupted process. I also represent a query path into the state of the process. The debugger is typically viewed through a window that views the stack of suspended contexts, the code for, and execution point in, the currently selected message, and inspectors on both the receiver of the currently selected message, and the variables in the current context.

Special note on recursive errors:
Some errors affect Squeak's ability to present a debugger.  This is normally an unrecoverable situation.  However, if such an error occurs in an isolation layer, Squeak will attempt to exit from the isolation layer and then present a debugger.  Here is the chain of events in such a recovery.

	* A recursive error is detected.
	* The current project is queried for an isolationHead
	* Changes in the isolationHead are revoked
	* The parent project of isolated project is returned to
	* The debugger is opened there and execution resumes.

If the user closes that debugger, execution continues in the outer project and layer.  If, after repairing some damage, the user proceeds from the debugger, then the isolationHead is re-invoked, the failed project is re-entered, and execution resumes in that world. !

!Debugger class methodsFor: 'settings' stamp: 'CamilloBruni 2/21/2011 11:29'!
alwaysOpenFullDebugger
	^ AlwaysOpenFullDebugger ifNil: [AlwaysOpenFullDebugger := false]! !

!Debugger class methodsFor: 'settings' stamp: 'MarcusDenker 3/27/2011 17:20'!
alwaysOpenFullDebugger: aBoolean
	AlwaysOpenFullDebugger := aBoolean.! !

!Debugger class methodsFor: 'utilities' stamp: 'JB 2/12/2010 17:04'!
closeAllDebuggers
	"Debugger closeAllDebuggers"
	(SystemWindow allSubInstances select: [:w | ToolSet isDebugger: w model])
			do: [:w | w delete]! !

!Debugger class methodsFor: 'instance creation' stamp: 'StephaneDucasse 1/8/2010 16:04'!
context: aContext
	"Answer an instance of me for debugging the active process starting with the given context."

	^ self new
		process: Processor activeProcess
		controller: nil
		context: aContext
		! !

!Debugger class methodsFor: 'settings' stamp: 'IgorStasenko 1/21/2011 17:16'!
defaultLogFileName
	^ 'PharoDebug.log'! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 21:55'!
extraDebuggerButtons
	^ ExtraDebuggerButtons ifNil: [ExtraDebuggerButtons := false]! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 21:54'!
extraDebuggerButtons: aBoolean
	ExtraDebuggerButtons := aBoolean! !

!Debugger class methodsFor: 'instance creation' stamp: 'MarcusDenker 1/23/2011 09:14'!
informExistingDebugger: aContext label: aString
	"Walking the context chain, we try to find out if we're in a debugger stepping situation.
	If we find the relevant contexts, we must rearrange them so they look just like they would
	if the methods were excuted outside of the debugger."
	| ctx quickStepMethod oldSender baseContext |
	ctx := thisContext.
	quickStepMethod := ContextPart compiledMethodAt: #quickSend:to:with:super:.
	[ctx sender == nil or: [ctx sender method == quickStepMethod]] whileFalse: [ctx := ctx sender].
	ctx sender == nil ifTrue: [^self].
	baseContext := ctx.
	"baseContext is now the context created by the #quickSend... method."
	oldSender := ctx := ctx sender home sender.
	"oldSender is the context which originally sent the #quickSend... method"
	[ctx isNil or: [ctx receiver isKindOf: self]] whileFalse: [ctx := ctx sender].
	ctx ifNil: [^self].
	"ctx is the context of the Debugger method #doStep"
	ctx receiver labelString: aString.
	ctx receiver externalInterrupt: false; proceedValue: aContext receiver.
	baseContext swapSender: baseContext sender sender sender.	"remove intervening contexts"
	thisContext swapSender: oldSender.	"make myself return to debugger"
	ErrorRecursion := false.
	^aContext! !

!Debugger class methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:28'!
initialize
	ErrorRecursion := false.
	ContextStackKeystrokes := Dictionary new
		at: $e put: #send;
		at: $t put: #doStep;
		at: $T put: #stepIntoBlock;
		at: $p put: #proceed;
		at: $r put: #restart;
		at: $f put: #fullStack;
		at: $w put: #where;
		yourself.

	"Debugger initialize"! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 21:57'!
logDebuggerStackToFile
	^ LogDebuggerStackToFile ifNil: [LogDebuggerStackToFile := true]! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 21:57'!
logDebuggerStackToFile: aBoolean
	 LogDebuggerStackToFile := aBoolean! !

!Debugger class methodsFor: 'settings' stamp: 'IgorStasenko 1/21/2011 17:16'!
logFileName
	^ LogFileName ifNil: [ self defaultLogFileName ] ! !

!Debugger class methodsFor: 'settings' stamp: 'IgorStasenko 1/21/2011 17:17'!
logFileName: newName
	LogFileName := newName! !

!Debugger class methodsFor: 'initialization' stamp: 'IgorStasenko 3/6/2011 18:39'!
openContext: aContext label: aString contents: contentsStringOrNil

	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."
	<primitive: 19> "Simulation guard"
	ErrorRecursion not & self logDebuggerStackToFile ifTrue:
		[Smalltalk logError: aString inContext: aContext].
	  ErrorRecursion ifTrue:[
               ErrorRecursion := false.
               self primitiveError: aString].
	ErrorRecursion := true.
	self informExistingDebugger: aContext label: aString.
	(self context: aContext)
		openNotifierContents: contentsStringOrNil
		label: aString.
	ErrorRecursion := false.
	Processor activeProcess suspend.
! !

!Debugger class methodsFor: 'opening' stamp: 'IgorStasenko 1/21/2011 19:05'!
openInterrupt: aString onProcess: interruptedProcess 
	"Open a notifier in response to an interrupt. An interrupt occurs when
	the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other
	systems) or when the low-space watcher detects that memory is low."
	| debugger |
	<primitive: 19>
	"Simulation guard"
	debugger := self new.
	debugger
		process: interruptedProcess
		controller: nil
		context: interruptedProcess suspendedContext.
	debugger externalInterrupt: true.
	self logDebuggerStackToFile
		ifTrue: [(aString includesSubString: 'Space')
					& (aString includesSubString: 'low')
				ifTrue: [Smalltalk
						logError: aString
						inContext: debugger interruptedContext ]].
	^ debugger openNotifierContents: nil label: aString! !

!Debugger class methodsFor: 'opening' stamp: 'CamilloBruni 2/23/2011 10:52'!
openOn: process context: context label: title contents: contentsStringOrNil fullView: bool 
	"Open a notifier in response to an error, halt, or notify. A notifier view
	just shows a short view of the sender stack and provides a menu that
	lets the user open a full debugger."
	|fullView|
	fullView := (bool or: [self alwaysOpenFullDebugger]).
	UIManager default openDebuggerOn: process context: context label: title contents: contentsStringOrNil fullView: fullView! !

!Debugger class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:08'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightRed
! !

!Debugger class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:02'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #debugger
! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 22:06'!
restartAlsoProceeds
	^ RestartAlsoProceeds ifNil: [RestartAlsoProceeds := false]! !

!Debugger class methodsFor: 'settings' stamp: 'AlainPlantec 12/6/2009 22:06'!
restartAlsoProceeds: aBoolean
	RestartAlsoProceeds:= aBoolean! !

!Debugger methodsFor: 'context stack menu' stamp: 'EstebanLorenzano 11/21/2009 17:01'!
askForCategoryIn: aClass default: aString
	| categories category |
	categories := OrderedCollection withAll: (aClass allMethodCategoriesIntegratedThrough: Object).	
	category := UIManager default  
		chooseOrRequestFrom: categories
		title: 'Please provide a good category for the new method!!' translated.
	category ifNil: [^aString].
	^ category isEmptyOrNil 
		ifTrue: [^ aString] 
		ifFalse: [ category ]! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
askForSuperclassOf: aClass toImplement: aSelector ifCancel: cancelBlock
	| classes chosenClassIndex |
	classes := aClass withAllSuperclasses.
	chosenClassIndex := UIManager default 
		chooseFrom: (classes collect: [:c | c name])
		title: 'Define #', aSelector, ' in which class?'.
	chosenClassIndex = 0 ifTrue: [^ cancelBlock value].
	^ classes at: chosenClassIndex! !

!Debugger methodsFor: 'context stack menu' stamp: 'wod 5/15/1998 00:24'!
browseMessages
	"Present a menu of all messages sent by the currently selected message.
	Open a message set browser of all implementors of the message chosen.
	Do nothing if no message is chosen."

	contextStackIndex = 0 ifTrue: [^ self].
	super browseMessages.! !

!Debugger methodsFor: 'context stack menu' stamp: 'wod 5/15/1998 00:23'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all
	messages sent by it.  Open a message set browser of all implementors
	of the message chosen."

	contextStackIndex = 0 ifTrue: [^ self].
	super browseSendersOfMessages! !

!Debugger methodsFor: 'context stack menu' stamp: 'IgorStasenko 3/6/2011 18:39'!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."

	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	Smalltalk tools versionBrowser
		browseVersionsOf: (class compiledMethodAt: selector)
		class: self selectedClass theNonMetaClass
		meta: class isMeta
		category: self selectedMessageCategoryName
		selector: selector! !

!Debugger methodsFor: 'toolbuilder' stamp: 'MarcusDenker 1/23/2011 09:12'!
buildFullWith: builder
	| windowSpec listSpec textSpec panelSpec extent |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Debugger'.
	extent := Display height < 800
		ifTrue: [ RealEstateAgent standardWindowExtent ]
		ifFalse: [ 600 @ 700 ].	"a small screen"
	windowSpec extent: extent.
	windowSpec children: OrderedCollection new.
	listSpec := builder pluggableListSpec new.
	listSpec
		model: self;
		list: #contextStackList;
		getIndex: #contextStackIndex;
		setIndex: #toggleContextStackIndex:;
		menu: #contextStackMenu:shifted:;
		keyPress: #contextStackKey:from:;
		frame: (0 @ 0 corner: 1 @ 0.25).
	windowSpec children add: listSpec.
	panelSpec := self buildOptionalButtonsWith: builder.
	panelSpec frame: (0 @ 0.25 corner: 1 @ 0.3).
	windowSpec children add: panelSpec.
	textSpec := builder pluggableTextSpec new.
	textSpec
		model: self;
		getText: #contents;
		setText: #contents:notifying:;
		selection: #contentsSelection;
		menu: #codePaneMenu:shifted:;
		frame: (0 @ 0.3 corner: 1 @ 0.8).
	windowSpec children add: textSpec.
	listSpec := builder pluggableListSpec new.
	listSpec
		model: self receiverInspector;
		list: #fieldList;
		getIndex: #selectionIndex;
		setIndex: #toggleIndex:;
		menu: #fieldListMenu:;
		keyPress: #inspectorKey:from:;
		frame: (0 @ 0.8 corner: 0.2 @ 1).
	windowSpec children add: listSpec.
	textSpec := builder pluggableTextSpec new.
	textSpec
		model: self receiverInspector;
		getText: #contents;
		setText: #accept:;
		selection: #contentsSelection;
		menu: #codePaneMenu:shifted:;
		frame: (0.2 @ 0.8 corner: 0.5 @ 1).
	windowSpec children add: textSpec.
	listSpec := builder pluggableListSpec new.
	listSpec
		model: self contextVariablesInspector;
		list: #fieldList;
		getIndex: #selectionIndex;
		setIndex: #toggleIndex:;
		menu: #fieldListMenu:;
		keyPress: #inspectorKey:from:;
		frame: (0.5 @ 0.8 corner: 0.7 @ 1).
	windowSpec children add: listSpec.
	textSpec := builder pluggableTextSpec new.
	textSpec
		model: self contextVariablesInspector;
		getText: #contents;
		setText: #accept:;
		selection: #contentsSelection;
		menu: #codePaneMenu:shifted:;
		frame: (0.7 @ 0.8 corner: 1 @ 1).
	windowSpec children add: textSpec.
	^ builder build: windowSpec! !

!Debugger methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 16:24'!
buildWith: aBuilder
	^self buildFullWith: aBuilder! !

!Debugger methodsFor: 'private' stamp: 'yo 8/12/2003 16:34'!
checkContextSelection

	contextStackIndex = 0 ifTrue: [self contextStackIndex: 1 oldContextWas: nil].
! !

!Debugger methodsFor: 'context stack menu' stamp: ''!
close: aScheduledController 
	"The argument is a controller on a view of the receiver.
	That view is closed."

	aScheduledController close
! !

!Debugger methodsFor: 'code pane menu' stamp: 'nk 8/6/2003 13:52'!
codePaneMenu: aMenu shifted: shifted
	aMenu add: 'run to here' target: self selector: #runToSelection: argument: thisContext sender receiver selectionInterval.
	aMenu addLine.
	super codePaneMenu: aMenu shifted: shifted.
	^aMenu.! !

!Debugger methodsFor: 'accessing' stamp: 'di 10/9/1998 17:15'!
contents 
	"Depending on the current selection, different information is retrieved.
	Answer a string description of that information.  This information is the
	method in the currently selected context."

	contents == nil ifTrue: [^ String new].
	^ contents copy! !

!Debugger methodsFor: 'accessing' stamp: 'HernanWilkinson 9/30/2010 14:11'!
contents: aText notifying: aController
	"The retrieved information has changed and its source must now be updated.
	 In this case, the retrieved information is the method of the selected context."
	| result selector classOfMethod category h ctxt newMethod |
	contextStackIndex = 0 ifTrue:
		[^false].
	self selectedContext isExecutingBlock ifTrue:
		[h := self selectedContext activeHome.
		 h ifNil:
			[self inform: 'Method for block not found on stack, can''t edit and continue'.
			 ^false].
		 (self confirm: 'I will have to revert to the method from\which this block originated.  Is that OK?' withCRs) ifFalse:
			[^false].
		self resetContext: h.
		result := self contents: aText notifying: aController.
		self contentsChanged.
		^result].

	classOfMethod := self selectedClass.
	category := self selectedMessageCategoryName.
	selector := self selectedClass parserClass new parseSelector: aText.
	(selector == self selectedMessageName
	 or: [(self selectedMessageName beginsWith: 'DoIt')
		and: [selector numArgs = self selectedMessageName numArgs]]) ifFalse:
		[self inform: 'can''t change selector'.
		 ^false].
	selector := classOfMethod
				compile: aText
				classified: category
				notifying: aController.
	selector ifNil: [^false]. "compile cancelled"
	contents := aText.
	newMethod := classOfMethod compiledMethodAt: selector.
	newMethod isQuick ifTrue:
		[self down.
		 self selectedContext jump: (self selectedContext previousPc - self selectedContext pc)].
	ctxt := interruptedProcess popTo: self selectedContext.
	ctxt == self selectedContext
		ifFalse:
			[self inform: 'Method saved, but current context unchanged\because of unwind error. Click OK to see error' withCRs]
		ifTrue:
			[newMethod isQuick ifFalse:
				[interruptedProcess
					restartTopWith: newMethod;
				 	stepToSendOrReturn].
			contextVariablesInspector object: nil].
	self resetContext: ctxt.
	"Issue 3015 - Hernan"
	self isInterruptedContextATest ifTrue: [ self prepareTestToRunAgain ].	

	World
			addAlarm: #changed:
			withArguments: #(contentsSelection)
			for: self
			at: (Time millisecondClockValue + 200).
	^true! !

!Debugger methodsFor: 'code pane' stamp: 'tk 4/15/1998 18:31'!
contentsSelection

	^ self pcRange! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
contextStackIndex
	"Answer the index of the selected context."

	^contextStackIndex! !

!Debugger methodsFor: 'private' stamp: 'LucFabresse 10/1/2010 12:15'!
contextStackIndex: anInteger oldContextWas: oldContext 
	"Change the context stack index to anInteger, perhaps in response to
	user selection."
	| isNewMethod selectedContextSlotName index |
	contextStackIndex := anInteger.
	anInteger = 0
		ifTrue: [currentCompiledMethod := contents := nil.
			self changed: #contextStackIndex.
			self decorateButtons.
			self contentsChanged.
			contextVariablesInspector object: nil.
			receiverInspector object: self receiver.
			^ self].
	selectedContextSlotName := contextVariablesInspector selectedSlotName.
	isNewMethod := oldContext == nil
				or: [oldContext method ~~ (currentCompiledMethod := self selectedContext method)].
	isNewMethod
		ifTrue: [contents := self selectedMessage.
			self contentsChanged.
			self pcRange].
	self changed: #contextStackIndex.
	
	"update the state of create button"
	self isInterruptedContextDoesNotUnderstand
		ifTrue: [self changed: #isInterruptedContextDoesNotUnderstand].
	self decorateButtons.
	contextVariablesInspector object: self selectedContext.
	((index := contextVariablesInspector fieldList indexOf: selectedContextSlotName) ~= 0
			and: [index ~= contextVariablesInspector selectionIndex])
		ifTrue: [contextVariablesInspector toggleIndex: index].
	receiverInspector object: self receiver.
	isNewMethod
		ifFalse: [self changed: #contentsSelection]! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 11/20/2005 21:27'!
contextStackKey: aChar from: view
	"Respond to a keystroke in the context list"

 	| selector |
	selector := ContextStackKeystrokes at: aChar ifAbsent: [nil].
	selector ifNil: [self messageListKey: aChar from: view]
		ifNotNil: [self perform: selector]! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
contextStackList
	"Answer the array of contexts."

	^contextStackList! !

!Debugger methodsFor: 'context stack menu' stamp: 'MarianoMartinezPeck 4/26/2011 21:44'!
contextStackMenu: aMenu shifted: shifted 
	"Set up the menu appropriately for the context-stack-list, either shifted
	or unshifted as per the parameter provided"
	^ shifted
		ifTrue: [aMenu
				labels: 'Browse class hierarchy
Browse class
Browse method (O)
Implementors of sent messages
Change sets with this method
Inspect instances
Inspect subinstances
Revert to previous version
Remove from current change set
Revert & remove from changes
More...'
				lines: #(5 7 10 )
				selections: #(#classHierarchy #browseClass #openSingleMessageBrowser #browseAllMessages #findMethodInChangeSets #inspectInstances #inspectSubInstances #revertToPreviousVersion #removeFromCurrentChanges #revertAndForget #unshiftedYellowButtonActivity )]
		ifFalse: [self selectedContext selector = #doesNotUnderstand:
				ifTrue: [aMenu
						add: 'Implement in...'
						subMenu: (self
								populateImplementInMenu: (UIManager default menuClass new defaultTarget: self))
						target: nil
						selector: nil
						argumentList: #(nil )].
			aMenu
				labels: 'FullStack (f)
Restart (r)
Proceed (p)
Step (t)
Step through (T)
Send (e)
Where (w)
Peel to first like this
Return entered value
Toggle break on entry
Senders of... (n)
Implementors of... (m)
Inheritance (i)
Versions (v)
Inst var refs...
Inst var defs...
Class var refs...
Class variables
Class refs (N)
Browse full (b)
File out 
Mail out bug report
Copy to clipboard
More...'
				lines: #(8 9 13 15 18 21 )
				selections: #(#fullStack #restart #proceed #doStep #stepIntoBlock #send #where #peelToFirst #returnValue #toggleBreakOnEntry #browseSendersOfMessages #browseMessages #methodHierarchy #browseVersions #browseInstVarRefs #browseInstVarDefs #browseClassVarRefs #browseClassVariables #browseClassRefs #browseMethodFull #fileOutMessage #mailOutBugReport #copyToClipboard #shiftedYellowButtonActivity )]! !

!Debugger methodsFor: 'accessing' stamp: ''!
contextVariablesInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context."

	^contextVariablesInspector! !

!Debugger methodsFor: 'context stack menu' stamp: 'MarianoMartinezPeck 4/26/2011 21:45'!
copyToClipboard
	| stream |
	stream := String new writeStream. 
	self interruptedContext errorReportOn: stream.
	Clipboard clipboardText: stream contents asString.
! !

!Debugger methodsFor: 'private' stamp: 'HernanWilkinson 9/28/2010 13:23'!
createMethod
	"Should only be called when this Debugger was created in response to a
	MessageNotUnderstood exception. Create a stub for the method that was
	missing and proceed into it."
	
	| msg chosenClass |

	"Added due to error 3011 - Hernan"
	self selectedContext = self interruptedContext ifFalse: [ ^ UIManager default inform: 'Please select the #doesNotUnderstand: context'].
	
	msg := contextStackTop tempAt: 1.
	chosenClass := self
		askForSuperclassOf: contextStackTop receiver class
		toImplement: msg selector
		ifCancel: [^self].
	self implement: msg inClass: chosenClass.
! !

!Debugger methodsFor: 'initialize' stamp: 'tbn 7/29/2010 21:50'!
customButtonSpecs
	"Answer an array of elements of the form wording, selector, help-message, that characterize the custom button row of a debugger."

	| list |
	list := #(('Proceed'	proceed				'Close the debugger and proceed.')
		('Restart'		restart				'Reset this context to its start.')
		('Into'			send				'Step Into message sends')
		('Over'			doStep				'Step Over message sends')
		('Through'		stepIntoBlock		'Step into a block')
		('Full Stack'		fullStack			'Show full stack')
		('Run to Here'	runToSelection		'Run to selection')
		('Where'		where				'Select current pc range')).
	self class restartAlsoProceeds ifTrue:
		[list := list collect: [:each |
			each second == #restart
				ifTrue: [each copy at: 3 put: 'Proceed from the beginning of this context.'; yourself]
				ifFalse: [each]]].
	^ list! !

!Debugger methodsFor: 'notifier menu' stamp: 'alain.plantec 5/30/2008 11:43'!
debug
	"Open a full DebuggerView."
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	^ self openFullMorphicLabel: topView label! !

!Debugger methodsFor: 'initialize' stamp: 'kfr 10/4/2000 22:13'!
debugAt: anInteger
	self toggleContextStackIndex: anInteger. 
	 ^ self debug.! !

!Debugger methodsFor: 'context stack menu' stamp: 'MarcusDenker 1/23/2011 09:11'!
debugProceedMenu: aMenu
	^ aMenu
		labels:
			('Proceed
Debug' collect: [ :l | l translated ])
		lines: #()
		selections: #(#proceed #debug)! !

!Debugger methodsFor: 'code pane' stamp: ''!
doItContext
	"Answer the context in which a text selection can be evaluated."

	contextStackIndex = 0
		ifTrue: [^super doItContext]
		ifFalse: [^self selectedContext]! !

!Debugger methodsFor: 'code pane' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^self receiver! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 11/20/2005 21:27'!
doStep
	"Send the selected message in the accessed method, and regain control 
	after the invoked method returns."
	
	| currentContext newContext |
	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	currentContext := self selectedContext.
	newContext := interruptedProcess completeStep: currentContext.
	newContext == currentContext ifTrue: [
		newContext := interruptedProcess stepToSendOrReturn].
	self contextStackIndex > 1
		ifTrue: [self resetContext: newContext]
		ifFalse: [newContext == currentContext
				ifTrue: [self changed: #contentsSelection.
						self updateInspectors]
				ifFalse: [self resetContext: newContext]].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/15/1998 16:55'!
down
	"move down the context stack to the previous (enclosing) context"

	self toggleContextStackIndex: contextStackIndex+1! !

!Debugger methodsFor: 'initialize' stamp: 'sd 11/20/2005 21:27'!
errorWasInUIProcess: boolean

	errorWasInUIProcess := boolean! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'IgorStasenko 3/6/2011 18:38'!
expandStack
	"A Notifier is being turned into a full debugger.  Show a substantial amount of stack in the context pane."

	self newStack: (contextStackTop stackOfSize: 20).
	contextStackIndex := 0.
	receiverInspector := Smalltalk tools inspector inspect: nil.
	contextVariablesInspector := ContextVariablesInspector inspect: nil.
	proceedValue := nil! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
externalInterrupt: aBoolean

	externalInterrupt := aBoolean ! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 5/15/2010 23:06'!
fullStack
	"Change from displaying the minimal stack to a full one."

	contextStack last sender isNil
		ifTrue:
			[self changed: #flash]
		ifFalse:
			[| oldContextStackIndex |
          oldContextStackIndex := contextStackIndex.
          self fullyExpandStack.
          oldContextStackIndex = contextStackIndex
               ifFalse: [self toggleContextStackIndex: oldContextStackIndex]]! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'sd 5/15/2010 23:01'!
fullyExpandStack
	"Expand the stack to include all of it, rather than the first four or five
	contexts."

	self okToChange ifFalse: [^ self].
	self newStack: (contextStackTop stackOfSize: contextStack size + 100000).
	self changed: #contextStackList! !

!Debugger methodsFor: 'tally support' stamp: 'ab 3/23/2005 16:43'!
getTextMorph
	^ (self dependents select: [:m| m class == PluggableTextMorph]) first! !

!Debugger methodsFor: 'context stack menu' stamp: 'HernanWilkinson 10/13/2010 10:49'!
implement: aMessage inClass: aClass
	
	aClass
		compile: (DynamicMessageImplementor for: aMessage in: aClass) value
		classified: (self askForCategoryIn: aClass default: 'as yet unclassified').
	self setContentsToForceRefetch.
	self selectedContext privRefreshWith: (aClass lookupSelector: aMessage selector).
	self selectedContext method numArgs > 0 ifTrue:
		[(self selectedContext tempAt: 1) arguments withIndexDo:
			[:arg :index|
			self selectedContext tempAt: index put: arg]].
	self resetContext: self selectedContext.
	self debug.
! !

!Debugger methodsFor: 'initialize' stamp: 'tk 5/9/2003 11:20'!
initialExtent
	"Make the full debugger longer!!"

	dependents size < 9 ifTrue: [^ super initialExtent].	"Pre debug window"
	RealEstateAgent standardWindowExtent y < 400 "a tiny screen" 
		ifTrue: [^ super initialExtent].
	
	^ 600@700
! !

!Debugger methodsFor: 'accessing' stamp: ''!
interruptedContext
	"Answer the suspended context of the interrupted process."

	^contextStackTop! !

!Debugger methodsFor: 'accessing' stamp: 'StephaneDucasse 2/6/2011 21:52'!
interruptedProcess
	
	^ interruptedProcess! !

!Debugger methodsFor: 'testing' stamp: 'HernanWilkinson 9/30/2010 13:54'!
isInterruptedContextATest

	^ (self isTestObject: self interruptedContext receiver) and: [ self isTestMethod: self interruptedContext method of: self interruptedContext receiver ]! !

!Debugger methodsFor: 'testing' stamp: 'HernanWilkinson 9/28/2010 12:11'!
isInterruptedContextDoesNotUnderstand	
	
	^ self interruptedContext selector == #doesNotUnderstand:! !

!Debugger methodsFor: 'testing' stamp: 'HernanWilkinson 9/30/2010 13:57'!
isTestMethod: aCompiledMethod of: aTestCase

	^ aCompiledMethod selector = aTestCase selector! !

!Debugger methodsFor: 'testing' stamp: 'HernanWilkinson 9/30/2010 13:56'!
isTestObject: anObject

	"I'm not sure this is the best way to doit because it creates a coupling with TestCase, but due that SUnit is part of the core I think it is not bad after all - Hernan'"
	^ anObject isKindOf: TestCase! !

!Debugger methodsFor: 'accessing' stamp: 'hmm 7/16/2001 21:54'!
labelString
	^labelString! !

!Debugger methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
labelString: aString
	labelString := aString.
	self changed: #relabel! !

!Debugger methodsFor: 'private' stamp: 'adrian_lienhard 7/18/2009 15:53'!
lowSpaceChoices
	"Return a notifier message string to be presented when space is running low."

	^ 'Warning!! Pharo is almost out of memory!!

Low space detection is now disabled. It will be restored when you close or proceed from this error notifier. Don''t panic, but do proceed with caution.

Here are some suggestions:

 If you suspect an infinite recursion (the same methods calling each other again and again), then close this debugger, and fix the problem.

 If you want this computation to finish, then make more space available (read on) and choose "proceed" in this debugger. Here are some ways to make more space available...
   > Close any windows that are not needed.
   > Get rid of some large objects (e.g., images).
   > Leave this window on the screen, choose "save as..." from the screen menu, quit, restart the Pharo VM with a larger memory allocation, then restart the image you just saved, and choose "proceed" in this window.

 If you want to investigate further, choose "debug" in this window.  Do not use the debugger "fullStack" command unless you are certain that the stack is not very deep. (Trying to show the full stack will definitely use up all remaining memory if the low-space problem is caused by an infinite recursion!!).

'
! !

!Debugger methodsFor: 'context stack menu' stamp: 'nice 1/5/2010 15:59'!
mailOutBugReport
	"Compose a useful bug report showing the state of the process as well as vital image statistics as suggested by Chris Norton - 
'Squeak could pre-fill the bug form with lots of vital, but
oft-repeated, information like what is the image version, last update
number, VM version, platform, available RAM, author...'

and address it to the list with the appropriate subject prefix."

	
	MailSender default ifNil: [^self].

	Cursor write
		showWhile: 
			[ | messageStrm |"Prepare the message"
			messageStrm := (String new: 1500) writeStream.
			messageStrm nextPutAll: 'From: ';
			 nextPutAll: MailSender userName;
			 cr;
			 nextPutAll: 'To: Pharo-project@lists.gforge.inria.fr';
			 cr;
			 nextPutAll: 'Subject: ';
			 nextPutAll: '[BUG]'; nextPutAll: self interruptedContext printString;
			 cr;cr;
			 nextPutAll: 'here insert explanation of what you were doing, suspect changes you''ve made and so forth.';cr;cr.
			self interruptedContext errorReportOn: messageStrm.

			MailSender sendMessage: (MailMessage from: messageStrm contents)].
! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
messageListIndex
	"Answer the index of the currently selected context."

	^contextStackIndex! !

!Debugger methodsFor: 'context stack menu' stamp: 'sw 3/16/2001 17:20'!
messageListMenu: aMenu shifted: shifted
	"The context-stack menu takes the place of the message-list menu in the debugger, so pass it on"

	^ self contextStackMenu: aMenu shifted: shifted! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
newStack: stack
	| oldStack diff |
	oldStack := contextStack.
	contextStack := stack.
	(oldStack == nil or: [oldStack last ~~ stack last])
		ifTrue: [contextStackList := contextStack collect: [:ctx | ctx printString].
				^ self].
	"May be able to re-use some of previous list"
	diff := stack size - oldStack size.
	contextStackList := diff <= 0
		ifTrue: [contextStackList copyFrom: 1-diff to: oldStack size]
		ifFalse: [diff > 1
				ifTrue: [contextStack collect: [:ctx | ctx printString]]
				ifFalse: [(Array with: stack first printString) , contextStackList]]! !

!Debugger methodsFor: 'initialize' stamp: 'JuanVuletich 10/22/2010 14:06'!
openFullMorphicLabel: aLabelString
	"Open a full morphic debugger with the given label"

	^UIManager default openDebugger: self fullMorphicLabel: aLabelString! !

!Debugger methodsFor: 'initialize' stamp: 'JuanVuletich 11/1/2010 10:08'!
openFullNoSuspendLabel: aString 
	"Create and schedule a full debugger with the given label. Do not
	terminate the current active process."
	self openFullMorphicLabel: aString.
	errorWasInUIProcess := UIManager default spawnNewProcessIfThisIsUI: interruptedProcess! !

!Debugger methodsFor: 'initialize' stamp: 'FernandoOlivero 3/6/2011 22:02'!
openNotifierContents: msgString label: label
	"Create and schedule a notifier view with the given label and message.
	A notifier view shows just the message or the first several lines of the
	stack, with a menu that allows the user to open a full debugger if so
	desired. "

	"NOTE: When this method returns, a new process has been scheduled to
	run the windows, and thus this notifier, but the previous active porcess
	has not been suspended. The sender will do this."

	| msg |
	Sensor flushKeyboard.
	savedCursor := Cursor currentCursor.
	Cursor normal show.
	msg := (label beginsWith: 'Space is low')
		ifTrue: [ self lowSpaceChoices , (msgString ifNil: [ '' ]) ]
		ifFalse: [ msgString ].
	self buildMorphicNotifierLabelled: label message: msg.
	errorWasInUIProcess := UIManager default spawnNewProcessIfThisIsUI: interruptedProcess! !

!Debugger methodsFor: 'code pane' stamp: 'tk 5/2/1998 10:04'!
pc

	^ self pcRange! !

!Debugger methodsFor: 'code pane' stamp: 'eem 3/12/2009 14:54'!
pcRange
	"Answer the indices in the source code for the method corresponding to 
	the selected context's program counter value."

	(selectingPC and: [contextStackIndex ~= 0]) ifFalse:
		[^1 to: 0].
	self selectedContext isDead ifTrue:
		[^1 to: 0].
	^self selectedContext debuggerMap
		rangeForPC: self selectedContext pc
		contextIsActiveContext: contextStackIndex = 1! !

!Debugger methodsFor: 'context stack menu' stamp: 'md 2/20/2006 20:23'!
peelToFirst
	"Peel the stack back to the second occurance of the currently selected message.  Very useful for an infinite recursion.  Gets back to the second call so you can see one complete recursion cycle, and how it was called at the beginning.  Also frees a lot of space!!"

	| ctxt |
	contextStackIndex = 0 ifTrue: [^ Beeper beep].
	"self okToChange ifFalse: [^ self]."
	ctxt := interruptedProcess popTo: self selectedContext findSecondToOldestSimilarSender.
	self resetContext: ctxt.
! !

!Debugger methodsFor: 'code pane menu' stamp: 'sd 11/20/2005 21:27'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then perform it on myself.  If not, send it to otherTarget, presumably the editPane from which the menu was invoked." 

	| result |
	(#(debug proceed) includes: selector)		"When I am a notifier window"
		ifTrue: [^ self perform: selector]
		ifFalse: [result := super perform: selector orSendTo: otherTarget.
				selector == #doIt ifTrue: [
					result ~~ #failedDoit ifTrue: [self proceedValue: result]].
				^ result]! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 11/20/2005 21:27'!
populateImplementInMenu: aMenu

	| msg |
	msg := self selectedContext at: 1.
	self selectedContext receiver class withAllSuperclasses do:
		[:each |
		aMenu add: each name target: self selector: #implement:inClass: argumentList: (Array with: msg with: each)].
	^ aMenu

! !

!Debugger methodsFor: 'initialize' stamp: 'tbn 7/29/2010 21:50'!
preDebugButtonQuads
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'	 translated.		#debug.		#red. 	'Bring up a debugger' translated}
		}
! !

!Debugger methodsFor: 'initialize' stamp: 'stephane.ducasse 10/26/2008 15:33'!
preDebugNotifierContentsFrom: messageString

	^ messageString
		! !

!Debugger methodsFor: 'private' stamp: 'HernanWilkinson 9/30/2010 13:57'!
prepareTestToRunAgain

	self interruptedContext receiver prepareToRunAgain
	! !

!Debugger methodsFor: 'context stack menu' stamp: 'di 5/5/1998 00:07'!
proceed
	"Proceed execution of the receiver's model, starting after the expression at 
	which an interruption occurred."

	Smalltalk okayToProceedEvenIfSpaceIsLow ifTrue: [
		self proceed: self topView].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 1/24/2003 12:29'!
proceed: aTopView 
	"Proceed from the interrupted state of the currently selected context. The 
	argument is the topView of the receiver. That view is closed."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	self resumeProcess: aTopView! !

!Debugger methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
proceedValue: anObject 
	"Set the value to be returned to the selected context when the interrupted 
	process proceeds."

	proceedValue := anObject! !

!Debugger methodsFor: 'private' stamp: 'lr 3/14/2010 21:13'!
process: aProcess controller: aController context: aContext
	super initialize.
	Smalltalk globals at: #MessageTally ifPresent: [ :c | c new close ].
	contents := nil.
	interruptedProcess := aProcess.
	interruptedController := aController.
	contextStackTop := aContext.
	self newStack: (contextStackTop stackOfSize: 1).
	contextStackIndex := 1.
	externalInterrupt := false.
	selectingPC := true.
	errorWasInUIProcess := false! !

!Debugger methodsFor: 'accessing' stamp: ''!
receiver
	"Answer the receiver of the selected context, if any. Answer nil 
	otherwise."

	contextStackIndex = 0
		ifTrue: [^nil]
		ifFalse: [^self selectedContext receiver]! !

!Debugger methodsFor: 'accessing' stamp: ''!
receiverInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context's receiver."

	^receiverInspector! !

!Debugger methodsFor: 'initialize' stamp: 'jm 8/20/1998 18:31'!
release

	self windowIsClosing.
	super release.
! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
resetContext: aContext 
	"Used when a new context becomes top-of-stack, for instance when the
	method of the selected context is re-compiled, or the simulator steps or
	returns to a new method. There is room for much optimization here, first
	to save recomputing the whole stack list (and text), and secondly to avoid
	recomposing all that text (by editing the paragraph instead of recreating it)."

	| oldContext |
	oldContext := self selectedContext.
	contextStackTop := aContext.
	self newStack: contextStackTop contextStack.
	self changed: #contextStackList.
	self contextStackIndex: 1 oldContextWas: oldContext.
	self contentsChanged.
! !

!Debugger methodsFor: 'context stack menu' stamp: 'HernanWilkinson 9/30/2010 14:08'!
restart
	"Proceed from the initial state of the currently selected context. The 
	argument is a controller on a view of the receiver. That view is closed."
	"Closing now depends on a setting (RestartAlsoProceeds class variable)"

	| ctxt noUnwindError |
	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	ctxt := interruptedProcess popTo: self selectedContext.
	noUnwindError := false.
	ctxt == self selectedContext ifTrue: [
		noUnwindError := true.
		interruptedProcess restartTop; stepToSendOrReturn].
	self resetContext: ctxt.
	"Issue 3015 - Hernan"
	self isInterruptedContextATest ifTrue: [ self prepareTestToRunAgain ].	

	(self class restartAlsoProceeds and: [noUnwindError]) ifTrue: [self proceed].
! !

!Debugger methodsFor: 'private' stamp: 'FernandoOlivero 3/6/2011 22:03'!
resumeProcess: aTopView 
	savedCursor notNil ifTrue:[ savedCursor show].
	interruptedProcess isTerminated
		ifFalse: [errorWasInUIProcess
				ifTrue: [UIManager default resumeUIProcess: interruptedProcess]
				ifFalse: [interruptedProcess resume]].
	"if old process was terminated, just terminate current one"
	interruptedProcess := nil.
	"Before delete, so release doesn't terminate it"
	aTopView delete.
	World displayWorld.
	Smalltalk installLowSpaceWatcher.
	"restart low space handler"
	errorWasInUIProcess == false
		ifFalse: [Processor terminateActive]! !

!Debugger methodsFor: 'context stack menu' stamp: 'rbb 3/1/2005 10:50'!
returnValue
	"Force a return of a given value to the previous context!!"

	| previous selectedContext expression value |
	contextStackIndex = 0 ifTrue: [^Beeper beep].
	selectedContext := self selectedContext.
	expression := UIManager default request: 'Enter expression for return value:'.
	value := Compiler new 
				evaluate: expression
				in: selectedContext
				to: selectedContext receiver.
	previous := selectedContext sender.
	self resetContext: previous.
	interruptedProcess popTo: previous value: value! !

!Debugger methodsFor: 'accessing' stamp: 'hfm 12/21/2008 22:57'!
runToSelection

	| currentContext selectionInterval |
	
	selectionInterval := self codeTextMorph selectionInterval.
	self pc first >= selectionInterval first 
		ifTrue: [ ^self ].
	currentContext := self selectedContext.
	[ currentContext == self selectedContext and: [ self pc first < selectionInterval first ] ] 
		whileTrue: [ self doStep ].! !

!Debugger methodsFor: 'code pane menu' stamp: 'sd 11/20/2005 21:27'!
runToSelection: selectionInterval
	| currentContext |
	self pc first >= selectionInterval first ifTrue: [ ^self ].
	currentContext := self selectedContext.
	[ currentContext == self selectedContext and: [ self pc first < selectionInterval first ] ] whileTrue: [ self doStep ].! !

!Debugger methodsFor: 'class list' stamp: 'md 2/17/2006 09:32'!
selectedClass
	"Answer the class in which the currently selected context's method was 
	found."

	^self selectedContext methodClass! !

!Debugger methodsFor: 'private' stamp: ''!
selectedContext

	contextStackIndex = 0
		ifTrue: [^contextStackTop]
		ifFalse: [^contextStack at: contextStackIndex]! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'eem 3/12/2009 14:54'!
selectedMessage
	"Answer the source code of the currently selected context."
	^contents := self selectedContext debuggerMap sourceText asText makeSelectorBold! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'eem 9/5/2008 13:57'!
selectedMessageName
	"Answer the message selector of the currently selected context.
	 If the method is unbound we can still usefully answer its old selector."

	| selector |
	selector := self selectedContext methodSelector.
	^(selector ~~ self selectedContext method selector
	    and: [selector beginsWith: 'DoIt'])
		ifTrue: [self selectedContext method selector]
		ifFalse: [selector]! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 1/24/2003 12:29'!
send
	"Send the selected message in the accessed method, and take control in 
	the method invoked to allow further step or send."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	interruptedProcess step: self selectedContext.
	self resetContext: interruptedProcess stepToSendOrReturn.
! !

!Debugger methodsFor: 'dependents access' stamp: 'di 1/14/1999 09:28'!
step 
	"Update the inspectors."

	receiverInspector ifNotNil: [receiverInspector step].
	contextVariablesInspector ifNotNil: [contextVariablesInspector step].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 1/24/2003 12:46'!
stepIntoBlock
	"Send messages until you return to the present method context.
	 Used to step into a block in the method."

	interruptedProcess stepToHome: self selectedContext.
	self resetContext: interruptedProcess stepToSendOrReturn.! !

!Debugger methodsFor: 'notifier menu' stamp: 'IgorStasenko 1/21/2011 19:03'!
storeLog
 	Smalltalk logError: labelString printString inContext: contextStackTop
 ! !

!Debugger methodsFor: 'tally support' stamp: 'ab 3/23/2005 16:42'!
tally

	self getTextMorph tallyIt.
! !

!Debugger methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 09:44'!
taskbarIcon
	^ self theme smallDebugIcon! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
toggleContextStackIndex: anInteger 
	"If anInteger is the same as the index of the selected context, deselect it. 
	Otherwise, the context whose index is anInteger becomes the selected 
	context."

	self contextStackIndex: 
		(contextStackIndex = anInteger
			ifTrue: [0]
			ifFalse: [anInteger])
		oldContextWas:
		(contextStackIndex = 0
			ifTrue: [nil]
			ifFalse: [contextStack at: contextStackIndex])! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/15/1998 16:55'!
up
	"move up the context stack to the next (enclosed) context"

	contextStackIndex > 1 ifTrue: [self toggleContextStackIndex: contextStackIndex-1]! !

!Debugger methodsFor: 'dependents access' stamp: 'hmm 7/15/2001 19:48'!
updateInspectors 
	"Update the inspectors on the receiver's variables."

	receiverInspector == nil ifFalse: [receiverInspector update].
	contextVariablesInspector == nil ifFalse: [contextVariablesInspector update]! !

!Debugger methodsFor: 'initialize' stamp: 'sw 1/24/2001 21:22'!
wantsOptionalButtons
	"The debugger benefits so majorly from the optional buttons that we put them up regardless of the global setting.  Some traditionalists will want to change this method manually!!"

	^ true! !

!Debugger methodsFor: 'dependents access' stamp: 'di 1/14/1999 09:25'!
wantsSteps
 
	^ true! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 11/20/2005 21:27'!
where
	"Select the expression whose evaluation was interrupted."

	selectingPC := true.
	self contextStackIndex: contextStackIndex oldContextWas: self selectedContext
! !

!Debugger methodsFor: 'initialize' stamp: 'sd 11/20/2005 21:27'!
windowIsClosing
	"My window is being closed; clean up. Restart the low space watcher."

	interruptedProcess == nil ifTrue: [^ self].
	interruptedProcess terminate.
	interruptedProcess := nil.
	interruptedController := nil.
	contextStack := nil.
	contextStackTop := nil.
	receiverInspector := nil.
	contextVariablesInspector := nil.
	Smalltalk installLowSpaceWatcher.  "restart low space handler"
! !

!StringHolder class methodsFor: 'instance creation' stamp: 'MarcusDenker 10/30/2010 17:15'!
open
	^self new openLabel: 'Edit'! !

!StringHolder class methodsFor: 'instance creation' stamp: 'wiz 1/3/2007 14:23'!
openLabel: aString

	^self new openLabel: aString! !

!StringHolder class methodsFor: 'window color' stamp: 'AlainPlantec 12/16/2009 22:10'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightYellow ! !

!StringHolder class methodsFor: 'themes' stamp: 'FernandoOlivero 4/12/2011 09:52'!
theme
	"Answer the ui theme that provides controls."

	^UITheme current! !

!StringHolder methodsFor: 'accessing' stamp: 'di 5/19/1998 15:34'!
acceptContents: aString 
	"Set aString to be the contents of the receiver.  Return true cuz happy"

	self contents: aString.
	^ true! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
browseAllMessages
	"Create and schedule a message set browser on all implementors of all the messages sent by the current method."

	| aClass aName method filteredList |
	(aName := self selectedMessageName) ifNotNil: [
		method := (aClass := self selectedClassOrMetaClass) compiledMethodAt: aName.
		filteredList := method messages reject: 
			[:each | #(new initialize = ) includes: each].
		self systemNavigation browseAllImplementorsOfList: filteredList asSortedCollection
			 title: 'All messages sent in ', aClass name, '.', aName]
! !

!StringHolder methodsFor: '*Tools' stamp: 'tk 4/18/1998 16:11'!
browseClass
	"Open an class browser on this class and method"

	self selectedClassOrMetaClass ifNotNil: [
		Browser newOnClass: self selectedClassOrMetaClass 
			selector: self selectedMessageName]! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'MarcusDenker 2/16/2010 16:15'!
browseClassRefs

	| class |
	class := self selectedClass theNonMetaClass ifNil: [^self].
	class isTrait 
		ifTrue: [self systemNavigation browseAllUsersOfTrait: class]
		ifFalse: [self systemNavigation browseAllCallsOnClass: class]
! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'al 12/6/2005 22:23'!
browseClassVarRefs
	"1/17/96 sw: devolve responsibility to the class, so that the code that does the real work can be shared"

	| cls |
	cls := self selectedClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVarRefs: cls]! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'al 12/6/2005 22:22'!
browseClassVariables
	"Browse the class variables of the selected class. 2/5/96 sw"
	| cls |
	cls := self selectedClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVariables: cls]
! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'sd 5/10/2008 17:29'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the
	receiver's current selection."
	
	^ self spawnFullProtocol! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'al 12/6/2005 22:24'!
browseInstVarDefs 

	| cls |
	cls := self selectedClassOrMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseInstVarDefs: cls]! !

!StringHolder methodsFor: '*Tools-Traits' stamp: 'al 12/6/2005 22:24'!
browseInstVarRefs
	"1/26/96 sw: real work moved to class, so it can be shared"
	| cls |
	cls := self selectedClassOrMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseInstVarRefs: cls]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 08:42'!
browseLocalImplementors
	"Present a menu of all messages sent by the currently selected message. 
	Open a message set browser of all implementors of the message chosen in or below
	the selected class.
	Do nothing if no message is chosen."
	self getSelectorAndSendQuery: #browseAllImplementorsOf:localTo:
		to: self systemNavigation
		with: { self selectedClass }! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 20:41'!
browseLocalSendersOfMessages
	"Present a menu of the currently selected message, as well as all
	messages sent by it.  Open a message set browser of all implementors
	of the message chosen in or below the selected class"

	self getSelectorAndSendQuery: #browseAllCallsOn:localTo:
		to: self systemNavigation
		with: { self selectedClass }! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 08:45'!
browseMessages
	"Present a menu of all messages sent by the currently selected message. 
	Open a message set browser of all implementors of the message chosen."

	self getSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[Browser fullOnClass: myClass selector: self selectedMessageName]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 20:40'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self getSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation! !

!StringHolder methodsFor: '*Tools' stamp: 'stephane.ducasse 10/12/2008 21:02'!
browseUnusedMethods
	| classes unsent messageList cls |
	(cls := self selectedClass)
		ifNil: [^ self].
	classes := Array with: cls with: cls class.
	unsent := Set new.
	classes
		do: [:c | unsent addAll: c selectors].
	unsent := self systemNavigation allUnsentMessagesIn: unsent.
	messageList := OrderedCollection new.
	classes
		do: [:c | (c selectors
				select: [:s | unsent includes: s]) asSortedCollection
				do: [:sel | messageList add: c name , ' ' , sel]].
	self systemNavigation browseMessageList: messageList name: 'Unsent Methods in ' , cls name! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
browseVersions
	"Create and schedule a Versions Browser, showing all versions of the 
	currently selected message. Answer the browser or nil."
	| selector class | 
	self classCommentIndicated
		ifTrue: [ ClassCommentVersionsBrowser browseCommentOf: self selectedClass.
			^nil ].

	(selector := self selectedMessageName)
		ifNil:[ self inform: 'Sorry, only actual methods have retrievable versions.'. ^nil ]
		ifNotNil: [
			class := self selectedClassOrMetaClass.
			^VersionsBrowser
				browseVersionsOf: (class compiledMethodAt: selector)
				class: self selectedClass
				meta: class isMeta
				category: (class organization categoryOfElement: selector)
				selector: selector]! !

!StringHolder methodsFor: 'initialize-release' stamp: 'BenjaminVanRyseghem 2/15/2011 13:35'!
buildTextMorph

	self textMorph: (PluggableTextMorph 
			on: self 
			text: #contents 
			accept: #acceptContents:
			readSelection: nil 
			menu: #codePaneMenu:shifted:).
	^ self textMorph 
			! !

!StringHolder methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 20:36'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Workspace'.
	windowSpec children: OrderedCollection new.
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #acceptContents:; 
		selection: nil; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!StringHolder methodsFor: 'accessing' stamp: 'nk 4/29/2004 12:32'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."
	^false! !

!StringHolder methodsFor: '*Tools' stamp: 'MarcusDenker 2/16/2010 16:31'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self systemNavigation
		browseHierarchy: self selectedClassOrMetaClass "OK if nil"
		selector: self selectedMessageName
! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 5/8/2000 02:16'!
classListKey: aChar from: view 
	"Respond to a Command key.  I am a model with a list of classes and a 
	code pane, and I also have a listView that has a list of methods.  The 
	view knows how to get the list and selection."

	aChar == $f ifTrue: [^ self findMethod].
	aChar == $r ifTrue: [^ self recent].
	aChar == $h ifTrue: [^ self spawnHierarchy].
	aChar == $x ifTrue: [^ self removeClass].
	^ self messageListKey: aChar from: view! !

!StringHolder methodsFor: 'user edits' stamp: 'di 4/21/1998 11:30'!
clearUserEditFlag
	"Clear the hasUnacceptedEdits flag in all my dependent views."

	self changed: #clearUserEdits! !

!StringHolder methodsFor: 'code pane menu' stamp: 'AlainPlantec 11/16/2010 22:25'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	| donorMenu |
	donorMenu := shifted
				ifTrue: [SmalltalkEditor shiftedYellowButtonMenu]
				ifFalse: [SmalltalkEditor yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!StringHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/29/2011 13:20'!
codeTextMorph
	^ self dependents
		detect: [:dep | (dep isKindOf: PluggableTextMorph)
				and: [dep getTextSelector == #contents]]
		ifNone: []! !

!StringHolder methodsFor: 'accessing' stamp: ''!
contents
	"Answer the contents that the receiver is holding--presumably a string."

	^contents! !

!StringHolder methodsFor: 'accessing' stamp: 'AlexandreBergel 1/16/2009 10:25'!
contents: textOrString 
	"Set textOrString to be the contents of the receiver."

	contents := textOrString "asString"! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/3/98 22:50'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0  "null selection"! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
copyName
	"Copy the current selector to the clipboard"
	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString asText]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
copySelector
	"Copy the selected selector to the clipboard"

	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString]! !

!StringHolder methodsFor: 'initialization' stamp: ''!
defaultContents

	^''! !

!StringHolder methodsFor: 'evaluation' stamp: ''!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!StringHolder methodsFor: 'evaluation' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a 
	text selection."

	^nil! !

!StringHolder methodsFor: 'initialize-release' stamp: 'jmv 5/10/2009 09:32'!
embeddedInMorphicWindowLabeled: labelString

	^self embeddedInMorphicWindowLabeled: labelString wrap: true! !

!StringHolder methodsFor: 'initialize-release' stamp: 'StephaneDucasse 2/14/2011 12:38'!
embeddedInMorphicWindowLabeled: labelString wrap: aBoolean
	| window |
	window :=  (SystemWindow labelled: labelString) model: self.
	window 
		addMorph: (self buildTextMorph wrapFlag: aBoolean)
		frame: (0@0 corner: 1@1).
	^ window! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 7/1/2001 08:24'!
fileOutMessage
	"Put a description of the selected message on a file"

	self selectedMessageName ifNotNil:
		[Cursor write showWhile:
			[self selectedClassOrMetaClass fileOutMethod: self selectedMessageName]]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
findMethodInChangeSets
	"Find and open a changeSet containing the current method."

	| aName |
	(aName := self selectedMessageName) ifNotNil: [
		ChangeSorter browseChangeSetsWithClass: self selectedClassOrMetaClass
					selector: aName]! !

!StringHolder methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 11:07'!
initialize
	"Initialize the state of the receiver with its default contents."

	super initialize.
	contents := self defaultContents.
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
inspectInstances
	"Inspect all instances of the selected class."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[myClass theNonMetaClass inspectAllInstances]. 
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil: [
		aClass theNonMetaClass inspectSubInstances].
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
messageListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also
	have a listView that has a list of methods.  The view knows how to get
	the list and selection."

	| sel class |
	aChar == $D ifTrue: [^ self toggleDiffing].

	sel := self selectedMessageName.
	aChar == $m ifTrue:  "These next two put up a type in if no message selected"
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation].
	aChar == $n ifTrue: 
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation].

	"The following require a class selection"
	(class := self selectedClassOrMetaClass) ifNil: [^ self arrowKey: aChar from: view].
	aChar == $b ifTrue: [^ Browser fullOnClass: class selector: sel].
	aChar == $N ifTrue: [^ self browseClassRefs].
	aChar == $i ifTrue: [^ self methodHierarchy].
	aChar == $h ifTrue: [^ self classHierarchy].
	aChar == $p ifTrue: [^ self browseFullProtocol].

	"The following require a method selection"
	sel ifNotNil: 
		[aChar == $o ifTrue: [^ self fileOutMessage].
		aChar == $c ifTrue: [^ self copySelector].
		aChar == $v ifTrue: [^ self browseVersions].
		aChar == $O ifTrue: [^ self openSingleMessageBrowser].
		aChar == $x ifTrue: [^ self removeMessage]].

	^ self arrowKey: aChar from: view! !

!StringHolder methodsFor: '*Tools' stamp: 'marcus.denker 11/29/2008 23:23'!
messageListSelectorTitle
	| selector aString aStamp aSize |

	(selector := self selectedMessageName) ifNil: [
			aSize := self messageList size.
			^ (aSize == 0 ifTrue: ['no'] ifFalse: [aSize printString]), ' message', (aSize == 1 ifTrue: [''] ifFalse: ['s'])] ifNotNil: [
			aString := selector truncateWithElipsisTo: 28.
			^ (aStamp := self timeStamp) size > 0
				ifTrue: [aString, String cr, aStamp]
				ifFalse: [aString]]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 1/16/2004 21:10'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation 
			methodHierarchyBrowserForClass: self selectedClassOrMetaClass 
			selector: self selectedMessageName
! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/9/2000 23:59'!
noteAcceptanceOfCodeFor: aSelector
	"A method has possibly been submitted for the receiver with aSelector as its selector; If the receiver wishes to take soem action here is a chance for it to do so"
! !

!StringHolder methodsFor: '*Tools' stamp: 'JuanVuletich 10/26/2010 18:27'!
offerDurableMenuFrom: menuRetriever shifted: aBoolean
	"Pop up (morphic only) a menu whose target is the receiver and whose contents are provided by sending the menuRetriever to the receiver.  The menuRetriever takes two arguments: a menu, and a boolean representing the shift state; put a stay-up item at the top of the menu."

	| aMenu |
	aMenu := UIManager default menuClass new defaultTarget: self.
	aMenu addStayUpItem.
	self perform: menuRetriever with: aMenu with: aBoolean.
		aMenu popUpInWorld! !

!StringHolder methodsFor: '*Tools' stamp: 'JuanVuletich 10/26/2010 18:27'!
offerMenuFrom: menuRetriever shifted: aBoolean 
	"Pop up a menu whose target is
	the receiver and whose contents are provided by sending the
	menuRetriever to the receiver. The menuRetriever takes two arguments:
	a menu, and a boolean representing the shift state."
	| aMenu |
	aMenu := UIManager default menuClass new defaultTarget: self.
	self
		perform: menuRetriever
		with: aMenu
		with: aBoolean.
	aMenu popUpInWorld! !

!StringHolder methodsFor: 'user edits' stamp: 'tk 4/13/1998 23:07'!
okToChange

	self canDiscardEdits ifTrue: [^ true].
	self changed: #wantToChange.  "Solicit cancel from view"
	^ self canDiscardEdits
! !

!StringHolder methodsFor: 'initialization' stamp: 'wiz 1/3/2007 14:27'!
openAsMorphLabel: labelString 
	"Workspace new openAsMorphLabel: 'Workspace'"
	^(self embeddedInMorphicWindowLabeled: labelString) openInWorld! !

!StringHolder methodsFor: 'initialize-release' stamp: 'jmv 5/10/2009 09:21'!
openLabel: aString 
	"Create a standard system view of the model, me, a StringHolder and open it.  If in mvc, terminate the active controller so that the new window will immediately be activated."
	self openAsMorphLabel: aString! !

!StringHolder methodsFor: 'initialize-release' stamp: 'jmv 5/10/2009 09:37'!
openLabel: aString wrap: aBoolean

	"Create a standard system view of the model, me, a StringHolder and open it."
	(self embeddedInMorphicWindowLabeled: aString wrap: aBoolean) openInWorld! !

!StringHolder methodsFor: '*Tools' stamp: 'StephaneDucasse 1/7/2011 18:06'!
openSingleMessageBrowser
	| msgName mr |
	"Create and schedule a message list browser populated only by the currently selected message"

	(msgName := self selectedMessageName) ifNil: [^ self].

	mr := SourcedMethodReference new
		setClass: self selectedClassOrMetaClass
		methodSymbol: msgName.

	self systemNavigation 
		browseMessageList: (Array with: mr)
		name: mr asStringOrText
		autoSelect: nil! !

!StringHolder methodsFor: '*Tools' stamp: 'RAA 12/10/1999 09:36'!
packageListKey: aChar from: view
	"Respond to a Command key in the package pane in the PackageBrowser"
	aChar == $f ifTrue: [^ self findClass].
	^ self classListKey: aChar from: view
! !

!StringHolder methodsFor: 'code pane menu' stamp: 'AlexandreBergel 1/16/2009 10:37'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	^ (self respondsTo: selector)
		ifTrue: [ self perform: selector]
		ifFalse: [ otherTarget perform: selector]! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/1/2000 11:04'!
reformulateList
	"If the receiver has a way of reformulating its message list, here is a chance for it to do so"! !

!StringHolder methodsFor: 'accessing' stamp: 'sw 12/6/2000 17:48'!
reformulateListNoting: newSelector
	"A method has possibly been submitted for the receiver with newSelector as its selector; If the receiver has a way of reformulating its message list, here is a chance for it to do so"

	^ self reformulateList! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 5/23/2003 14:42'!
removeFromCurrentChanges
	"Tell the changes mgr to forget that the current msg was changed."

	ChangeSet current removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
	self changed: #annotation! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 1/28/1999 12:34'!
revertAndForget
	"Revert to the previous version of the current method, and tell the changes mgr to forget that it was ever changed.  Danger!!  Use only if you really know what you're doing!!"

	self okToChange ifFalse: [^ self].
	self revertToPreviousVersion.
	self removeFromCurrentChanges.
	self contentsChanged
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
revertToPreviousVersion
	"Revert to the previous version of the current method"
	| aClass aSelector  changeRecords |
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	aClass ifNil: [^ self changed: #flash].
	aSelector := self selectedMessageName.
	changeRecords := aClass changeRecordsAt: aSelector.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [self changed: #flash.  ^ Beeper beep].
	changeRecords second fileIn.
	self contentsChanged
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 11/20/2005 21:26'!
selectMessageAndEvaluate: aBlock
	"Allow the user to choose one selector, chosen from the currently selected message's selector, as well as those of all messages sent by it, and evaluate aBlock on behalf of chosen selector.  If there is only one possible choice, simply make it; if there are multiple choices, put up a menu, and evaluate aBlock on behalf of the the chosen selector, doing nothing if the user declines to choose any"

	| selector method messages |
	(selector := self selectedMessageName) ifNil: [^ self].
	method := (self selectedClassOrMetaClass ifNil: [^ self])
		compiledMethodAt: selector
		ifAbsent: [].
	(method isNil or: [(messages := method messages) size == 0])
		 ifTrue: [^ aBlock value: selector].
	(messages size == 1 and: [messages includes: selector])
		ifTrue:
			[^ aBlock value: selector].  "If only one item, there is no choice"

	self systemNavigation 
		showMenuOf: messages
		withFirstItem: selector
		ifChosenDo: [:sel | aBlock value: sel]! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 14:59'!
selectedClassName
	"I may know what class is currently selected"

	self selectedClass ifNotNil: [^ self selectedClass name].
	^ nil! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:01'!
selectedClassOrMetaClass

	^ self selectedClass	"I don't know any better"! !

!StringHolder methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:22'!
selectedMessageName

	^ nil! !

!StringHolder methodsFor: '*Tools' stamp: 'nk 11/15/2002 12:23'!
systemCatListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also have a listView that has a list of methods.  The view knows how to get the list and selection."

	aChar == $f ifTrue: [^ self findClass].
	aChar == $x ifTrue: [^ self removeSystemCategory].
	^ self classListKey: aChar from: view! !

!StringHolder methodsFor: 'accessing' stamp: 'AlexandreBergel 1/16/2009 10:24'!
textContents: aStringOrText 
	"Set aStringOrText to be the contents of the receiver."

	contents := aStringOrText! !

!StringHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/15/2011 13:34'!
textMorph
	
	^ textMorph! !

!StringHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/15/2011 13:34'!
textMorph: aMorph

	textMorph := aMorph! !

!StringHolder methodsFor: '*Tools' stamp: 'MarcusDenker 2/20/2010 03:16'!
timeStamp
	"Answer the time stamp for the chosen class and method, if any, else an empty string"

	|  selector  aMethod |
	(selector := self selectedMessageName) ifNotNil:
		[self selectedClassOrMetaClass 
			ifNil:
				[^ String new]
			ifNotNil:
				[aMethod := self selectedClassOrMetaClass compiledMethodAt: selector ifAbsent: [nil].
				aMethod ifNotNil: [^ aMethod timeStamp]]].
	^ String new! !

StringHolder subclass: #Workspace
	instanceVariableNames: 'bindings acceptDroppedMorphs acceptAction mustDeclareVariables fileName lineEnding encoding stylingActive'
	classVariableNames: 'LastContents'
	poolDictionaries: ''
	category: 'Tools-Base'!

!Workspace commentStamp: 'ls 10/14/2003 12:13' prior: 0!
A Workspace is a text area plus a lot of support for executable code.  It is a great place to execute top-level commands to compute something useful, and it is a great place to develop bits of a program before those bits get put into class methods.

To open a new workspace, execute:

	Workspace open


A workspace can have its own variables, called "workspace variables", to hold intermediate results.  For example, if you type into a workspace "x := 5" and do-it, then later you could type in "y := x * 2" and y would become 10.

Additionally, in Morphic, a workspace can gain access to morphs that are on the screen.  If acceptDroppedMorphss is turned on, then whenever a morph is dropped on the workspace, a variable will be created which references that morph.  This functionality is toggled with the window-wide menu of a workspace.


The instance variables of this class are:

	bindings  -  holds the workspace variables for this workspace

	acceptDroppedMorphss - whether dropped morphs should create new variables!

!Workspace class methodsFor: 'instance creation' stamp: 'GeorgeHerolyants 1/14/2010 18:57'!
openContents: aString
	"Open a new workspace with the given contents"
	
	^ self new
		open;
	 	contents: aString;
		changed: #contents;
		yourself! !

!Workspace class methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/14/2010 18:53'!
openFile: aFileName
	"Open a new workspace associated with the given file"

	^ self new 
		open;
		openFile: aFileName;
		yourself! !

!Workspace class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 03:01'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #workspace
! !

!Workspace methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:56'!
acceptAction: anAction
	acceptAction := anAction.! !

!Workspace methodsFor: 'accessing' stamp: 'StephaneDucasse 1/21/2011 17:01'!
acceptContents: aString

	self fileName ifNotNil: [ self saveString: aString toFile: self fileName ].

	acceptAction ifNotNil:[acceptAction value: aString].
	self contents: aString.
	^ true! !

!Workspace methodsFor: 'as yet unclassified' stamp: 'tbn 7/6/2010 16:40'!
acceptDroppedMorphsWording

	^ self acceptsDroppingMorphForReference
		ifTrue: ['<yes> Create textual references to dropped morphs' translated]
		ifFalse: ['<no> Create textual references to dropped morphs' translated]
! !

!Workspace methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:27'!
acceptDroppingMorph: dropee event: evt inMorph: targetMorph 
	"Return the dropee to its old position, and add a reference to it at the cursor point."

	| bindingName externalName |
	externalName := dropee externalName.
	externalName := externalName isOctetString
		ifTrue: [externalName] ifFalse: ['a' , externalName].
	bindingName := externalName translateToLowercase, dropee identityHash printString.
	targetMorph correctSelectionWithString: bindingName, ' '.
	(self bindingOf: bindingName) value: dropee.
	dropee rejectDropMorphEvent: evt.
	^ true "success"
! !

!Workspace methodsFor: 'drag and drop' stamp: 'jcg 7/8/2000 00:10'!
acceptsDroppingMorphForReference

	^ acceptDroppedMorphs
		
! !

!Workspace methodsFor: 'menu commands' stamp: 'BenjaminVanRyseghem 2/15/2011 13:28'!
addModelItemsToWindowMenu: aMenu 
	aMenu addLine.
	aMenu
		add: 'Open...'
		target: self
		action: #openCommand.

	aMenu
		add: 'Save as...'
		target: self
		action: #saveAsCommand.
	
	self fileName ifNotNil: 
		[aMenu
			add: 'Reload'
			target: self
			action: #reloadCommand].
	
	aMenu addLine.
	
	aMenu
		add: 'Encoding' subMenu: 
		([ :menu | 
		TextConverter allEncodingNames do: 
			[ :ea |
			menu
				addUpdating: #itemForEncoding: 
				target: self 
				selector: #useEncoding: 
				argumentList: {ea}].
		menu] value: MenuMorph new).
	
	aMenu addLine.
	
	aMenu 
		addUpdating: #itemForCr
		target: self
		selector: #useLineEnding:
		argumentList: {#cr}.
	
	aMenu 
		addUpdating: #itemForLf
		target: self
		selector: #useLineEnding:
		argumentList: {#lf}.
	
	aMenu 
		addUpdating: #itemForCrLf
		target: self
		selector: #useLineEnding:
		argumentList: {#crlf}.
	
	aMenu addLine.
	aMenu
		add: 'Inspect variables'
		target: self
		action: #browseBindings.
	aMenu
		add: 'Reset variables'
		target: self
		action: #initializeBindings.
	aMenu
		addUpdating: #mustDeclareVariableWording
		target: self
		action: #toggleVariableDeclarationMode.
	aMenu
		addUpdating: #acceptDroppedMorphsWording
		target: self
		action: #toggleDroppingMorphForReference.

	aMenu 
		add: 'Previous contents...'
		target: self
		selector: #selectPreviousContent.
		
	aMenu addLine.
	
	aMenu
		addUpdating: #isStylingActive
		target: self
		action: #toggleStylingActive.! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/14/2010 18:34'!
askForFileNameToOpen
	"Ask user for file name to load content from"

	^ UIManager default
			chooseFileMatching: nil 
			label: 'Choose file to open in the workspace' translated! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/14/2010 18:34'!
askForFileNameToSave: suggestedFileName
	"Ask user for the name of the file to save workspace content to. Use the given file name as the initial answer"

	^ UIManager default
			request: 'Enter file name to save the workspace to' translated 
			initialAnswer: suggestedFileName! !

!Workspace methodsFor: 'binding' stamp: 'sd 3/20/2004 15:20'!
bindingOf: aString 
	
	mustDeclareVariables ifTrue: [^ nil].
	"I want to have workspace that force the user to declare  
	variables. Still subclasses may want to do something else"
	bindings isNil
		ifTrue: [self initializeBindings].
	(bindings includesKey: aString)
		ifFalse: [bindings at: aString put: nil].
	^ bindings associationAt: aString! !

!Workspace methodsFor: 'binding' stamp: 'yann.monclair 7/17/2010 12:37'!
browseBindings
	bindings inspect! !

!Workspace methodsFor: 'toolbuilder' stamp: 'BenjaminVanRyseghem 2/15/2011 13:16'!
buildTextMorph 

	| morph |
	morph := super buildTextMorph.
	morph 
		font: StandardFonts codeFont;
		beStyled.
	morph styler workspace: self.
	^ morph! !

!Workspace methodsFor: 'drag and drop' stamp: 'StephaneDucasse 1/29/2011 12:09'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	| donorMenu |
	donorMenu := shifted
				ifTrue: [self shiftedYellowButtonMenu]
				ifFalse: [self yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!Workspace methodsFor: 'accessing' stamp: 'StephaneDucasse 1/21/2011 17:20'!
contents: stringOrText
	contents := stringOrText.
	self changed: #contents.! !

!Workspace methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/20/2010 18:47'!
fileName
	^ fileName! !

!Workspace methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/20/2010 18:47'!
fileName: anObject
	fileName := anObject! !

!Workspace methodsFor: 'binding' stamp: 'JuanVuletich 10/11/2010 20:17'!
hasBindingOf: aString
	^bindings notNil and: [
		bindings includesKey: aString ]! !

!Workspace methodsFor: 'binding' stamp: 'BenjaminVanRyseghem 2/15/2011 13:17'!
hasBindingThatBeginsWith: aString

	bindings ifNil:[^false].
	bindings keysDo:[:each | 
		(each beginsWith: aString) 
			ifTrue:[^true]].
	^false
! !

!Workspace methodsFor: 'configuration' stamp: 'AlexandreBergel 1/16/2009 10:32'!
historyLength
	"Number of contents being stored"
	^ 5! !

!Workspace methodsFor: 'initialization' stamp: 'LaurentLaffont 3/14/2010 17:23'!
initialExtent
 	"Start small.  Window aspect ratio is 5 sqrt::1 . Good asthetics. -wiz"
	
	^ 450@200! !

!Workspace methodsFor: 'initialization' stamp: 'GuillermoPolito 8/13/2010 00:26'!
initialize
	| os |
	
	super initialize.
	contents := ''.
	acceptDroppedMorphs := false.
	mustDeclareVariables := false.
	lineEnding := #cr.
	os := OSPlatform platformName asUppercase. "get OS name"
	"try to guess default line ending"
	(os beginsWith: 'WIN') ifTrue: [lineEnding := #crlf].
	(os beginsWith: 'UNIX') ifTrue: [lineEnding := #lf].
	
	encoding := TextConverter default class encodingNames first.! !

!Workspace methodsFor: 'binding' stamp: 'sd 3/20/2004 14:31'!
initializeBindings
	
	bindings := Dictionary new! !

!Workspace methodsFor: 'styling' stamp: 'BenjaminVanRyseghem 2/15/2011 13:28'!
isStylingActive
	
	^self stylingActive 
		ifTrue: ['<yes> syntax highlighting' translated]
		ifFalse: ['<no> syntax highlighting' translated]! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 17:28'!
itemForCr
	^ lineEnding = #cr
		ifTrue: ['<yes> CR']
		ifFalse: ['<no> CR']! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 17:28'!
itemForCrLf
	^ lineEnding = #crlf
		ifTrue: ['<yes> CR+LF']
		ifFalse: ['<no> CR+LF']! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 16:43'!
itemForEncoding: aString
	^ encoding = aString
		ifTrue: ['<yes> ', aString]
		ifFalse: ['<no> ', aString]! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 17:28'!
itemForLf
	^ lineEnding = #lf
		ifTrue: ['<yes> LF']
		ifFalse: ['<no> LF']! !

!Workspace methodsFor: 'accessing' stamp: 'GeorgeHerolyants 1/14/2010 18:45'!
label: aString
	"Set the window label to the given string"

	self containingWindow setLabel: aString
	
	! !

!Workspace methodsFor: 'accessing' stamp: 'AlexandreBergel 1/16/2009 10:33'!
lastContents
	LastContents ifNil: [LastContents := OrderedCollection new].
	^ LastContents! !

!Workspace methodsFor: 'variable declarations' stamp: 'tbn 7/6/2010 16:40'!
mustDeclareVariableWording
	
	^ mustDeclareVariables
		ifTrue: ['<yes> Automatically create variable declaration' translated]
		ifFalse: ['<no> Automatically create variable declaration' translated]! !

!Workspace methodsFor: 'accessing' stamp: 'sd 3/20/2004 15:17'!
mustDeclareVariables: aBoolean

	mustDeclareVariables := aBoolean! !

!Workspace methodsFor: 'history' stamp: 'AlexandreBergel 1/16/2009 10:36'!
okToChange
	"This method is called by SystemWindow just before closing a workspace window. 
	The caller of this method is SystemWindow>>delete"
	
	| textMorphs textMorph contentAsString |
	textMorphs := self dependents select: [:c | c isKindOf: PluggableTextMorph].
	textMorphs ifEmpty: [ ^ true ]. "This case should normally not happen"
	
	textMorph := textMorphs first.
	contentAsString := (textMorph text) asString.
	self lastContents addFirst: contentAsString.
	self trimHistoryIfNecessary.
	^ true! !

!Workspace methodsFor: 'initialize-release' stamp: 'GeorgeHerolyants 1/14/2010 18:53'!
open
	
	self openLabel: 'Untitled'! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 14:56'!
openCommand
	"Open content of the file choosed by user in this workspace and associate it's name with this workspace"
	| userWarned |

	userWarned := true.
	(self fileName isNil and: [self contents isEmpty not or: [self hasUnacceptedEdits]]) ifTrue: [userWarned := self warnUser].
	(self fileName isNil not and: [self hasUnacceptedEdits]) ifTrue: [userWarned := self warnUser].
	
	userWarned ifTrue: [self openFile: self askForFileNameToOpen]
! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/18/2010 14:50'!
openFile: aFileName
	"Load file content into this workspace. If successful, associate the workspace with this file and change its label"

	aFileName isEmptyOrNil ifTrue: [ ^ self ].
	
	self contents: (self readStringFromFile: aFileName).
	self fileName: aFileName.
	self label: aFileName.
	self changed: #contents! !

!Workspace methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 2/15/2011 13:09'!
openLabel: labelString
	| window pane |
	window := (SystemWindow labelled: labelString) model: self.
	pane := self buildTextMorph.
	window addMorph: pane frame: (0@0 corner: 1@1).
	window openInWorld.
	^ window! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/21/2010 12:27'!
readStringFromFile: aFileName
	"Read the content of the given file. Perform line endings normalization."
	| content |
	
	content := '' writeStream.
	
	FileStream oldFileNamed: aFileName do: 
		[ :f |
		f converter: (TextConverter newForEncoding: encoding).
		
		[f atEnd] whileFalse: 
			[ | char |
			char := f next.
			char = Character cr ifTrue: 
				[f peek = Character lf 
					ifTrue: [lineEnding := #crlf. f next]
					ifFalse: [lineEnding := #cr]].
			char = Character lf ifTrue: 
				[lineEnding := #lf. 
				char := Character cr].
			content nextPut: char]].
	
	^ content contents! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/14/2010 19:14'!
reloadCommand
	"Replaces unsaved content with the content of the associated file"
	
	self warnUser ifTrue: [self openFile: self fileName]! !

!Workspace methodsFor: 'menu commands' stamp: 'MarcusDenker 4/29/2011 00:37'!
saveAsCommand
	"Save the receiver's contents string to a file, prompting the user for a file-name.  Suggest a reasonable file-name."
	| suggestedFileName | 
		
	suggestedFileName := self askForFileNameToSave: self suggestedFileName.
	suggestedFileName ifNil: [^ self].
	
	FileStream newFileNamed: suggestedFileName  do: [ :s | "do nothing"].
	self fileName: suggestedFileName.
	self label: suggestedFileName.

	self codeTextMorph ifNotNil: 
		[ :morph | | stringToSave |
		stringToSave :=  morph text string.
		self saveString: stringToSave toFile: suggestedFileName.
		self contents: stringToSave.
		self changed: #contents]! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/21/2010 12:17'!
saveString: aString toFile: aFileName
	"Save the given string to the given file"
	
	FileStream forceNewFileNamed: aFileName do: 
	[ :stream | 
		stream converter: (TextConverter newForEncoding: encoding).
		stream wantsLineEndConversion: true.
		stream lineEndConvention: lineEnding.
		stream nextPutAll: aString ].
	! !

!Workspace methodsFor: 'history' stamp: 'AlexandreBergel 1/16/2009 17:05'!
selectPreviousContent

	| trimmedContents newContentOrNil |
	trimmedContents := self lastContents collect: [:c | c copyFrom: 1 to: (c size min: 20)].
	newContentOrNil := UIManager default chooseFrom: trimmedContents values: self lastContents.
	
	"newContentOrNil may be nil if the user cancel"
	newContentOrNil ifNil: [ newContentOrNil := '' ].
	
	self setContent: newContentOrNil! !

!Workspace methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
setBindings: aDictionary
	"Sets the Workspace to use the specified dictionary as its namespace"

	bindings := aDictionary.
! !

!Workspace methodsFor: 'history' stamp: 'AlexandreBergel 1/16/2009 16:53'!
setContent: aString
	| textMorphs textMorph |
	textMorphs := self dependents select: [:c | c isKindOf: PluggableTextMorph].
	textMorphs ifEmpty: [ ^ true ]. "This case should normally not happen"
	
	textMorph := textMorphs first.
	textMorph setText: aString! !

!Workspace methodsFor: 'drag and drop' stamp: 'StephaneDucasse 2/25/2011 18:02'!
shiftedYellowButtonMenu
	"Answer the menu to be presented when the yellow button is pressed while the shift key is down"

	^ (UIManager default
		newMenuIn: textMorph for: self)
		addList: {
		
		{'Browse it (b)' translated.					#browseIt}.
		{'Senders of it (n)' translated.				#sendersOfIt}.
		{'Implementors of it (m)' translated.		#implementorsOfIt}.
		{'References to it (N)' translated.			#referencesToIt}.
		#-.
		{'Selectors containing it (W)' translated.	#methodNamesContainingIt}.
		{'Method strings with it (E)' translated.	#methodStringsContainingit}.
		{'Method source with it' translated.		#methodSourceContainingIt}.
		{'Class names containing it' translated.	#classNamesContainingIt}.
		{'Class comments with it' translated.		#classCommentsContainingIt}.
		{'Change sets with it' translated.			#browseChangeSetsWithSelector}.
	"	#-.
		{'Pretty print' translated.					#prettyPrint}.
		{'Pretty print with color' translated.		#prettyPrintWithColor}.
		{'File it in (G)' translated.					#fileItIn}.
		#-.
		{'Back...' translated.						#yellowButtonActivity}.
	"
	}
! !

!Workspace methodsFor: 'styling' stamp: 'BenjaminVanRyseghem 2/15/2011 14:16'!
stylingActive

	^stylingActive ifNil:[stylingActive := true]! !

!Workspace methodsFor: 'file support' stamp: 'GeorgeHerolyants 1/14/2010 16:17'!
suggestedFileName
	"If the workspace has an associated fileName use that, otherwise derive a reasonable file-name from the window label"

	| labelToUse suggestedName lastIndex |

	self fileName ifNotNil: [ :f | ^ f ].
						
	^ (self containingWindow 
		ifNil: ['Untitled']
		ifNotNilDo:	[ :window |  window label ]), '.ws'! !

!Workspace methodsFor: 'drag and drop' stamp: 'sd 11/20/2005 21:27'!
toggleDroppingMorphForReference

	acceptDroppedMorphs := acceptDroppedMorphs not.
		
! !

!Workspace methodsFor: 'styling' stamp: 'BenjaminVanRyseghem 2/15/2011 14:16'!
toggleStylingActive

	stylingActive := self stylingActive not.
	self textMorph toggleStyling.
	^self isStylingActive
	! !

!Workspace methodsFor: 'variable declarations' stamp: 'sd 3/20/2004 14:39'!
toggleVariableDeclarationMode

	mustDeclareVariables := mustDeclareVariables not! !

!Workspace methodsFor: 'history' stamp: 'AlexandreBergel 1/16/2009 10:35'!
trimHistoryIfNecessary
	(self lastContents size > self historyLength)
		ifTrue: [ LastContents := LastContents copyFrom: 1 to: self historyLength ]! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 17:48'!
useEncoding: aString
	encoding := aString.
	self codeTextMorph hasUnacceptedEdits: true.! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/18/2010 17:48'!
useLineEnding: aSymbol
	lineEnding := aSymbol.
	self codeTextMorph hasUnacceptedEdits: true.! !

!Workspace methodsFor: 'drag and drop' stamp: 'jcg 7/7/2000 11:16'!
wantsDroppedMorph: dropee event: evt inMorph: target

	^ acceptDroppedMorphs
		
! !

!Workspace methodsFor: 'menu commands' stamp: 'GeorgeHerolyants 1/14/2010 19:16'!
warnUser
	"Warn the user about the possible content loss"

	^ UIManager confirm: 'This will replace the current workspace content.', String cr, 'Do you want to continue?' translated! !

!Workspace methodsFor: 'drag and drop' stamp: 'StephaneDucasse 2/25/2011 18:05'!
yellowButtonMenu
	^  (UIManager default
		newMenuIn: textMorph for: self)
		addList:  {
			{'Do it (d)' translated.					#doIt. nil. #smallDoItIcon}.
			{'Print it (p)' translated.					#printIt. nil. #smallPrintIcon}.
			{'Inspect it (i)' translated.				#inspectIt. nil. #smallInspectItIcon}.
			{'Explore it (I)' translated.				#exploreIt. nil. #smallInspectItIcon}.
			{'Debug it (D)' translated.				#debugIt. nil. #smallDebugItIcon}.
			{'Profile it' translated.					#tallyIt}.
		     {'Watch it' translated.					#watchIt}.	 
		    	 
			#-.
			{'Find...(f)' translated.					#find. nil. #smallFindIcon}.
			{'Find again (g)' translated.				#findAgain.nil. #smallFindIcon}.
			{'Extended search...' translated.			#shiftedTextPaneMenuRequest}.
			#-.
			{'Do again (j)' translated.				#again. nil. #smallDoItIcon}.
			{'Undo (z)' translated.					#undo. nil. #smallUndocon}.
			#-.
			{'Copy (c)' translated.					#copySelection. nil. #smallCopyIcon}.
			{'Cut (x)' translated.						#cut. nil. #smallCutIcon}.
			{'Paste (v)' translated.					#paste. nil. #smallPasteIcon}.
			{'Paste...' translated.					#pasteRecent. nil. #smallCopyIcon}.
			#-.
			{'Accept (s)' translated.					#accept. nil. #smallOkIcon}.
			{'Cancel (l)' translated.					#cancel. nil. #smallCancelIcon}.
		}.
! !

Model subclass: #ValueHolder
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!ValueHolder methodsFor: 'as yet unclassified' stamp: 'ls 8/5/1998 07:49'!
contents
	^contents! !

!ValueHolder methodsFor: 'as yet unclassified' stamp: 'alain.plantec 6/11/2008 15:40'!
contents: newContents
	contents := newContents.
	self contentsChanged! !

!SystemDictionary methodsFor: '*Tools-Inspector' stamp: 'StephaneDucasse 3/17/2010 20:42'!
inspectGlobals
	"Smalltalk  inspectGlobals"
	
	| associations aDict |
	associations := ((self  keys select: [:aKey | ((self  at: aKey) isKindOf: Class) not]) asArray sort 
															collect: [:aKey | self associationAt: aKey]).
	aDict := IdentityDictionary new.
	associations do: [:as | aDict add: as].
	aDict inspectWithLabel: 'The Globals'! !

MorphTreeNodeModel subclass: #TimeProfilerNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 21:54'!
asString
	^ String streamContents: [:s | self item printOn: s total: model tally totalTime: model time tallyExact: false] ! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/3/2010 06:44'!
browseItem
	ToolSet browse: self methodClass  selector: self selector! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 14:33'!
childrenItems
	^ (self item sonsOver: model threshold) asSortedCollection! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 21:37'!
exploreItem
	self item explore! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 21:37'!
inspectItem
	self item inspect! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/25/2011 17:10'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	c = $b
		ifTrue: [self browseItem. ^ true].
	c = $i
		ifTrue: [self inspectItem. ^ true].
	c = $I
		ifTrue: [self exploreItem. ^ true].
	^ false
! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 21:36'!
menu: menu shifted: b 
	menu
		add: 'Browse (b)' translated
		target: self
		selector: #browseItem.
	menu
		add: 'Inspect (i)' translated
		target: self
		selector: #inspectItem.
	menu
		add: 'Explore (I)' translated
		target: self
		selector: #exploreItem.
! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/3/2010 06:44'!
methodClass
	^self item method methodClass 
	! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 22:30'!
percentage 
	^ self item tally asFloat / model time * 100.0 roundTo: 0.1.
! !

!TimeProfilerNode methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/3/2010 06:44'!
selector
	^self item method selector 
	! !

Magnitude subclass: #MessageTally
	instanceVariableNames: 'class method process tally receivers senders time gcStats maxClassNameSize maxClassPlusSelectorSize maxTabs reportOtherProcesses'
	classVariableNames: 'DefaultPollPeriod Timer'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!MessageTally commentStamp: 'StephaneDucasse 9/27/2009 10:42' prior: 0!
My instances observe and report the amount of time spent in methods.

NOTE: a higher-level user interface (combining the MessageTally result tree with a method browser) is available from TimeProfileBrowser. Note that TimeProfileBrowser was not fancy with the different setting possibilities.

	TimeProfileBrowser spyOn:  [20 timesRepeat: 
			[Transcript show: 100 factorial printString]]
	

Strategies
-----------
MessageTally provides two different strategies available for profiling:

* spyOn: and friends use a high-priority Process to interrupt the block or process being spied on at periodic intervals. The interrupted call stack is then examined for caller information. See below for an example showing different settings

* tallySends: and friends use the interpreter simulator to run the block, recording every method call.

The two give you different results:

	* spyOn: gives you a view of where the time is being spent in your program, at least on a rough statistical level (assuming you've run the 	block for long enough and have a high enough poll rate). If you're trying to optimize your code, start here and optimize the methods where 	most of the time is being spent first.

	* tallySends: gives you accurate counts of how many times methods get called, and by exactly which route. If you're debugging, or trying to 	figure out if a given method is getting called too many times, this is your tool.

Q: How do you interpret MessageTally>>tallySends
A: The methods #tallySends and #spyOn: measure two very different quantities, but broken down in the same who-called-who format.  #spyOn: is approximate, but more indicative of real time spent, whereas #tallySends is exact and a precise record of how many times each method got executed.

Examples
----------

Here you can see all the processes computation time
	
		[1000 timesRepeat: [3.14159 printString. Processor yield]] fork.
		[1000 timesRepeat: [30 factorial. Processor yield]] fork.
		[1000 timesRepeat: [30 factorial. Processor yield]] fork.
		MessageTally spyAllOn: [ (Delay forMilliseconds: 100) wait] 


Settings
---------
You can change the printing format (that is, the whitespace and string compression) by using these instance methods: 
	maxClassNameSize:
	maxClassPlusSelectorSize:
	maxTabs:

You can change the default polling period (initially set to 1) by calling
	MessageTally defaultPollPeriod: numberOfMilliseconds


To understand the difference
----------------------------------
Here we see all the processes
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally spyAllOn: [10000 timesRepeat: [1.23 printString]]
	
	
Here we only see the execution of the expression [10000 timesRepeat: [1.23 printString]
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally spyOn: [10000 timesRepeat: [1.23 printString]]
	
Here we only check the exact message sends: this is not a pc-sampling approach
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally tallySends: [10000 timesRepeat: [1.23 printString]]
	


!

!MessageTally class methodsFor: 'defaults' stamp: 'StephaneDucasse 11/11/2010 22:29'!
defaultMaxClassNameSize
	"Return the default maximum width of the class name alone"
	^50! !

!MessageTally class methodsFor: 'defaults' stamp: 'StephaneDucasse 11/11/2010 22:29'!
defaultMaxClassPlusSelectorSize
	"Return the default maximum width of the class plus selector together (not counting the '>>')"
	^100! !

!MessageTally class methodsFor: 'defaults' stamp: 'jmv 3/2/2009 12:32'!
defaultMaxTabs
	"Return the default number of tabs after which leading white space is compressed"
	^120! !

!MessageTally class methodsFor: 'defaults' stamp: 'nk 3/8/2004 12:41'!
defaultPollPeriod
	"Answer the number of milliseconds between interrupts for spyOn: and friends.
	This should be faster for faster machines."
	^DefaultPollPeriod ifNil: [ DefaultPollPeriod := 1 ]! !

!MessageTally class methodsFor: 'defaults' stamp: 'nk 3/8/2004 12:41'!
defaultPollPeriod: numberOfMilliseconds
	"Set the default number of milliseconds between interrupts for spyOn: and friends.
	This should be faster for faster machines."
	DefaultPollPeriod := numberOfMilliseconds! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:23'!
spyAllOn: aBlock
	"Spy on all the processes in the system
	
	[1000 timesRepeat: [3.14159 printString. Processor yield]] fork.
	[1000 timesRepeat: [20 factorial. Processor yield]] fork.
	[1000 timesRepeat: [20 factorial. Processor yield]] fork.
	MessageTally spyAllOn: [ (Delay forMilliseconds: 100) wait]
	
	"
	^ self spyAllOn: aBlock cutoff: 1! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:19'!
spyAllOn: aBlock cutoff: aNumber

	| node result |
	node := self new.
	node reportOtherProcesses: true.	"Irrelevant in this case. All processes will be reported on their own."
	result := node spyAllEvery: self defaultPollPeriod on: aBlock.
	(CodeHolder new contents: (String streamContents: [:s | node report: s cutoff: aNumber; close]))
		openLabel: 'Spy Results' wrap: false.
	^ result! !

!MessageTally class methodsFor: 'spying' stamp: 'jmv 9/24/2009 15:59'!
spyOn: aBlock
	"
	Spy on aBlock, in the current process. Can include or not statistics on other processes in the report.
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally spyOn: [10000 timesRepeat: [1.23 printString]]
	"
	^self spyOn: aBlock reportOtherProcesses: false! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:20'!
spyOn: aBlock cutoff: aNumber

	^self spyOn: aBlock reportOtherProcesses: false cutoff: aNumber! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:23'!
spyOn: aBlock reportOtherProcesses: aBoolean
	"
	Spy on aBlock, in the current process. Can include or not statistics on other processes in the report.
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally spyOn: [10000 timesRepeat: [1.23 printString]] reportOtherProcesses: true
	"
	^ self spyOn: aBlock reportOtherProcesses: aBoolean cutoff: 1! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:24'!
spyOn: aBlock reportOtherProcesses: aBoolean cutoff: aNumber
	"
	Spy on aBlock, in the current process. Can include or not statistics on other processes in the report.
	[1000 timesRepeat: [
		100 timesRepeat: [120 factorial].
		(Delay forMilliseconds: 10) wait
		]] forkAt: 45 named: '45'.
	MessageTally spyOn: [10000 timesRepeat: [1.23 printString]] reportOtherProcesses: true
	"
	^ self 
		spyOn: aBlock 
		reportOtherProcesses: aBoolean 
		cutoff: aNumber 
		openResultWindow: true 
		closeAfter: true! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:21'!
spyOn: aBlock reportOtherProcesses: aBoolean cutoff: aNumber openResultWindow: openResultWindow

	^ self 
		spyOn: aBlock 
		reportOtherProcesses: aBoolean 
		cutoff: aNumber 
		openResultWindow: openResultWindow 
		closeAfter: true! !

!MessageTally class methodsFor: 'spying' stamp: 'MarcusDenker 1/23/2011 09:09'!
spyOn: aBlock reportOtherProcesses: aBoolean cutoff: aNumber openResultWindow: openResultWindow closeAfter: closeAfter

	| node |
	node := self new.
	node reportOtherProcesses: aBoolean.
	node spyEvery: self defaultPollPeriod on: aBlock.
	
	openResultWindow ifTrue:
	 [ (CodeHolder new contents: (String streamContents: [:s | node report: s cutoff: aNumber]))
		openLabel: 'Spy Results' wrap: false ].
	
	closeAfter ifTrue: [ node close ].
	
	^ node
! !

!MessageTally class methodsFor: 'public' stamp: 'jmv 9/24/2009 16:02'!
spyOn: aBlock toFileNamed: fileName reportOtherProcesses: aBoolean
	"Spy on the evaluation of aBlock. Write the data collected on a file
	named fileName."

	| file value node |
	node := self new.
	node reportOtherProcesses: aBoolean.
	value := node spyEvery: self defaultPollPeriod on: aBlock.
	file := FileStream newFileNamed: fileName.
	node report: file; close.
	file close.
	^value! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/16/2010 15:21'!
spyOnProcess: aProcess forMilliseconds: msecDuration 

	^self spyOnProcess: aProcess forMilliseconds: msecDuration reportOtherProcesses: false
! !

!MessageTally class methodsFor: 'spying' stamp: 'jmv 2/19/2010 14:41'!
spyOnProcess: aProcess forMilliseconds: msecDuration reportOtherProcesses: aBoolean
	"
	Spy on aProcess for a certain amount of time
	| p1 p2 |  
	p1 := [100000 timesRepeat: [3.14159 printString. Processor yield]] newProcess.  
	p2 := [100000 timesRepeat: [3.14159 printString. Processor yield]] newProcess.
	p1 resume.
	p2 resume.  
	(Delay forMilliseconds: 100) wait.  
	MessageTally spyOnProcess: p1 forMilliseconds: 1000 reportOtherProcesses: true
	"
	| node |
	node := self new.
	node reportOtherProcesses: aBoolean.
	node
		spyEvery: self defaultPollPeriod
		onProcess: aProcess
		forMilliseconds: msecDuration.
	(CodeHolder new
		contents: (String
				streamContents: [:s | node report: s]))
		openLabel: 'Spy Results' wrap: false! !

!MessageTally class methodsFor: 'spying' stamp: 'jmv 2/19/2010 14:41'!
spyOnProcess: aProcess forMilliseconds: msecDuration toFileNamed: fileName reportOtherProcesses: aBoolean
	"Spy on the evaluation of aProcess. Write the data collected on a file  
	named fileName. Will overwrite fileName"
	| file node |
	node := self new.
	node reportOtherProcesses: aBoolean.
	node
		spyEvery: self defaultPollPeriod
		onProcess: aProcess
		forMilliseconds: msecDuration.
	file := FileStream fileNamed: fileName.
	node report: file.
	file close! !

!MessageTally class methodsFor: 'public' stamp: 'Alexandre Bergel 3/5/2010 17:41'!
tallySends: aBlock
	"
	MessageTally tallySends: [3.14159 printString]
	"
	
	"This method uses the simulator to count the number of calls on each method
	invoked in evaluating aBlock. If receiver is not nil, then only sends
	to that receiver are tallied.
	Results are presented as leaves, sorted by frequency,
	preceded, optionally, by the whole tree."

	^ self tallySendsTo: nil inBlock: aBlock showTree: true! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/4/2010 18:47'!
tallySendsTo: receiver inBlock: aBlock showTree: treeOption
	^ self tallySendsTo: receiver inBlock: aBlock showTree: treeOption closeAfter: true! !

!MessageTally class methodsFor: 'spying' stamp: 'Alexandre Bergel 3/5/2010 17:41'!
tallySendsTo: receiver inBlock: aBlock showTree: treeOption closeAfter: closeAfter
	^ self tallySendsTo: receiver inBlock: aBlock showTree: treeOption closeAfter: closeAfter openResultWindow: true! !

!MessageTally class methodsFor: 'spying' stamp: 'MarcusDenker 1/28/2011 15:50'!
tallySendsTo: receiver inBlock: aBlock showTree: treeOption closeAfter: closeAfter openResultWindow: openResultWindow
	"
	MessageTally tallySends: [3.14159 printString]
	"
	"This method uses the simulator to count the number of calls on each method
	invoked in evaluating aBlock. If receiver is not nil, then only sends
	to that receiver are tallied.
	Results are presented as leaves, sorted by frequency,
	preceded, optionally, by the whole tree."
	| prev tallies startTime totalTime |
	startTime := Time millisecondClockValue.
	tallies := self new class: aBlock receiver class method: aBlock method.
	tallies reportOtherProcesses: true.	"Do NOT filter nodes with nil process"
	prev := aBlock.
	thisContext sender
		runSimulated: aBlock
		contextAtEachStep:
			[:current |
			current == prev ifFalse: 
				["call or return"
				prev sender isNil ifFalse: 
					["call only"
					(receiver isNil or: [current receiver == receiver])
						ifTrue: [tallies tally: current by: 1]].
				prev := current]].

	totalTime := Time millisecondClockValue - startTime // 1000.0 roundTo: 0.01.
	openResultWindow ifTrue: [
		(CodeHolder new contents:
			(String streamContents:
				[:s |
				s nextPutAll: 'This simulation took ' , totalTime printString
								, ' seconds.'; cr.
				treeOption
					ifTrue: [ tallies fullPrintExactOn: s ]
					ifFalse: [ tallies leavesPrintExactOn: s ].
				]))
			openLabel: 'Spy Results' wrap: false ].
	closeAfter ifTrue: [ tallies close ].
	^ tallies! !

!MessageTally class methodsFor: 'private' stamp: 'jmv 2/19/2010 14:43'!
terminateTimerProcess

	Timer ifNotNil: [ Timer terminate ].
	Timer := nil! !

!MessageTally class methodsFor: 'spying' stamp: ''!
time: aBlock

	^ Time millisecondsToRun: aBlock! !

!MessageTally methodsFor: 'comparing' stamp: ''!
< aMessageTally 
	"Refer to the comment in Magnitude|<."

	^tally > aMessageTally tally! !

!MessageTally methodsFor: 'comparing' stamp: 'Alexandre Bergel 3/5/2010 17:17'!
= aMessageTally

	self species == aMessageTally species ifFalse: [^ false].
	^ aMessageTally method == method 
		and: [ aMessageTally process == process ]! !

!MessageTally methodsFor: 'comparing' stamp: ''!
> aMessageTally 
	"Refer to the comment in Magnitude|>."

	^tally < aMessageTally tally! !

!MessageTally methodsFor: 'collecting leaves' stamp: ''!
bump: hitCount
	tally := tally + hitCount! !

!MessageTally methodsFor: 'collecting leaves' stamp: 'MarcusDenker 1/23/2011 09:13'!
bump: hitCount fromSender: senderTally
	"Add this hitCount to the total, and include a reference to the
	sender responsible for the increment"
	self bump: hitCount.
	senders ifNil: [senders := OrderedCollection new].
	senderTally ifNotNil: [senders add: (senderTally copyWithTally: hitCount)]! !

!MessageTally methodsFor: 'tallying' stamp: ''!
bumpBy: count

	tally := tally + count! !

!MessageTally methodsFor: 'private' stamp: ''!
class: aClass method: aMethod

	class := aClass.
	method := aMethod.
	tally := 0.
	receivers := Array new: 0! !

!MessageTally methodsFor: 'private' stamp: 'jmv 9/25/2009 08:48'!
close

	Timer ifNotNil: [ Timer terminate ].
	Timer := nil.
	class := method := tally := receivers := nil! !

!MessageTally methodsFor: 'private' stamp: 'MarcusDenker 1/23/2011 09:15'!
copyWithTally: hitCount
	^ (self class new class: class method: method)
		reportOtherProcesses: reportOtherProcesses;
		process: process;
		bump: hitCount! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 9/24/2009 15:49'!
fullPrintExactOn: aStream
	aStream nextPutAll: '**Tree**'; cr.
	self
		treePrintOn: aStream
		tabs: OrderedCollection new
		thisTab: ''
		total: tally
		totalTime: time
		tallyExact: true
		orThreshold: nil.
	aStream nextPut: Character newPage; cr.
	aStream nextPutAll: '**Leaves**'; cr.
	self leavesPrintExactOn: aStream! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 9/24/2009 15:49'!
fullPrintOn: aStream threshold: perCent
	| threshold |  
	threshold := (perCent asFloat / 100 * tally) rounded.
	aStream nextPutAll: '**Tree**'; cr.
	self
		rootPrintOn: aStream
		total: tally
		totalTime: time
		threshold: threshold.
	aStream nextPut: Character newPage; cr.
	aStream nextPutAll: '**Leaves**'; cr.
	self
		leavesPrintOn: aStream
		threshold: threshold! !

!MessageTally methodsFor: 'comparing' stamp: 'jmv 8/20/2009 08:20'!
hash
	"Hash is reimplemented because = is implemented."

	^method hash! !

!MessageTally methodsFor: 'initialization' stamp: 'Alexandre Bergel 3/15/2010 21:08'!
initialize
	"We do not do a super initialize since it is not strickly necessary and more importantly MessageTally must be instantiated quickly"
	"super initialize."
	
	maxClassNameSize := self class defaultMaxClassNameSize.
	maxClassPlusSelectorSize := self class defaultMaxClassPlusSelectorSize.
	maxTabs := self class defaultMaxTabs.
	reportOtherProcesses := true.
	time := 0.
	tally := 0.! !

!MessageTally methodsFor: 'collecting leaves' stamp: 'MarcusDenker 1/23/2011 09:15'!
into: leafDict fromSender: senderTally 
	| leafNode |
	leafNode := leafDict at: method
				ifAbsentPut: 
					[ (self class new class: class method: method)
						process: process;
						reportOtherProcesses: reportOtherProcesses ].
					
	leafNode bump: tally fromSender: senderTally! !

!MessageTally methodsFor: 'comparing' stamp: 'MarcusDenker 1/23/2011 09:14'!
isPrimitives
	"Detect pseudo node used to carry tally of local hits"
	^ receivers isNil! !

!MessageTally methodsFor: 'collecting leaves' stamp: 'jmv 9/24/2009 16:07'!
leavesInto: leafDict fromSender: senderTally

	| rcvrs |
	rcvrs := self sonsOver: 0.
	rcvrs size = 0
		ifTrue: [ self into: leafDict fromSender: senderTally ]
		ifFalse: [
			
			(reportOtherProcesses not and: [ rcvrs anyOne process isNil ]) ifTrue: [
				^self].
			
			rcvrs do: [ :node |
				node isPrimitives
					ifTrue: [ node leavesInto: leafDict fromSender: senderTally ]
					ifFalse: [ node leavesInto: leafDict fromSender: self ]]]! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 9/24/2009 15:47'!
leavesPrintExactOn: aStream
	| dict |
	dict := IdentityDictionary new: 100.
	self leavesInto: dict fromSender: nil.
	dict asSortedCollection
		do: [ :node |
			node printOn: aStream total: tally totalTime: nil tallyExact: true.
			node printSenderCountsOn: aStream ]! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 9/24/2009 15:48'!
leavesPrintOn: aStream threshold: threshold
	| dict |
	dict := IdentityDictionary new: 100.
	self leavesInto: dict fromSender: nil.
	(dict asOrderedCollection
			select: [:node | node tally > threshold])
		asSortedCollection do: [:node |
			node printOn: aStream total: tally totalTime: time tallyExact: false ]! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:29'!
maxClassNameSize
	^maxClassNameSize! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:30'!
maxClassNameSize: aNumber
	maxClassNameSize := aNumber! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:29'!
maxClassPlusSelectorSize
	^maxClassPlusSelectorSize! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:30'!
maxClassPlusSelectorSize: aNumber
	maxClassPlusSelectorSize := aNumber! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:29'!
maxTabs
	^maxTabs! !

!MessageTally methodsFor: 'printing format' stamp: 'nk 3/8/2004 12:30'!
maxTabs: aNumber
	maxTabs := aNumber! !

!MessageTally methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/5/2010 17:18'!
method
	"Return the compiled method associated to this tally"
	
	^method! !

!MessageTally methodsFor: 'private' stamp: ''!
primitives: anInteger

	tally := anInteger.
	receivers := nil! !

!MessageTally methodsFor: 'printing' stamp: 'LucFabresse 10/1/2010 15:40'!
printOn: aStream 
	| aSelector className aClass |
	(class isNil or: [method isNil]) ifTrue: [^super printOn: aStream].
	aSelector := method selector.
	className := method methodClass name.
	aStream
		nextPutAll: (aClass name contractTo: self maxClassNameSize);
		nextPutAll: '(';
		nextPutAll: (className  contractTo: self maxClassNameSize);
		nextPutAll: ')';
		nextPutAll: ' >> ';
		nextPutAll: (aSelector 
					contractTo: self maxClassPlusSelectorSize - className size)! !

!MessageTally methodsFor: 'printing' stamp: 'MarcusDenker 1/23/2011 09:13'!
printOn: aStream total: total totalTime: totalTime tallyExact: isExact 

	isExact 
		ifTrue: [
			| myTally |
			myTally := tally.
			receivers == nil 
				ifFalse: [receivers do: [:r | myTally := myTally - r tally]].
			aStream
				print: myTally;
				space]
		ifFalse: [
			| percentage |
			percentage := tally asFloat / total * 100.0.
			aStream
				nextPutAll: (percentage printShowingDecimalPlaces: 1);
				nextPutAll: '% {';
				print: (percentage * totalTime / 100) rounded;
				nextPutAll: 'ms} '].
	receivers isNil
		ifTrue: [
			aStream
				nextPutAll: 'primitives';
				cr]
		ifFalse: [
			| className aSelector aClass |
			aSelector := method selector.
			aClass := method methodClass.
			className := aClass name contractTo: self maxClassNameSize.
			aStream
				nextPutAll: class name;
				nextPutAll: (aClass = class 
							ifTrue: ['>>']
							ifFalse: ['(' , aClass name , ')>>']);
				nextPutAll: (aSelector 
							contractTo: self maxClassPlusSelectorSize - className size);
				cr]! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 2/6/2010 10:09'!
printSenderCountsOn: aStream
	| mergedSenders |
	mergedSenders := IdentityDictionary new.
	senders do: [ :node | | mergedNode |
		mergedNode := mergedSenders at: node method ifAbsent: [ nil ].
		mergedNode
			ifNil: [ mergedSenders at: node method put: node ]
			ifNotNil: [ mergedNode bump: node tally ]].
	mergedSenders asSortedCollection do: [ :node | 
		10 to: node tally printString size by: -1 do: [ :i | aStream space].
		node printOn: aStream total: tally totalTime: nil tallyExact: true]! !

!MessageTally methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/5/2010 17:19'!
process
	"Return the profiled process"
	
	^process! !

!MessageTally methodsFor: 'accessing' stamp: 'ar 3/3/2009 19:29'!
process: aProcess
	process := aProcess! !

!MessageTally methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/15/2010 21:02'!
receivers 
	^ receivers ! !

!MessageTally methodsFor: 'reporting' stamp: 'jmv 3/4/2009 09:27'!
report: strm 
	"Print a report, with cutoff percentage of each element of the tree 
	(leaves, roots, tree), on the stream, strm."

	self report: strm cutoff: 1! !

!MessageTally methodsFor: 'reporting' stamp: 'jmv 9/24/2009 15:35'!
report: strm cutoff: threshold 
	tally = 0
		ifTrue: [strm nextPutAll: ' - no tallies obtained']
		ifFalse: 
			[strm nextPutAll: ' - '; print: tally; nextPutAll: ' tallies, ', time printString, ' msec.'; cr; cr.
			self fullPrintOn: strm threshold: threshold].
		
	time isZero ifFalse:	
		[self reportGCStatsOn: strm].! !

!MessageTally methodsFor: 'reporting' stamp: 'ar 7/18/2001 22:12'!
reportGCStatsOn: str
	| oldSpaceEnd youngSpaceEnd memoryEnd fullGCs fullGCTime incrGCs incrGCTime tenureCount upTime rootOverflows |
	upTime := time.
	oldSpaceEnd			:= gcStats at: 1.
	youngSpaceEnd		:= gcStats at: 2.
	memoryEnd			:= gcStats at: 3.
	fullGCs				:= gcStats at: 7.
	fullGCTime			:= gcStats at: 8.
	incrGCs				:= gcStats at: 9.
	incrGCTime			:= gcStats at: 10.
	tenureCount			:= gcStats at: 11.
	rootOverflows		:= gcStats at: 22.

	str cr.
	str	nextPutAll: '**Memory**'; cr.
	str	nextPutAll:	'	old			';
		nextPutAll: oldSpaceEnd asStringWithCommasSigned; nextPutAll: ' bytes'; cr.
	str	nextPutAll: '	young		';
		nextPutAll: (youngSpaceEnd - oldSpaceEnd) asStringWithCommasSigned; nextPutAll: ' bytes'; cr.
	str	nextPutAll: '	used		';
		nextPutAll: youngSpaceEnd asStringWithCommasSigned; nextPutAll: ' bytes'; cr.
	str	nextPutAll: '	free		';
		nextPutAll: (memoryEnd - youngSpaceEnd) asStringWithCommasSigned; nextPutAll: ' bytes'; cr.

	str cr.
	str	nextPutAll: '**GCs**'; cr.
	str	nextPutAll: '	full			';
		print: fullGCs; nextPutAll: ' totalling '; nextPutAll: fullGCTime asStringWithCommas; nextPutAll: 'ms (';
		print: ((fullGCTime / upTime * 100) roundTo: 1.0);
		nextPutAll: '% uptime)'.
	fullGCs = 0 ifFalse:
		[str	nextPutAll: ', avg '; print: ((fullGCTime / fullGCs) roundTo: 1.0); nextPutAll: 'ms'].
	str	cr.
	str	nextPutAll: '	incr		';
		print: incrGCs; nextPutAll: ' totalling '; nextPutAll: incrGCTime asStringWithCommas; nextPutAll: 'ms (';
		print: ((incrGCTime / upTime * 100) roundTo: 1.0);
		nextPutAll: '% uptime)'.
	incrGCs = 0 ifFalse:
		[str nextPutAll:', avg '; print: ((incrGCTime / incrGCs) roundTo: 1.0); nextPutAll: 'ms'].
	str cr.
	str	nextPutAll: '	tenures		';
		nextPutAll: tenureCount asStringWithCommas.
	tenureCount = 0 ifFalse:
		[str nextPutAll: ' (avg '; print: (incrGCs / tenureCount) asInteger; nextPutAll: ' GCs/tenure)'].
	str	cr.
	str	nextPutAll: '	root table	';
		nextPutAll: rootOverflows asStringWithCommas; nextPutAll:' overflows'.
	str cr.
! !

!MessageTally methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/4/2010 19:19'!
reportOtherProcesses
	^ reportOtherProcesses! !

!MessageTally methodsFor: 'accessing' stamp: 'jmv 9/24/2009 16:02'!
reportOtherProcesses: aBoolean
	reportOtherProcesses := aBoolean! !

!MessageTally methodsFor: 'printing' stamp: 'jmv 2/6/2010 10:19'!
rootPrintOn: aStream total: total totalTime: totalTime threshold: threshold

	| groups |
	groups := (self sonsOver: threshold) groupBy: [ :aTally | aTally process] having: [ :g | true ].
	groups do: [ :g |
		| sons p |
		sons := g asArray sort.
		p := g anyOne process.
		(reportOtherProcesses or: [ p notNil ]) ifTrue: [
			aStream nextPutAll: '--------------------------------'; cr.
			aStream nextPutAll: 'Process: ',  (p ifNil: [ 'other processes'] ifNotNil: [ p browserPrintString]); cr.
			aStream nextPutAll: '--------------------------------'; cr.
			sons do: [ :son |
				son
					treePrintOn: aStream
					tabs: OrderedCollection new
					thisTab: ''
					total: total
					totalTime: totalTime
					tallyExact: false
					orThreshold: threshold]].
	]! !

!MessageTally methodsFor: 'comparing' stamp: 'MarcusDenker 1/23/2011 09:15'!
sonsOver: threshold

	| hereTally sons |
	(receivers isNil or: [ receivers size = 0 ]) ifTrue: [ ^#() ].
	hereTally := tally.
	sons := receivers select: [ :son | "subtract subNode tallies for primitive hits here"
		hereTally := hereTally - son tally.
		son tally > threshold ].
	hereTally > threshold
		ifTrue: [
			| last |
			last := self class new class: class method: method.
			last process: process.
			last reportOtherProcesses: reportOtherProcesses.
			^sons copyWith: (last primitives: hereTally)].
	^sons! !

!MessageTally methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:04'!
species
	^MessageTally! !

!MessageTally methodsFor: 'initialize-release' stamp: 'Janniklaval 10/23/2010 12:07'!
spyAllEvery: millisecs on: aBlock
	"Create a spy and spy on the given block at the specified rate."
	"Spy all the system processes"

	| myDelay time0 |
	aBlock isBlock
		ifFalse: [ self error: 'spy needs a block here' ].
	self class: aBlock receiver class method: aBlock method.
		"set up the probe"
	myDelay := Delay forMilliseconds: millisecs.
	time0 := Time millisecondClockValue.
	gcStats := Smalltalk getVMParameters.
	Timer ifNotNil: [ self error: 'it seems a tally is already running' ].
	Timer := [
		[true] whileTrue: [
			| startTime observedProcess |
			startTime := Time millisecondClockValue.
			myDelay wait.
			observedProcess := Processor preemptedProcess.
			self
				tally: observedProcess suspendedContext
				in: observedProcess
				"tally can be > 1 if ran a long primitive"
				by: (Time millisecondClockValue - startTime) // millisecs].
		nil] newProcess.
	Timer priority: Processor timingPriority-1.
		"activate the probe and evaluate the block"
	Timer resume.
	^ aBlock ensure: [
		"cancel the probe and return the value"
		"Could have already been terminated. See #terminateTimerProcess"
		Timer ifNotNil: [
			Timer terminate.
			Timer := nil ].
		"Collect gc statistics"
		Smalltalk getVMParameters keysAndValuesDo: [ :idx :gcVal | 
			gcVal ifNotNil: [ gcStats at: idx put: (gcVal - (gcStats at: idx))]].
		time := Time millisecondClockValue - time0]! !

!MessageTally methodsFor: 'initialize-release' stamp: 'Janniklaval 10/23/2010 12:08'!
spyEvery: millisecs on: aBlock
	"Create a spy and spy on the given block at the specified rate."
	"Spy only on the active process (in which aBlock is run)"

	| myDelay time0 observedProcess |
	aBlock isBlock
		ifFalse: [ self error: 'spy needs a block here' ].
	self class: aBlock receiver class method: aBlock method.
		"set up the probe"
	observedProcess := Processor activeProcess.
	myDelay := Delay forMilliseconds: millisecs.
	time0 := Time millisecondClockValue.
	gcStats := Smalltalk getVMParameters.
	Timer ifNotNil: [ self error: 'it seems a tally is already running' ].
	Timer := [
		[ true ] whileTrue: [
			| startTime |
			startTime := Time millisecondClockValue.
			myDelay wait.
			self
				tally: Processor preemptedProcess suspendedContext
				in: (observedProcess == Processor preemptedProcess ifTrue: [observedProcess] ifFalse: [nil])
				"tally can be > 1 if ran a long primitive"
				by: (Time millisecondClockValue - startTime) // millisecs].
		nil] newProcess.
	Timer priority: Processor timingPriority-1.
		"activate the probe and evaluate the block"
	Timer resume.
	^ aBlock ensure: [
		"cancel the probe and return the value"
		"Could have already been terminated. See #terminateTimerProcess"
		Timer ifNotNil: [
			Timer terminate.
			Timer := nil ].
		"Collect gc statistics"
		Smalltalk getVMParameters keysAndValuesDo: [ :idx :gcVal | 
			gcVal ifNotNil: [ gcStats at: idx put: (gcVal - (gcStats at: idx))]].
		time := Time millisecondClockValue - time0]! !

!MessageTally methodsFor: 'initialize-release' stamp: 'Janniklaval 10/23/2010 12:06'!
spyEvery: millisecs onProcess: aProcess forMilliseconds: msecDuration
	"Create a spy and spy on the given process at the specified rate."
	| myDelay time0 endTime observedProcess sem |
	(aProcess isKindOf: Process)
		ifFalse: [self error: 'spy needs a Process here'].
	self class: aProcess suspendedContext receiver class method: aProcess suspendedContext method.
	"set up the probe"
	observedProcess := aProcess.
	myDelay := Delay forMilliseconds: millisecs.
	time0 := Time millisecondClockValue.
	endTime := time0 + msecDuration.
	sem := Semaphore new.
	gcStats := Smalltalk getVMParameters.
	Timer ifNotNil: [ self error: 'it seems a tally is already running' ].
	Timer := [
			[
				| startTime |
				startTime := Time millisecondClockValue.
				myDelay wait.
				self
					tally: Processor preemptedProcess suspendedContext
					in: (observedProcess == Processor preemptedProcess ifTrue: [ observedProcess ] ifFalse: [nil])
					"tally can be > 1 if ran a long primitive"
					by: (Time millisecondClockValue - startTime) // millisecs.
				startTime < endTime
			] whileTrue.
			sem signal.
		] newProcess.
	Timer priority: Processor timingPriority-1.
		"activate the probe and evaluate the block"
	Timer resume.
	"activate the probe and wait for it to finish"
	sem wait.
	"Collect gc statistics"
	Smalltalk getVMParameters keysAndValuesDo: [ :idx :gcVal | 
		gcVal ifNotNil: [ gcStats at: idx put: (gcVal - (gcStats at: idx)) ] ].
	time := Time millisecondClockValue - time0! !

!MessageTally methodsFor: 'accessing' stamp: 'stp 05/08/1999 12:06'!
tally
	"Answer the receiver's number of tally."

	^tally! !

!MessageTally methodsFor: 'tallying' stamp: 'Alexandre Bergel 3/5/2010 17:37'!
tally: context by: count
	"Explicitly tally the specified context and its stack."
	| sender |
	
	"Add to this node if appropriate"
	context method == method ifTrue: [^self bumpBy: count].
	
	"No sender? Add new branch to the tree."
	(sender :=  context home sender) ifNil: [
		^ (self bumpBy: count) tallyPath: context by: count].
	
	"Find the node for the sending context (or add it if necessary)"
	^ (self tally: sender by: count) tallyPath: context by: count! !

!MessageTally methodsFor: 'tallying' stamp: 'jmv 3/4/2009 10:37'!
tally: context in: aProcess by: count
	"Explicitly tally the specified context and its stack."
	| sender |

	"Add to this node if appropriate"
	context method == method ifTrue: [^self bumpBy: count].
	
	"No sender? Add new branch to the tree."
	(sender := context home sender) ifNil: [
		^ (self bumpBy: count) tallyPath: context in: aProcess by: count].
	
	"Find the node for the sending context (or add it if necessary)"
	^ (self tally: sender in: aProcess by: count) tallyPath: context in: aProcess by: count! !

!MessageTally methodsFor: 'tallying' stamp: 'MarcusDenker 1/23/2011 09:15'!
tallyPath: context by: count
	| aMethod path |
	aMethod := context method.
	
	"Find the correct child (if there)"
	receivers do: [ :oldTally | 
		oldTally method == aMethod ifTrue: [path := oldTally]].
	
	"Add new child if needed"
	path ifNil: [
		path := self class new class: context receiver class method: aMethod.
		path reportOtherProcesses: reportOtherProcesses.
		receivers := receivers copyWith: path].
	
	^ path bumpBy: count! !

!MessageTally methodsFor: 'tallying' stamp: 'MarcusDenker 1/23/2011 09:15'!
tallyPath: context in: aProcess by: count
	| aMethod path |
	aMethod := context method.
	
	"Find the correct child (if there)"
	receivers do: [ :oldTally | 
		(oldTally method == aMethod and: [oldTally process == aProcess])
			ifTrue: [path := oldTally]].
		
	"Add new child if needed"
	path ifNil:[
		path := self class new class: context receiver class method: aMethod;
			process: aProcess;
			reportOtherProcesses: reportOtherProcesses;
			maxClassNameSize: maxClassNameSize;
			maxClassPlusSelectorSize: maxClassPlusSelectorSize;
			maxTabs: maxTabs.
		receivers := receivers copyWith: path].

	^ path bumpBy: count! !

!MessageTally methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/5/2010 17:19'!
theClass
	"Return the class of the object receiver related to this tally"
	
	 ^ class! !

!MessageTally methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:47'!
time
	"Answer the receiver's run time."

	^time! !

!MessageTally methodsFor: 'printing' stamp: 'nk 3/8/2004 12:23'!
treePrintOn: aStream tabs: tabs thisTab: myTab total: total totalTime: totalTime tallyExact: isExact orThreshold: threshold 
	| sons sonTab |
	tabs do: [:tab | aStream nextPutAll: tab].
	tabs size > 0 
		ifTrue: 
			[self 
				printOn: aStream
				total: total
				totalTime: totalTime
				tallyExact: isExact].
	sons := isExact ifTrue: [receivers] ifFalse: [self sonsOver: threshold].
	sons isEmpty 
		ifFalse: 
			[tabs addLast: myTab.
			sons := sons asSortedCollection.
			(1 to: sons size) do: 
					[:i | 
					sonTab := i < sons size ifTrue: ['  |'] ifFalse: ['  '].
					(sons at: i) 
						treePrintOn: aStream
						tabs: (tabs size < self maxTabs 
								ifTrue: [tabs]
								ifFalse: [(tabs select: [:x | x = '[']) copyWith: '['])
						thisTab: sonTab
						total: total
						totalTime: totalTime
						tallyExact: isExact
						orThreshold: threshold].
			tabs removeLast]! !

!SequenceableCollection methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:17'!
customizeExplorerContents

	^ true.
! !

!PasteUpMorph methodsFor: '*Tools' stamp: 'DamienCassou 5/14/2011 17:00'!
defaultDesktopCommandKeyTriplets
	"Answer a list of triplets of the form
		<key> <receiver> <selector>   [+ optional fourth element, a <description> for use in desktop-command-key-help]
that will provide the default desktop command key handlers.  If the selector takes an argument, that argument will be the command-key event"
 
	^ {
		{ $r.	ActiveWorld.						#restoreMorphicDisplay.					'Redraw the screen'}.		
		{ $b.	SystemBrowser.					#defaultOpenBrowser.						'Open a new System Browser'}.
		{ $k.	ToolSet.							#openWorkspace.							'Open a new, blank Workspace'}.
		{ $t.	ToolSet .	 						#openTranscript.							'Make a System Transcript visible'}.
		{ $C.	self.								#findAChangeSorter:.						'Make a Change Sorter visible'}.
		{ $R.	self. 								#openRecentSubmissionsBrowser:.			'Make a Recent Submissions browser visible'}.
		{ $W.	self. 								#findAMessageNamesWindow:.				'Make a MessageNames tool visible'}.
		{ $Z.	ChangeList. 						#browseRecentLog.						'Browse recently-logged changes'}.
		{ $\.	SystemWindow. 					#sendTopWindowToBack.					'Send the top window to the back'}.
	}.
! !

!TranscriptStream methodsFor: '*Tools-Base' stamp: 'di 5/27/1998 16:44'!
codePaneMenu: aMenu shifted: shifted
	"Note that unless we override perform:orSendTo:, PluggableTextController will respond to all menu items"
	^ StringHolder basicNew codePaneMenu: aMenu shifted: shifted
! !

MorphTreeModel subclass: #TimeProfiler
	instanceVariableNames: 'rootMessageTally threshold minPercentageToExpand treeMorph block blockCodePane withBlockCodePane blockSource reportOtherProcesses showLeavesOnly'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!TimeProfiler commentStamp: 'AlainPlantec 2/19/2010 14:44' prior: 0!
self new openOnBlock: [SimpleGridExample new open]
self new openOnBlock: [ClassTreeExample new openOn: Object]
 MessageTally spyAllOn: [ClassTreeExample new openOn: Object]!

!TimeProfiler class methodsFor: 'world menu' stamp: 'FernandoOlivero 4/12/2011 10:49'!
menuCommandOn: aBuilder 
	<worldMenu>
	(aBuilder item: #'Time profiler')
		parent: #Tools;  
		icon: self theme smallDebugIcon;
		action: [TimeProfiler new open]! !

!TimeProfiler class methodsFor: 'opening' stamp: 'AlainPlantec 2/15/2011 09:47'!
spyOn: aBlock 
	^ (self new withBlockCodePane: false)
		openOnBlock: aBlock ! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 02:54'!
blockCode
	^ block
		ifNil: ['']
		ifNotNil: [ blockSource ifNil: [blockSource := String 
			streamContents: [:s |  block decompile 
				statements 
					do: [:stmt | stmt printOn: s indent: 0]
					separatedBy: [s nextPutAll: '.', String cr]]]]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'nice 3/31/2011 22:46'!
blockCode: aString notifying: aRequestor
	"Treat the current selection as an expression; evaluate and tally it."
	| receiver context  |
	aString ifNil: [^ self].

	receiver := nil.
	context := self doItContext.
	blockSource := aString.

	[ | compiledMethod |
		compiledMethod := self compile: aString for: receiver in: context.
		compiledMethod ifNil: [^ self].
		compiledMethod valueWithReceiver: self arguments: #().
	] 
		on: OutOfScopeNotification 
		do: [ :ex | ex resume: true].
	self changed: #blockCode.
	self changed: #rootNodes.
	self changed: #summaryText.
	self selection: nil.
	self startState.

	! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/3/2010 06:49'!
browseItem
	self selectedNode ifNotNil: [:current | current browseItem].
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 9/4/2010 17:36'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	| donorMenu |
	donorMenu := shifted
				ifTrue: [SmalltalkEditor shiftedYellowButtonMenu]
				ifFalse: [SmalltalkEditor yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 00:43'!
compile: codeString for: anObject in: evalContext

	| methodNode |
	methodNode := [Compiler new
		compileNoPattern: ('self runBlock: [', codeString, ']')
		in: anObject class
		context: evalContext
		notifying: self
		ifFail: [^nil]]
			on: OutOfScopeNotification
			do: [:ex | ex resume: true].

	^ methodNode generateWithTempNames.! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:21'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0		"null selection"! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:47'!
correctFrom: start to: stop with: aString
	^ blockCodePane correctFrom: start to: stop with: aString! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:42'!
deselect
	blockCodePane  deselect! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:42'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'nice 3/31/2011 22:45'!
doItReceiver
	"This class's classPool has been jimmied to be the classPool of the class 
	being browsed. A doIt in the code pane will let the user see the value of 
	the class variables."

	^ self selectedClass! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 20:29'!
expandAll
	treeMorph expandAllSuchThat: [:node | node percentage > self minPercentageToExpand]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 20:17'!
expandAllFromNode: aNode
	treeMorph expandAllFromNode: aNode suchThat: [:node | node percentage > self minPercentageToExpand]
	! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:36'!
helpMessage

^'You can edit some code in the top editor and accept to profile it.
The result consists in a tree of method calls shown in the middle panel.
Select a method call to see or change its implementation in the bottom editor.

Tallies building:
The tallies are built according to the time threshold value that you can change at the top right (Treshold input field).

Tree expanding:
A tree node is expanded only if its duration percentage is greater that the minimum value that you can change at the top left (Min duration percentage input field). 
'! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/15/2011 10:09'!
initialExtent
	^ 750 @ 700! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/25/2011 17:10'!
keyStroke: anEvent from: aTreeMorph 
	self selectedNode ifNotNil: [:current | current keyStroke: anEvent from: aTreeMorph ].
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/6/2010 09:25'!
menu: menu shifted: b 
	"Set up the menu to apply to the receiver's, 
	honoring the #shifted boolean"
	self selectedNode ifNotNil: [:current | 
		current menu: menu shifted: b.
		menu addLine.
		menu
			add: 'Expand all from here' translated
			target: self
			selector: #expandAllFromNode:
			argument: current].
	^ menu! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:04'!
minPercentageToExpand
	^ minPercentageToExpand ifNil: [minPercentageToExpand := 3]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 20:14'!
minPercentageToExpand: aPercentage
	minPercentageToExpand := aPercentage.
	self changed: #minPercentageToExpand.
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 02:04'!
notify: aString at: anInteger in: aStream
	blockCodePane notify: aString at: anInteger - 'self runBlock: [' size in: aStream ! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 4/12/2011 10:48'!
open
	| win toolBar codePane vgap summaryPane blkPaneGap treeTopFraction |
	win := StandardWindow new model: self.
	win setLabel: 'Time profiler'.
	toolBar := self toolBarOn: win.
	blockCodePane := self theme
				newTextEditorIn: win
				for: self
				getText: #blockCode
				setText: #blockCode:notifying:
				getEnabled: nil.
	blockCodePane getMenuSelector: #codePaneMenu:shifted:.
	blockCodePane font: StandardFonts codeFont.
	blockCodePane alwaysAccept: true.
	treeMorph := self treeMorph buildContents; yourself.
	summaryPane := win
				newTextEditorFor: self
				getText: #summaryText
				setText: nil
				getEnabled: nil.
	codePane :=  self theme
				newTextEditorIn: win
				for: self
				getText: #selectedMethodCode
				setText: #selectedMethodCode:notifying:
				getEnabled: nil.
	codePane getMenuSelector: #codePaneMenu:shifted:.
	codePane font: StandardFonts codeFont.
	vgap := toolBar minExtent y.
	win
		addMorph: toolBar
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ 0 corner: 0 @ vgap)).
	blkPaneGap := 0.
	treeTopFraction := 0.0.
	self withBlockCodePane
		ifTrue: [blkPaneGap := 10.
			treeTopFraction := 0.15.
			win
				addMorph: blockCodePane
				fullFrame: (LayoutFrame
						fractions: (0 @ 0 corner: 1 @ treeTopFraction)
						offsets: (0 @ vgap corner: 0 @ (vgap + blkPaneGap)))].
	win
		addMorph: treeMorph
		fullFrame: (LayoutFrame
				fractions: (0 @ treeTopFraction corner: 1 @ 0.6)
				offsets: (0 @ (vgap + blkPaneGap) corner: 0 @ 0)).
	win
		addMorph: summaryPane
		fullFrame: (LayoutFrame
				fractions: (0 @ 0.6 corner: 1 @ 0.7)
				offsets: (0 @ 0 corner: 0 @ (blkPaneGap + 10))).
	win
		addMorph: codePane
		fullFrame: (LayoutFrame
				fractions: (0 @ 0.7 corner: 1 @ 1)
				offsets: (0 @ (blkPaneGap + 10) corner: 0 @ 0)).
	win openInWorld.
	self startState! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 00:26'!
openOnBlock: aBlock 
	self runBlock: aBlock pollingEvery: MessageTally defaultPollPeriod.
	self open! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/5/2010 23:11'!
openPathFrom: aNode
	self changed: {#rootNodes. #openNodeRecursively. aNode}! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 22:36'!
profileIt
	blockCodePane accept.
	! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:05'!
reportOtherProcesses
	^ reportOtherProcesses ifNil: [reportOtherProcesses := true]
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:26'!
reportOtherProcesses: aBoolean
	self reportOtherProcesses ~= aBoolean
		ifTrue: [
			reportOtherProcesses := aBoolean.
			self changed: #reportOtherProcesses.
			self changed: #rootNodes]
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 15:59'!
rootItems
	| sons result groups |
	result := OrderedCollection new.
	rootMessageTally 
		ifNotNil: [
			self showLeavesOnly 
				ifTrue: [ | dict |
					dict := IdentityDictionary new: 100.
					rootMessageTally leavesInto: dict fromSender: nil.
					result addAll: ((dict asOrderedCollection select: [:node | node tally > self threshold]) asSortedCollection)]
				ifFalse: [sons := rootMessageTally sonsOver: self threshold.
					groups := sons groupBy: [ :aTally | aTally process] having: [ :g | true].
					groups do:[:g | self reportOtherProcesses 
						ifTrue: [result addAll: g asSortedCollection]
						ifFalse: [result addAll: (g asSortedCollection reject: [:gg | gg process isNil])]]]].
	^ result
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/20/2010 08:44'!
rootNodeClassFromItem: anItem
	^ TimeProfilerNode ! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 14:20'!
runBlock: aBlock 
	self runBlock: aBlock pollingEvery: MessageTally defaultPollPeriod! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 14:51'!
runBlock: aBlock pollingEvery: pollPeriod 
	block := aBlock.
	rootMessageTally := MessageTally new.
	rootMessageTally
		reportOtherProcesses: true;
		maxClassNameSize: 1000;
		maxClassPlusSelectorSize: 1000;
		maxTabs: 100.
	rootMessageTally spyEvery: pollPeriod on: aBlock.
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:41'!
select
	blockCodePane  select! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:39'!
selectFrom: start to: stop
	blockCodePane  selectFrom: start to: stop! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:43'!
selectInvisiblyFrom:  first to:  last
	blockCodePane selectInvisiblyFrom: first to: last.! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:41'!
selectedClass
	^ self selectedNode 
		ifNotNil: [ :currNode | currNode methodClass]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:22'!
selectedMethodCode
	^ self selectedNode 
		ifNil: [self helpMessage]
		ifNotNil: [ :currNode | | class selector |
			class := currNode methodClass.
			selector := currNode selector.
			(class ultimateSourceCodeAt: selector ifAbsent: ['error'])
				copy asText makeSelectorBoldIn: class]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:32'!
selectedMethodCode: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Answer false if the compilation fails. Otherwise answer true."

	self selectedNode 
		ifNil: [^ false]
		ifNotNil: [ :currNode | | class oldSelector category newSelector |
			class := currNode methodClass.
			oldSelector := currNode selector.
			class ifNil: [^ false].
			"Normal method accept"
			category := class organization categoryOfElement: oldSelector.
			newSelector := class compile: aString
				classified: category
				notifying: aController.
			newSelector == nil ifTrue: [^ false].
			self changed: #selectedMethodCode.
			^ true]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 22:02'!
selection: aSelection
	super selection: aSelection.
	self changed: #selectedMethodCode
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 00:00'!
selectionInterval
	^ 1 to: 0! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:11'!
showLeavesOnly 
	^ showLeavesOnly ifNil: [showLeavesOnly := false]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:31'!
showLeavesOnly: aBoolean
	self showLeavesOnly ~= aBoolean
		ifTrue: [
			treeMorph treeLineWidth: (aBoolean ifTrue: [0] ifFalse: [1]).
			showLeavesOnly := aBoolean.
			self changed: #showLeavesOnly.
			self changed: #rootNodes]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 02:19'!
startMinPercentageToExpand
	^ 20! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 02:20'!
startState
	treeMorph expandAllSuchThat: [:node | node percentage > self startMinPercentageToExpand]
	! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 22:52'!
summaryText
	^ String streamContents: [:strm | rootMessageTally 
		ifNotNil: [rootMessageTally time > 0 
			ifTrue: [strm nextPutAll: ' - '; print: rootMessageTally tally; nextPutAll: ' tallies, ', rootMessageTally time printString, ' msec.'; cr.
				rootMessageTally reportGCStatsOn: strm]]]
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 17:12'!
tally
	^ rootMessageTally tally! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/18/2010 01:46'!
text
	^ blockSource! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 20:12'!
threshold
	^ threshold ifNil: [threshold := 0]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/17/2010 23:02'!
threshold: aTreshold
	(aTreshold ~= threshold and: [aTreshold between: 0 and: 1000])
		ifTrue: [
			threshold := aTreshold.
			self changed: #rootNodes].
	self changed: #threshold! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/2/2010 17:12'!
time
	^ rootMessageTally time! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 4/12/2011 10:49'!
toolBarOn: aWindow
	| uiTheme toolBar |
	uiTheme := UITheme current .
	toolBar := toolBar := aWindow newToolbar: {
		( uiTheme
			newButtonIn: World 
			for: self
			getState: nil
			action: #profileIt
			arguments: {}
			getEnabled: nil
			getLabel: nil
			help: nil)		 		
				label: ( uiTheme
					newRowIn: World 
					for: {AlphaImageMorph new image:  uiTheme smallDoItIcon. uiTheme buttonLabelForText: 'Profile it' translated});		 			yourself.
		aWindow newToolSpacer.
		aWindow newToolSpacer.
		aWindow newLabel: 'Min time % ' translated.
		 (uiTheme
				newTextEntryIn: aWindow
				for: self 
				get: #minPercentageToExpand
				set: #minPercentageToExpand: 
				class: Float
				getEnabled: nil
				help: nil).
		aWindow newToolSpacer.
		aWindow newToolSpacer.
		aWindow newLabel: 'Threshold: ' translated.
		 (uiTheme
				newTextEntryIn: aWindow
				for: self 
				get: #threshold
				set: #threshold: 
				class: Integer
				getEnabled: nil
				help: nil).
		aWindow newToolSpacer.
		aWindow newToolSpacer.
		aWindow newDropListFor: self 
			list: #whatToShowList
			getSelected: #whatToShow 
			setSelected: #whatToShow:
			getEnabled: nil 
			useIndex: false 
			help: 'Which kind of result'.
		aWindow newToolSpacer.
		aWindow newToolSpacer.
		uiTheme
			newCheckboxIn: World
			for: self
			getSelected: #reportOtherProcesses
			setSelected: #reportOtherProcesses:
			getEnabled: nil
			label: 'All processes' 
			help: nil.
		aWindow newToolSpacer}.
	^ toolBar! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/24/2011 08:34'!
treeMorph
	^ (MorphTreeMorph on: self)
		rowInset: 2;
		columnInset: 4;
		autoDeselection: true;
		getMenuSelector:#menu:shifted:;
		keystrokeActionSelector: #keyStroke:from:;
		treeLineWidth: 1;
		treeLineDashes: {5. 1};
		lineColorBlock: [:node | {Color gray. Color orange. Color brown. Color magenta. Color blue} at: ((node level \\ 5) + 1)];
		doubleClickSelector: #browseItem;
		rowColorForEven: Color white darker;
		yourself
! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:21'!
whatToShow
	^ self showLeavesOnly ifTrue: [#'Leaves'] ifFalse: [#'Full tree']! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:22'!
whatToShow: aSymbol
	self showLeavesOnly: aSymbol = #'Leaves'! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/19/2010 16:20'!
whatToShowList
	^ #(#'Full tree' #'Leaves')! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/15/2011 09:38'!
withBlockCodePane
	^ withBlockCodePane ifNil: [withBlockCodePane := true]! !

!TimeProfiler methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 2/15/2011 09:38'!
withBlockCodePane: aBoolean
	withBlockCodePane := aBoolean! !

!ToolSet class methodsFor: '*Tools-ViewHierarchyExplorer' stamp: 'SW 5/22/2009 12:07'!
exploreViewHierarchy: anObject
	"Open an explorer on the given object."
	self default ifNil:[^self inform: 'Cannot explore view hierarchy- no ToolSet present'].
	^self default exploreViewHierarchy: anObject! !

!MethodContext methodsFor: '*Tools-Inspector' stamp: 'StephaneDucasse 8/21/2010 20:49'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ ContextInspector! !

!TraitBehavior methodsFor: '*Tools-Browser' stamp: 'al 2/4/2006 11:43'!
browse
	self systemNavigation browseClass: self! !

!Form class methodsFor: '*Tools-FileList' stamp: 'nk 7/16/2003 18:01'!
fileReaderServicesForFile: fullName suffix: suffix

	^((ImageReadWriter allTypicalFileExtensions add: '*'; add: 'form'; yourself)
		includes: suffix)
		ifTrue: [ self services ]
		ifFalse: [#()]
! !

!Form class methodsFor: '*Tools-FileList' stamp: 'StephaneDucasse 11/13/2009 18:42'!
openImageInWindow: fullName 
	"Handle five file formats: GIF, JPG, PNG, Form storeOn: (run coded), and
	BMP. Fail if file format is not recognized."
	
	| image |
	image := self fromFileNamed: fullName.
	(World drawingClass withForm: image) openInWorld! !

!Form class methodsFor: '*Tools-FileList' stamp: 'tbn 8/11/2010 10:18'!
serviceImageAsBackground
	"Answer a service for setting the desktop background from a given graphical file's contents"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Use graphic as background'
		selector: #setBackgroundFromImageFileNamed:
		description: 'Use the graphic as the background for the desktop'
		buttonLabel: 'Background'! !

!Form class methodsFor: '*Tools-FileList' stamp: 'tbn 8/11/2010 10:19'!
serviceOpenImageInWindow
	"Answer a service for opening a graphic in a window"

	^ SimpleServiceEntry 
		provider: self 
		label: 'Open graphic in a window'
		selector: #openImageInWindow:
		description: 'Open a graphic file in a window'
		buttonLabel: 'Open'! !

!Form class methodsFor: '*Tools-FileList' stamp: 'MarcusDenker 3/23/2011 18:28'!
services

	^ Array 
		with: self serviceOpenImageInWindow
		with: self serviceImageAsBackground ! !

!Form class methodsFor: '*Tools-FileList' stamp: 'GabrielOmarCotelli 6/4/2009 20:42'!
setBackgroundFromImageFileNamed: aFileName

	(self fromFileNamed: aFileName) setAsBackground! !

StringMorph subclass: #CodeAnnotationMorph
	instanceVariableNames: 'codeHolder process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!CodeAnnotationMorph commentStamp: 'LaurentLaffont 2/13/2011 16:46' prior: 0!
I display code annotations that can be embedded in a browser (basic core image browser) to show useful informations about the current method or class. In order to show me in your browser, the "show annotation pane" setting must be set to true (search for 'annotation' in the setting browser). This setting is set to false by default.

The informations to show are given by CodeHolder>>annotation.
A methods annotation is built according to a list of requests which consists in a list of symbols. These symbols are interpreted by CodeHolder>>#annotationForSelector: ofClass:. See CodeHolder class >>#annotationRequests for more details about this list of requests.
By default, the author, the time stamp, the message category, the senders and implementors count and the list of change sets in which the method is referenced are shown.

Because the building of an annotation can be very time consuming (especially the implementors count),an annotation is built in background. So, I am also a good example of how a process can be used in order to build some data in background.

Instance Variables
   codeHolder: <CodeHolder>
   process: <Process>

codeHolder
   - The CodeHolder (a Browser, a MessageList ..) in which I'm embedded

process
   - The process which is building the annotation in background

COTDC - A.Plantec!

!CodeAnnotationMorph class methodsFor: 'instance creation' stamp: 'AlainPlantec 11/13/2009 23:34'!
on: aCodeHolder
	^ self new codeHolder: aCodeHolder! !

!CodeAnnotationMorph methodsFor: 'accessing' stamp: 'AlainPlantec 11/13/2009 23:42'!
codeHolder: aCodeHolder
	codeHolder := aCodeHolder.
	aCodeHolder ifNotNil: [aCodeHolder addDependent: self]! !

!CodeAnnotationMorph methodsFor: 'initialization' stamp: 'AlainPlantec 11/14/2009 00:03'!
initialize
	super initialize.
	self borderWidth: 0.
	self contents: ''.
! !

!CodeAnnotationMorph methodsFor: 'updating' stamp: 'Igor.Stasenko 5/9/2010 23:01'!
syncContents
	| ch |
	process ifNotNil: [ process terminate ].
	process := nil.
	
	ch := codeHolder ifNil: [ ^ self contents: '' ].
		
	process := [  | ann |
		ann := ch annotation ifNil: [''].
		WorldState addDeferredUIMessage: [ self contents: ann ].
	] newProcess.
	process priority: Processor userBackgroundPriority.
	process resume.
	! !

!CodeAnnotationMorph methodsFor: 'updating' stamp: 'Igor.Stasenko 10/12/2010 23:29'!
update: anAspect
	super update: anAspect.
	anAspect == #contents ifFalse: [ ^self ].
	self syncContents.
! !

StringMorph subclass: #IndentingListItemMorph
	instanceVariableNames: 'indentLevel isExpanded complexContents firstChild container nextSibling icon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!IndentingListItemMorph commentStamp: 'AlainPlantec 1/7/2010 22:16' prior: 0!
An IndentingListItemMorph is a StringMorph that draws itself with an optional toggle at its left, as part of the display of the SimpleHierarchicalListMorph.

It will also display lines around the toggle depending on UITheme settings

Instance variables:

indentLevel <SmallInteger> 	the indent level, from 0 at the root and increasing by 1 at each level of the hierarchy.

isExpanded <Boolean>		true if this item is expanded (showing its children)

complexContents <ListItemWrapper>	an adapter wrapping my represented item that can answer its children, etc.
	
firstChild <IndentingListItemMorph|nil>	my first child, or nil if none
	
container <SimpleHierarchicalListMorph>	my container
	
nextSibling <IndentingListItemMorph|nil>	the next item in the linked list of siblings, or nil if none.

Contributed by Bob Arning as part of the ObjectExplorer package.
Don't blame him if it's not perfect.  We wanted to get it out for people to play with.!

!IndentingListItemMorph methodsFor: 'drag and drop' stamp: 'nk 6/12/2004 16:49'!
acceptDroppingMorph: toDrop event: evt
	complexContents acceptDroppingObject: toDrop complexContents.
	toDrop delete.
	self highlightForDrop: false.! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'RAA 7/30/2000 19:49'!
addChildrenForList: hostList addingTo: morphList withExpandedItems: expandedItems

	firstChild ifNotNil: [
		firstChild withSiblingsDo: [ :aNode | aNode delete].
	].
	firstChild := nil.
	complexContents hasContents ifFalse: [^self].
	firstChild := hostList 
		addMorphsTo: morphList
		from: complexContents contents 
		allowSorting: true
		withExpandedItems: expandedItems
		atLevel: indentLevel + 1.
	! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 7/21/2000 11:00'!
balloonText

	^complexContents balloonText ifNil: [super balloonText]! !

!IndentingListItemMorph methodsFor: 'halos and balloon help' stamp: 'RAA 7/21/2000 11:13'!
boundsForBalloon

	"some morphs have bounds that are way too big"
	container ifNil: [^super boundsForBalloon].
	^self boundsInWorld intersect: container boundsInWorld! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 3/31/1999 17:44'!
canExpand

	^complexContents hasContents! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'panda 4/28/2000 15:30'!
children
	| children |
	children := OrderedCollection new.
	self childrenDo: [:each | children add: each].
	^children! !

!IndentingListItemMorph methodsFor: 'enumeration' stamp: 'panda 4/28/2000 15:29'!
childrenDo: aBlock

	firstChild ifNotNil: [
		firstChild withSiblingsDo: [ :aNode | aBlock value: aNode].
	]! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'RAA 7/11/1998 14:34'!
complexContents

	^complexContents! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'gvc 7/30/2009 13:55'!
drawLineToggleToTextOn: aCanvas lineColor: lineColor hasToggle: hasToggle
	"If I am not the only item in my container, draw the line between:
		- my toggle (if any) or my left edge (if no toggle)
		- and my text left edge.
	Only draw now if no toggle."

	| myBounds myCenter hLineY hLineLeft myTheme|
	self isSoleItem ifTrue: [ ^self ].
	self hasToggle ifTrue: [^self].
	myBounds := self toggleBounds.
	myCenter := myBounds center.
	hLineY := myCenter y - 1.
	hLineLeft := myCenter x.
	"Draw line from toggle to text. Use optimised form since vertical."
	myTheme := self theme.
	aCanvas
		frameRectangle: (hLineLeft @ hLineY corner: myBounds right + 3 @ (hLineY + 1))
		width: myTheme treeLineWidth
		colors: (myTheme treeLineColorsFrom: lineColor)
		dashes: myTheme treeLineDashes! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'nk 3/8/2004 11:43'!
drawLinesOn: aCanvas lineColor: lineColor 
	| hasToggle |
	hasToggle := self hasToggle.
	"Draw line from toggle to text"
	self drawLineToggleToTextOn: aCanvas lineColor: lineColor hasToggle: hasToggle.

	"Draw the line from my toggle to the nextSibling's toggle"
	self nextSibling ifNotNil: [ self drawLinesToNextSiblingOn: aCanvas lineColor: lineColor hasToggle: hasToggle ].

	"If I have children and am expanded, draw a line to my first child"
	(self firstChild notNil and: [ self isExpanded ])
		ifTrue: [ self drawLinesToFirstChildOn: aCanvas lineColor: lineColor]! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'MarcusDenker 1/23/2011 09:19'!
drawLinesToFirstChildOn: aCanvas lineColor: lineColor
	"Draw line from me to first child.
	Don't bother if the first child has a toggle.."

	| vLineX vLineTop vLineBottom childBounds childCenter myTheme |
	self firstChild hasToggle
		ifTrue: [ ^ self ].
	childBounds := self firstChild toggleBounds.
	childCenter := childBounds center.
	vLineX := childCenter x.
	vLineTop := bounds bottom.
	vLineBottom := self firstChild hasToggle
		ifTrue: [ childCenter y - (childBounds height // 2) + 1 ]
		ifFalse: [ childCenter y - 2 ].
	myTheme := self theme.
	aCanvas
		frameRectangle: (vLineX @ vLineTop corner: (vLineX + 1) @ vLineBottom)
		width: myTheme treeLineWidth
		colors: (myTheme treeLineColorsFrom: lineColor)
		dashes: myTheme treeLineDashes! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'gvc 7/30/2009 13:55'!
drawLinesToNextSiblingOn: aCanvas lineColor: lineColor hasToggle: hasToggle
	"Draw line from me to next sibling"
	
	| myBounds nextSibBounds vLineX myCenter vLineTop vLineBottom myTheme|
	myBounds := self toggleBounds.
	nextSibBounds := self nextSibling toggleBounds.
	myCenter := myBounds center.
	vLineX := myCenter x.
	vLineTop := myCenter y + 1.
	vLineBottom := nextSibBounds center y - 1.
	"Draw line from me to next sibling"
	myTheme := self theme.
	aCanvas
		frameRectangle: (vLineX @ vLineTop corner: vLineX + 1 @ vLineBottom)
		width: myTheme treeLineWidth
		colors: (myTheme treeLineColorsFrom: lineColor)
		dashes: myTheme treeLineDashes! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'nice 1/5/2010 15:59'!
drawOn: aCanvas 
	| tRect sRect columnScanner colorToUse columnLeft |
	tRect := self toggleRectangle.
	sRect := bounds withLeft: tRect right + 4.
	self 
		drawToggleOn: aCanvas
		in: tRect.
	colorToUse := complexContents preferredColor ifNil: [ color ].
	icon isNil ifFalse: 
		[ aCanvas 
			translucentImage: icon
			at: sRect left @ (self top + ((self height - icon height) // 2)).
		sRect := sRect left: sRect left + icon width + 2 ].
	(container columns isNil or: [ (contents asString indexOf: Character tab) = 0 ]) 
		ifTrue: 
			[ sRect := sRect top: (sRect top + sRect bottom - self fontToUse height) // 2.
			aCanvas 
				drawString: contents asString
				in: sRect
				font: self fontToUse
				color: colorToUse ]
		ifFalse: 
			[ columnLeft := sRect left.
			columnScanner := contents asString readStream.
			container columns do: 
				[ :width | | columnData columnRect | 
				columnRect := columnLeft @ sRect top extent: width @ sRect height.
				columnData := columnScanner upTo: Character tab.
				columnData isEmpty ifFalse: 
					[ aCanvas 
						drawString: columnData
						in: columnRect
						font: self fontToUse
						color: colorToUse ].
				columnLeft := columnRect right + 5 ] ]! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'AlainPlantec 2/11/2011 15:05'!
drawToggleOn: aCanvas in: aRectangle

	| aForm centeringOffset |
	complexContents hasContents ifFalse: [^self].
	aForm := isExpanded 
		ifTrue: [container expandedFormForMorph: self]
		ifFalse: [container notExpandedFormForMorph: self].
	centeringOffset := ((aRectangle height - aForm extent y) / 2.0) truncated.
	^aCanvas 
		translucentImage: aForm 
		at: (aRectangle topLeft translateBy: 0 @ centeringOffset).
! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'CarloTeixeira 7/2/2010 23:44'!
findExactPathMatchIn: anArray 
	self
		withSiblingsDo: [:each | (each complexContents asString = anArray first
					or: [anArray first isNil])
				ifTrue: [^ each]].
	^ nil! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'CarloTeixeira 7/2/2010 23:53'!
findPathIn: anArray 
	| found |
	found := self findExactPathMatchIn: anArray.
	found
		ifNil: ["try again with no case sensitivity"
			found := self findSimilarPathMatchIn: anArray].
	^ found! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'CarloTeixeira 7/2/2010 23:46'!
findSimilarPathMatchIn: anArray 
	self
		withSiblingsDo: [:each | (each complexContents asString sameAs: anArray first)
				ifTrue: [^ each]].
	^ nil! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 8/2/1999 16:48'!
firstChild

	^firstChild! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'dgd 9/25/2004 22:25'!
hasIcon
	"Answer whether the receiver has an icon."
	^ icon notNil! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'nk 2/19/2004 18:29'!
hasToggle
	^ complexContents hasContents! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'RAA 8/3/1999 09:47'!
highlight

	complexContents highlightingColor ifNotNil: [self color: complexContents highlightingColor].
	self changed.
	
! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'dgd 9/25/2004 22:27'!
icon
	"answer the receiver's icon"
	^ icon! !

!IndentingListItemMorph methodsFor: 'mouse events' stamp: 'ar 3/17/2001 17:32'!
inToggleArea: aPoint

	^self toggleRectangle containsPoint: aPoint! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 7/30/2000 19:15'!
indentLevel

	^indentLevel! !

!IndentingListItemMorph methodsFor: 'initialization' stamp: 'AlainPlantec 11/29/2009 21:59'!
initWithContents: anObject prior: priorMorph forList: hostList indentLevel: newLevel

	container := hostList.
	complexContents := anObject.
	self initWithContents: anObject asString font: StandardFonts listFont emphasis: nil.
	indentLevel := 0.
	isExpanded := false.
 	nextSibling := firstChild := nil.
	priorMorph ifNotNil: [
		priorMorph nextSibling: self.
	].
	indentLevel := newLevel.
	icon := anObject icon.
	self extent: self minWidth @ self minHeight! !

!IndentingListItemMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 20:42'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	indentLevel := 0.
	isExpanded := false! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 7/31/1998 00:30'!
isExpanded

	^isExpanded! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 7/31/1998 00:48'!
isExpanded: aBoolean

	isExpanded := aBoolean! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'nk 3/8/2004 09:14'!
isFirstItem
	^owner submorphs first == self! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'nk 3/8/2004 09:15'!
isSoleItem
	^self isFirstItem and: [ owner submorphs size = 1 ]! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'gvc 6/15/2007 13:17'!
minHeight
	"Answer the minimum height of the receiver."
	
	| iconHeight |
	iconHeight := self hasIcon
				ifTrue: [self icon height + 2]
				ifFalse: [0].
	^(self fontToUse height max: iconHeight) max: super minHeight! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'gvc 6/15/2007 13:15'!
minWidth
	"Fixed to work such that guessed width is unnecessary in
	#adjustSubmorphPositions."

	| iconWidth |
	iconWidth := self hasIcon
				ifTrue: [self icon width + 2]
				ifFalse: [0].
	^(13 * indentLevel + 15 + (self fontToUse widthOfString: contents)
		+ iconWidth) max: super minWidth! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 7/11/1998 12:15'!
nextSibling

	^nextSibling! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'RAA 8/1/1998 01:05'!
nextSibling: anotherMorph

	nextSibling := anotherMorph! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'CarloTeixeira 7/2/2010 23:54'!
openPath: anArray 
	| found |
	anArray isEmpty
		ifTrue: [^ container setSelectedMorph: nil].
	found := self findPathIn: anArray.
	found
		ifNil: [^ container setSelectedMorph: nil].
	found isExpanded
		ifTrue: [found refreshExpandedState]
		ifFalse: [found toggleExpandedState].
	container adjustSubmorphPositions.
	found changed.
	anArray size = 1
		ifTrue: [^ container setSelectedMorph: found].
	^ found firstChild
		ifNil: [container setSelectedMorph: nil]
		ifNotNil: [found firstChild openPath: anArray allButFirst]! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'RAA 6/21/1999 14:54'!
recursiveAddTo: aCollection

	firstChild ifNotNil: [
		firstChild withSiblingsDo: [ :aNode | aNode recursiveAddTo: aCollection].
	].
	aCollection add: self
	! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'RAA 4/2/1999 18:02'!
recursiveDelete

	firstChild ifNotNil: [
		firstChild withSiblingsDo: [ :aNode | aNode recursiveDelete].
	].
	self delete
	! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'CarloTeixeira 7/2/2010 23:11'!
refreshExpandedState

 	| newChildren toDelete c |
	toDelete := OrderedCollection new.
	firstChild ifNotNil: [
		firstChild withSiblingsDo: [ :aNode | aNode recursiveAddTo: toDelete].
	].
	container noteRemovalOfAll: toDelete.
	(isExpanded and: [complexContents hasContents]) ifFalse: [
		^self changed
	].
	(c := complexContents contents) isEmpty ifTrue: [^self changed].
	newChildren := container 
		addSubmorphsAfter: self 
		fromCollection: c 
		allowSorting: true.
	firstChild := newChildren first.
! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'nk 12/5/2002 15:16'!
toggleBounds
	^self toggleRectangle! !

!IndentingListItemMorph methodsFor: 'private-container protocol' stamp: 'MarcusDenker 1/23/2011 09:19'!
toggleExpandedState

	isExpanded := isExpanded not.
	self refreshExpandedState.! !

!IndentingListItemMorph methodsFor: 'action' stamp: 'gvc 4/25/2007 19:42'!
toggleRectangle

	| h |
	h := bounds height.
	^(bounds left + (13 * indentLevel)) @ bounds top extent: 9@h! !

!IndentingListItemMorph methodsFor: 'drawing' stamp: 'RAA 8/3/1999 09:46'!
unhighlight

	complexContents highlightingColor ifNotNil: [self color: Color black].
	self changed.
	
	
! !

!IndentingListItemMorph methodsFor: 'accessing' stamp: 'bf 2/9/2004 10:55'!
userString
	"Add leading tabs to my userString"
	^ (String new: indentLevel withAll: Character tab), super userString
! !

!IndentingListItemMorph methodsFor: 'private' stamp: 'RAA 7/11/1998 14:25'!
withSiblingsDo: aBlock

	| node |
	node := self.
	[node isNil] whileFalse: [
		aBlock value: node.
		node := node nextSibling
	].! !

!IndentingListItemMorph methodsFor: 'converting' stamp: 'RAA 3/31/1999 12:13'!
withoutListWrapper

	^complexContents withoutListWrapper! !

!Dictionary methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:16'!
customizeExplorerContents

	^ true.
! !

!Dictionary methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:44'!
explorerContentsWithIndexCollect: twoArgBlock

	| sortedKeys |
	sortedKeys := self keys asSortedCollection: [:x :y |
		((x isString and: [y isString])
			or: [x isNumber and: [y isNumber]])
			ifTrue: [x < y]
			ifFalse: [x class == y class
				ifTrue: [x printString < y printString]
				ifFalse: [x class name < y class name]]].
	^ sortedKeys collect: [:k | twoArgBlock value: (self at: k) value: k].
! !

!Dictionary methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:32'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ DictionaryInspector! !

PluggableTextMorph subclass: #BrowserCommentTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!BrowserCommentTextMorph commentStamp: '<historical>' prior: 0!
I am a PluggableTextMorph that knows enough to make myself invisible when necessary.!

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'AlainPlantec 11/17/2010 11:34'!
defaultLayoutFrame
	^ (LayoutFrame fractions: (0@0.75 corner: 1@1)).
! !

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'nk 2/15/2004 13:41'!
hideOrShowPane
	(self model editSelection == #editClass)
		ifTrue: [ self showPane ]
		ifFalse: [ self hidePane ]! !

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'marcus.denker 11/10/2008 10:04'!
hidePane
	"Fixed to not keep doing the splitters. If we are hiden don't hide again!!"

	| win |
	self owner ifNotNil: [
		win := self window ifNil: [^self].
		self window ifNotNil: [:window | window removePaneSplitters].
		self lowerPane ifNotNil: [:lp | 
			lp layoutFrame bottomFraction: self layoutFrame bottomFraction.
			lp layoutFrame bottomOffset: SystemWindow borderWidth negated].
		self delete.
		win updatePanesFromSubmorphs.
		win addPaneSplitters]! !

!BrowserCommentTextMorph methodsFor: 'accessing' stamp: 'nk 2/15/2004 14:12'!
lowerPane
	"Answer the AlignmentMorph that I live beneath"
	^self valueOfProperty: #browserLowerPane! !

!BrowserCommentTextMorph methodsFor: 'updating' stamp: 'stephane.ducasse 10/9/2008 18:50'!
noteNewOwner: win
	"Dirty fix for when the 'lower pane' hasn't been reset to the bottom at the
	time the receiver is added"

	super noteNewOwner: win.
	self setProperty: #browserWindow toValue: win.
	win ifNil: [ ^self ].
	win setProperty: #browserClassCommentPane toValue: self.
	self setProperty: #browserLowerPane toValue: (win submorphThat: [ :m | m isAlignmentMorph and: [ m layoutFrame bottomFraction = 1 or: [ m layoutFrame bottomFraction = self layoutFrame topFraction]]] ifNone: []).
! !

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'AlainPlantec 11/17/2010 11:35'!
showPane
	"Fixed to not keep doing the splitters. If we are showing don't show again!!"
	
	| win |
	self owner ifNil: [
		win := self window ifNil: [ ^self ].
		win addMorph: self fullFrame: self defaultLayoutFrame.
		win updatePanesFromSubmorphs.
		self lowerPane ifNotNil: [ :lp | lp layoutFrame bottomFraction: self layoutFrame topFraction ].
		win addPaneSplitters]! !

!BrowserCommentTextMorph methodsFor: 'updating' stamp: 'nk 2/15/2004 13:42'!
update: anAspect
	super update: anAspect.
	anAspect == #editSelection ifFalse: [ ^self ].
	self hideOrShowPane! !

!BrowserCommentTextMorph methodsFor: 'accessing' stamp: 'nk 2/15/2004 14:07'!
window
	^self owner ifNil: [ self valueOfProperty: #browserWindow ].! !

Object subclass: #AbstractTool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!AbstractTool commentStamp: '<historical>' prior: 0!
I'm an abstract class grouping generic methods for managing packages/classes/groups/methods from a browser!

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:12'!
addCategoryIn: aClass

	self addCategoryIn: aClass before: nil! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 5/5/2011 14:48'!
addCategoryIn: aClass before: aCategory

	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| labels reject lines oldIndex newName |
	
	labels := OrderedCollection new.
	reject := Set new.
	reject
		addAll: aClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	aClass allSuperclasses do: [:cls | | cats |
		cls = Object ifFalse: [
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse: [
				lines add: labels size.
				labels addAll: cats asSortedCollection.
				reject addAll: cats ]]].
	
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: 'Add Category'.
	newName ifNil: [ ^ nil ].
	newName := newName asSymbol.

	aClass organization
		addCategory: newName
		before: aCategory.
	self selectedCategory: newName.
	
	^ newName! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:14'!
addClassIn: aPackage
	" add a new class in the provided package. A dialog will be opened to let the user set the class definition "

	| classSymbol systemCategory classDefinition |
	classSymbol := #NameOfSubclass.
	systemCategory := aPackage
		ifNil: [ 'Unknown' ].
	classDefinition := 'Object subclass: #' , classSymbol , '
		instanceVariableNames: '''' 
		classVariableNames: ''''
		poolDictionaries: ''''
		category: ''' , systemCategory name , ''''.
	classDefinition := UIManager default 
		multiLineRequest: 'Define class definition:'
		centerAt: Sensor cursorPoint
		initialAnswer: classDefinition
		answerHeight: 250.
	(classDefinition isNil or: [ classDefinition isEmpty ])
		ifTrue: [ ^ self ].
	Compiler evaluate: classDefinition.! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/13/2011 12:20'!
addPackageBasedOn: aPackage

	| name initialAnswer package |
	initialAnswer := aPackage ifNil: [''] ifNotNil: [:p | p name].
	name := UIManager default request: 'Name of the new package:' initialAnswer: initialAnswer title: 'Create a new package'.
	(name isNil or: [name isEmpty])
		ifFalse: [ package := self browsedEnvironment packageOrganizer createPackageNamed: name ].
	^ package! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/14/2011 22:47'!
addPackagesAsGroup: aCollection

	aCollection
		ifNotEmpty: [| packages list blocks name | 
			packages := aCollection sort: [:a :b | a name <= b name ].
			blocks := OrderedCollection new.
			packages do: [:package | blocks add: [ package orderedClasses]].
			name := ( packages collect: [:package | package name]) joinUsing: ' + '.
			^ (self groupsManager	
					addADynamicGroupSilentlyNamed: name
					blocks: blocks)]! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/13/2011 12:21'!
addPackagesAsGroup: aCollection named: aString

	aCollection
		ifNotEmpty: [:packages || blocks | 
			blocks := OrderedCollection new.
			packages do: [:package | blocks add: [ package orderedClasses]].
			^ (self groupsManager	
					addADynamicGroupSilentlyNamed: aString 
					blocks: blocks)]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/14/2011 21:48'!
addTraitIn: aPackage
	" add a new class in the provided package. A dialog will be opened to let the user set the class definition "

	| classSymbol systemCategory classDefinition |
	classSymbol := #NameOfSubclass.
	systemCategory := aPackage
		ifNil: [ 'Unknown' ].
	classDefinition := 'Trait named: #TSortable
	uses: {}
	category: ''' , systemCategory name , ''''.
	classDefinition := UIManager default 
		multiLineRequest: 'Define class definition:'
		centerAt: Sensor cursorPoint
		initialAnswer: classDefinition
		answerHeight: 250.
	(classDefinition isNil or: [ classDefinition isEmpty ])
		ifTrue: [ ^ self ].
	Compiler evaluate: classDefinition.! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:25'!
browseClassRefsOf: aClass

	| class |
	class := aClass theNonMetaClass ifNil: [^self].
	class isTrait 
		ifTrue: [self systemNavigation browseAllUsersOfTrait: class]
		ifFalse: [self systemNavigation browseAllCallsOnClass: class]
! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:26'!
browseClassVarRefsOf: aClass

	| cls |
	cls := aClass theNonMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseClassVarRefs: cls]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:27'!
browseCommentVersionsOf: aCollection

	aCollection do: [:class | ClassCommentVersionsBrowser browseCommentOf: class ]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:28'!
browseInstVarDefsOf: aClass

	(aClass notNil and: [aClass isTrait not])
		ifTrue: [self systemNavigation browseInstVarDefs: aClass]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:29'!
browseInstVarRefsOf: aClass

	(aClass notNil and: [aClass isTrait not])
		ifTrue: [self systemNavigation browseInstVarRefs: aClass]! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 12:34'!
browseMessagesFrom: aMethod
	" badly named, it browses implementors "

	self getSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation with: {} method: aMethod! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 12:34'!
browseSendersOfMessagesFrom: aMethod
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self getSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation with: {} method: aMethod! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:35'!
browseUnusedMethodsOf: aClass
	
	aClass
		ifNil: [^ self]
		ifNotNil: [:cls || classes unsent messageList |
			classes := Array with: cls with: cls class.
			unsent := Set new.
			classes
				do: [:c | unsent addAll: c selectors ].
			unsent := self systemNavigation allUnsentMessagesIn: unsent.
			messageList := OrderedCollection new.
			classes
				do: [:c | (c selectors
						select: [:s | unsent includes: s]) asSortedCollection
						do: [:sel | messageList add: c name , ' ' , sel]].
			self systemNavigation browseMessageList: messageList name: 'Unsent Methods in ' , cls name]! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 5/3/2011 15:55'!
browseVersionsFrom: aMethod
	"Create and schedule a Versions Browser, showing all versions of the 
	currently selected message. Answer the browser or nil."
	| selector class | 

	(selector := aMethod selector)
		ifNil:[ self inform: 'Sorry, only actual methods have retrievable versions.'. ^nil ]
		ifNotNil: [
			class := aMethod methodClass.
			
			(VersionsBrowser
				browseVersionsOf: aMethod
				class: class
				meta: class isMeta
				category: aMethod category
				selector: selector)]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:37'!
categorizeAllUncategorizedMethodsOf: aClass
	"Categorize methods by looking in parent classes for a method category."

	| organizer organizers |
	organizer := aClass organization.
	organizers := aClass withAllSuperclasses collect: [:ea | ea organization].
	(organizer listAtCategoryNamed: ClassOrganizer default) do: [:sel | | found |
		found := (organizers collect: [ :org | org categoryOfElement: sel])
			detect: [:ea | ea ~= ClassOrganizer default and: [ ea ~= nil]]
			ifNone: [].
		found ifNotNil: [organizer classify: sel under: found]].! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/14/2011 11:28'!
categorizeMethods: aCollection of: aClass from: aCategory

	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| labels reject lines oldIndex newName |
	aCollection ifEmpty: [^ self].
	aClass ifNil: [^self].
	
	labels := OrderedCollection new.
	labels
		addAll: aClass organization categories copy sort;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	lines add: labels size -1.
	
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: ('Change Protocol (current: ', aCategory, ')').
	newName ifNil: [ ^self ].
	newName := newName asSymbol.
	
	aCollection do: [:item | 
		item methodClass organization
					classify: item selector
					under: newName 
					suppressIfDefault: true ].! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/14/2011 14:59'!
compileANewClassFrom: aString

	^ Compiler evaluate: aString! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/14/2011 15:09'!
compileANewMethodInClass: aClass categorized: aCategory from: aString notifyng: aController

	^ aClass compile: aString classified: aCategory notifying: aController.! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:46'!
copyClasses: aCollection
	
	aCollection
		ifNotEmpty:[:classes || needsRefresh lastDuplicate | 
			needsRefresh := false.
			classes do: [:class || copysName |
				copysName := UIManager default request: 'Please type new class name' initialAnswer: class name.
				copysName isEmptyOrNil
					ifFalse: [
						needsRefresh ifFalse: [ needsRefresh := true ].
						lastDuplicate := class duplicateClassWithNewName: copysName ]].

			^(needsRefresh -> lastDuplicate)].
	^ false -> nil! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 12:48'!
createInstVarAccessorsOf: aClass
	"Create getters and setters for all inst vars defined at the level of the current class selection, except do NOT clobber or override any selectors already understood by the instances of the selected class"

	aClass
		ifNotNil:	[
			aClass instVarNames do: [:aName || setter newMessage |
				(aClass canUnderstand: aName asSymbol)
					ifFalse: [
						newMessage := self buildGetterFor: aName.
						aClass compile: newMessage classified: 'accessing' notifying: nil].
				(aClass canUnderstand: (aName, ':') asSymbol)
					ifFalse:[
						newMessage :=self buildSetterFor: aName.
						aClass compile: newMessage classified: 'accessing' notifying: nil]]]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 5/3/2011 11:27'!
fileOutClasses: aCollection
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .st."

	Cursor write 
		showWhile: [
			aCollection
				ifNotEmpty: [
					aCollection do: [:class | class theNonMetaClass fileOut ]]]! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 12:52'!
fileOutMethods: aCollection

	Cursor write 
		showWhile:[
			aCollection
				ifNotEmpty: [:methods |
					 methods do: [:method |
						method methodClass fileOutMethod: method selector ]]]! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/13/2011 12:52'!
fileOutPackages: aCollection

	Cursor write 
		showWhile:[
			aCollection
				ifNotEmpty: [
					aCollection do: [:package | package fileOut ]]]! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/14/2011 16:38'!
findMethodIn: aClass

	| recentList pattern foundMethod |
	aClass ifNil: [^ nil ].
	recentList := aClass selectors.
	recentList size == 0
		ifTrue: [pattern := UIManager default request: 'Message name or fragment?']
		ifFalse: [pattern := UIManager default 
				enterOrRequestFrom: recentList 
				lines: #() 
				title: 'Message name or fragment?'].
			
	pattern isEmptyOrNil ifTrue: [^ nil ].
	foundMethod := self systemNavigation selectorFromPattern: pattern withCaption: '' startingFrom: self selectedClass.
	foundMethod ifNil: [^ nil ].
	^ foundMethod! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/13/2011 12:57'!
findPackageIn: aPackageNamesList

	| pattern foundPackage |
	aPackageNamesList size == 0
		ifTrue: [pattern := UIManager default request: 'Package name or fragment?']
		ifFalse: [pattern := UIManager default 
				enterOrRequestFrom: aPackageNamesList 
				lines: #() 
				title: 'Package name or fragment?'].
	pattern isEmptyOrNil ifTrue: [^ self flashPackage ].
	foundPackage := self systemNavigation packageFromPattern: pattern withCaption: ''.
	foundPackage ifNil: [^ self flashPackage].
	^ foundPackage! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 12:32'!
getSelectorAndSendQuery: querySelector to: queryPerformer with: queryArgs method: aMethod
	"Obtain a selector relevant to the current context, and then send the querySelector to the queryPerformer with the selector obtained and queryArgs as its arguments.  If no message is currently selected, then obtain a method name from a user type-in"

	| strm array |
	strm := (array := Array new: queryArgs size + 1) writeStream.
	strm nextPut: nil.
	strm nextPutAll: queryArgs.

	aMethod ifNil: [ | selector |
		selector := UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
		selector ifNil: [selector := String new].
		selector := selector copyWithout: Character space.
		^ selector isEmptyOrNil ifFalse: [
			(Symbol hasInterned: selector
				ifTrue: [ :aSymbol |
					array at: 1 put: aSymbol.
					queryPerformer perform: querySelector withArguments: array])
				ifFalse: [ self inform: 'no such selector']
		]
	].

	array at: 1 put: aMethod selector.
	queryPerformer perform: querySelector withArguments: array! !

!AbstractTool methodsFor: 'group' stamp: 'BenjaminVanRyseghem 4/13/2011 12:59'!
mergeGroups: aCollection

	aCollection
		ifNotEmpty: [:groups || group |
			group := groups reduce: [:a :b | a or: b].
			groups do: [:gp | 
				gp removable
					ifTrue: [self groupsManager removeAGroupSilently: gp]].
				^ group].
	^ nil! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 13:00'!
methodHierarchyFrom: aMethod
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation 
			methodHierarchyBrowserForClass: aMethod methodClass
			selector: aMethod selector! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/14/2011 16:30'!
moveInNewPackage: aCollection

	| labels reject lines oldIndex newName item aPackage |
	aCollection ifEmpty: [^self].
	aPackage := aCollection first package.	
	labels := self model packages collect: [:each | each name].
	lines := {0}.
		
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: ('Change Package (current: ', aPackage name,')').
	newName ifNil: [ ^self ].
	newName := newName asSymbol.

	item := self browsedEnvironment packageOrganizer packageNamed: newName.
	aCollection do: [:aClass |
		aPackage removeClass: aClass.
		item addClassDefinition: aClass.
		aClass category: item name ].
	^ item! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 13:06'!
moveMethod: aMethod inPackage: aPackage

	aMethod methodClass organization
				classify: aMethod selector
				under: '*', aPackage name
				suppressIfDefault: true.! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 13:05'!
moveMethodsToPackage: aCollection in: packagesList from: aPackage

	"Search for a package from a pattern or from the recent list"
	| pattern foundPackage |

	aCollection ifEmpty: [^ self ].

	packagesList size == 0
		ifTrue: [pattern := UIManager default request: 'Package name or fragment?']
		ifFalse: [pattern := UIManager default 
				enterOrRequestFrom: packagesList 
				lines: #() 
				title: 'Package name or fragment?'].
	pattern isEmptyOrNil ifTrue: [^ self flashPackage ].
	foundPackage := self systemNavigation packageFromPattern: pattern withCaption: ''.
	foundPackage ifNil: [^ self flashPackage].
	
	aPackage = foundPackage ifTrue: [ ^ self ].
	
	aCollection do: [:each |
		self moveMethod: each inPackage: foundPackage ].
		
! !

!AbstractTool methodsFor: 'category' stamp: 'BenjaminVanRyseghem 4/13/2011 13:09'!
removeCategory: aCategory inClass: aClass

	| messageCategoryName |
	aCategory ifNil: [^ self].
	self okToChange ifFalse: [^ self].
	messageCategoryName := aCategory.
	(self confirm: 'Are you sure you want to
remove the protocol ', aCategory,'
and all its methods?')
		ifTrue: 
			[aClass removeCategory: messageCategoryName.
			"self selectedCategory: nil.
			methodsTree update"
			^ true].
	^ false! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 16:15'!
removeClass: aClass
	"Remove the selected class from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."

	| message  className classToRemove result |
	classToRemove := aClass ifNil: [Beeper beep. ^ false].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you
want to REMOVE the class ', className, '
from the system?'.
	(result := self confirm: message)
		ifTrue: 
			[classToRemove subclasses size > 0
				ifTrue: [(self confirm: 'class has subclasses: ' , message)
					ifFalse: [^ false]].
			classToRemove removeFromSystem ].
	^ result! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 16:18'!
removeClasses: aCollection
	"Remove the selected classes from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."

	| result |
	result := false.
	aCollection
		ifNotEmpty: [
			aCollection do: [:class | (self removeClass: class)
										ifTrue: [ result := true ]]].
	^ result! !

!AbstractTool methodsFor: 'category' stamp: 'BenjaminVanRyseghem 4/13/2011 13:10'!
removeEmptyCategoriesFrom: aClass

	aClass organization removeEmptyCategories.! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 13:12'!
removeMethod: aMethod inClass: aClass
	"If a message is selected, create a Confirmer so the user can verify that  
	the currently selected message should be removed from the system. If 
	so, remove it. "
	| messageName confirmation |
	aMethod ifNil: [^ false].
	messageName := aMethod selector.
	confirmation := self systemNavigation confirmRemovalOf: messageName on: aClass.
	confirmation == 3
		ifTrue: [^ false].
	(aClass includesLocalSelector: messageName)
		ifTrue: [ aClass removeSelector: messageName ]
		ifFalse: [ self removeNonLocalSelector: messageName ].

	"In case organization not cached"
	confirmation == 2
		ifTrue: [self systemNavigation browseAllCallsOn: messageName].
	^ true! !

!AbstractTool methodsFor: 'method' stamp: 'BenjaminVanRyseghem 4/13/2011 13:13'!
removeNonLocalSelector: aSymbol from: aClass
	| traits isAlias |
	traits := aClass traitsProvidingSelector: aSymbol.
	isAlias := aClass isLocalAliasSelector: aSymbol.
	isAlias
		ifTrue: [
			[traits size = 1] assert.
			aClass removeAlias: aSymbol of: traits first]
		ifFalse: [
			traits do: [:each |
				aClass addExclusionOf: aSymbol to: each ]]! !

!AbstractTool methodsFor: 'category' stamp: 'BenjaminVanRyseghem 4/13/2011 13:20'!
renameCategory: aCategory from: aClass
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	
	| oldIndex oldName newName |
	aClass ifNil: [^ self].
	aCategory ifNil: [^ self].
	oldName := aCategory.
	newName := UIManager default
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmptyOrNil
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	newName = oldName ifTrue: [^ self].
	aClass organization
		renameCategory: oldName
		toBe: newName.
	^ newName! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 16:06'!
renameClass: aClass
	| oldName newName obs |

	aClass ifNil: [ ^ self ].
	
	oldName := aClass name.
	newName := UIManager default  request: 'Please type new class name' initialAnswer: oldName.
	newName isEmptyOrNil ifTrue: [ ^ self ].	"Cancel returns"
	newName := newName asSymbol.
	newName = oldName ifTrue: [ ^ self ].
	(self class environment includesKey: newName)
		ifTrue: [ ^ self error: newName , ' already exists' ].
	aClass rename: newName.

	obs := self systemNavigation allReferencesTo: (aClass environment associationAt: newName).
	obs isEmpty
		ifFalse: [ self systemNavigation browseMessageList: obs name: 'Obsolete References to ' , oldName autoSelect: oldName ].
	^ newName! !

!AbstractTool methodsFor: 'package' stamp: 'BenjaminVanRyseghem 4/13/2011 16:08'!
renamePackages: aCollection

	| name |
	aCollection
		 ifNotEmpty: [:packages |
			packages do: [:selectedPackage | 
				name := UIManager default request: 'New name of the package' initialAnswer: selectedPackage name title: 'Rename a package'.
				(name isNil or: [name isEmpty])
					ifFalse: [ | package |
						package := self browsedEnvironment packageOrganizer renamePackage: selectedPackage in: name ]]]! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 16:11'!
showUnreferencedClassVarsOf: aClass
	"Search for all class variables known to the selected class, and put up a 
	list of those that have no references anywhere in the system. The 
	search includes superclasses, so that you don't need to navigate your 
	way to the class that defines each class variable in order to determine 
	whether it is unreferenced"
	| aList aReport |
	( aClass isNil or: [aClass isTrait]) ifTrue: [^ self].
	aList := self systemNavigation allUnreferencedClassVariablesOf: aClass.
	aList size == 0
		ifTrue: [^ UIManager default inform: 'There are no unreferenced
class variables in
' , aClass name].
	aReport := String
				streamContents: [:aStream | 
					aStream nextPutAll: 'Unreferenced class variable(s) in ' translated, aClass name;
						 cr.
					aList
						do: [:el | aStream tab; nextPutAll: el; cr]].
	UIManager default inform: aReport! !

!AbstractTool methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/13/2011 16:11'!
showUnreferencedInstVarsOf: aClass
	"Search for all instance variables known to the selected class, and put up a list of those that have no references anywhere in the system.  The search includes superclasses, so that you don't need to navigate your way to the class that defines each inst variable in order to determine whether it is unreferenced"

	|  aList aReport |
	(aClass isNil or: [aClass isTrait]) ifTrue: [^ self].
	aList := aClass allUnreferencedInstanceVariables.
	aList size == 0 ifTrue: [^ UIManager default inform: 'There are no unreferenced
instance variables in
', aClass name].
	aReport := String streamContents:
		[:aStream |
			aStream nextPutAll: 'Unreferenced instance variable(s) in ' translated, aClass name; cr.
			aList do: [:el | aStream tab; nextPutAll: el; cr]].
	UIManager default inform: aReport! !

Object subclass: #DynamicMessageImplementor
	instanceVariableNames: 'message class argumentNames stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!DynamicMessageImplementor class methodsFor: 'instance creation' stamp: 'HernanWilkinson 10/12/2010 16:31'!
for: aMessage in: aClass

	^ self new initializeFor: aMessage in: aClass! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/13/2010 10:53'!
argumentClassNameOf: argument

	^ (argument isKindOf: Class) 
		ifTrue: [ argument name, 'Class' ]
		ifFalse: [ argument class name ]! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:19'!
argumentNameAt: anIndex

	| argumentName argument argumentClassName |
	
	argument := message arguments at: anIndex.
	argumentClassName := self argumentClassNameOf: argument.
	argumentName := (self argumentNamePrefixOf: argumentClassName), argumentClassName.
	
	[ argumentNames includes: argumentName ] whileTrue: [ argumentName := argumentName , anIndex asString ].
	argumentNames add: argumentName.

	^ argumentName! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:19'!
argumentNamePrefixOf: argumentClassName

	^ argumentClassName first isVowel
		ifTrue: [ 'an' ]
		ifFalse: [ 'a' ]! !

!DynamicMessageImplementor methodsFor: 'testing' stamp: 'HernanWilkinson 10/12/2010 17:11'!
hasParameter: aKeyword

	^ aKeyword last = $: or: [ message selector isInfix ]! !

!DynamicMessageImplementor methodsFor: 'initialization' stamp: 'HernanWilkinson 10/12/2010 16:32'!
initializeFor: aMessage in: aClass

	message := aMessage.
	class := aClass! !

!DynamicMessageImplementor methodsFor: 'testing' stamp: 'HernanWilkinson 10/12/2010 16:38'!
isMessageAGetter
	
	^ message numArgs = 0 and: [ class instVarNames includes: message selector ]! !

!DynamicMessageImplementor methodsFor: 'testing' stamp: 'HernanWilkinson 10/12/2010 16:52'!
isMessageASetter
	
	^ message numArgs = 1 and: [ class instVarNames includes: message selector allButLast ]! !

!DynamicMessageImplementor methodsFor: 'evaluating' stamp: 'HernanWilkinson 10/12/2010 17:02'!
value

	argumentNames := Set new.
	stream := WriteStream on: String new.
	self writeSourceCode.
	
	^ stream contents! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:14'!
write: aKeyword of: anIndex

	stream nextPutAll: aKeyword.
	self writeArgumentNameIfNecessaryOf: aKeyword at: anIndex! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:16'!
writeArgumentNameAt: anIndex

	| argumentName |
	
	argumentName := self argumentNameAt: anIndex.
	stream
		nextPutAll: ' ';
		nextPutAll: argumentName;
		space! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:14'!
writeArgumentNameIfNecessaryOf: aKeyword at: anIndex

	(self hasParameter: aKeyword) ifTrue: [ self writeArgumentNameAt: anIndex ]! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:08'!
writeGetterSourceCode

	stream 
		cr;
		tab;
		nextPutAll: '^ ';
		nextPutAll: message selector ! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:08'!
writeGetterSourceCodeIfNecessary

	self isMessageAGetter ifTrue: [ self writeGetterSourceCode ]! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:09'!
writeMethodName

	message selector keywords doWithIndex: [ :aKeyword :anIndex | self write: aKeyword of: anIndex ]! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:08'!
writeSetterSourceCode

	stream 
		cr;
		tab;
		nextPutAll: message selector allButLast;
		nextPutAll: ' := ';
		nextPutAll: argumentNames anyOne ! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:08'!
writeSetterSourceCodeIfNecessary

	self isMessageASetter ifTrue: [ self writeSetterSourceCode ]! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:04'!
writeShouldBeImplemented

	stream
		cr;
		tab;
		nextPutAll: 'self ';
		nextPutAll: #shouldBeImplemented;
		nextPut: $.! !

!DynamicMessageImplementor methodsFor: 'evaluating-private' stamp: 'HernanWilkinson 10/12/2010 17:07'!
writeSourceCode

	self writeMethodName.
	self writeShouldBeImplemented.
	self writeGetterSourceCodeIfNecessary.
	self writeSetterSourceCodeIfNecessary! !

Object subclass: #Inspector
	instanceVariableNames: 'contents object selectionIndex timeOfLastListUpdate selectionUpdateTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!Inspector commentStamp: '<historical>' prior: 0!
I represent a query path into the internal representation of an object. As a StringHolder, the string I represent is the value of the currently selected variable of the observed object.!

Inspector subclass: #BasicInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!BasicInspector class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 02:59'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #basicInspector
! !

!BasicInspector methodsFor: 'as yet unclassified' stamp: 'sd 11/20/2005 21:26'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no 
	current selection."

	self initialize.
	object := anObject.
	selectionIndex := 0.
	contents := ''! !

Inspector subclass: #CompiledMethodInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!CompiledMethodInspector class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 2/20/2011 15:12'!
registerToolsOn: registry
	"Register ourselves as inspector for CompiledMethod"
	
	 
	registry registerInspector: self for: CompiledMethod
! !

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'ajh 3/20/2003 00:17'!
contentsIsString
 	"Hacked so contents empty when deselected"
 
 	^ #(0 2 3) includes: selectionIndex! !

!CompiledMethodInspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
fieldList

	| keys |
	keys := OrderedCollection new.
	keys add: 'self'.
	keys add: 'all bytecodes'.
	keys add: 'header'.
	1 to: object numLiterals do: [ :i |
		keys add: 'literal', i printString ].
	object initialPC to: object size do: [ :i |
		keys add: i printString ].
	^ keys asArray
	! !

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
selection

	| bytecodeIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object ].
	selectionIndex = 2 ifTrue: [^ object symbolic].
	selectionIndex = 3 ifTrue: [^ object headerDescription].
	selectionIndex <= (object numLiterals + 3) 
		ifTrue: [ ^ object objectAt: selectionIndex - 2 ].
	bytecodeIndex := selectionIndex - object numLiterals - 3.
	^ object at: object initialPC + bytecodeIndex - 1! !

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'ajh 3/20/2001 11:56'!
selectionUnmodifiable
	"Answer if the current selected variable is unmodifiable via acceptance in the code pane.  For most inspectors, no selection and a selection of self (selectionIndex = 1) are unmodifiable"
 
 	^ true! !

Inspector subclass: #ContextInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!ContextInspector class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 2/20/2011 15:13'!
registerToolsOn: registry
	"Register ourselves as inspector for MethodContext"
	
	 
	registry registerInspector: self for: MethodContext
! !

!ContextInspector methodsFor: 'accessing' stamp: 'StephaneDucasse 8/21/2010 20:48'!
fieldList
	"Answer the base field list plus an abbreviated list of indices."

	^ self baseFieldList , (object tempNames collect: [:t| '[',t,']'])! !

!ContextInspector methodsFor: 'accessing' stamp: 'StephaneDucasse 8/21/2010 20:48'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."
	
	| basicIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintStringLimitedTo: 20000].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].
	basicIndex := selectionIndex - 2 - object class instSize.
	^object debuggerMap namedTempAt: basicIndex in: object
! !

Inspector subclass: #ContextVariablesInspector
	instanceVariableNames: 'fieldList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!ContextVariablesInspector commentStamp: '<historical>' prior: 0!
I represent a query path into the internal representation of a ContextPart. Typically this is a context at a point in the query path of a Debugger. As a StringHolder, the string I represent is the value of the currently selected variable of the observed temporary variable of the context.!

!ContextVariablesInspector methodsFor: 'nil' stamp: 'HenrikSperreJohansen 10/17/2009 10:39'!
contentsIsString
       "Hacked so contents empty when deselected"

       ^ #(0 3) includes: selectionIndex! !

!ContextVariablesInspector methodsFor: 'code' stamp: ''!
doItContext

	^object! !

!ContextVariablesInspector methodsFor: 'code' stamp: ''!
doItReceiver

	^object receiver! !

!ContextVariablesInspector methodsFor: 'accessing' stamp: 'eem 5/21/2008 12:31'!
fieldList 
	"Refer to the comment in Inspector|fieldList."

	object == nil ifTrue: [^Array with: 'thisContext'].
	^fieldList ifNil:[fieldList := (Array with: 'thisContext' with: 'stack top' with: 'all temp vars') , object tempNames]! !

!ContextVariablesInspector methodsFor: 'accessing' stamp: 'ar 4/11/2006 02:33'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no 
	current selection.
	
	Because no object's inspectorClass method answers this class, it is OK for this method to
	override Inspector >> inspect: "
	fieldList := nil.
	object := anObject.
	self initialize.
	! !

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'eem 7/18/2008 11:18'!
replaceSelectionValue: anObject 
	"Refer to the comment in Inspector|replaceSelectionValue:."

	^selectionIndex = 1
		ifTrue: [object]
		ifFalse: [object namedTempAt: selectionIndex - 3 put: anObject]! !

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'eem 6/10/2008 09:37'!
selection 
	"Refer to the comment in Inspector|selection."
	selectionIndex = 0 ifTrue:[^''].
	selectionIndex = 1 ifTrue: [^object].
	selectionIndex = 2 ifTrue: [^object stackPtr > 0 ifTrue: [object top]].
	selectionIndex = 3 ifTrue: [^object tempsAndValues].
	^object debuggerMap namedTempAt: selectionIndex - 3 in: object! !

Inspector subclass: #DictionaryInspector
	instanceVariableNames: 'keyArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!DictionaryInspector commentStamp: 'LaurentLaffont 3/4/2011 22:44' prior: 0!
I provide a custom inspector for classes of type Dictionary.
These customizations are tools to interactively manipulate the Dictionary I'm inspecting and they are accessed in the context menu of the currently selected association.

The customizations that I provide are as follows:
inspect -> Create and schedule an Inspector on my currently selected association
copy name -> copy the name of my currently selected association so that it can be pasted somewhere else
references -> Create a browser on all references to the association of the current selection
objects pointing to this value -> Open a list inspector on all the objects that point to the value of the selected instance variable, if any.
senders of this key -> Create a browser on all senders of the selected key
refresh view
add key
rename key
remove
basic inspect -> Bring up a non-special inspector
!

!DictionaryInspector class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 2/20/2011 15:12'!
registerToolsOn: registry
	"Register ourselves as inspector for Dictionary (and its subclasses)"
	
	 
	registry registerInspector: self for: Dictionary
! !

!DictionaryInspector methodsFor: 'menu' stamp: 'rbb 3/1/2005 10:51'!
addEntry
	| newKey aKey |

	newKey := UIManager default request:
'Enter new key, then type RETURN.
(Expression will be evaluated for value.)
Examples:  #Fred    ''a string''   3+4'.
	aKey := Compiler evaluate: newKey.
	object at: aKey put: nil.
	self calculateKeyArray.
	selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self update! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
addEntry: aKey
	object at: aKey put: nil.
	self calculateKeyArray.
	selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self update! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
calculateKeyArray
	"Recalculate the KeyArray from the object being inspected"

	keyArray := object keysSortedSafely asArray.
	selectionIndex := 0.
! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'di 9/22/1998 21:25'!
contentsIsString
	"Hacked so contents empty when deselected"

	^ (selectionIndex = 0)! !

!DictionaryInspector methodsFor: 'menu' stamp: 'apb 8/20/2004 21:19'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel |
	self selectionIndex <= self numberOfFixedFields
		ifTrue: [super copyName]
		ifFalse: [sel := String streamContents: [:strm | 
							strm nextPutAll: '(self at: '.
							(keyArray at: selectionIndex - self numberOfFixedFields)
								storeOn: strm.
							strm nextPutAll: ')'].
			Clipboard clipboardText: sel asText 			"no undo allowed"]! !

!DictionaryInspector methodsFor: 'accessing' stamp: 'apb 8/20/2004 23:06'!
fieldList
	^ self baseFieldList
		, (keyArray collect: [:key | key printString])! !

!DictionaryInspector methodsFor: 'menu' stamp: 'ar 10/31/2004 17:25'!
fieldListMenu: aMenu

	^ aMenu labels:
'inspect
copy name
references
objects pointing to this value
senders of this key
refresh view
add key
rename key
remove
basic inspect'
	lines: #(6 9)
	selections: #(inspectSelection copyName selectionReferences objectReferencesToSelection sendersOfSelectedKey refreshView addEntry renameEntry removeSelection inspectBasic)
! !

!DictionaryInspector methodsFor: 'initialization' stamp: 'PHK 7/21/2004 18:00'!
initialize
	super initialize.
	self calculateKeyArray! !

!DictionaryInspector methodsFor: 'private' stamp: 'apb 8/20/2004 21:15'!
numberOfFixedFields
	^ 2 + object class instSize! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
refreshView
	| i |
	i := selectionIndex.
	self calculateKeyArray.
	selectionIndex := i.
	self changed: #fieldList.
	self changed: #contents.! !

!DictionaryInspector methodsFor: 'menu' stamp: 'sd 11/20/2005 21:27'!
removeSelection
	selectionIndex = 0 ifTrue: [^ self changed: #flash].
	object removeKey: (keyArray at: selectionIndex - self numberOfFixedFields).
	selectionIndex := 0.
	contents := ''.
	self calculateKeyArray.
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self changed: #selection.! !

!DictionaryInspector methodsFor: 'menu' stamp: 'yann.monclair 7/17/2010 13:59'!
renameEntry
	| newKey aKey value |

	value := object at: (keyArray at: selectionIndex - self numberOfFixedFields).
	newKey := UIManager default request: 
'Enter new key, then type RETURN.
(Expression will be evaluated for value.)
Examples:  #Fred    ''a string''   3+4'
		 initialAnswer: (keyArray at: selectionIndex - self numberOfFixedFields) printString.
	newKey isNil ifFalse: [
		aKey := Compiler evaluate: newKey.
		object removeKey: (keyArray at: selectionIndex - self numberOfFixedFields).
		object at: aKey put: value.
		self calculateKeyArray.
		selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
		self changed: #selectionIndex.
		self changed: #inspectObject.
		self changed: #fieldList.
		self update ]! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 22:37'!
replaceSelectionValue: anObject 
	selectionIndex <= self numberOfFixedFields
		ifTrue: [^ super replaceSelectionValue: anObject].
	^ object
		at: (keyArray at: selectionIndex - self numberOfFixedFields)
		put: anObject! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 21:55'!
selection

	selectionIndex <= (self numberOfFixedFields) ifTrue: [^ super selection].
	^ object at: (keyArray at: selectionIndex - self numberOfFixedFields) ifAbsent:[nil]! !

!DictionaryInspector methodsFor: 'menu' stamp: 'ar 10/31/2004 17:26'!
selectionReferences
	"Create a browser on all references to the association of the current selection."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	object class == MethodDictionary ifTrue: [^ self changed: #flash].
	self systemNavigation browseAllCallsOn: (object associationAt: (keyArray at: selectionIndex  - self numberOfFixedFields)).
! !

!DictionaryInspector methodsFor: 'menu' stamp: 'ar 4/10/2005 22:17'!
sendersOfSelectedKey
	"Create a browser on all senders of the selected key"
	| aKey |
	self selectionIndex = 0
		ifTrue: [^ self changed: #flash].
	((aKey := keyArray at: selectionIndex  - self numberOfFixedFields) isSymbol)
		ifFalse: [^ self changed: #flash].
	SystemNavigation default browseAllCallsOn: aKey! !

!Inspector class methodsFor: 'instance creation' stamp: 'al 9/21/2008 19:41'!
horizontalDividerProportion
	^ 0.4! !

!Inspector class methodsFor: 'instance creation' stamp: 'PHK 7/22/2004 17:04'!
inspect: anObject 
	"Answer an instance of me to provide an inspector for anObject."
	
	"We call basicNew to avoid a premature initialization; the instance method 
	inspect: anObject will do a self initialize."

	^self basicNew inspect: anObject! !

!Inspector class methodsFor: 'instance creation' stamp: 'apb 7/14/2004 12:54'!
openAsMorphOn: anObject withEvalPane: withEval withLabel: label valueViewClass: valueViewClass
	"Note: for now, this always adds an eval pane, and ignores the valueViewClass"

	^ (self openAsMorphOn: anObject withLabel: label) openInWorld! !

!Inspector class methodsFor: 'instance creation' stamp: 'JuanVuletich 10/26/2010 14:19'!
openAsMorphOn: anObject withLabel: aLabel 
	"(Inspector openAsMorphOn: SystemOrganization withLabel: 'Test') openInWorld"

	^UIManager default openInspector: self asMorphOn: anObject withLabel: aLabel! !

!Inspector class methodsFor: 'instance creation' stamp: 'ar 9/27/2005 18:30'!
openOn: anObject
	"Create and schedule an instance of me on the model, anInspector. "

	^ self openOn: anObject withEvalPane: true! !

!Inspector class methodsFor: 'instance creation' stamp: ''!
openOn: anObject withEvalPane: withEval 
	"Create and schedule an instance of me on the model, anInspector. "

	^ self openOn: anObject withEvalPane: withEval withLabel: anObject defaultLabelForInspector! !

!Inspector class methodsFor: 'instance creation' stamp: 'alain.plantec 6/10/2008 18:35'!
openOn: anObject withEvalPane: withEval withLabel: label 
	^ self
		openAsMorphOn: anObject
		withEvalPane: withEval
		withLabel: label
		valueViewClass: nil! !

!Inspector class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 02:58'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #inspector
! !

!Inspector class methodsFor: 'instance creation' stamp: 'sw 1/19/1999 14:38'!
verticalDividerProportion
	^ 0.7! !

!Inspector methodsFor: 'selecting' stamp: 'damiencassou 5/30/2008 16:29'!
accept: aString 
	| result |
	result := self doItReceiver class evaluatorClass new 
		evaluate: aString readStream
		in: self doItContext
		to: self doItReceiver
		notifying: nil
		ifFail: 
			[ "fix this"
			self changed: #flash.
			^ false ].
	result == #failedDoit ifTrue: [ ^ false ].
	self replaceSelectionValue: result.
	self changed: #contents.
	^ true! !

!Inspector methodsFor: 'menu commands' stamp: 'tbn 7/29/2010 22:07'!
addCollectionItemsTo: aMenu
	"If the current selection is an appropriate collection, add items to aMenu that cater to that kind of selection"

	| sel |
	((((sel := self selection) isMemberOf: Array) or: [sel isMemberOf: OrderedCollection]) and: 
		[sel size > 0]) ifTrue: [
			aMenu addList: #(
				('Inspect element...'					inspectElement))].

	(sel isKindOf: MorphExtension) ifTrue: [
			aMenu addList: #(
				('Inspect property...'				inspectElement))].! !

!Inspector methodsFor: 'accessing' stamp: ''!
baseFieldList
	"Answer an Array consisting of 'self'
	and the instance variable names of the inspected object."

	^ (Array with: 'self' with: 'all inst vars')
			, object class allInstVarNames! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 5/10/2008 17:33'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection."

	^ self spawnProtocol

	! !

!Inspector methodsFor: 'menu commands' stamp: 'CamilloBruni 2/21/2011 16:29'!
browseMethodFull
	SystemNavigation default browseClass: self selection class.! !

!Inspector methodsFor: 'menu commands' stamp: 'tk 4/10/1998 17:53'!
classOfSelection
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
classVarRefs
	"Request a browser of methods that store into a chosen instance variable"

	| aClass |
	(aClass := self classOfSelection) ifNotNil:
		[self systemNavigation  browseClassVarRefs: aClass].
! !

!Inspector methodsFor: 'menu commands' stamp: 'IgorStasenko 2/17/2011 16:03'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	| donorMenu |
	donorMenu := shifted
				ifTrue: [SmalltalkEditor shiftedYellowButtonMenu]
				ifFalse: [SmalltalkEditor yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!Inspector methodsFor: 'accessing' stamp: 'GuillermoPolito 8/12/2010 14:52'!
contents
	^contents! !

!Inspector methodsFor: 'selecting' stamp: 'di 9/22/1998 21:24'!
contentsIsString
	"Hacked so contents empty when deselected and = long printString when item 2"

	^ (selectionIndex = 2) | (selectionIndex = 0)! !

!Inspector methodsFor: 'accessing' stamp: 'GuillermoPolito 8/12/2010 14:52'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0  "null selection"! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel aClass variableNames |
	self selectionUnmodifiable
		ifTrue: [^ self changed: #flash].
	aClass := self object class.
	variableNames := aClass allInstVarNames.
	(aClass isVariable and: [selectionIndex > (variableNames size + 2)])
		ifTrue: [sel := '(self basicAt: ' , (selectionIndex - (variableNames size + 2)) asString , ')']
		ifFalse: [sel := variableNames at: selectionIndex - 2].
	(self selection isKindOf: Collection)
		ifTrue: [sel := '(' , sel , ' at: 1)'].
	Clipboard clipboardText: sel asText! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
defsOfSelection
	"Open a browser on all defining references to the selected instance variable, if that's what currently selected. "
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation  browseAllStoresInto: sel from: aClass! !

!Inspector methodsFor: 'menu commands' stamp: 'GuillermoPolito 8/13/2010 00:12'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!Inspector methodsFor: 'code' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^object! !

!Inspector methodsFor: 'menu commands' stamp: 'IgorStasenko 3/6/2011 18:54'!
explorePointers
	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	Smalltalk tools pointerExplorer new openExplorerFor: self selection! !

!Inspector methodsFor: 'menu commands' stamp: 'sw 9/21/1999 12:16'!
exploreSelection

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	^ self selection explore! !

!Inspector methodsFor: 'accessing' stamp: ''!
fieldList
	"Answer the base field list plus an abbreviated list of indices."

	object class isVariable ifFalse: [^ self baseFieldList].
	^ self baseFieldList ,
		(object basicSize <= (self i1 + self i2)
			ifTrue: [(1 to: object basicSize)
						collect: [:i | i printString]]
			ifFalse: [(1 to: self i1) , (object basicSize-(self i2-1) to: object basicSize)
						collect: [:i | i printString]])! !

!Inspector methodsFor: 'menu commands' stamp: 'AlainPlantec 10/19/2010 23:50'!
fieldListMenu: aMenu
	"Arm the supplied menu with items for the field-list of the receiver"

	aMenu addStayUpItemSpecial.

	aMenu addList: #(
		('Inspect (i)'						inspectSelection. nil. #smallInspectItIcon)
		('Explore (I)'						exploreSelection. nil. #smallInspectItIcon)).

	self addCollectionItemsTo: aMenu.

	aMenu addList: #(
		-
		('Method refs to this inst var'		referencesToSelection)
		('Methods storing into this inst var'	defsOfSelection)
		('Objects pointing to this value'		objectReferencesToSelection)
		('Explore pointers'					explorePointers)
		-
		('Browse full (b)'					browseMethodFull)
		('Browse class'						browseClass)
		('Browse hierarchy (h)'				classHierarchy)
		('Browse protocol (p)'				browseFullProtocol)
		-
		('Inst var refs...'					browseInstVarRefs)
		('Inst var defs...'					browseInstVarDefs)
		('Class var refs...'					classVarRefs)
		('Class variables'					browseClassVariables)
		('Class refs (N)'						browseClassRefs)
		-
		('Copy name (c)'					copyName)		
		('Basic inspect'						inspectBasic)
		).
	^ aMenu


! !

!Inspector methodsFor: 'accessing' stamp: ''!
i1
	"This is the max index shown before skipping to the 
	last i2 elements of very long arrays"
	^ 100! !

!Inspector methodsFor: 'accessing' stamp: ''!
i2
	"This is the number of elements to show at the end
	of very long arrays"
	^ 10! !

!Inspector methodsFor: 'accessing' stamp: 'al 9/21/2008 19:40'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 350 @ 300! !

!Inspector methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
initialize
	
	selectionIndex := 0.
	super initialize! !

!Inspector methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no current selection.
	
	Normally the receiver will be of the correct class (as defined by anObject inspectorClass),
	because it will have just been created by sedning inspect to anObject.   However, the
	debugger uses two embedded inspectors, which are re-targetted on the current receiver
	each time the stack frame changes.  The left-hand inspector in the debugger has its
	class changed by the code here.  Care should be taken if this method is overridden to
	ensure that the overriding code calls 'super inspect: anObject', or otherwise ensures that 
	the class of these embedded inspectors are changed back."

	| c |
	c := anObject inspectorClass.
	(self class ~= c and: [self class format = c format]) ifTrue: [
		self primitiveChangeClassTo: c basicNew].
	
	"Set 'object' before sending the initialize message, because some implementations
	of initialize (e.g., in DictionaryInspector) require 'object' to be non-nil."
	
	object := anObject. 
	self initialize! !

!Inspector methodsFor: 'menu commands' stamp: 'tk 4/12/1998 08:49'!
inspectBasic
	"Bring up a non-special inspector"

	selectionIndex = 0 ifTrue: [^ object basicInspect].
	self selection basicInspect! !

!Inspector methodsFor: 'menu commands' stamp: 'damiencassou 5/30/2008 16:29'!
inspectElement
	"Create and schedule an Inspector on an element of the receiver's model's currently selected collection."
	| sel selSize countString count nameStrs |
	self selectionIndex = 0 ifTrue: [ ^ self changed: #flash ].
	((sel := self selection) isKindOf: SequenceableCollection) ifFalse: 
		[ (sel isKindOf: MorphExtension) ifTrue: [ ^ sel inspectElement ].
		^ sel inspect ].
	(selSize := sel size) == 1 ifTrue: [ ^ sel first inspect ].
	selSize <= 20 ifTrue: 
		[ nameStrs := (1 to: selSize) asArray collect: 
			[ :ii | 
			ii printString , '   ' , (((sel at: ii) printStringLimitedTo: 25) 
					replaceAll: Character cr
					with: Character space) ].
		count := UIManager default 
			chooseFrom: nameStrs substrings
			title: 'which element?'.
		count = 0 ifTrue: [ ^ self ].
		^ (sel at: count) inspect ].
	countString := UIManager default 
		request: 'Which element? (1 to ' , selSize printString , ')'
		initialAnswer: '1'.
	countString isEmptyOrNil ifTrue: [ ^ self ].
	count := Integer readFrom: countString readStream.
	(count > 0 and: [ count <= selSize ]) 
		ifTrue: [ (sel at: count) inspect ]
		ifFalse: [ Beeper beep ]! !

!Inspector methodsFor: 'menu commands' stamp: 'apb 7/14/2004 13:16'!
inspectSelection
	"Create and schedule an Inspector on the receiver's model's currently selected object."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	self selection inspect.
	^ self selection! !

!Inspector methodsFor: 'menu commands' stamp: 'stephane.ducasse 9/20/2008 22:23'!
inspectorKey: aChar from: view
	"Respond to a Command key issued while the cursor is over my field list"

	aChar == $i ifTrue: [^ self selection inspect].
	aChar == $I ifTrue: [^ self selection explore].
	aChar == $b ifTrue:	[^ self browseMethodFull].
	aChar == $h ifTrue:	[^ self classHierarchy].
	aChar == $c ifTrue: [^ self copyName].
	aChar == $p ifTrue: [^ self browseFullProtocol].
	aChar == $N ifTrue: [^ self browseClassRefs].

	^ self arrowKey: aChar from: view! !

!Inspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
modelWakeUpIn: aWindow
	| newText |
	self updateListsAndCodeIn: aWindow.
	newText := self contentsIsString
		ifTrue: [newText := self selection]
		ifFalse: ["keep it short to reduce time to compute it"
			self selectionPrintString ].
	newText = contents ifFalse:
		[contents := newText.
		self changed: #contents]! !

!Inspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
noteSelectionIndex: anInteger for: aSymbol
	aSymbol == #fieldList
		ifTrue:
			[selectionIndex := anInteger]! !

!Inspector methodsFor: 'accessing' stamp: ''!
object
	"Answer the object being inspected by the receiver."

	^object! !

!Inspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
object: anObject 
	"Set anObject to be the object being inspected by the receiver."

	| oldIndex |
	anObject == object
		ifTrue: [self update]
		ifFalse:
			[oldIndex := selectionIndex <= 2 ifTrue: [selectionIndex] ifFalse: [0].
			self inspect: anObject.
			oldIndex := oldIndex min: self fieldList size.
			self changed: #inspectObject.
			oldIndex > 0
				ifTrue: [self toggleIndex: oldIndex].
			self changed: #fieldList.
			self changed: #contents]! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 4/16/2003 11:41'!
objectReferencesToSelection
	"Open a list inspector on all the objects that point to the value of the selected instance variable, if any.  "

	self selectionIndex == 0 ifTrue: [^ self changed: #flash].
	self systemNavigation
		browseAllObjectReferencesTo: self selection
		except: (Array with: self object)
		ifNone: [:obj | self changed: #flash].
! !

!Inspector methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
printStringErrorText
	| nm |
	nm := self selectionIndex < 3
					ifTrue: ['self']
					ifFalse: [self selectedSlotName].
	^ ('<error in printString: evaluate "' , nm , ' printString" to debug>') asText.! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
referencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what currently selected.  1/25/96 sw"
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation   browseAllAccessesTo: sel from: aClass! !

!Inspector methodsFor: 'selecting' stamp: 'nice 11/8/2009 15:17'!
replaceSelectionValue: anObject 
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. The value of the selected variable is set to the value, 
	anObject."
	| basicIndex si instVarIndex |
	selectionIndex <= 2 ifTrue: [
		self toggleIndex: (si := selectionIndex).  
		self toggleIndex: si.
		^ object].
	instVarIndex := selectionIndex - 2.
	instVarIndex > object class instSize
		ifFalse: [^ object instVarAt: instVarIndex put: anObject].
	object class isVariable or: [self error: 'Cannot replace selection'].
	basicIndex := selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^object basicAt: basicIndex put: anObject]
		ifFalse: [^object basicAt: object basicSize - (self i1 + self i2) + basicIndex
					put: anObject]! !

!Inspector methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:37'!
selectedClass
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!Inspector methodsFor: 'accessing' stamp: 'MarcusDenker 4/14/2011 10:52'!
selectedClassOrMetaClass

	^ self selectedClass! !

!Inspector methodsFor: 'selecting' stamp: 'eem 5/21/2008 11:46'!
selectedSlotName

	^ self fieldList at: self selectionIndex ifAbsent: []! !

!Inspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."
	| basicIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintStringLimitedTo: 20000].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].
	basicIndex := selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^ object basicAt: basicIndex]
		ifFalse: [^ object basicAt: object basicSize - (self i1 + self i2) + basicIndex]! !

!Inspector methodsFor: 'selecting' stamp: ''!
selectionIndex
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. Answer the index into the list of the selected variable."

	^selectionIndex! !

!Inspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 22:03'!
selectionPrintString
	| text |
	selectionUpdateTime := [text := [self selection printStringLimitedTo: 5000]
						on: Error
						do: [text := self printStringErrorText.
							text
								addAttribute: TextColor red
								from: 1
								to: text size.
							text]] timeToRun.
	^ text! !

!Inspector methodsFor: 'selecting' stamp: 'PHK 6/30/2004 11:50'!
selectionUnmodifiable
	"Answer if the current selected variable is modifiable via acceptance in the code pane.  For most inspectors, no selection and a selection of 'self' (selectionIndex = 1) and 'all inst vars' (selectionIndex = 2) are unmodifiable"

	^ selectionIndex <= 2! !

!Inspector methodsFor: 'menu commands' stamp: 'BenjaminVanRyseghem 5/6/2011 16:13'!
spawnFullProtocol
	"Spawn a window showing full protocol for the receiver's selection"

	self spawnProtocol! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
spawnProtocol
	"Spawn a protocol on browser on the receiver's selection"

	| objectToRepresent |
	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	ProtocolBrowser openSubProtocolForClass: objectToRepresent class! !

!Inspector methodsFor: 'stepping' stamp: 'AlainPlantec 12/1/2009 22:37'!
stepAt: millisecondClockValue in: aWindow
	| newText |

	(CodeHolder smartUpdating and: [(millisecondClockValue - self timeOfLastListUpdate) > 8000]) "Not more often than once every 8 seconds"
		ifTrue:
			[self updateListsAndCodeIn: aWindow.
			timeOfLastListUpdate := millisecondClockValue].

	newText := self contentsIsString
		ifTrue: [self selection]
		ifFalse: ["keep it short to reduce time to compute it"
			self selectionPrintString ].
	newText = contents ifFalse:
		[contents := newText.
		self changed: #contents]! !

!Inspector methodsFor: 'accessing' stamp: 'sma 6/15/2000 16:48'!
stepTimeIn: aSystemWindow
	^ (selectionUpdateTime ifNil: [0]) * 10 max: 1000! !

!Inspector methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 09:47'!
taskbarIcon 
	^ self theme smallInspectItIcon! !

!Inspector methodsFor: 'theme' stamp: 'MarcusDenker 4/14/2011 10:46'!
theme	
	^ UITheme current! !

!Inspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
timeOfLastListUpdate
	^ timeOfLastListUpdate ifNil: [timeOfLastListUpdate := 0]! !

!Inspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
toggleIndex: anInteger
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. If anInteger is the index of this variable, then deselect it. 
	Otherwise, make the variable whose index is anInteger be the selected 
	item."

	selectionUpdateTime := 0.
	selectionIndex = anInteger
		ifTrue: 
			["same index, turn off selection"
			selectionIndex := 0.
			contents := '']
		ifFalse:
			["different index, new selection"
			selectionIndex := anInteger.
			self contentsIsString
				ifTrue: [contents := self selection]
				ifFalse: [contents := self selectionPrintString]].
	self changed: #selection.
	self changed: #contents.
	self changed: #selectionIndex.! !

!Inspector methodsFor: 'accessing' stamp: 'GuillermoPolito 8/12/2010 14:53'!
trash
	"What should be displayed if a trash pane is restored to initial state"

	^ ''! !

!Inspector methodsFor: 'accessing' stamp: 'tk 6/11/1998 22:23'!
trash: newText
	"Don't save it"
	^ true! !

!Inspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
update
	"Reshow contents, assuming selected value may have changed."

	selectionIndex = 0
		ifFalse:
			[self contentsIsString
				ifTrue: [contents := self selection]
				ifFalse: [contents := self selectionPrintString].
			self changed: #contents.
			self changed: #selection.
			self changed: #selectionIndex]! !

!Inspector methodsFor: 'accessing' stamp: 'di 1/13/1999 14:36'!
wantsSteps
	^ true! !

Inspector subclass: #OrderedCollectionInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!OrderedCollectionInspector class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/20/2011 15:14'!
registerToolsOn: registry
	"Register ourselves as inspector for OrderedCollection"
	
	 
	registry registerInspector: self for: OrderedCollection
! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'dew 9/19/2001 03:27'!
fieldList
	object ifNil: [ ^ OrderedCollection new].
	^ self baseFieldList ,
		(object size <= (self i1 + self i2)
			ifTrue: [(1 to: object size)
						collect: [:i | i printString]]
			ifFalse: [(1 to: self i1) , (object size-(self i2-1) to: object size)
						collect: [:i | i printString]])
"
OrderedCollection new inspect
(OrderedCollection newFrom: #(3 5 7 123)) inspect
(OrderedCollection newFrom: (1 to: 1000)) inspect
"! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sw 9/16/97 22:38'!
replaceSelectionValue: anObject 
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. The value of the selected variable is set to the value, anObject."

	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ super replaceSelectionValue: anObject].
	object at: self selectedObjectIndex put: anObject! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sd 11/20/2005 21:27'!
selectedObjectIndex
	"Answer the index of the inspectee's collection that the current selection refers to."

	| basicIndex |
	basicIndex := selectionIndex - 2 - object class instSize.
	^ (object size <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [basicIndex]
		ifFalse: [object size - (self i1 + self i2) + basicIndex]! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sw 9/16/97 22:39'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."

	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ super selection].
	^ object at: self selectedObjectIndex! !

Inspector subclass: #SetInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!SetInspector commentStamp: '<historical>' prior: 0!
A verison of the Inspector specialized for inspecting Sets.  It displays the elements of the set like elements of an array.  Note that the indices, being phyical locations in the hash table, are not meaningful outside of the set.!

!SetInspector class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/20/2011 15:15'!
registerToolsOn: registry
	"Register ourselves as inspector for Set"
	
	 
	registry registerInspector: self for: Set
! !

!SetInspector methodsFor: 'selecting' stamp: 'PHK 6/29/2004 15:33'!
arrayIndexForSelection
	^ (self fieldList at: selectionIndex) asInteger! !

!SetInspector methodsFor: 'menu commands' stamp: 'sd 11/20/2005 21:27'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel |
	self selectionIndex <= (2 + object class instSize)
		ifTrue: [super copyName]
		ifFalse: [sel := '(self array at: '
						, (String streamContents: 
							[:strm | self arrayIndexForSelection storeOn: strm]) , ')'.
			Clipboard clipboardText: sel asText]! !

!SetInspector methodsFor: 'accessing' stamp: 'PHK 6/29/2004 14:50'!
fieldList
	object
		ifNil: [^ Set new].
	^ self baseFieldList
		, (object array
				withIndexCollect: [:each :i | each ifNotNil: [i printString]])
		  select: [:each | each notNil]! !

!SetInspector methodsFor: 'menu' stamp: 'PHK 6/30/2004 12:16'!
fieldListMenu: aMenu

	^ aMenu labels:
'inspect
copy name
objects pointing to this value
refresh view
remove
basic inspect'
	lines: #( 5 8)
	selections: #(inspectSelection copyName objectReferencesToSelection update removeSelection inspectBasic)
! !

!SetInspector methodsFor: 'menu' stamp: 'sd 11/20/2005 21:27'!
removeSelection
	(selectionIndex <= object class instSize) ifTrue: [^ self changed: #flash].
	object remove: self selection.
	selectionIndex := 0.
	contents := ''.
	self changed: #inspectObject.
	self changed: #fieldList.
	self changed: #selection.
	self changed: #selectionIndex.! !

!SetInspector methodsFor: 'selecting' stamp: 'PHK 6/29/2004 15:38'!
replaceSelectionValue: anObject
	^ object array at: self arrayIndexForSelection put: anObject! !

!SetInspector methodsFor: 'selecting' stamp: 'PHK 6/29/2004 15:35'!
selection
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintString].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].

	^ object array at: self arrayIndexForSelection! !

SetInspector subclass: #WeakSetInspector
	instanceVariableNames: 'flagObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!WeakSetInspector commentStamp: '<historical>' prior: 0!
A verison of the SetInspector specialized for inspecting WeakSets.  It knows about the flag object used to indicate empty locations in the hash table.!

!WeakSetInspector class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/20/2011 15:15'!
registerToolsOn: registry
	"Register ourselves as inspector for WeakSet"
	
	 
	registry registerInspector: self for: WeakSet
! !

!WeakSetInspector methodsFor: 'accessing' stamp: 'nice 12/15/2007 11:59'!
fieldList
	| slotIndices |
	object ifNil: [^ Set new].
	
	"Implementation note: do not use objectArray withIndexCollect: as super
	because this might collect indices in a WeakArray, leading to constantly changing fieldList
	as explained at http://bugs.squeak.org/view.php?id=6812"
	
	slotIndices := (Array new: object size) writeStream.
	object array withIndexDo: [:each :i |
		(each notNil and: [each ~= flagObject]) ifTrue: [slotIndices nextPut: i printString]].
	
	^ self baseFieldList
		, slotIndices contents! !

!WeakSetInspector methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
initialize
	super initialize.
	flagObject := object instVarNamed: 'flag'. ! !

Object subclass: #MetacelloConfigurationBrowser
	instanceVariableNames: 'window configTree selectedConfiguration repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-ConfigurationBrowser'!

!MetacelloConfigurationBrowser commentStamp: 'StephaneDucasse 8/10/2010 21:09' prior: 0!
A MetacelloConfigurationBrowser is simple tool to browse Metacello configurations published at http://www.squeaksource.com/MetaRepoForPharoXX where XX denotes different pharo versions.

MetaRepoForPharo1.0 acts as a distribution of all the packages/projects that can be loaded in Pharo1.0. 

We are sorry for the name but the source limits the length of the project name: The full name is 
MetacelloRepositoriesForPharoXX.

Metacello is a configuration language for packages. It allows one to define dependencies 
between packages as well as between complete projects. !

!MetacelloConfigurationBrowser class methodsFor: 'registration' stamp: 'FernandoOlivero 4/12/2011 10:01'!
menuCommandOn: aBuilder  
	<worldMenu>
	(aBuilder item: 'Configuration Browser')
		parent: #Tools;
		order: 0.5;
		action: [self open]; 
		icon:  self theme smallLoadProjectIcon! !

!MetacelloConfigurationBrowser class methodsFor: 'instance creation' stamp: 'tbn 8/4/2010 10:56'!
open
	^(self new)
		open;
		yourself
		! !

!MetacelloConfigurationBrowser class methodsFor: 'registration' stamp: 'FernandoOlivero 4/12/2011 10:01'!
theme
	^ UITheme current ! !

!MetacelloConfigurationBrowser methodsFor: 'menu' stamp: 'tbn 8/4/2010 14:14'!
configurationMenu: aMenu
	selectedConfiguration isNil ifTrue: [^aMenu labels: 'Refresh' lines: #() selections: #(refresh)].
	^aMenu labels:
		'Load configuration
		Load configuration and latest version
		Refresh' 
		lines: #(2)
		selections:
		#(loadSelectedConfiguration loadLatestFromSelectedConfiguration refresh)! !

!MetacelloConfigurationBrowser methodsFor: 'initialize' stamp: 'tbn 8/4/2010 14:28'!
initWindow
	"Create the window"
	
	| leftPanel tabs |
	window := StandardWindow new.
	window title: 'Browser on ', self repository.
	leftPanel := PanelMorph new changeTableLayout.		
	tabs := TabGroupMorph new hResizing: #spaceFill; vResizing: #spaceFill.				
	leftPanel addMorph: tabs.
	window 
		addMorph: leftPanel
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@0 corner: 0 @ 0)).		

	configTree := PluggableTreeMorph new.
	tabs addPage: configTree label: 'Configurations'.		
	configTree 
		model: self; 
		setSelectedSelector: #onConfigurationSelected:;
		getMenuSelector: #configurationMenu:.
		
	window 
		addMorph: configTree 
		fullFrame: (LayoutFrame fractions: (0@0 corner: 0@1) offsets: (0@0 corner: 0 @ 0)).		
	tabs selectedPageIndex: 1.									! !

!MetacelloConfigurationBrowser methodsFor: 'initialize' stamp: 'StephaneDucasse 8/10/2010 21:26'!
initialize
	"Initializes the receiver"
	
	super initialize.
	self repository: self pharoDistribution.
	self initWindow! !

!MetacelloConfigurationBrowser methodsFor: 'private' stamp: 'tbn 8/4/2010 14:27'!
loadConfigurationWithLatest: loadLatest
	"Load the configuration and (depending on the flag) load the latest version"
	
	| configName |
	configName := selectedConfiguration readStream upTo: $-.
	Gofer new
		url: self repository; 
		package: configName;
		load.
	loadLatest ifFalse: [^self].
	
	"Now load the latest version"
	(Smalltalk at: (configName) asSymbol) project latestVersion load.! !

!MetacelloConfigurationBrowser methodsFor: 'actions' stamp: 'tbn 8/4/2010 11:31'!
loadLatestFromSelectedConfiguration
	self loadConfigurationWithLatest: true! !

!MetacelloConfigurationBrowser methodsFor: 'actions' stamp: 'tbn 8/4/2010 11:30'!
loadSelectedConfiguration 

	self loadConfigurationWithLatest: false! !

!MetacelloConfigurationBrowser methodsFor: 'events' stamp: 'tbn 8/4/2010 11:17'!
onConfigurationSelected: aString 
	selectedConfiguration := aString! !

!MetacelloConfigurationBrowser methodsFor: 'actions' stamp: 'tbn 8/4/2010 11:06'!
open
	self refresh.
	window openInWorld! !

!MetacelloConfigurationBrowser methodsFor: 'defaults' stamp: 'StephaneDucasse 8/10/2010 21:26'!
pharoDistribution
	"Returns the correct pharo distribution url"
	
	^'http://www.squeaksource.com/MetaRepoFor', ((SystemVersion current majorMinorVersion) copyWithRegex: '[.-]' matchesReplacedWith: '').  
	
	! !

!MetacelloConfigurationBrowser methodsFor: 'actions' stamp: 'tbn 8/4/2010 15:50'!
refresh
	|repo reductionMap split configName  author version last topMostItems item |
	repo := MCHttpRepository 
				location: self repository				
				user: '' 
				password: ''.
				
	"Reduce to display only the latest"			
	reductionMap := Dictionary new.	
	repo allVersionNames reverse do: [:each |				 
		split := each findTokens: '-.'.
		configName := split first.
		author := split second. 
		version := Integer readFrom: split last.
		last := reductionMap at: configName ifAbsentPut: [ author -> version].
		version > last value ifTrue: [
			reductionMap at: configName put: (author -> version).
		]
	].
	topMostItems := SortedCollection sortBlock: [:e1 :e2 | e1 asString <= e2 asString].
	reductionMap keysAndValuesDo: [:key :val | 
		item := key, '-', val key, '.', val value asString.
		topMostItems add: (PluggableListItemWrapper with: item)].	
		
	"Set as list contents"	
	configTree list: topMostItems.
	window title: self repository.! !

!MetacelloConfigurationBrowser methodsFor: 'accessing' stamp: 'tbn 8/4/2010 14:24'!
repository 
	^repository ! !

!MetacelloConfigurationBrowser methodsFor: 'accessing' stamp: 'tbn 8/4/2010 14:24'!
repository: aURLString
	repository := aURLString! !

!Object class methodsFor: '*Tools-FileList' stamp: 'md 2/15/2006 17:20'!
services
	"Backstop"
	^#()! !

!Object methodsFor: '*Tools-Explorer' stamp: 'sma 11/12/2000 11:43'!
asExplorerString
	^ self printString! !

!Object methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:31'!
basicInspect
	"Create and schedule an Inspector in which the user can examine the 
	receiver's variables. This method should not be overriden."
	^ToolSet basicInspect: self! !

!Object methodsFor: '*Tools-Browser' stamp: 'mu 3/6/2004 15:13'!
browse
	self systemNavigation browseClass: self class! !

!Object methodsFor: '*Tools-Browser' stamp: 'mu 3/11/2004 16:00'!
browseHierarchy
	self systemNavigation browseHierarchy: self class! !

!Object methodsFor: '*Tools-Explorer' stamp: 'yo 8/27/2008 23:16'!
customizeExplorerContents

	^ false.
! !

!Object methodsFor: '*Tools-Base' stamp: 'MarianoMartinezPeck 4/15/2011 17:12'!
defaultBackgroundColor
	"Answer the color to be used as the base window color for a window whose model is an object of the receiver's class"
	
	"I don't want to do a self theme  because otherwise I will need to implement it on Object"
	^  UITheme current  windowColorFor: self! !

!Object methodsFor: '*Tools-Inspector' stamp: ''!
defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the receiver."

	^ self class name! !

!Object methodsFor: '*Tools-Inspector' stamp: 'sbw 6/2/2004 08:45'!
doExpiredInspectCount
	self clearHaltOnce.
	self removeHaltCount.
	self inspect! !

!Object methodsFor: '*Tools-Explorer' stamp: 'ar 9/27/2005 20:29'!
explore
	^ToolSet explore: self! !

!Object methodsFor: '*Tools-Explorer' stamp: 'stephaneducasse 9/17/2005 21:48'!
exploreWithLabel: label

	^ ObjectExplorer new openExplorerFor: self withLabel:
label! !

!Object methodsFor: '*Tools-Explorer' stamp: 'md 8/13/2008 21:39'!
hasContentsInExplorer

	^self basicSize > 0 or: [self class allInstVarNames notEmpty]
! !

!Object methodsFor: '*Tools-Inspector' stamp: 'md 1/18/2006 19:09'!
inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."
	ToolSet inspect: self! !

!Object methodsFor: '*Tools-Inspector' stamp: 'sbw 6/2/2004 08:46'!
inspectOnCount: int 
	self haltOnceEnabled
		ifTrue: [self hasHaltCount
				ifTrue: [self decrementAndCheckHaltCount
						ifTrue: [self doExpiredInspectCount]]
				ifFalse: [int = 1
						ifTrue: [self doExpiredInspectCount]
						ifFalse: [self setHaltCountTo: int - 1]]]! !

!Object methodsFor: '*Tools-Inspector' stamp: 'igorStasenko 1/30/2010 14:04'!
inspectOnce
	"Inspect unless we have already done it once."
	self haltOnceEnabled
		ifTrue: [self clearHaltOnce.
				^ self inspect]! !

!Object methodsFor: '*Tools-Inspector' stamp: 'sbw 6/2/2004 13:20'!
inspectUntilCount: int 
	self haltOnceEnabled
		ifTrue: [self hasHaltCount
				ifTrue: [self decrementAndCheckHaltCount
						ifTrue: [self doExpiredInspectCount]
						ifFalse: [self inspect]]
				ifFalse: [int = 1
						ifTrue: [self doExpiredInspectCount]
						ifFalse: [self setHaltCountTo: int - 1]]]! !

!Object methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 20:30'!
inspectWithLabel: aLabel
	"Create and schedule an Inspector in which the user can examine the receiver's variables."
	^ToolSet inspect: self label: aLabel! !

!Object methodsFor: '*Tools-Inspector' stamp: 'apb 7/14/2004 12:19'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Inspector! !

!Object methodsFor: '*Tools-Base' stamp: 'sw 1/18/2001 13:43'!
showDiffs
	"Answer whether the receiver, serving as the model of a text-bearing entity, is 'showing differences' -- if it is, the editor may wish to show special feedback"

	^ false! !

!Object methodsFor: '*Tools-Base' stamp: 'sw 10/19/1999 14:39'!
updateListsAndCodeIn: aWindow
	self canDiscardEdits ifFalse: [^ self].
	aWindow updatablePanes do: [:aPane | aPane verifyContents]! !

Object subclass: #StandardToolSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!StandardToolSet commentStamp: '<historical>' prior: 0!
Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.!

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 18:57'!
basicInspect: anObject
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^BasicInspector openOn: anObject! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'md 3/10/2006 21:43'!
browse: aClass selector: aSelector
	"Open a browser"
	^SystemBrowser default fullOnClass: aClass selector: aSelector! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/16/2005 15:20'!
browseChangeSetsWithClass: aClass selector: aSelector
	"Browse all the change sets with the given class/selector"
	^ChangeSorter browseChangeSetsWithClass: aClass selector: aSelector! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'MarcusDenker 2/16/2010 16:22'!
browseHierarchy: aClass selector: aSelector
	"Open a browser"
	aClass ifNil: [^ self].
	(SystemBrowser default new setClass: aClass selector: aSelector) spawnHierarchy.! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'davidroethlisberger 2/11/2009 12:00'!
browseImplementorsOf: aSymbol name: titleString autoSelect: autoSelectString
	^self browseMessageSet: (SystemNavigation default allImplementorsOf: aSymbol) asSortedCollection name: titleString autoSelect: autoSelectString! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/17/2005 11:12'!
browseMessageNames: aString
	^(MessageNames methodBrowserSearchingFor: aString) openInWorld! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/17/2005 11:13'!
browseMessageSet: messageList name: title autoSelect: autoSelectString
	"Open a message set browser"
	^MessageSet
		openMessageList: messageList 
		name: title 
		autoSelect: autoSelectString! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'JB 2/18/2010 13:52'!
browseSendersOf: aSymbol name: titleString autoSelect: autoSelectString
	
	^self 
		browseMessageSet: (SystemNavigation default allCallsOn: aSymbol) asSortedCollection
		name: titleString 
		autoSelect: autoSelectString! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/17/2005 11:35'!
browseVersionsOf: aClass selector: aSelector
	"Open a browser"
	VersionsBrowser
		browseVersionsOf: (aClass compiledMethodAt: aSelector)
		class: aClass theNonMetaClass
		meta: aClass isMeta
		category: (aClass organization categoryOfElement: aSelector)
		selector: aSelector! !

!StandardToolSet class methodsFor: 'completion' stamp: 'damiencassou 7/29/2009 16:12'!
codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt
	^ aBlock value! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/15/2005 18:57'!
debug: aProcess context: aContext label: aString contents: contents fullView: aBool
	"Open a debugger on the given process and context."
	^Debugger openOn: aProcess context: aContext label: aString contents: contents fullView: aBool! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'MarianoMartinezPeck 1/19/2010 19:17'!
debug: aGuineaPig controller: aController context: aContext

	^Debugger new
		process: aGuineaPig
		controller: aController
		context: aContext! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'MikeRoberts 4/1/2009 21:30'!
debugContext: aContext

	^Debugger context: aContext! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/15/2005 19:15'!
debugContext: aContext label: aString contents: contents
	"Open a debugger on the given process and context."
	^Debugger openContext: aContext label: aString contents: contents! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/17/2005 11:16'!
debugError: anError
	"Handle an otherwise unhandled error"
	^Processor activeProcess
		debug: anError signalerContext
		title: anError description! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 9/27/2005 19:18'!
debugSyntaxError: anError
	"Handle a syntax error"
	| notifier |
	notifier :=  SyntaxError new
		setClass: anError errorClass
		code: anError errorCode
		debugger: (Debugger context: anError signalerContext)
		doitFlag: anError doitFlag.
	notifier category: anError category.
	SyntaxError open: notifier.! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 19:34'!
explore: anObject
	"Open an explorer on the given object."
	^ObjectExplorer new openExplorerFor: anObject! !

!StandardToolSet class methodsFor: '*Tools-ViewHierarchyExplorer' stamp: 'SW 5/22/2009 12:09'!
exploreViewHierarchy: aMorph
	"Open an explorer on the given object."
	^ViewHierarchyExplorer new openExplorerFor: aMorph! !

!StandardToolSet class methodsFor: 'initialization' stamp: 'AlainPlantec 12/16/2009 21:26'!
initialize
	ToolSet register: self.
! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 18:58'!
inspect: anObject
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^(self inspectorClassOf: anObject) openOn: anObject! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 19:57'!
inspect: anObject label: aString
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^(self inspectorClassOf: anObject) openOn: anObject withEvalPane: true withLabel: aString! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'lr 3/14/2010 21:13'!
inspectorClassOf: anObject
	"Answer the inspector class for the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."

	| map |
	map := Dictionary new.
	#(#(#CompiledMethod #CompiledMethodInspector) #(#CompositeEvent #OrderedCollectionInspector) #(#Dictionary #DictionaryInspector) #(#ExternalStructure #ExternalStructureInspector) #(#FloatArray #OrderedCollectionInspector) #(#OrderedCollection #OrderedCollectionInspector) #(#Set #SetInspector) #(#WeakSet #WeakSetInspector))
		do: [ :spec | map at: spec first put: spec last ].
	anObject class
		withAllSuperclassesDo: [ :cls | map at: cls name ifPresent: [ :inspectorName | ^ Smalltalk globals classNamed: inspectorName ] ].
	^ Inspector! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/17/2005 11:16'!
interrupt: aProcess label: aString
	"Open a debugger on the given process and context."
	Debugger
		openInterrupt: aString
		onProcess: aProcess! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'MikeRoberts 4/1/2009 21:18'!
isDebugger: aModel

	^aModel isKindOf: Debugger! !

!StandardToolSet class methodsFor: 'menu' stamp: 'DavidRoethlisberger 12/23/2009 16:53'!
mainMenuItems
	"Answer the most important menu items available for this tool set"
	^#(
		('System Browser' 			#openClassBrowser)
		('Workspace'				#openWorkspace)
		('Test Runner'				#openTestRunner)
		('Monticello Browser'		#openMonticelloBrowser)
	)
! !

!StandardToolSet class methodsFor: 'world menu' stamp: 'FernandoOlivero 4/12/2011 10:31'!
menuCommandOn: aBuilder  
	<worldMenu>
	(aBuilder item: #Tools)
		order: 1.0;
		target: self;
		icon: self theme smallDoItIcon.! !

!StandardToolSet class methodsFor: 'menu' stamp: 'StephaneDucasse 1/28/2011 16:42'!
menuItems
	"Answer the menu items available for this tool set"
	
	^#(
		('System Browser' 			#openClassBrowser)
		('Message Names'				#openMessageNames)
		-
		('Workspace'					#openWorkspace)
		('Transcript' 				#openTranscript)
		('File Browser'				#openFileList)
		-
		('Test Runner'				#openTestRunner)
		('Process Browser' 			#openProcessBrowser)
		-
		('Monticello Browser'		#openMonticelloBrowser)
		"('Monticello Configurations' #openMonticelloConfigurations)"
		('Recover lost changes...'	#openRecentChangesLog)
		"('Simple Change Sorter'		#openChangeSorter)"
		('Change Sorter'				#openDualChangeSorter)
	)
! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/15/2005 18:58'!
openChangedMessageSet: aChangeSet
	"Open a ChangedMessageSet for aChangeSet"
	ChangedMessageSet openFor: aChangeSet! !

!StandardToolSet class methodsFor: 'menu' stamp: 'md 3/10/2006 21:47'!
openClassBrowser
	SystemBrowser default open! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/15/2005 18:58'!
openClassListBrowser: anArray title: aString
	"Open a class list browser"
	^ClassListBrowser new initForClassesNamed: anArray title: aString
! !

!StandardToolSet class methodsFor: 'menu' stamp: 'JuanVuletich 10/21/2010 17:09'!
openDualChangeSorter
	DualChangeSorter new openAsMorph! !

!StandardToolSet class methodsFor: 'menu' stamp: 'MarcusDenker 12/12/2009 07:29'!
openFileList
	FileList open! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 12:59'!
openMessageNames
	"Bring a MessageNames tool to the front"
	MessageNames openMessageNames! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 12:59'!
openProcessBrowser
	ProcessBrowser open! !

!StandardToolSet class methodsFor: 'menu' stamp: 'al 10/12/2008 21:11'!
openRecentChangesLog
	ChangeList browseRecentLog! !

!StandardToolSet class methodsFor: 'menu' stamp: 'BartGauquie 11/1/2009 10:55'!
openTestRunner
	TestRunnerBrowser default ifNotNil: [^ TestRunnerBrowser default open].
	self inform: 'No test runner installed.'! !

!StandardToolSet class methodsFor: 'menu' stamp: 'FernandoOlivero 3/29/2011 12:44'!
openTranscript
	UIManager default openTranscript! !

!StandardToolSet class methodsFor: 'menu' stamp: 'md 1/18/2006 19:08'!
openWorkspace
	Workspace open! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'FernandoOlivero 4/12/2011 10:31'!
theme
	^ UITheme current ! !

!StandardToolSet class methodsFor: 'initialization' stamp: 'ar 7/16/2005 16:18'!
unload
	ToolSet unregister: self.! !

Object subclass: #SyntaxError
	instanceVariableNames: 'contents class selector category debugger doitFlag dependents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!SyntaxError commentStamp: '<historical>' prior: 0!
I represent syntax error report for syntax errors encountered when filing in class descriptions from a non-interactive source such as an external file. As a StringHolder, the string to be viewed is the method code or expression containing the error.

The user may fix the error and accept the method to continue the fileIn.
!

!SyntaxError class methodsFor: 'instance creation' stamp: 'GuillermoPolito 8/13/2010 01:08'!
buildMorphicViewOn: aSyntaxError
	"Answer an Morphic view on the given SyntaxError."
	| window |
	window := (SystemWindow labelled: 'Syntax Error') model: aSyntaxError.

	window addMorph: (PluggableListMorph on: aSyntaxError list: #list
			selected: #listIndex changeSelected: nil menu: #listMenu:)
		frame: (0@0 corner: 1@0.15).

	window addMorph: (PluggableTextMorph on: aSyntaxError text: #contents
			accept: #contents:notifying: readSelection: #contentsSelection
			menu: #codePaneMenu:shifted:)
		frame: (0@0.15 corner: 1@1).

	aSyntaxError addDependant: window.
	^ window openInWorldExtent: 380@220! !

!SyntaxError class methodsFor: 'instance creation' stamp: 'MikeRoberts 4/1/2009 21:30'!
errorInClass: aClass withCode: codeString doitFlag: doit
	"Open a view whose model is a syntax error. The error occurred when trying to add the given method code to the given class."

	self open:
		(self new setClass: aClass
			code: codeString
			debugger: (ToolSet debugContext: thisContext)
			doitFlag: doit).
! !

!SyntaxError class methodsFor: 'instance creation' stamp: 'JuanVuletich 11/1/2010 10:09'!
open: aSyntaxError 
	"Answer a standard system view whose model is an instance of me."
	<primitive: 19>
	"Simulation guard"
	self buildMorphicViewOn: aSyntaxError.
	UIManager default spawnNewProcessIfThisIsUI: Processor activeProcess.
	^ Processor activeProcess suspend! !

!SyntaxError class methodsFor: 'tools registry' stamp: 'IgorStasenko 2/19/2011 02:57'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]" 
	registry register: self as: #syntaxError
! !

!SyntaxError methodsFor: 'dependents access' stamp: 'GuillermoPolito 8/13/2010 01:10'!
addDependant: aDependant
	dependents add: aDependant! !

!SyntaxError methodsFor: 'menu' stamp: 'GuillermoPolito 8/13/2010 01:11'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[Browser fullOnClass: myClass selector: self selectedMessageName]! !

!SyntaxError methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
category: aSymbol
	"Record the message category of method being compiled. This is used when the user corrects the error and accepts."

	category := aSymbol.
! !

!SyntaxError methodsFor: 'accessing' stamp: 'StephaneDucasse 1/29/2011 12:08'!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	
	| donorMenu |
	donorMenu := shifted
				ifTrue: [self shiftedYellowButtonMenu]
				ifFalse: [self yellowButtonMenu].
	^ aMenu addAllFrom: donorMenu! !

!SyntaxError methodsFor: 'initialization' stamp: 'GuillermoPolito 8/13/2010 00:57'!
contents
	^contents! !

!SyntaxError methodsFor: 'initialization' stamp: 'GuillermoPolito 8/13/2010 00:57'!
contents: stringOrText
	contents := stringOrText! !

!SyntaxError methodsFor: 'other' stamp: 'StephaneDucasse 3/12/2011 13:34'!
contents: aString notifying: aController
	"Compile the code in aString and proceed. Do not notify anybody of errors, because nobody would have been notified of errors if this syntax error had not arisen"

	doitFlag
		ifTrue: [Compiler evaluate: aString]
		ifFalse: [class compile: aString classified: category].
	aController hasUnacceptedEdits: false.
	self proceed! !

!SyntaxError methodsFor: 'other' stamp: 'GuillermoPolito 8/13/2010 01:00'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	^ 1 to: 0  "null selection"! !

!SyntaxError methodsFor: 'menu' stamp: 'alain.plantec 6/1/2008 20:38'!
debug
	"Show the stack of the process leading to this syntax editor, typically showing the stack of the compiler as called from fileIn."

	debugger openFullNoSuspendLabel: 'Stack of the Syntax Error'.
! !

!SyntaxError methodsFor: 'accessing' stamp: 'GuillermoPolito 8/13/2010 01:10'!
initialize
	dependents := Set new.! !

!SyntaxError methodsFor: 'message list' stamp: 'MarcusDenker 2/21/2010 19:32'!
list
	"Answer an array of one element made up of the class name, message category, and message selector in which the syntax error was found. This is the single item in the message list of a view/browser on the receiver."

	selector ifNil: [^ Array with: (class name, '  ', (category ifNil: ['']), '  ', '<none>')].
	category ifNil: [^ Array with: (class name, '    ', '<none>')].
	^ Array with: (class name, '  ', category, '  ', selector)
! !

!SyntaxError methodsFor: 'message list' stamp: 'jm 5/3/1998 13:48'!
listIndex
	"There is always exactly one element in my list and it is always selected."

	^ 1
! !

!SyntaxError methodsFor: 'menu' stamp: 'StephaneDucasse 1/28/2011 16:54'!
listMenu: aMenu
	^ aMenu
		labels:
			({ 'Proceed' .
'Debug calling process' .
'Browse full'} collect: [ :l | l translated ])
		lines: #()
		selections: #(#proceed #debug #browseMethodFull)! !

!SyntaxError methodsFor: 'other' stamp: 'MikeRoberts 4/1/2009 21:35'!
notify: error at: location in: source
	"Open a syntax error view, inserting the given error message into the given source at the given location. This message is sent to the 'requestor' when the parser or compiler finds a syntax error."

	| aClass aString |
	aClass := thisContext sender receiver encoder classEncoding.
	aString :=
		source contents
			copyReplaceFrom: location
			to: location - 1
			with: error.
	self setClass: aClass
		code: aString
		debugger: (ToolSet debugContext: thisContext)
		doitFlag: false.
	self class open: self.
! !

!SyntaxError methodsFor: 'menu' stamp: 'di 5/5/1998 00:06'!
proceed
	"The user has has edited and presumably fixed the syntax error and the filein can now proceed."

	debugger proceed: self topView.
! !

!SyntaxError methodsFor: 'initialization' stamp: 'StephaneDucasse 2/6/2011 21:53'!
release
	"Prevent that syntaxError window lets a suspended compiler process running, when the window is closed."
	| p |
	(debugger isNil or: [	(p := debugger interruptedProcess) isNil 
							or: [p isTerminated]]) ifTrue: [^self].
       p terminate! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:15'!
selectedClass
	"Answer the class in which the syntax error occurred."

	^ class
! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:33'!
selectedClassOrMetaClass
	"Answer the class of the method being compiled."

	^ class
! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:17'!
selectedMessageName
	"Answer the selector of the method being compiled."

	^ selector
! !

!SyntaxError methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:30'!
setClass: aClass code: aString debugger: aDebugger doitFlag: flag

	| types printables badChar |
	class := aClass.
	debugger := aDebugger.
	selector := aClass parserClass new parseSelector: aString.
	types := Scanner classPool at: #TypeTable.	"dictionary"
	printables := '!!@#$%&*-_=+<>{}?/\,گ׿`~`' asSet.
	badChar := aString detect: [:aChar | (types at: aChar asciiValue ifAbsent: [#xLetter]) == #xBinary and: [
			(printables includes: aChar) not]] ifNone: [nil].
	contents := badChar 
		ifNil: [aString]
		ifNotNil: ['<<<This string contains a character (ascii value ', 
			badChar asciiValue printString,
			') that is not normally used in code>>> ', aString].
	category ifNil: [category := aClass organization categoryOfElement: selector].
	category ifNil: [category := ClassOrganizer default].
	doitFlag := flag! !

!SyntaxError methodsFor: 'accessing' stamp: 'StephaneDucasse 1/29/2011 12:09'!
shiftedYellowButtonMenu
	"Answer the menu to be presented when the yellow button is pressed while the shift key is down"

	^ MenuMorph fromArray: {
		
		{'Browse it (b)' translated.					#browseIt}.
		{'Senders of it (n)' translated.				#sendersOfIt}.
		{'Implementors of it (m)' translated.		#implementorsOfIt}.
		{'References to it (N)' translated.			#referencesToIt}.
		#-.
		{'Selectors containing it (W)' translated.	#methodNamesContainingIt}.
		{'Method strings with it (E)' translated.	#methodStringsContainingit}.
		{'Method source with it' translated.		#methodSourceContainingIt}.
		{'Class names containing it' translated.	#classNamesContainingIt}.
		{'Class comments with it' translated.		#classCommentsContainingIt}.
		{'Change sets with it' translated.			#browseChangeSetsWithSelector}.
	"	#-.
		{'Pretty print' translated.					#prettyPrint}.
		{'Pretty print with color' translated.		#prettyPrintWithColor}.
		{'File it in (G)' translated.					#fileItIn}.
		#-.
		{'Back...' translated.						#yellowButtonActivity}.
	"
	}
! !

!SyntaxError methodsFor: 'accessing' stamp: 'GuillermoPolito 8/13/2010 01:10'!
topView
	"Find the first top view on me. Is there any danger of their being two
	with the same model? Any danger from ungarbage collected old views?
	Ask if schedulled?"
	dependents
		ifNil: [^ nil].
	dependents
		do: [:v | (v isSystemWindow
					and: [v isInWorld])
				ifTrue: [^ v]].
	^ nil! !

!SyntaxError methodsFor: 'accessing' stamp: 'StephaneDucasse 1/28/2011 17:10'!
yellowButtonMenu
	^ MenuMorph fromArray: {
			{'Do it (d)' translated.					#doIt. nil. #smallDoItIcon}.
			{'Print it (p)' translated.					#printIt. nil. #smallPrintIcon}.
			{'Inspect it (i)' translated.				#inspectIt. nil. #smallInspectItIcon}.
			{'Explore it (I)' translated.				#exploreIt. nil. #smallInspectItIcon}.
			{'Debug it (D)' translated.				#debugIt. nil. #smallDebugItIcon}.
			{'Profile it' translated.					#tallyIt}.
		     {'Watch it' translated.					#watchIt}.	 
		    	 
			#-.
			{'Find...(f)' translated.					#find. nil. #smallFindIcon}.
			{'Find again (g)' translated.				#findAgain.nil. #smallFindIcon}.
			{'Extended search...' translated.			#shiftedTextPaneMenuRequest}.
			#-.
			{'Do again (j)' translated.				#again. nil. #smallDoItIcon}.
			{'Undo (z)' translated.					#undo. nil. #smallUndocon}.
			#-.
			{'Copy (c)' translated.					#copySelection. nil. #smallCopyIcon}.
			{'Cut (x)' translated.						#cut. nil. #smallCutIcon}.
			{'Paste (v)' translated.					#paste. nil. #smallPasteIcon}.
			{'Paste...' translated.					#pasteRecent. nil. #smallCopyIcon}.
			#-.
			{'Accept (s)' translated.					#accept. nil. #smallOkIcon}.
			{'Cancel (l)' translated.					#cancel. nil. #smallCancelIcon}.
		}.
! !

!ParagraphEditor methodsFor: '*tools' stamp: 'alain.plantec 6/11/2008 14:59'!
browseChangeSetsWithSelector
	"Determine which, if any, change sets have at least one change for the selected selector, independent of class"

	| aSelector |
	self lineSelectAndEmptyCheck: [^ self].
	(aSelector := self selectedSelector) == nil ifTrue: [^ self flash].
	self terminateAndInitializeAround: [ChangeSorter browseChangeSetsWithSelector: aSelector]! !

!ParagraphEditor methodsFor: '*tools' stamp: 'lr 3/14/2010 21:13'!
browseItHere
	"Retarget the receiver's window to look at the selected class, if appropriate.  3/1/96 sw"

	| aSymbol b |
	(((b := model) isKindOf: Browser) and: [ b couldBrowseAnyClass ])
		ifFalse: [ ^ self flash ].
	model okToChange
		ifFalse: [ ^ self flash ].
	self selectionInterval isEmpty
		ifTrue: [ self selectWord ].
	(aSymbol := self selectedSymbol) isNil
		ifTrue: [ ^ self flash ].
	self
		terminateAndInitializeAround: [ 
			| foundClass |
			foundClass := Smalltalk globals at: aSymbol ifAbsent: [ nil ].
			foundClass isNil
				ifTrue: [ ^ self flash ].
			(foundClass isKindOf: Class)
				ifTrue: [ 
					model systemCategoryListIndex: (model systemCategoryList indexOf: foundClass category).
					model classListIndex: (model classList indexOf: foundClass name) ] ]! !

!ParagraphEditor methodsFor: '*tools' stamp: 'MarianoMartinezPeck 1/19/2010 19:17'!
debug: aCompiledMethod receiver: anObject in: evalContext

	| selector guineaPig debugger context |
	selector := evalContext isNil ifTrue: [#DoIt] ifFalse: [#DoItIn:].
	anObject class addSelectorSilently: selector withMethod: aCompiledMethod.
	guineaPig := evalContext isNil
		ifTrue: [[anObject DoIt] newProcess]
		ifFalse: [[anObject DoItIn: evalContext] newProcess].
	context := guineaPig suspendedContext.
	debugger := ToolSet 
		debug: guineaPig
		controller: nil
		context: context.
	debugger openFullNoSuspendLabel: 'Debug it'.
	[debugger interruptedContext method == aCompiledMethod]
		whileFalse: [debugger send].
	anObject class basicRemoveSelector: selector! !

!ParagraphEditor methodsFor: '*tools' stamp: 'StephaneDucasse 3/1/2011 19:39'!
debugIt

	| method receiver context |
	(model respondsTo: #doItReceiver) 
		ifTrue: 
			[ receiver := model doItReceiver.
			context := model doItContext]
		ifFalse:
			[receiver := context := nil].
	self lineSelectAndEmptyCheck: [^self].
	method := self compileSelectionFor: receiver in: context.
	method notNil ifTrue:
		[self debug: method receiver: receiver in: context].! !
Debugger initialize!
ChangeList initialize!
Browser initialize!
FileContentsBrowser initialize!
FileList initialize!
CPUWatcher initialize!
StandardToolSet initialize!
ChangeSorter initialize!
ProcessBrowser initialize!
RecentMessageSet initialize!
